C ICAM (Idealized Coupleable Atmospheric Model)
C couplesrc $Revision: 1.6 $ created $Date: 1996/04/29 02:01:59 $
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C acouple.F: Fortran source module
C Coupler module for communicating with (or invoking)
C the atmospheric model from an ocean model.
C
 
C
C Top-level routines:
C ATMBGN: Initialize parameters for model integration/communication
C ATMINT: Integrates atmospheric model over one synchronization interval
C ATMDIA: Writes out diagnostic file for one part of run
C ATMEND: Terminates atmospheric model integration, saves model state
C
 
C C preprocessor options
C
C  OCEAN_ONLY: Define this option when this module is invoked
C          from a stand-alone ocean model that communicates with
C          an atmospheric model using a non-COMMON-block option.
C          Do not define this option when COMMON blocks are used
C          for communication between the atmosphere and the ocean,
C          or when creating a stand-alone atmospheric model
C          (which may or may not communicate with an ocean model).
C
C  MAXBAS: Maximum number of (rectangular) ocean basins
C
C  JOMAX:  Maximum number of temperature grid-points in X-direction
C           in ocean basin (>= 1)
C  KOMAX:  Maximum number of temperature grid-points in Y-direction
C           in ocean basin (>= 2)
C  NFOCN:  Maximum number of fluxes passed from atmosphere to ocean
C           (Tau-x, Tau-y, heat flux, E-P, tracer fluxes)
C  DOUBLEPREC: Defining this option allows an ocean model running at
C          double precision to communicate with an atmospheric model
C          running at single precision
C
 
#ifndef MAXBAS
#   define MAXBAS 1
#endif
#ifndef JOMAX
#   define JOMAX 1
#endif
#ifndef KOMAX
#   define KOMAX 65
#endif
#ifndef NFOCN
#   define NFOCN 4
#endif
 
 
      SUBROUTINE ATMBGN( ERRFLG, IASTEP, COMSTR, RUNNAM, IPART, IUNITR,
     1                   DIRSTR, OLDRUN, IOPART, NSAMPL, NSAVE )
C Initialize parameters for model integration/communication.
C 1. New run: read model parameters from model parameter file,
C    and set prognostic variables to default values.
C 2. Continuation run: read model parameters and prognostic
C    variables from the save file.
C 3. Restart run: read new model parameters from model parameter file,
C    and the prognostic variables from the save file.
C
C Input:  CHARACTER*(*) COMSTR -- communication option string
C         [Options 1: <blank>  => use COMMON blocks for communication;
C                  2: 'PVM'    => use PVM (not yet implemented);
C                  3: <other>  => use disk files for communication
C                        (COMSTR is assumed to contain the name of
C                         the directory where these files reside)]
C         CHARACTER*5 RUNNAM -- run name (used for filenames)
C         (the model parameter file is assumed to be named <rrrrr>.par,
C          where <rrrrr> denotes the run name)
C         INTEGER IPART -- new run part number (>=0, used for filenames)
C         (the model save files are named <rrrrrpppp>.sv,
C          where <pppp> denotes the part number)
C         [set to zero for new/restart run]
C         INTEGER IUNITR -- starting Fortran unit number for model use
C (11 unit numbers, from IUNITR to IUNITR+10 are reserved for model use)
C         CHARACTER*(*) DIRSTR -- name of directory containing
C            the model parameter file and other initialization files.
C         CHARACTER*5 OLDRUN   -- old run name
C         (set to non-blank value for restart runs)
C         INTEGER IOPART -- old run part number (for restart runs only)
C         [The following two parameters, NSAMPL and NSAVE,
C          are meaningful only if the C-preprocessor DIAGNOS option
C          is defined]
C         INTEGER NSAMPL -- sampling interval, in time-steps (> 0)
C             Model data are sampled every NSAMPL time-steps
C             for averaging horizontal section data.
C         INTEGER NSAVE  -- horizontal section data saving interval,
C             in units of NSAMPL time-steps (>=0). Horizontal
C             section data are saved every NSAMPL*NSAVE time-steps,
C             after averaging over the NSAVE samples.
C            [set to zero to suppress horizontal section saves]
C         (the horizontal section save files are named <rrrrrpppp>.hs)
C Output: LOGICAL ERRFLG -- error flag (TRUE on error)
C         INTEGER IASTEP -- current atmospheric model time-step count
C         (Set to -1 for ocean-only version)
C
C
#include "mcouple.h"
C
      LOGICAL ERRFLG
      CHARACTER COMSTR*(*), RUNNAM*5
      INTEGER IPART, IUNITR
      CHARACTER DIRSTR*(*), OLDRUN*5
      INTEGER IOPART, NSAMPL, NSAVE, IASTEP
C
#ifndef OCEAN_ONLY
      INTEGER IUSAV
      LOGICAL RESTRT
      CHARACTER SAVFIL*12, PARFIL*9
#endif
C
C     Clear error flag
      ERRFLG= .FALSE.
C
C     Save run info in COMMON block
      IUNIT0= IUNITR
      IPART0= IPART
      RUNNM0= RUNNAM
      NSAMP0= NSAMPL
      NSAVE0= NSAVE
C     Diagnostics file save flag
      DIAFLG= .TRUE.
C
C     Initialize packet communications;
      CALL INIPAK( COMSTR, RUNNAM )
C
#ifndef OCEAN_ONLY
C
C     Initialize atmospheric model
      IF (IPART .GT. 0) THEN
C
C       Continuation run; open save file
        RESTRT= .FALSE.
        WRITE(SAVFIL,'(4X,I5,A3)') 10000+IPART-1, '.sv'
        SAVFIL(1:5)= RUNNAM
        IUSAV= IUNIT0
        OPEN( IUSAV, FILE=SAVFIL, STATUS='OLD' )
        WRITE(*,'(A,A)') 'Continuing run from save file ', SAVFIL
      ELSE IF (OLDRUN .NE. ' ') THEN
C
C       Restart run; open save file
        RESTRT= .TRUE.
        WRITE(SAVFIL,'(4X,I5,A3)') 10000+IOPART, '.sv'
        SAVFIL(1:5)= OLDRUN
        IUSAV= IUNIT0
        OPEN( IUSAV, FILE=SAVFIL, STATUS='OLD' )
        WRITE(*,'(A,A)') 'Restarting model from save file ', SAVFIL
      ELSE
C
C       New run; no save file
        RESTRT= .FALSE.
        IUSAV= 0
        WRITE(*,*) 'Starting new run'
      ENDIF
C
C     Create model parameter file name
      PARFIL= RUNNAM//'.par'
      WRITE(*,'(A,A)') 'Model parameter file = ', PARFIL
C
C     Initialize atmospheric model parameters
      CALL ATMINI( RESTRT, IUNIT0+1, IUSAV, PARFIL,
     1             DIRSTR, NSAMPL, NSAVE, DIAFLG,
     2             IASTEP, ERRFLG )
      IF (ERRFLG) RETURN
C
C     Close save file, if any
      IF (IUSAV .GT. 0) CLOSE( IUSAV )
#else
C     No time-step count available for ocean-only version
      IASTEP= -1
#endif
C
      RETURN
      END
 
      SUBROUTINE ATMINT( SYNFLG, IOSTEP, NOSYNC, NASYNC, NXMAX, NX, NY,
     1                   X, XMID, Y, YMID, SST,
     2                   HEAT, EMINP, TAUX, TAUY, ERRFLG )
C Integrates the atmospheric model to the end of one synchronization
C interval using the supplied SST values and returns the surface fluxes
C HEAT, EMINP, TAUX, and TAUY at the end of the current synchronization
C interval for synchronous integrations (SYNFLG=.TRUE.) or at the end of
C previous synchronization interval for asynchronous (SYNFLG=.FALSE.)
C integrations, i.e., without waiting.
C (All the REAL input/output parameters to this routine can be declared
C  to be double precision through the DOUBLEPREC option, thus allowing
C  it to be called from an ocean model running at double precision.)
C
C Input:  LOGICAL SYNFLG -- synchronous integration flag
C                           (set to TRUE for synchronous integrations)
C         INTEGER IOSTEP -- current oceanic time-step count
C                 (should be zero at the beginning of the run,
C                  and be a multiple of NOSYNC otherwise.
C                  a special value of IOSTEP=-1 is allowed before
C                  the very first time-step of asynchronous
C                  integrations, in which case the SSTs are sent
C                  to the atmospheric model, but no flux values are
C                  returned)
C         INTEGER NOSYNC -- no. of oceanic time-steps between
C                           synchronizations (>= 1)
C         INTEGER NASYNC -- no. of atmospheric time-steps between
C                           synchronizations (>= 1)
C         INTEGER NXMAX   -- declared X dimension
C         INTEGER NX, NY  -- actual no. of X/Y ocean grid-points
C         REAL X(NX)      -- Longitudes of grid-points (degrees)
C         REAL XMID(0:NX) -- Longitude of grid-box edges (degrees)
C         REAL Y(NY)      -- Sine-latitude of grid-points
C         REAL YMID(0:NY) -- Sine-latitude of grid-box edges
C [NOTE: Ocean grid-points correspond to the temperature grid;
C        ocean grid box edges correspond to the velocity grid ]
C         REAL SST(NXMAX,NY)  -- oceanic SST values (K)
C [NOTE: SST values should be set to zero over land points]
C Output: (NOTE: no flux values are returned if IOSTEP=-1)
C         REAL HEAT(NXMAX,NY)  -- heat flux into ocean (W/m**2)
C         REAL EMINP(NXMAX,NY) -- evaporation-precipitation
C                                   over ocean, with runoff included
C                                  (kg/(s*m**2))
C         REAL TAUX(NXMAX,NY)  -- zonal wind stress over ocean (Pa)
C         REAL TAUY(NXMAX,NY)  -- meridional wind stress over ocean (Pa)
C [NOTE:  The wind-stress values TAUx/y correspond to the mid-point grid
C         XMID(1:*), YMID(1:*), as initialized by ATMBGN.
C         Therefore, the effective array size is usually (NX-1)*(NY-1),
C         Except for ocean basins that are periodic in the X-direction,
C         where the effective array size is NX*(NY-1).
C         The extra values corresponding to TAUx/y(*,NY) are the total
C         wind-stress over the two half grid-intervals at the northern
C         and southern boundaries of the basin. For ocean basins that
C         are not periodic in the X-direction, the extra values
C         corresponding to TAUx/y(*,NX) are the total wind-stress over
C         the two half grid-intervals at the western and eastern
C         boundaries of the ocean basin. These values may be useful in
C         checking flux conservation.
C         LOGICAL ERRFLG -- error flag (TRUE on error)
C
#include "mcouple.h"
C
      INTEGER IOSTEP, NOSYNC, NASYNC
      INTEGER NXMAX, NX, NY
      LOGICAL SYNFLG, ERRFLG
#ifdef DOUBLEPREC
      DOUBLE PRECISION X(NX), XMID(0:NX)
      DOUBLE PRECISION Y(NY), YMID(0:NY), SST(NXMAX,NY)
      DOUBLE PRECISION HEAT(NXMAX,NY), EMINP(NXMAX,NY)
      DOUBLE PRECISION TAUX(NXMAX,NY), TAUY(NXMAX,NY)
#else
      REAL X(NX), XMID(0:NX)
      REAL Y(NY), YMID(0:NY), SST(NXMAX,NY)
      REAL HEAT(NXMAX,NY), EMINP(NXMAX,NY)
      REAL TAUX(NXMAX,NY), TAUY(NXMAX,NY)
#endif
C
      REAL XBAS(JOMAX), XBMID(0:JOMAX)
      REAL YBAS(KOMAX), YBMID(0:KOMAX), SSTBAS(JOMAX,KOMAX)
      REAL FLXTOT(NFOCN), FLXBAS(JOMAX,KOMAX,NFOCN)
      INTEGER NBASX, NBASY, NFLX
      INTEGER JO, KO, IPACKT
C
C     Clear error flag
      ERRFLG= .FALSE.
C
C     Check basin resolution
      IF ( (NX .GT. JOMAX) .OR. (NY .GT. KOMAX) ) THEN
        WRITE(*,*) 'ATMINT: JOMAX/KOMAX too small for basin; recompile'
        STOP
      ENDIF
C
C     Check time-stepping information
      IF (IOSTEP .EQ. -1) THEN
        IF (SYNFLG) STOP 'SYNFLG'
      ELSE IF (MOD(IOSTEP,NOSYNC) .NE. 0) THEN
        WRITE(*,*) 'ATMINT: IOSTEP out of sync'
        STOP
      ENDIF
C
C     Compute packet number (0,1,...)
      IF (SYNFLG) THEN
C       Synchronous integration
        IPACKT= IOSTEP / NOSYNC
      ELSE
C       Asynchronous integration
        IF (IOSTEP .EQ. -1) THEN
          IPACKT= 0
        ELSE
          IPACKT= 1 + IOSTEP / NOSYNC
        ENDIF
      ENDIF
C
C     Copy coordinate information and SST values
      DO 110 JO=1,NX
        XBAS(JO)= X(JO)
  110   XBMID(JO)= XMID(JO)
      XBMID(0)= XMID(0)
C
      DO 120 KO=1,NY
        YBAS(KO)= Y(KO)
  120   YBMID(KO)= YMID(KO)
      YBMID(0)= YMID(0)
C
      DO 130 KO=1,NY
        DO 130 JO=1,NX
  130     SSTBAS(JO,KO)= SST(JO,KO)
C
C     Send SST to atmospheric model
      CALL SNDSST( IUNIT0, IPACKT, 1, JOMAX, NX, NY,
     1             XBAS, XBMID, YBAS, YBMID, SSTBAS,
     2             IOSTEP, NOSYNC, ERRFLG)
      IF (ERRFLG) RETURN
C
      IF (SYNFLG) THEN
C       Synchronous integration; increment packet number
        IPACKT= IPACKT + 1
      ENDIF
C
#ifndef OCEAN_ONLY
C
C     Integrate the atmospheric model to end of synchronization interval
      CALL ATMRUN( SYNFLG, IPACKT*NASYNC, ' ', ' ', ERRFLG )
      IF (ERRFLG) RETURN
#endif
C
C     Return, if first packet of asynchronous integration
      IF (.NOT.SYNFLG .AND. (IPACKT .EQ. 0)) RETURN
C
C     Receive fluxes from atmospheric model
      CALL RCVFLX( IUNIT0, IPACKT, 1, JOMAX, KOMAX, NFOCN,
     1             FLXTOT, FLXBAS, NBASX, NBASY, NFLX,
     2             ERRFLG)
      IF (ERRFLG) RETURN
C
C     Check flux array size
      IF ( (NBASX .NE. NX) .OR.
     1     (NBASY .NE. NY) .OR.
     2     (NFLX  .NE. NFOCN )   ) THEN
        WRITE(*,*) 'ATMINT: Flux array size mismatch'
        STOP
      ENDIF
C
C     Copy surface wind stress
      DO 210 KO=1,NY
        DO 210 JO=1,NX
          TAUX(JO,KO)= FLXBAS(JO,KO,1)
  210     TAUY(JO,KO)= FLXBAS(JO,KO,2)
C
C     Copy heat/E-P fluxes
      DO 220 KO=1,NY
        DO 220 JO=1,NX
          HEAT(JO,KO)= FLXBAS(JO,KO,3)
  220     EMINP(JO,KO)= FLXBAS(JO,KO,4)
C
C
      RETURN
      END
 
      SUBROUTINE ATMDIA
C Terminates one part of atmospheric model integration, and writes
C out diagnostic file, if diagnostics were initialized. Also,
C re-initializes diagnostics for the next part of integration.
C
C Input: (mcouple.h)
C
#include "mcouple.h"
C
#ifndef OCEAN_ONLY
      CHARACTER DIAFIL*72
#endif
C
#ifndef OCEAN_ONLY
C
C     Terminate run part
      CALL ATMFIN( 0, DIAFIL )
      IF (DIAFIL .NE. ' ') THEN
        WRITE(*,*) 'Model diagnostics written to file ', DIAFIL
      ENDIF
C     Increment part count in mcouple.h
      IPART0= IPART0 + 1
C     Re-initialize diagnostics
      CALL ATMDIN( NSAMP0, NSAVE0, DIAFLG )
#endif
C
      RETURN
      END
 
      SUBROUTINE ATMEND
C Ends atmospheric model integration by saving the model state
C to a file, and possibly writing out model diagnostics to a file.
C
C Input: (mcouple.h)
C Output: (Save file is written to <rrrrrpppp>.sv,
C          where <rrrrr> denotes the run name, and <pppp> denotes
C          the run part number)
C
#include "mcouple.h"
C
#ifndef OCEAN_ONLY
      INTEGER IUSAV
      CHARACTER SAVFIL*12, DIAFIL*72
#endif
C
#ifndef OCEAN_ONLY
C
C     Create save file name
      WRITE(SAVFIL,'(4X,I5,A3)') 10000+IPART0, '.sv'
      SAVFIL(1:5)= RUNNM0
C     Open save file
      IUSAV= IUNIT0
      OPEN( IUSAV, FILE=SAVFIL, STATUS='NEW' )
C     Terminate model integration, save model state to file
      CALL ATMFIN( IUSAV, DIAFIL )
C     Close save file
      CLOSE( IUSAV )
C
      WRITE(*,*) 'Model state saved to file ', SAVFIL
#endif
C
C     End packet communications
      CALL ENDPAK
C
      RETURN
      END
 
C --------------------------------------------------------------------
C For GNU Emacs:
C $Msg-digest-checksum: cce056a8854f899728d22d3c3083dc4d $
C --------------------------------------------------------------------
C Local Variables:
C msg-digest-active: t
C End:
C --------------------------------------------------------------------
