C ICAM (Idealized Coupleable Atmospheric Model)
C fluxsrc $Revision: 1.1 $ created $Date: 1996/01/28 20:53:43 $
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C aflux.F: Fortran source module
C Contains interpolation routines for exchanging fluxes/SST
C between the atmosphere and the ocean
C
 
#include "mmain.h"
 
C Routines:
C INIFLX: Initialize flux/SST exchange module
C         (called by ATMINI)
C UPDSST: Update surface temperature by communicating with ocean model
C         (called by ATMRUN)
C FLXOUT: Send surface fluxes to ocean model
C         (called by ATMRUN)
C
C Internal routine:
C SETINT: Set-up interpolation arrays for an ocean basin
C         (called by UPDSST)
C
 
      SUBROUTINE INIFLX
C Initialize flux/SST exchange module
C
C Input:  (mparams.h)
C Output: (mflux.h, minterp.h)
C
#include "mparams.h"
#include "mflux.h"
#include "minterp.h"
C
      INTEGER IB
C
C     For all ocean basins
      DO 10 IB=1,NBASIN
C       Clear flux accumulator
        NFACC(IB)= 0
C       Clear interpolation coefficients table
        INTINI(IB)= .FALSE.
   10   CONTINUE
C
      RETURN
      END
 
      SUBROUTINE UPDSST( ISTP, TSURF, ERRFLG)
C Update surface temperature by communicating with ocean model
C
C Input:  INTEGER ISTP -- model time-step count
C         REAL TSURF(K1MAX,K2MAX) -- current surface temperature (K)
C Output: REAL TSURF(K1MAX,K2MAX) -- surface temperatures
C           updated over regions with prognostic SST
C           [updates occur only when MOD(ISTP,NYSNC) = 0]
C         LOGICAL ERRFLG -- error flag
C
#include "mgrid.h"
#include "mparams.h"
#include "minterp.h"
C
      INTEGER ISTP
      REAL TSURF(K1MAX,K2MAX)
      LOGICAL ERRFLG
C
C     Parameters determining which power of SST is to be interpolated
      INTEGER NP
      REAL PINV
      PARAMETER (NP=4, PINV=0.25)
C
      REAL SSTBAS(JOMAX,KOMAX)
      REAL XBAS(JOMAX), XBMID(0:JOMAX)
      REAL YBAS(KOMAX), YBMID(0:KOMAX)
      REAL T4ATM(0:JBMAX+1,0:KBMAX+1)
      INTEGER NBASX, NBASY
      INTEGER IB, IPACKT, JO, KO, JA, KA, JB, KB
C
#ifdef ZONAVG
      REAL ZT4ATM(KBMAX), ZT4OCN(KOMAX)
#else
      REAL T4OCN(0:JOMAX+1,0:KOMAX+1)
#endif
C
#ifdef DEBUG
      REAL SUM, AREAWT
#endif
C
C
C     Clear error flag
      ERRFLG= .FALSE.
C
      DO 500 IB=1,NBCOUP
C       For each coupled ocean basin
        IF (MOD(ISTP, NSYNC) .EQ. 0) THEN
C
C         Compute packet number (0,1,...)
          IPACKT= ISTP / NSYNC
C         Receive SST values from ocean model for each basin
          CALL RCVSST( IUPAK, IPACKT, IB, JOMAX, KOMAX,
     1                 NBASX, NBASY, XBAS, XBMID, YBAS, YBMID,
     2                 SSTBAS, ERRFLG )
          IF (ERRFLG) RETURN
C
C         Check if interpolation coefficients are already set-up
          IF (.NOT. INTINI(IB)) THEN
C           Set-up interpolation coefficients for this basin
            CALL SETINT( IB, NBASX, NBASY,
     1                   XBAS, XBMID, YBAS, YBMID )
          ENDIF
C
#ifdef DEBUG
          WRITE(*,*)
          WRITE(*,*) 'Subroutine UPDSST: debugging information'
          WRITE(*,*) 'Received SST packet ', IPACKT, ' for basin ', IB
C
C         Compute area-weighted integral of interpolant
          SUM= 0.0
          DO 105 KO=1,NOCNY(IB)
            DO 105 JO=1,NOCNX(IB)
C             Take NP-th power of T
              SUM= SUM + (SSTBAS(JO,KO)**NP)
     1                  *(VGRIDY(KO)-VGRIDY(KO-1))
     2                  *(VGRIDX(JO)-VGRIDX(JO-1))
  105         CONTINUE
          WRITE(*,'(1X,A,1PE15.6)')
     1            'SST**NP integral prior to interpolation = ', SUM
#endif
C
C
C         Copy current surface temperature**NP
          DO 120 KB=1,NATMY(IB)
            KA= KB + KB0(IB)
            DO 110 JB=1,NATMX(IB)
              JA= 1 + MOD(JB-1+JB0(IB), K1)
C             Take NP-th power of T
  110         T4ATM(JB,KB)= TSURF(JA,KA)**NP
C
            IF (CYCLIC(IB)) THEN
C             Cyclic basin; periodic padding in X-direction
              T4ATM(0,KB)= T4ATM(NATMX(IB),KB)
              T4ATM(NATMX(IB)+1,KB)= T4ATM(1,KB)
            ELSE
C             Non-cyclic basin; duplicate boundary values
              T4ATM(0,KB)= T4ATM(1,KB)
              T4ATM(NATMX(IB)+1,KB)= T4ATM(NATMX(IB),KB)
            ENDIF
  120       CONTINUE
C
C         Padding in Y-direction; duplicate boundary values
          DO 130 JB=0,NATMX(IB)+1
            T4ATM(JB,0)= T4ATM(JB,1)
  130       T4ATM(JB,NATMY(IB)+1)= T4ATM(JB,NATMY(IB))
C
#ifdef ZONAVG
C
C *** ZONALLY SYMMETRIC SST FIELD ***
C
C         Copy current surface temperature**NP (from basin edge)
          DO 210 KB=1,NATMY(IB)
  210       ZT4ATM(KB)= T4ATM(1,KB)
C
C         Interpolate current temperature**NP to oceanic grid
C         (for smoother interpolation back to atmospheric grid)
          CALL FLUX1D( ZT4OCN(1), NOCNY(IB),
     1                 ZT4ATM(1), NATMY(IB),
     2                 MINA2O(0,IB), FINA2O(0,IB),
     3                 DYNA2O(1,IB), DYA2O(0,IB),
     4                 LWTA2O(1,IB), RWTA2O(1,IB) )
C
C         Copy non-zero values of SST**NP from packet
          DO 220 KO=1,NOCNY(IB)
            IF (SSTBAS(1,KO) .NE. 0.) THEN
C             Take NP-th power of T
              ZT4OCN(KO)= SSTBAS(1,KO)**NP
            ENDIF
  220       CONTINUE
C
C         Interpolate temperature**NP to atmospheric grid
          CALL FLUX1D( ZT4ATM(1), NATMY(IB),
     1                 ZT4OCN(1), NOCNY(IB),
     2                 MINO2A(0,IB), FINO2A(0,IB),
     3                 DYNO2A(1,IB), DYO2A(0,IB),
     4                 LWTO2A(1,IB), RWTO2A(1,IB) )
C
C         Copy zonally-symmetric temperature**NP
          DO 230 KB=1,NATMY(IB)
            DO 230 JB=1,NATMX(IB)
  230         T4ATM(JB,KB)= ZT4ATM(KB)
C
#else
C *** 2-D SST FIELD ***
C
C         Interpolate current temperature**NP to oceanic grid
C         (for smoother interpolation back to atmospheric grid)
          CALL FLUX2D( T4OCN(1,1), JOMAX+2, NOCNX(IB), NOCNY(IB),
     1                 T4ATM(0,0), JBMAX,   NATMX(IB), NATMY(IB),
     2         MXA2O(0,IB), FXA2O(1,0,IB), CFXA2O(1,0,IB), DXA2O(0,IB),
     3         MYA2O(0,IB), FYA2O(1,0,IB), CFYA2O(1,0,IB), DYA2O(0,IB),
     4         YWTA2O(0,IB) )
C
C         Copy non-zero values of SST**NP from packet
          DO 310 KO=1,NOCNY(IB)
            DO 310 JO=1,NOCNX(IB)
              IF (SSTBAS(JO,KO) .NE. 0.) THEN
C               Take NP-th power of T
                T4OCN(JO,KO)= SSTBAS(JO,KO)**NP
              ENDIF
  310         CONTINUE
C
          DO 320 KO=1,NOCNY(IB)
            IF (CYCLIC(IB)) THEN
C             Cyclic basin; periodic padding in X-direction
              T4OCN(0,KO)= T4OCN(NOCNX(IB),KO)
              T4OCN(NOCNX(IB)+1,KO)= T4OCN(1,KO)
            ELSE
C             Non-cyclic basin; duplicate boundary values
              T4OCN(0,KO)= T4OCN(1,KO)
              T4OCN(NOCNX(IB)+1,KO)= T4OCN(NOCNX(IB),KO)
            ENDIF
  320       CONTINUE
C
C         Padding in Y-direction; duplicate boundary values
          DO 330 JO=0,NOCNX(IB)+1
            T4OCN(JO,0)= T4OCN(JO,1)
  330       T4OCN(JO,NOCNY(IB)+1)= T4OCN(JO,NOCNY(IB))
C
C         Interpolate temperature**NP to atmospheric grid
          CALL FLUX2D( T4ATM(1,1), JBMAX+2, NATMX(IB), NATMY(IB),
     1                 T4OCN(0,0), JOMAX,   NOCNX(IB), NOCNY(IB),
     2         MXO2A(0,IB), FXO2A(1,0,IB), CFXO2A(1,0,IB), DXO2A(0,IB),
     3         MYO2A(0,IB), FYO2A(1,0,IB), CFYO2A(1,0,IB), DYO2A(0,IB),
     4         YWTO2A(0,IB) )
C
#endif
C         Compute and copy interpolated temperature
          DO 400 KB=1,NATMY(IB)
            KA= KB + KB0(IB)
            DO 400 JB=1,NATMX(IB)
              JA= 1 + MOD(JB-1+JB0(IB), K1)
C             Update only points with prognostic SST
              IF (PROSST(JA,KA)) THEN
C               Take NP-th root of T
                TSURF(JA,KA)= T4ATM(JB,KB)**PINV
              ENDIF
  400         CONTINUE
C
#ifdef DEBUG
C         Compute area-weighted integral after interpolation
          SUM= 0.0
          DO 403 KB=1,NATMY(IB)
            KA= KB + KB0(IB)
C           Area weight (MU * degrees latitude; total = 720)
            AREAWT= G(KA) * 720. / FLOAT(K1)
            DO 403 JB=1,NATMX(IB)
              JA= 1 + MOD(JB-1+JB0(IB), K1)
C             Sum only points with prognostic SST
              IF (PROSST(JA,KA)) THEN
                SUM= SUM + AREAWT*T4ATM(JB,KB)
              ENDIF
  403         CONTINUE
          WRITE(*,'(1X,A,1PE15.6)')
     1            'SST**NP integral after interpolation    = ', SUM
#endif
        ENDIF
  500   CONTINUE
C
      RETURN
      END
 
      SUBROUTINE FLXOUT( ISTP, IB, OCFLUX, ERRFLG )
C Send surface fluxes to ocean model
C
C Input:  INTEGER ISTP -- new model time-step count (should be >= 1)
C         INTEGER IB   -- ocean basin number
C         REAL OCFLUX(JBMAX,KBMAX,JTAUX:NFOCN)
C                      -- surface fluxes over ocean basin,
C                         as computed by routine FLXCAL
C Output: LOGICAL ERRFLG -- error flag
C [ Fluxes are accumulated and periodically sent to ocean model,
C   after interpolating to the ocean grid ]
C
#include "mgrid.h"
#include "mparams.h"
#include "mflux.h"
#include "minterp.h"
C
      INTEGER ISTP, IB
      REAL OCFLUX(JBMAX,KBMAX,JTAUX:NFOCN)
      LOGICAL ERRFLG
C
      REAL FLXTOT(JTAUX:NFOCN), FLXBAS(JOMAX,KOMAX,JTAUX:NFOCN)
      REAL AREAWT
      INTEGER NXB, NYB, NXO, NYO
      INTEGER IFX, IPACKT, JA, KA, JB, KB, JO, KO
#ifdef ZONAVG
      REAL ZOCFLX(KBMAX), ZFXBAS(KOMAX+1)
#else
      REAL TEMFXA(0:JBMAX+1,0:KBMAX+1), TEMFXO(JOMAX+1,KOMAX+1)
#endif
#ifdef DEBUG
      REAL SUM
#endif
C
C
C     Clear error flag
      ERRFLG= .FALSE.
C
C     Basin extent, atmospheric coordinates
      NXB= NATMX(IB)
      NYB= NATMY(IB)
C     Basin extent, oceanic coordinates
      NXO= NOCNX(IB)
      NYO= NOCNY(IB)
C
C
      IF (MOD(ISTP-1, NSYNC) .EQ. 0) THEN
C       Initialize flux accumulators
        IF (NFACC(IB) .GT. 0) THEN
          CALL ATMERR( 'FLXOUT', 'Flux accumulator not cleared' )
        ENDIF
        NFACC(IB)= 1
        IFACC0(IB)= ISTP-1
        IFACC1(IB)= ISTP-1
        DO 20 IFX=JTAUX,NFOCN
          DO 20 KB=1,NYB
            DO 20 JB=1,NXB
   20         BOFLUX(JB,KB,IFX,IB)= OCFLUX(JB,KB,IFX)
      ELSE
C       Accumulate fluxes
        IF (NFACC(IB) .LE. 0) THEN
          CALL ATMERR( 'FLXOUT', 'Flux accumulator empty' )
        ENDIF
        NFACC(IB)= NFACC(IB) + 1
        IFACC1(IB)= ISTP-1
        DO 30 IFX=JTAUX,NFOCN
          DO 30 KB=1,NYB
            DO 30 JB=1,NXB
              BOFLUX(JB,KB,IFX,IB)=  BOFLUX(JB,KB,IFX,IB)
     1                             + OCFLUX(JB,KB,IFX)
   30         CONTINUE
      ENDIF
C
      IF (MOD(ISTP, NSYNC) .EQ. 0) THEN
C       Normalize fluxes
        DO 40 IFX=JTAUX,NFOCN
          DO 40 KB=1,NYB
            DO 40 JB=1,NXB
   40         BOFLUX(JB,KB,IFX,IB)= BOFLUX(JB,KB,IFX,IB) / NFACC(IB)
C       Reset flux accumulator
        NFACC(IB)= -1
C
C       Prepare to send flux data packet
C       Compute packet number (1,...)
        IPACKT= ISTP / NSYNC
C
C       Compute area-weighted flux integral (over ocean points only)
        DO 70 IFX=JTAUX,NFOCN
          FLXTOT(IFX)= 0.0
          DO 50 KB=1,NYB
            KA= KB + KB0(IB)
C           Area weight (MU * degrees latitude; total = 720)
            AREAWT= G(KA) * 720. / FLOAT(K1)
            DO 50 JB=1,NXB
              JA= 1 + MOD(JB-1+JB0(IB), K1)
              IF (SEAMSK(JA,KA)) THEN
                FLXTOT(IFX)= FLXTOT(IFX) + AREAWT*BOFLUX(JB,KB,IFX,IB)
              ENDIF
   50         CONTINUE
C
#ifdef DEBUG
C         Display area-weighted integral of interpolant
          WRITE(*,*)
          WRITE(*,*) 'Subroutine FLXOUT: debugging information'
          WRITE(*,'(1X,A,I2,A,1PE15.6)')
     1   'Flux integral', IFX, ' before interpolation = ', FLXTOT(IFX)
#endif
C
   70         CONTINUE
#ifdef ZONAVG
C *** ZONALLY SYMMETRIC FLUX FIELDS ***
C
C       First interpolate wind stress to velocity grid
        DO 150 IFX=JTAUX,JTAUY
C         Copy zonally-uniform fluxes from basin edge
          DO 110 KB=1,NYB
  110       ZOCFLX(KB)= BOFLUX(1,KB,IFX,IB)
C         Interpolate to ocean velocity grid
          CALL FLUX1D( ZFXBAS(1), NYO+1,
     1                 ZOCFLX(1), NYB,
     2                 MINA2V(0,IB), FINA2V(0,IB),
     3                 DYNA2V(1,IB), DYA2V(0,IB),
     4                 LWTA2V(1,IB), RWTA2V(1,IB) )
C
C         Copy zonally-uniform fluxes (offset 1 in Y-direction)
          DO 120 KO=1,NYO-1
            DO 120 JO=1,NXO
  120         FLXBAS(JO,KO,IFX)= ZFXBAS(KO+1)
C         Fold back extra Y-edge value
          DO 130 JO=1,NXO
            FLXBAS(JO,NYO,IFX)=
     1             YFFAC(2,IB)*ZFXBAS(NYO+1)
     2           + YFFAC(1,IB)*ZFXBAS(1)
  130       CONTINUE
#ifdef DEBUG
C         Compute area-weighted flux integral after interpolation
          SUM= 0.0
          DO 140 KO=1,NYO+1
            SUM= SUM + ZFXBAS(KO)
     1                * (TGRIDY(KO) - TGRIDY(KO-1))
  140       CONTINUE
          SUM= SUM * NXB * (360. / K1)
          WRITE(*,'(1X,A,I2,A,1PE15.6)')
     1     'Flux integral', IFX, ' after interpolation  = ', SUM
#endif
  150     CONTINUE
C
C       Next, interpolate scalar fluxes to ocean temperature grid
        DO 190 IFX=JFS,NFOCN
C         Copy zonally-uniform fluxes from basin edge
          DO 160 KB=1,NYB
  160       ZOCFLX(KB)= BOFLUX(1,KB,IFX,IB)
C         Interpolate to temperature grid
          CALL FLUX1D( ZFXBAS(1), NYO,
     1                 ZOCFLX(1), NYB,
     2                 MINA2O(0,IB), FINA2O(0,IB),
     3                 DYNA2O(1,IB), DYA2O(0,IB),
     4                 LWTA2O(1,IB), RWTA2O(1,IB) )
C
C         Copy zonally-uniform fluxes to whole basin
          DO 170 KO=1,NYO
            DO 170 JO=1,NXO
  170         FLXBAS(JO,KO,IFX)= ZFXBAS(KO)
#ifdef DEBUG
C         Compute area-weighted flux integral after interpolation
          SUM= 0.0
          DO 180 KO=1,NYO
            SUM= SUM + ZFXBAS(KO)
     1                * (VGRIDY(KO) - VGRIDY(KO-1))
  180       CONTINUE
          SUM= SUM * NXB * (360. / K1)
          WRITE(*,'(1X,A,I2,A,1PE15.6)')
     1   'Flux integral', IFX, ' after interpolation  = ', SUM
#endif
  190     CONTINUE
C
#else
C *** 2-D FLUX FIELDS ***
C
        DO 290 IFX=JTAUX,NFOCN
C         Copy flux on atmospheric grid (for padding)
          DO 210 KB=1,NYB
            DO 210 JB=1,NXB
  210         TEMFXA(JB,KB)= BOFLUX(JB,KB,IFX,IB)
C
          DO 220 KB=1,NYB
            IF (CYCLIC(IB)) THEN
C             Cyclic basin; periodic padding in X-direction
              TEMFXA(0,KB)= TEMFXA(NXB,KB)
              TEMFXA(NXB+1,KB)= TEMFXA(1,KB)
            ELSE
C             Non-cyclic basin; duplicate boundary values
              TEMFXA(0,KB)= TEMFXA(1,KB)
              TEMFXA(NXB+1,KB)= TEMFXA(NXB,KB)
            ENDIF
  220       CONTINUE
C
C         Padding in Y-direction; duplicate boundary values
          DO 230 JB=0,NXB+1
            TEMFXA(JB,0)= TEMFXA(JB,1)
  230       TEMFXA(JB,NYB+1)= TEMFXA(JB,NYB)
C
          IF ( (IFX .EQ. JTAUX) .OR. (IFX .EQ. JTAUY) ) THEN
C
C           Wind stress; interpolate to oceanic velocity grid
            CALL FLUX2D( TEMFXO(1,1), JOMAX+1, NXO+1, NYO+1,
     1                   TEMFXA(0,0), JBMAX,   NXB,   NYB,
     2         MXA2V(0,IB), FXA2V(1,0,IB), CFXA2V(1,0,IB), DXA2V(0,IB),
     3         MYA2V(0,IB), FYA2V(1,0,IB), CFYA2V(1,0,IB), DYA2V(0,IB),
     4         YWTA2V(0,IB) )
C           Copy flux (offset 1 in X/Y direction)
            DO 240 KO=1,NYO-1
              DO 240 JO=1,NXO-1
  240           FLXBAS(JO,KO,IFX)= TEMFXO(JO+1,KO+1)
C           Fold back extra value at Y-edge
            DO 250 JO=1,NXO-1
              FLXBAS(JO,NYO,IFX)=  YFFAC(2,IB)*TEMFXO(JO+1,NYO+1)
     1                           + YFFAC(1,IB)*TEMFXO(JO+1,    1)
  250         CONTINUE
C           Fold back extra value at X-edge
            DO 260 KO=1,NYO-1
              FLXBAS(NXO,KO,IFX)=  XFFAC(2,IB)*TEMFXO(NXO+1,KO+1)
     1                           + XFFAC(1,IB)*TEMFXO(1,    KO+1)
  260         CONTINUE
C           Fold back corner value
            FLXBAS(NXO,NYO,IFX)=
     1         XFFAC(2,IB)*YFFAC(2,IB)*TEMFXO(NXO+1,NYO+1)
     2       + XFFAC(1,IB)*YFFAC(2,IB)*TEMFXO(    1,NYO+1)
     3       + XFFAC(2,IB)*YFFAC(1,IB)*TEMFXO(NXO+1,    1)
     4       + XFFAC(1,IB)*YFFAC(1,IB)*TEMFXO(    1,    1)
C
#ifdef DEBUG
C           Compute area-weighted flux integral after interpolation
            SUM= 0.0
            DO 270 KO=1,NYO+1
              DO 270 JO=1,NXO+1
                SUM= SUM + TEMFXO(JO,KO)
     1                    * (TGRIDY(KO) - TGRIDY(KO-1))
     2                    * (TGRIDX(JO) - TGRIDX(JO-1))
  270           CONTINUE
            WRITE(*,'(1X,A,I2,A,1PE15.6)')
     1     'Flux integral', IFX, ' after interpolation  = ', SUM
#endif
          ELSE
C
C           Scalar flux; interpolate to oceanic temperature grid
            CALL FLUX2D( FLXBAS(1,1,IFX), JOMAX, NXO, NYO,
     1                   TEMFXA(0,0),     JBMAX, NXB, NYB,
     2         MXA2O(0,IB), FXA2O(1,0,IB), CFXA2O(1,0,IB), DXA2O(0,IB),
     3         MYA2O(0,IB), FYA2O(1,0,IB), CFYA2O(1,0,IB), DYA2O(0,IB),
     4         YWTA2O(0,IB) )
C
#ifdef DEBUG
C           Compute area-weighted flux integral after interpolation
            SUM= 0.0
            DO 280 KO=1,NYO
              DO 280 JO=1,NXO
                SUM= SUM + FLXBAS(JO,KO,IFX)
     1                    * (VGRIDY(KO) - VGRIDY(KO-1))
     2                    * (VGRIDX(JO) - VGRIDX(JO-1))
  280         CONTINUE
            WRITE(*,'(1X,A,I2,A,1PE15.6)')
     1     'Flux integral', IFX, ' after interpolation  = ', SUM
#endif
          ENDIF
  290     CONTINUE
#endif
C
#ifndef OLD_VERSION
C       Multiply all fluxes by X/Y grid stretch factors
        DO 500 IFX=JTAUX,NFOCN
          DO 500 KO=1,NYO
            DO 500 JO=1,NXO
            FLXBAS(JO,KO,IFX)= XSFAC(IB)*YSFAC(IB)*FLXBAS(JO,KO,IFX)
  500       CONTINUE
#endif
C
C       Send fluxes to ocean model
        CALL SNDFLX( IUPAK, IPACKT, IB, JOMAX, KOMAX,
     1               FLXTOT, FLXBAS, NXO, NYO, NFOCN,
     2               ISTP, NSYNC, ERRFLG)
        IF (ERRFLG) RETURN
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE SETINT( IB, NXO, NYO,
     1                   XOCN, XOMID, YOCN, YOMID )
C Set-up interpolation arrays for an ocean basin
C
C Input:  INTEGER IB       -- ocean basin number
C         INTEGER NXO, NYO -- no. of X/Y oceanic grid-points
C         REAL XOCN(NXO)   -- Longitudes of grid-points (degrees)
C         REAL XOMID(0:NXO)-- Longitude of grid-box edges (degrees)
C         REAL YOCN(NYO)   -- Sine-latitude (mu) of grid-points
C         REAL YOMID(0:NYO)-- Sine-latitude (mu) grid-box edges
C Output: (minterp.h)
C
#include "mgrid.h"
#include "mparams.h"
#include "minterp.h"
C
      INTEGER IB, NXO, NYO
      REAL XOCN(NXO), XOMID(0:NXO)
      REAL YOCN(NYO), YOMID(0:NYO)
C
C
      REAL DGMIDB(0:JBMAX), DEGB(0:JBMAX+1)
      REAL MUMIDB(0:KBMAX), MUB(0:KBMAX+1), TGRDXP(0:JOMAX+1)
      REAL FAC1, FAC2
      INTEGER JLO, JHI, KLO, KHI, NXB, NYB
      INTEGER JB, KB, JO, KO
C
      IF (INTINI(IB)) THEN
          CALL ATMERR( 'SETINT',
     1        'Attempt to re-initialize interpolation coefficients' )
      ENDIF
C     Check that longitudinal basin boundaries lie within range
      IF ( (XOMID(0)   .LT.   0.0) .OR.
     1     (XOMID(NXO) .GT. 720.0)      ) THEN
            CALL ATMERR( 'SETINT',
     1                   'Longitudinal basin boundaries out of bounds')
      ENDIF
C     Check that latitudinal basin boundaries lie within range
      IF ( (YOMID(0)   .LT. -1.001) .OR.
     1     (YOMID(NYO) .GT.  1.001)      ) THEN
            CALL ATMERR( 'SETINT',
     1                   'Latitudinal basin boundaries out of bounds')
      ENDIF
C
C     Set coefficient initialization flag
      INTINI(IB)= .TRUE.
C
C     Copy number of basin grid-points
      NOCNX(IB)= NXO
      NOCNY(IB)= NYO
C
C     Basin extent, atmospheric coordinates
      NXB= NATMX(IB)
      NYB= NATMY(IB)
C
C     Determine basin boundaries (in atmospheric grid indices)
      JLO= JB0(IB) + 1
      KLO= KB0(IB) + 1
      JHI= JB0(IB) + NXB
      KHI= KB0(IB) + NYB
C
C     Copy Y coordinates for atmospheric grid (Sine-latitude)
      MUMIDB(0)= MUAMID(KLO-1)
      DO 10 KB=1,NYB
        MUB(KB)= MU(KB+KLO-1)
   10   MUMIDB(KB)= MUAMID(KB+KLO-1)
C
C     Pad atmospheric MU-grid with extra values
      MUB(0)= MUMIDB(0)
      MUB(NYB+1)= MUMIDB(NYB)
C
C     Copy Y coordinates for ocean grid (Sine-latitude)
      DO 20 KO=1,NYO
   20   TGRIDY(KO)  = YOCN(KO)
      DO 30 KO=0,NYO
   30   VGRIDY(KO) = YOMID(KO)
C
#ifdef DEBUG
      WRITE(*,*)
      WRITE(*,*) 'Subroutine SETINT: debugging information'
      WRITE(*,'(1X,A,I2)') 'Basin IB=', IB
      WRITE(*,*) 'NXO, NYO= ', NXO, NYO
      WRITE(*,*) 'JLO, JHI, KLO, KHI= ', JLO, JHI, KLO, KHI
      WRITE(*,*)
      WRITE(*,*) 'TGRIDY(1:NYO)='
      WRITE(*,'(1X,10F7.3)') (TGRIDY(KO),KO=1,NYO)
      WRITE(*,*) 'VGRIDY(*)='
      WRITE(*,'(1X,10F7.3)') (VGRIDY(KO),KO=0,NYO)
C
#endif
C
C     Stretch oceanic Y-grid to match atmospheric boundaries
      CALL STRETC( YSFAC(IB), TGRIDY(1), VGRIDY(0), NYO,
     1             MUMIDB(0), MUMIDB(NYB) )
C
C     Pad TGRIDY with extra values
      TGRIDY(0)= VGRIDY(0)
      TGRIDY(NYO+1)= VGRIDY(NYO)
C
C     Compute Y foldback factors, for flux conservation at edges
      FAC1= TGRIDY(1)     - TGRIDY(0)
      FAC2= TGRIDY(NYO+1) - TGRIDY(NYO)
      YFFAC(1,IB)= FAC1 / (FAC1 + FAC2)
      YFFAC(2,IB)= FAC2 / (FAC1 + FAC2)
C
#ifdef DEBUG
      WRITE(*,*) 'After stretching'
      WRITE(*,*) 'MUB(*)='
      WRITE(*,'(1X,10F7.3)') (MUB(KB),KB=0,NYB+1)
      WRITE(*,*) 'MUMIDB(*)='
      WRITE(*,'(1X,10F7.3)') (MUMIDB(KB),KB=0,NYB)
      WRITE(*,'(1X,A,F7.4)') 'YSFAC(IB)= ', YSFAC(IB)
      WRITE(*,'(1X,A,2F7.4)') 'YFFAC(*,IB)= ', YFFAC(1,IB), YFFAC(2,IB)
      WRITE(*,*) 'TGRIDY(*)='
      WRITE(*,'(1X,10F7.3)') (TGRIDY(KO),KO=0,NYO+1)
      WRITE(*,*) 'VGRIDY(*)='
      WRITE(*,'(1X,10F7.3)') (VGRIDY(KO),KO=0,NYO)
#endif
C
C     Copy Y coordinates for atmospheric grid (Sine-latitude)
      DGMIDB(0)= DGAMID(JLO-1)
      DO 40 JB=1,NXB
        DEGB(JB)= DGATM(JB+JLO-1)
   40   DGMIDB(JB)= DGAMID(JB+JLO-1)
C
C     Copy X coordinates for ocean grid (degrees longitude)
C     Temperature grid-points
      DO 50 JO=1,NXO
   50   TGRIDX(JO)= XOCN(JO)
C     Velocity grid-points
      DO 60 JO=0,NXO
   60   VGRIDX(JO)= XOMID(JO)
C
#ifdef DEBUG
      WRITE(*,*)
      WRITE(*,*) 'TGRIDX(1:NXO)='
      WRITE(*,'(1X,10F7.2)') (TGRIDX(JO),JO=1,NXO)
      WRITE(*,*) 'VGRIDX(*)='
      WRITE(*,'(1X,10F7.2)') (VGRIDX(JO),JO=0,NXO)
#endif
C
C     Stretch oceanic X-grid to match atmospheric boundaries
      CALL STRETC( XSFAC(IB), TGRIDX(1), VGRIDX(0), NXO,
     1             DGMIDB(0), DGMIDB(NXB) )
C
C     Pad TGRIDX with extra values
      TGRIDX(0)= VGRIDX(0)
      TGRIDX(NXO+1)= VGRIDX(NXO)
C
C     Compute X foldback factors, for flux conservation at edges
      FAC1= TGRIDX(1)     - TGRIDX(0)
      FAC2= TGRIDX(NXO+1) - TGRIDX(NXO)
      XFFAC(1,IB)= FAC1 / (FAC1 + FAC2)
      XFFAC(2,IB)= FAC2 / (FAC1 + FAC2)
C
C     Define "periodic" temperature grid-points (for SST)
      DO 70 JO=1,NXO
   70   TGRDXP(JO)= TGRIDX(JO)
C
      IF (NXB .LT. K1) THEN
C
C       Non-cyclic basin
        CYCLIC(IB)= .FALSE.
C       Pad atmospheric longitude-grid with extra values
        DEGB(0)=     DGMIDB(0)
        DEGB(NXB+1)= DGMIDB(NXB)
C       Pad oceanic longitude-grid with extra values (for SST)
        TGRDXP(0)=     VGRIDX(0)
        TGRDXP(NXO+1)= VGRIDX(NXO)
C
#ifdef DEBUG
        WRITE(*,*) 'Non-cyclic basin'
#endif
      ELSE
C
C       Cyclic basin; occupies entire X-grid, periodically
        CYCLIC(IB)= .TRUE.
C       Pad atmospheric longitude-grid with extra values (periodic)
        DEGB(0)=      DGMIDB(NXB) - 360.
        DEGB(NXB+1)= DGMIDB(1)   + 360.
C       Pad oceanic longitude-grid with periodic values (for SST)
        TGRDXP(0)=     TGRIDX(NXO) - 360.
        TGRDXP(NXO+1)= TGRIDX(1)   + 360.
C
#ifdef DEBUG
        WRITE(*,*) 'Cyclic basin'
#endif
      ENDIF
C
#ifdef DEBUG
      WRITE(*,*) 'DEGB(*)='
      WRITE(*,'(1X,10F7.2)') (DEGB(JB),JB=0,NXB+1)
      WRITE(*,*) 'DGMIDB(*)='
      WRITE(*,'(1X,10F7.2)') (DGMIDB(JB),JB=0,NXB)
      WRITE(*,*) 'After stretching'
      WRITE(*,'(1X,A,F7.4)') 'XSFAC(IB)= ', XSFAC(IB)
      WRITE(*,'(1X,A,2F7.4)') 'XFFAC(*,IB)= ', XFFAC(1,IB), XFFAC(2,IB)
      WRITE(*,*) 'TGRIDX(*)='
      WRITE(*,'(1X,10F7.2)') (TGRIDX(JO),JO=0,NXO+1)
      WRITE(*,*) 'VGRIDX(*)='
      WRITE(*,'(1X,10F7.2)') (VGRIDX(JO),JO=0,NXO)
      WRITE(*,*) 'TGRDXP(*)='
      WRITE(*,'(1X,10F7.2)') (TGRDXP(JO),JO=0,NXO+1)
#endif
C
#ifdef ZONAVG
      IF (NXO .GT. 1) THEN
        CALL ATMERR( 'SETINT',
     1      'Zonally-averaged basin must have single X-interval')
      ENDIF
C
C     Initialize ocean -> atmosphere interpolation arrays
      CALL FLXIN1( MINO2A(0,IB), FINO2A(0,IB),
     1             DYNO2A(1,IB), DYO2A(0,IB),
     2             LWTO2A(1,IB), RWTO2A(1,IB),
     3             MUMIDB(0), NYB,
     4             VGRIDY(0), TGRIDY(0), NYO )
C
C     Initialize atmosphere -> ocean interpolation arrays (T,S)
      CALL FLXIN1( MINA2O(0,IB), FINA2O(0,IB),
     1             DYNA2O(1,IB), DYA2O(0,IB),
     2             LWTA2O(1,IB), RWTA2O(1,IB),
     3             VGRIDY(0), NYO,
     4             MUMIDB(0), MUB(0), NYB )
C
C     Initialize atmosphere -> ocean interpolation arrays (U,V)
      CALL FLXIN1( MINA2V(0,IB), FINA2V(0,IB),
     1             DYNA2V(1,IB), DYA2V(0,IB),
     2             LWTA2V(1,IB), RWTA2V(1,IB),
     3             TGRIDY(0), NYO+1,
     4             MUMIDB(0), MUB(0), NYB )
C
#else
C
C     Initialize ocean -> atmosphere interpolation arrays
      CALL FLXIN2(
     1         MXO2A(0,IB), FXO2A(1,0,IB), CFXO2A(1,0,IB), DXO2A(0,IB),
     2         MYO2A(0,IB), FYO2A(1,0,IB), CFYO2A(1,0,IB), DYO2A(0,IB),
     3         YWTO2A(0,IB),
     4         DGMIDB(0), NXB, TGRDXP(0), NXO,
     5         MUMIDB(0), NYB, TGRIDY(0), NYO, VGRIDY(0) )
C
C     Initialize atmosphere -> ocean interpolation arrays (T,S)
      CALL FLXIN2(
     1         MXA2O(0,IB), FXA2O(1,0,IB), CFXA2O(1,0,IB), DXA2O(0,IB),
     2         MYA2O(0,IB), FYA2O(1,0,IB), CFYA2O(1,0,IB), DYA2O(0,IB),
     3         YWTA2O(0,IB),
     4         VGRIDX(0), NXO, DEGB(0), NXB,
     5         VGRIDY(0), NYO, MUB(0),  NYB, MUMIDB(0) )
C
C     Initialize atmosphere -> ocean interpolation arrays (U,V)
      CALL FLXIN2(
     1         MXA2V(0,IB), FXA2V(1,0,IB), CFXA2V(1,0,IB), DXA2V(0,IB),
     2         MYA2V(0,IB), FYA2V(1,0,IB), CFYA2V(1,0,IB), DYA2V(0,IB),
     3         YWTA2V(0,IB),
     4         TGRIDX(0), NXO+1, DEGB(0), NXB,
     5         TGRIDY(0), NYO+1, MUB(0),  NYB, MUMIDB(0) )
#endif
C
      RETURN
      END
