C ICAM (Idealized Coupleable Atmospheric Model)
C icamsrc $Revision: 1.8 $ created $Date: 1996/05/02 21:31:03 $
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C amarch.F: Fortran source module
C Contains time-marching routines for the atmospheric model
C
 
#include "mmain.h"
 
C Routines: (all called by ATMRUN)
C TIMDEP: Compute time-dependent parameters/forcing
C SRFPAR: Compute surface parameters
C ATMSTP: Integrate atmospheric prognostic variables over one time-step
C ->ZDIV: Set vertically integrated atmospheric divergence to zero
C LANSTP: Integrate land surface variables over one time-step
C FLXCAL: Compute surface fluxes over an ocean basin
C ROBERT: Apply Robert time-filter
C
      SUBROUTINE TIMDEP( ISTP, SSTVAL, LSTVAL,
     1                   CO2FAC, TSURF, SOLRAD, OTFADJ, OSFADJ )
C Compute time-dependent parameters/forcing
C
C Input:  INTEGER ISTP             -- current model time-step count
C         REAL SSTVAL(K1MAX,K2MAX) -- old prognostic SST values (K)
C         REAL LSTVAL(K1MAX,K2MAX) -- Land Surface Temperatures (K)
C Output: REAL CO2FAC -- CO2 concentration multiplication factor
C         REAL TSURF(K1MAX,K2MAX)  -- land/sea surface temperature (K)
C         REAL SOLRAD(K1MAX,K2MAX) -- incident solar radiation (W/m**2)
C         REAL OTFADJ(K1MAX,K2MAX) -- heat flux adjustment (W/m**2)
C         REAL OSFADJ(K1MAX,K2MAX) -- E-P flux adjustment (kg/(s*m**2))
C
#include "mgrid.h"
#include "mparams.h"
#include "mcoef.h"
C
      INTEGER ISTP
      REAL SSTVAL(K1MAX,K2MAX), LSTVAL(K1MAX,K2MAX)
      REAL CO2FAC, TSURF(K1MAX,K2MAX), SOLRAD(K1MAX,K2MAX)
      REAL         OTFADJ(K1MAX,K2MAX), OSFADJ(K1MAX,K2MAX)
C
      DOUBLE PRECISION DTIM
      REAL TIM, YRDAY
      INTEGER J, K
C
C     Compute current model time
      DTIM= DBLE(ISTP)*DBLE(DT1)
      TIM= DTIM
C     Compute time-dependent CO2 multiplication factor
      IF (TIM .GE. MULTIM) THEN
        CO2FAC= 1.0 + (DLWCO2/DLWAIR)*MULCO2
      ELSE
        CO2FAC= 1.0 + (DLWCO2/DLWAIR)*EXP((TIM/MULTIM)*ALOG(MULCO2))
      ENDIF
C
C We use the convention that the year starts at the NH spring equinox,
C i.e., TIM = 0 corresponds to March 21. If time is measured in years,
C 0.25 corresponds to the NH summer solstice and 0.75 to the NH winter
C solstice. However, North & Coakley use the convention that the year
C starts at the NH winter solstice (December 22). Hence we subtract
C 0.75 from our normalized time (YRDAY) when we use their formulas.
C
        IF (SSNFLG) THEN
C         Seasonally varying forcing
#ifdef OLD_VERSION
          YRDAY= (DTIM/DYEARL - INT(DTIM/DYEARL)) + (YPHASE - 0.75)
#else
C         Compute phase of year using time-step count to avoid
C         slow drifts due to machine roundoff at single precision
          YRDAY=  FLOAT(MOD(ISTP,NSPDAY*NDPYR))/FLOAT(NSPDAY*NDPYR)
     1          + (YPHASE - 0.75)
#endif
        ELSE
C         Perpetual season forcing
          YRDAY= YPHASE - 0.75
        ENDIF
C
        IF (FADSPF) THEN
C         Compute time-dependent heat and E-P flux adjustment terms
          DO 110 K=1,K2
            DO 110 J=1,K1
              OTFADJ(J,K)= OTFA0(J,K)
     1         + OTFA1A(J,K)*COS(2.*PI*(1.*(YRDAY+0.75)-OTFA1P(J,K)))
     2         + OTFA2A(J,K)*COS(2.*PI*(2.*(YRDAY+0.75)-OTFA2P(J,K)))
              OSFADJ(J,K)= OSFA0(J,K)
     1         + OSFA1A(J,K)*COS(2.*PI*(1.*(YRDAY+0.75)-OSFA1P(J,K)))
     2         + OSFA2A(J,K)*COS(2.*PI*(2.*(YRDAY+0.75)-OSFA2P(J,K)))
  110         CONTINUE
        ELSE
C         No flux adjustment
          DO 120 K=1,K2
            DO 120 J=1,K1
              OTFADJ(J,K)= 0.0
              OSFADJ(J,K)= 0.0
  120         CONTINUE
        ENDIF
 
C
C       Incident solar radiation (North & Coakley, JAS, 1979)
        DO 200 K=1,K2
          DO 200 J=1,K1
            SOLRAD(J,K)= 0.25*SOLFLX*AMAX1(0.0,
     1              1.0 - PPFAC*0.796*COS(2.0*PI*YRDAY)*MU(K)
     2                  + (0.147*COS(4.0*PI*YRDAY) - EQPFAC*0.477)
     3                    *0.5*(3.0*MU(K)**2 - 1.0) )
  200       CONTINUE
C
C       Initialize surface temperature over all land/sea grid points
        DO 300 K=1,K2
          DO 300 J=1,K1
            IF (PROSST(J,K)) THEN
C
C             Copy "old" prognostic SST vales
              TSURF(J,K)=  SSTVAL(J,K)
C
            ELSE IF (SEAMSK(J,K)) THEN
C
C             Set prescribed SST values
              IF (SSTSPF) THEN
C               Specified (mean, annual/semi-annual harmonics)
                TSURF(J,K)=  TSRF0(J,K)
     1           + TSRF1A(J,K)*COS(2.*PI*(1.*(YRDAY+0.75)-TSRF1P(J,K)))
     2           + TSRF2A(J,K)*COS(2.*PI*(2.*(YRDAY+0.75)-TSRF2P(J,K)))
              ELSE
#ifdef OLD_VERSION
C               Default SST (2 month phase lag)
                TSURF(J,K)=  292.0
     1                     - 3.0*COS(2.0*PI*(YRDAY-0.167))*MU(K)
     2                     - 19.0*0.5*(3.0*MU(K)**2 - 1.0)
#else
C               Default SST profile
                IF ((NBASIN .EQ. 2) .AND. (J .LE. K1/2)) THEN
C                 Two-basin geometry; "cold Pacific"
                  TSURF(J,K)=  301.0
     1                       - 33.0*MU(K)**2
     2                       - (7.0*MU(K) - 6.6*MU(K)**3)
     3                        *COS(2.0*PI*(YRDAY-0.175))
                ELSE
C
                  TSURF(J,K)=  301.0
     1                       - 31.5*MU(K)**2
     2                       - (7.0*MU(K) - 6.6*MU(K)**3)
     3                        *COS(2.0*PI*(YRDAY-0.175))
                ENDIF
#endif
              ENDIF
C
            ELSE
C
C             Copy land surface temperature values
              TSURF(J,K)=  LSTVAL(J,K)
            ENDIF
  300       CONTINUE
      RETURN
      END
 
      SUBROUTINE SRFPAR( RHSURF, ALBEDO, TSURF, WLEVEL, ICEMSK )
C Compute surface parameters (relative humidity and albedo),
C given surface properties (temperature, water-level, ice mask)
C
C Input:  REAL TSURF(K1MAX,K2MAX)  -- land/sea surface temperature (K)
C         REAL WLEVEL(K1MAX,K2MAX) -- land surface water level (m)
C         LOGICAL ICEMSK(K1MAX,K2MAX) -- ice mask
C Output: REAL RHSURF(K1MAX,K2MAX) -- surface relative humidity (0 .. 1)
C         REAL ALBEDO(K1MAX,K2MAX) -- surface albedo (0 .. 1)
C
#include "mgrid.h"
#include "mparams.h"
      REAL RHSURF(K1MAX,K2MAX), ALBEDO(K1MAX,K2MAX)
      REAL TSURF(K1MAX,K2MAX), WLEVEL(K1MAX,K2MAX)
      LOGICAL ICEMSK(K1MAX,K2MAX)
C
      INTEGER J, K
C
C     Set relative humidity values over sea/land
      DO 10 K=1,K2
        DO 10 J=1,K1
          IF (SEAMSK(J,K)) THEN
C           Sea surface relative humidity (100%)
            RHSURF(J,K)= 1.0
          ELSE
C           Land surface relative humidity
            IF (BUCKHT(J,K) .EQ. 0.) THEN
C             Empty (dry) bucket; relative humidity (0%)
              RHSURF(J,K)= 0.0
            ELSE
C???          Bucket model not yet implemented
              STOP 'BUCKHT'
            ENDIF
          ENDIF
   10     CONTINUE
C
C     Set albedo values over sea/land
C     Simple linear dependence on surface temperature
C     [Fits the North & Coakley (JAS 1979) values quite well]
      DO 20 K=1,K2
        DO 20 J=1,K1
          IF (ICEMSK(J,K) .OR. (TSURF(J,K) .LE. 248.)) THEN
C           Ice albedo
            ALBEDO(J,K)= ALBICE
          ELSE IF (SEAMSK(J,K)) THEN
C           Sea albedo
            ALBEDO(J,K)= ALBSEA + AMAX1( 0.0, (ALBICE - ALBSEA)
     1                                       *(298. - TSURF(J,K))
     2                                       /50. )
          ELSE
C           Land albedo
            ALBEDO(J,K)= ALBLAN + AMAX1( 0.0, (ALBICE - ALBLAN)
     1                                       *(298. - TSURF(J,K))
     2                                       /50. )
          ENDIF
   20     CONTINUE
C
        RETURN
        END
 
      SUBROUTINE ATMSTP( DTATM, SEAMSK, ICEMSK, SOLRAD, ALBEDO,
     1                   TSURF, RHSURF, CO2FAC, DMPBOT, LTROP,
     2                   PGQSP0, PGQSP1,
     3                   PGQSP2, DPGQSP, TAUSRF, EMINP,
     4                   CDRAG, QEVAP, MSAPCP, MCAPCP,
     5                   FEVAP, FSENS, FRAD, FNET, FPSSRF, FMSSRF,
     6                   FPLSRF, FMLSRF, FPSTOP, FPLTOP,
     7                   SWHET, LWHET, MSAHET, MCAHET, DCAHET )
C
C Integrate atmospheric prognostic variables over
C one (leap-frog) time-step, and returns the tendencies
C and various fluxes.
C
C Input:  REAL DTATM -- time-step (sec)
C         LOGICAL SEAMSK(K1MAX,K2MAX) -- sea mask (.TRUE. over sea)
C         LOGICAL ICEMSK(K1MAX,K2MAX) -- ice mask (.TRUE. over ice)
C         REAL SOLRAD(K1MAX,K2MAX) -- incident solar radiation (W/m**2)
C         REAL ALBEDO(K1MAX,K2MAX) -- surface albedo (0 .. 1)
C         REAL TSURF(K1MAX,K2MAX)  -- land/sea surface temperature (K)
C         REAL RHSURF(K1MAX,K2MAX) -- surface relative humidity (0 .. 1)
C         REAL CO2FAC -- CO2 concentration multiplication factor
C         REAL DMPBOT -- Rayleigh friction damping coefficient at the
C                        surface (1/sec) [used to compute TAUSRF]
C         INTEGER LTROP --  Model tropopause level number
C         COMPLEX PGQSP0(0:M1MAX,0:N1MAX,NPGQ,L1MAX) -- spectral
C          representation of prognostic quantities at (current time)-DT1
C         COMPLEX PGQSP1(0:M1MAX,0:N1MAX,NPGQ,L1MAX) -- spectral
C          representation of prognostic quantities at current model time
C Output: COMPLEX PGQSP2(0:M1MAX,0:N1MAX,NPGQ,L1MAX) -- spectral
C          representation of prognostic quantities at (current time)+DT1
C         COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) -- spectral
C          tendency of prognostic quantities at "current" model time
C          (including all adiabatic and diabatic contributions)
C         REAL TAUSRF(K1MAX,K2MAX,JX:JY) -- surface wind stress (Pa)
C         REAL EMINP(K1MAX,K2MAX)      -- E-P values (kg/(s*m**2))
C         REAL CDRAG(K1MAX,K2MAX) -- surface drag coefficient (in 1/s)
C         REAL QEVAP(K1MAX,K2MAX) -- evaporative flux of water vapour
C                                    (in kg/(s*m**2))
C         REAL MSAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                static adjustment (in kg/(s*m**2))
C         REAL MCAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                convective adjustment (in kg/(s*m**2))
C         REAL FEVAP(K1MAX,K2MAX) -- heat flux associated with the
C                                    evaporative flux of water vapour
C                                    (in W/m**2)
C         REAL FSENS(K1MAX,K2MAX) -- upward surface sensible heat flux
C                                    (in W/m**2)
C         REAL FRAD(K1MAX,K2MAX) -- upward surface radiative heat flux
C                                   (in W/m**2)
C         REAL FNET(K1MAX,K2MAX) -- Net heat flux into land/sea
C                                   (in W/m**2)
C         REAL FPSSRF(K1MAX,K2MAX) -- upward SW radiation at surface
C         REAL FMSSRF(K1MAX,K2MAX) -- downward SW radiation at surface
C         REAL FPLSRF(K1MAX,K2MAX) -- upward LW radiation at surface
C         REAL FMLSRF(K1MAX,K2MAX) -- downward LW radiation at surface
C         REAL FPSTOP(K1MAX,K2MAX) -- upward SW radiation at model top
C         REAL FPLTOP(K1MAX,K2MAX) -- upward LW radiation at model top
C         REAL SWHET(K2MAX,L1MAX)  -- zonally-averaged vertical profile
C              of heating due to SW radiation (in K/s)
C         REAL LWHET(K2MAX,L1MAX)  -- zonally-averaged vertical profile
C              of heating due to LW radiation (in K/s)
C         REAL MSAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to moist static adjustment (in K/s)
C         REAL MCAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to moist convective adjustment (in K/s)
C         REAL DCAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to dry convective adjustment (in K/s)
C
C
#include "mgrid.h"
C
      REAL DTATM
      LOGICAL SEAMSK(K1MAX,K2MAX), ICEMSK(K1MAX,K2MAX)
      REAL SOLRAD(K1MAX,K2MAX), ALBEDO(K1MAX,K2MAX)
      REAL TSURF(K1MAX,K2MAX), RHSURF(K1MAX,K2MAX), CO2FAC, DMPBOT
      INTEGER LTROP
      COMPLEX PGQSP0(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX PGQSP1(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX PGQSP2(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      REAL TAUSRF(K1MAX,K2MAX,JX:JY), EMINP(K1MAX,K2MAX)
      REAL CDRAG(K1MAX,K2MAX),  QEVAP(K1MAX,K2MAX)
      REAL MSAPCP(K1MAX,K2MAX), MCAPCP(K1MAX,K2MAX)
      REAL FEVAP(K1MAX,K2MAX),  FSENS(K1MAX,K2MAX)
      REAL FRAD(K1MAX,K2MAX),   FNET(K1MAX,K2MAX)
      REAL FPSSRF(K1MAX,K2MAX), FMSSRF(K1MAX,K2MAX)
      REAL FPLSRF(K1MAX,K2MAX), FMLSRF(K1MAX,K2MAX)
      REAL FPSTOP(K1MAX,K2MAX), FPLTOP(K1MAX,K2MAX)
      REAL SWHET(K2MAX,L1MAX),  LWHET(K2MAX,L1MAX)
      REAL MSAHET(K2MAX,L1MAX), MCAHET(K2MAX,L1MAX)
      REAL DCAHET(K2MAX,L1MAX)
C
C --- LOCAL VARIABLES ---
C     Adiabatic + mechanistic forcing tendencies
      COMPLEX DPGQSA(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
C
C     "Interim" low-level wind (m/s)
      REAL ULOW(K1MAX,K2MAX,JX:JY)
C
      REAL TEMP
      COMPLEX FSP1(0:M1MAX,0:N1MAX), FSP2(0:M1MAX,0:N1MAX)
      INTEGER J, K, L, M, N, IPGQ
C
C     Compute "adiabatic" time tendency
      CALL DDTPGQ( DPGQSA, PGQSP0, PGQSP1 )
C     Make implicit corrections to "adiabatic" tendency
      CALL DDTIMP( DPGQSA )
C     Compute "interim" values at next time-step (leap frog)
      DO 110 L=1,L1
        DO 110 IPGQ=1,NPGQ
          DO 110 N=0,N1
            DO 110 M=0,MIN0(N,M1)
              PGQSP2(M,N,IPGQ,L)=  PGQSP0(M,N,IPGQ,L)
     1                           + 2.0 * DTATM * DPGQSA(M,N,IPGQ,L)
  110         CONTINUE
C
C     Compute effects of mechanistic forcing/damping
C     (using "interim" fields)
      CALL DDTFRC( DPGQSA, PGQSP2 )
C
C     Compute new "interim" values for rad-con adjustment
      DO 120 L=1,L1
        DO 120 IPGQ=1,NPGQ
          DO 120 N=0,N1
            DO 120 M=0,MIN0(N,M1)
              PGQSP2(M,N,IPGQ,L)=  PGQSP2(M,N,IPGQ,L)
     1                           + 2.0*DTATM*DPGQSA(M,N,IPGQ,L)
  120         CONTINUE
C
C     Compute "interim" horizontal velocity at level L1
C
C     First compute streamfunction PSI and velocity potential CHI
      CALL IDELSQ( FSP1, PGQSP2(0,0,JVOR,L1), N1)
      CALL IDELSQ( FSP2, PGQSP2(0,0,JDIV,L1), N1)
C     Then compute horizontal velocity from PSI and CHI
      CALL HVELOC( ULOW(1,1,JX), ULOW(1,1,JY), FSP1, FSP2 )
C
C       Initialize "physics" time-tendencies to zero
        DO 210 L=1,L1
          DO 210 IPGQ=1,NPGQ
  210       CALL ZEROSP( DPGQSP(0,0,IPGQ,L), N1)
C
C       Compute tendencies due to model "physics"
C       (Radiative-convective adjustment, ...)
        CALL PHYCAL( DPGQSP, CDRAG, QEVAP, MSAPCP, MCAPCP,
     1               FEVAP, FSENS, FRAD, FNET, FPSSRF, FMSSRF,
     2               FPLSRF, FMLSRF, FPSTOP, FPLTOP,
     3               SWHET, LWHET, MSAHET, MCAHET, DCAHET,
     4               PGQSP2, SEAMSK, ICEMSK, SOLRAD, ALBEDO,
     5               TSURF, RHSURF, ULOW, CO2FAC, 2.0*DTATM,
     6               LTROP, LTROP )
C
C       March atmosphere on to next time-step (from "interim" values)
        DO 220 L=1,L1
          DO 220 IPGQ=1,NPGQ
            DO 220 N=0,N1
              DO 220 M=0,MIN0(N,M1)
                PGQSP2(M,N,IPGQ,L)=  PGQSP2(M,N,IPGQ,L)
     1                             + 2.0 * DTATM * DPGQSP(M,N,IPGQ,L)
  220           CONTINUE
C
C       Set vertically integrated atmospheric divergence to zero
C       [required for consistency in the Lorenz (1960) truncation]
        CALL ZDIV( PGQSP2 )
C
C       Compute total forcing tendency (physics + relaxation)
        DO 230 L=1,L1
          DO 230 IPGQ=1,NPGQ
            DO 230 N=0,N1
              DO 230 M=0,MIN0(N,M1)
                DPGQSP(M,N,IPGQ,L)=  DPGQSP(M,N,IPGQ,L)
     1                             + DPGQSA(M,N,IPGQ,L)
  230           CONTINUE
C
C       Compute surface wind stress (Rayleigh friction)
        TEMP= DMPBOT*DP(L1)/G0
        DO 310 K=1,K2
          DO 310 J=1,K1
            TAUSRF(J,K,JX)= TEMP*ULOW(J,K,JX)
  310       TAUSRF(J,K,JY)= TEMP*ULOW(J,K,JY)
C
C       Compute E-P (evaporation-precipitation) (kg/(s*m**2))
        DO 320 K=1,K2
          DO 320 J=1,K1
  320       EMINP(J,K)= QEVAP(J,K) - MSAPCP(J,K) - MCAPCP(J,K)
C
      RETURN
      END
 
      SUBROUTINE ZDIV( PGQSP )
C Set vertically integrated atmospheric divergence to zero
C [for the Lorenz (1960) vertical truncation]
C
C Input:  COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C         spectral representation of prognostic quantities
C Output: COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C         spectral representation of prognostic quantities,
C         with the vertically integrated divergence set to zero
C
#include "mgrid.h"
C
      COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX   FSP(0:M1MAX,0:N1MAX)
      REAL TEMP
      INTEGER L, M, N
C
C       Compute vertical mass-weighted average of divergence field
        CALL ZEROSP( FSP, N1 )
        DO 20 L=1,L1
          TEMP= DP(L)/PSURF
          DO 10 N=0,N1
            DO 10 M=0,MIN0(N,M1)
   10         FSP(M,N)= FSP(M,N) + TEMP * PGQSP(M,N,JDIV,L)
   20     CONTINUE
C       Ensure that vertically integrated divergence is zero,
C       by removing the vertical average.
        DO 30 L=1,L1
          DO 30 N=0,N1
            DO 30 M=0,MIN0(N,M1)
   30         PGQSP(M,N,JDIV,L) = PGQSP(M,N,JDIV,L) - FSP(M,N)
C
        RETURN
        END
 
      SUBROUTINE LANSTP( DTLAN, FNET, EMINP, LAND0, LAND1, LAND2,
     1                   RUNOFF, EMPLAN, EMPLAV,
     2                   QEVAP, MSAPCP, MCAPCP )
C Integrate land surface variables over one time-step
C Input:  REAL DTLAN -- effective time-step (sec)
C         REAL FNET(K1MAX,K2MAX)  -- Net heat flux into surface (W/m**2)
C         REAL EMINP(K1MAX,K2MAX) -- E-P values (kg/(s*m**2))
C         REAL LAND0(K1MAX,K2MAX,JLANT:JLANWL) -- prognostic
C              land temperature/water-level at (current time)-DT1
C         REAL LAND1(K1MAX,K2MAX,JLANT:JLANWL) -- prognostic
C              land temperature/water-level at current model time
C (The following fluxes are actually unnecessary, but are present for
C  compatibility with previous versions)
C         REAL QEVAP(K1MAX,K2MAX)  -- evaporative flux of water vapour
C                                     (in kg/(s*m**2))
C         REAL MSAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                static adjustment (in kg/(s*m**2))
C         REAL MCAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                convective adjustment (in kg/(s*m**2))
C Output: REAL LAND2(K1MAX,K2MAX,JLANT:JLANWL) -- prognostic
C              land temperature/water-level at (current time)+DT1
C         REAL RUNOFF(K1MAX,K2MAX) -- Runoff (kg/(s*m**2))
C         REAL EMPLAN(K2MAX) -- total E-P over land at each latitude
C         REAL EMPLAV -- EMPLAN, averaged over all latitudes
C
#include "mgrid.h"
#include "mparams.h"
C
      REAL DTLAN
      REAL LAND0(K1MAX,K2MAX,JLANT:JLANWL),
     1     LAND1(K1MAX,K2MAX,JLANT:JLANWL),
     2     LAND2(K1MAX,K2MAX,JLANT:JLANWL)
      REAL FNET(K1MAX,K2MAX), EMINP(K1MAX,K2MAX)
      REAL RUNOFF(K1MAX,K2MAX), EMPLAN(K2MAX), EMPLAV
      REAL QEVAP(K1MAX,K2MAX), MSAPCP(K1MAX,K2MAX), MCAPCP(K1MAX,K2MAX)
C
      REAL OVFLOW
      INTEGER J, K
C
      REAL WTOTAL
#ifdef DEBUG
      REAL GAVGE
#endif
C
      DO 10 K=1,K2
C       Initialize land E-P accumulator
        EMPLAN(K)= 0.0
        DO 10 J=1,K1
          IF (.NOT. SEAMSK(J,K)) THEN
C
C           Time-step land surface temperature
            LAND2(J,K,JLANT)=  LAND0(J,K,JLANT)
     1                       + 2.0 * DTLAN * FNET(J,K) / LANHCP
C
            IF (BUCKHT(J,K) .EQ. 0.0) THEN
C
C             No bucket
              LAND2(J,K,JLANWL)=  LAND1(J,K,JLANWL)
              RUNOFF(J,K)= -EMINP(J,K)
C
C             Accumulate E-P over land at each latitude
#ifdef OLD_VERSION
              EMPLAN(K)=  EMPLAN(K)
     1                  + (QEVAP(J,K)-MSAPCP(J,K)-MCAPCP(J,K))
#else
              EMPLAN(K)=  EMPLAN(K) + EMINP(J,K)
#endif
            ELSE
C
C             Time-step bucket water level (forward differencing)
              LAND2(J,K,JLANWL)=  LAND1(J,K,JLANWL)
     1                          - DTLAN * EMINP(J,K) / RHOWAT
C
C             Check bucket for overflow
              OVFLOW= AMAX1( 0., LAND2(J,K,JLANWL) - BUCKHT(J,K) )
C             Decrement water level, if there is overflow
              LAND2(J,K,JLANWL)= LAND2(J,K,JLANWL) - OVFLOW
C             Release overflow as runoff
              RUNOFF(J,K)= OVFLOW * RHOWAT / DTLAN
C             Accumulate -runoff over land at each latitude
              EMPLAN(K)=  EMPLAN(K) - RUNOFF(J,K)
            ENDIF
          ELSE
C           Set runoff to zero over sea points
            RUNOFF(J,K)= 0.
          ENDIF
   10     CONTINUE
C       Compute global average E-P over land sector
        EMPLAV= WTOTAL(G,EMPLAN,K2)
C
#ifdef DEBUG
        WRITE(*,*)
        WRITE(*,*) 'LANSTP: Debugging information'
        WRITE(*,*) 'Latitudinal profile of total land E-P'
        WRITE(*,'(1X,1P,5E15.6)') (EMPLAN(K)*YEARL/RHOWAT,K=1,K2)
        WRITE(*,'(1X,A,1P,E15.6)')'Global total land E-P (m/yr)     = ',
     1                          EMPLAV*YEARL/RHOWAT
        WRITE(*,'(1X,A,1P,E15.6)')'Global average runoff (m/yr)     = ',
     1                          GAVGE(RUNOFF)*YEARL/RHOWAT
        WRITE(*,'(1X,A,1P,E15.6)')'Global average bucket level1 (m) = ',
     1                          GAVGE(LAND1(1,1,JLANWL))*YEARL/RHOWAT
        WRITE(*,'(1X,A,1P,E15.6)')'Global average bucket level2 (m) = ',
     1                          GAVGE(LAND2(1,1,JLANWL))*YEARL/RHOWAT
#endif
C
        RETURN
        END
 
      SUBROUTINE FLXCAL( OCFLUX, IB, TAUSRF, FNET,
     1                   EMINP, EMPLAN, EMPLAV, OTFADJ, OSFADJ,
     2                   QEVAP, MSAPCP, MCAPCP )
C Compute surface fluxes over an ocean basin
C Input:  INTEGER IB -- ocean basin number
C         REAL TAUSRF(K1MAX,K2MAX,JX:JY) -- surface wind stress (Pa)
C         REAL FNET(K1MAX,K2MAX) -- Net heat flux into surface (W/m**2)
C         REAL EMINP(K1MAX,K2MAX) -- E-P values (kg/(s*m**2))
C         REAL EMPLAN(K2MAX) -- total E-P over land at each latitude
C         REAL EMPLAV -- EMPLAN, averaged over all latitudes
C         REAL OTFADJ(K1MAX,K2MAX) -- heat flux adjustment (W/m**2)
C         REAL OSFADJ(K1MAX,K2MAX) -- E-P flux adjustment (kg/(s*m**2))
C (The following fluxes are actually unnecessary, but are present for
C  compatibility with previous versions)
C         REAL QEVAP(K1MAX,K2MAX)  -- evaporative flux of water vapour
C                                     (in kg/(s*m**2))
C         REAL MSAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                static adjustment (in kg/(s*m**2))
C         REAL MCAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                convective adjustment (in kg/(s*m**2))
C Output: REAL OCFLUX(JBMAX,KBMAX,JTAUX:NFOCN) --
C         fluxes over the different basins, after adjustments,
C         redistribution of run-off, and possible zonal averaging.
C         [JTAUX:  surface-stress in X direction, in Pa;
C          JTAUY:  surface-stress in Y direction, in Pa;
C          JFS:    heat flux into ocean, in W/m**2;
C          JEMINP: E-P over ocean, including runoff, in kg/(s*m**2)]
C
#include "mgrid.h"
#include "mparams.h"
      INTEGER IB
      REAL TAUSRF(K1MAX,K2MAX,JX:JY), FNET(K1MAX,K2MAX)
      REAL EMINP(K1MAX,K2MAX), EMPLAN(K2MAX), EMPLAV
      REAL OTFADJ(K1MAX,K2MAX), OSFADJ(K1MAX,K2MAX)
      REAL OCFLUX(JBMAX,KBMAX,JTAUX:NFOCN)
      REAL QEVAP(K1MAX,K2MAX), MSAPCP(K1MAX,K2MAX), MCAPCP(K1MAX,K2MAX)
C
      INTEGER JB, KB, JA, KA
      REAL EMPBAV, AREAWT
      REAL WTOTAL
#ifdef ZONAVG
      REAL ZOFLUX(KBMAX,JTAUX:NFOCN)
      INTEGER IFX
#else
      REAL EMPBAS(KBMAX), EMPCOR
#endif
C
#ifdef ZONAVG
C     Zonally-averaged ocean-basin;
C     Average wind stress (Pa) and net heat flux (W/m**2)
C     over ocean sector only (with possible heat flux adjustment)
      DO 10 KB=1,NATMY(IB)
        ZOFLUX(KB,JTAUX)= 0.0
        ZOFLUX(KB,JTAUY)= 0.0
        ZOFLUX(KB,JFS)= 0.0
        KA= KB + KB0(IB)
        DO 10 JB=1,NATMX(IB)
          JA= 1 + MOD(JB-1+JB0(IB), K1)
          IF (SEAMSK(JA,KA)) THEN
            ZOFLUX(KB,JTAUX)= ZOFLUX(KB,JTAUX)
     1                      + TAUSRF(JA,KA,JX) / NOCNP(KB,IB)
            ZOFLUX(KB,JTAUY)= ZOFLUX(KB,JTAUY)
     1                      + TAUSRF(JA,KA,JY) / NOCNP(KB,IB)
            ZOFLUX(KB,JFS)=   ZOFLUX(KB,JFS)
     1                      + (FNET(JA,KA)-OTFADJ(JA,KA)) / NOCNP(KB,IB)
          ENDIF
   10     CONTINUE
C
C     Accumulate (NOT average) E-P over ocean sector (kg/(s*m**2))
C     (with possible flux adjustment)
      AREAWT= 0.0
      DO 20 KB=1,NATMY(IB)
        ZOFLUX(KB,JEMINP)= 0.0
        KA= KB + KB0(IB)
        AREAWT= AREAWT + G(KA)*NOCNP(KB,IB)
        DO 20 JB=1,NATMX(IB)
          JA= 1 + MOD(JB-1+JB0(IB), K1)
          IF (SEAMSK(JA,KA)) THEN
#ifdef OLD_VERSION
            ZOFLUX(KB,JEMINP)= ZOFLUX(KB,JEMINP)
     1     + ((QEVAP(JA,KA)-MSAPCP(JA,KA)-MCAPCP(JA,KA))-OSFADJ(JA,KA))
#else
            ZOFLUX(KB,JEMINP)=  ZOFLUX(KB,JEMINP)
     1                        + (EMINP(JA,KA) - OSFADJ(JA,KA))
#endif
          ENDIF
   20     CONTINUE
C
C     Compute basin-average E-P
      EMPBAV= WTOTAL( G(KB0(IB)+1), ZOFLUX(1,JEMINP), NATMY(IB) )
C
C     Include runoff in the oceanic E-P term
      DO 30 KB=1,NATMY(IB)
        KA= KB + KB0(IB)
        IF (IRUNOF .EQ. 0) THEN
C         Balance E-P separately over each basin
          ZOFLUX(KB,JEMINP)=  ZOFLUX(KB,JEMINP) / NOCNP(KB,IB)
     1                      - EMPBAV / AREAWT
        ELSE IF (IRUNOF .EQ. 1) THEN
C         Distribute run-off uniformly over entire ocean region
#ifdef OLD_VERSION
          ZOFLUX(KB,JEMINP)=  (ZOFLUX(KB,JEMINP) + ROFAC(IB)*EMPLAV)
     1                      / NOCNP(KB,IB)
#else
          ZOFLUX(KB,JEMINP)=  ZOFLUX(KB,JEMINP) / NOCNP(KB,IB)
     1                      + ROFAC(IB)*EMPLAV / AREAWT
#endif
        ELSE
C         Distribute run-off uniformly over ocean at each latitude
          ZOFLUX(KB,JEMINP)=  (ZOFLUX(KB,JEMINP) + ROFAC(IB)*EMPLAN(KA))
     1                      / NOCNP(KB,IB)
        ENDIF
   30   CONTINUE
C
C     Copy zonally-averaged fluxes to entire ocean basin
      DO 40 IFX=JTAUX,NFOCN
        DO 40 KB=1,NATMY(IB)
          DO 40 JB=1,NATMX(IB)
   40       OCFLUX(JB,KB,IFX)= ZOFLUX(KB,IFX)
#else
C     3-D ocean basin
C     Compute momentum/heat/E-P fluxes over ocean sector
C     (with possible heat/E-P flux adjustments)
      AREAWT= 0.0
      DO 50 KB=1,NATMY(IB)
        KA= KB + KB0(IB)
        EMPBAS(KB)= 0.0
        DO 50 JB=1,NATMX(IB)
          JA= 1 + MOD(JB-1+JB0(IB), K1)
C
C         Copy fluxes over all points (regardless of SEAMSK value)
C         (with possible flux adjustments)
          OCFLUX(JB,KB,JTAUX)=  TAUSRF(JA,KA,JX)
          OCFLUX(JB,KB,JTAUY)=  TAUSRF(JA,KA,JY)
          OCFLUX(JB,KB,JFS)=    FNET(JA,KA)-OTFADJ(JA,KA)
          OCFLUX(JB,KB,JEMINP)= EMINP(JA,KA)-OSFADJ(JA,KA)
C
          IF (SEAMSK(JA,KA)) THEN
C           Over sea points only; accumulate E-P and area weight
            EMPBAS(KB)= EMPBAS(KB) + OCFLUX(JB,KB,JEMINP)
            AREAWT= AREAWT + G(KA)
          ENDIF
   50     CONTINUE
C
C     Compute basin-average E-P
      EMPBAV= WTOTAL( G(KB0(IB)+1), EMPBAS(1), NATMY(IB) )
C
      DO 60 KB=1,NATMY(IB)
        KA= KB + KB0(IB)
        IF (IRUNOF .EQ. 0) THEN
C         Balance E-P separately over each basin
          EMPCOR= - EMPBAV / AREAWT
        ELSE IF (IRUNOF .EQ. 1) THEN
C         Distribute run-off uniformly over entire ocean region
          EMPCOR= ROFAC(IB)*EMPLAV / AREAWT
        ELSE
C         Distribute run-off uniformly over ocean at each latitude
          EMPCOR= ROFAC(IB)*EMPLAN(KA)/NOCNP(KB,IB)
        ENDIF
        DO 60 JB=1,NATMX(IB)
          JA= 1 + MOD(JB-1+JB0(IB), K1)
          IF (SEAMSK(JA,KA)) THEN
C           Over sea points only; modify E-P flux to include runoff
            OCFLUX(JB,KB,JEMINP)=  OCFLUX(JB,KB,JEMINP) + EMPCOR
          ENDIF
   60     CONTINUE
#endif
C
      RETURN
      END
