C ICAM (Idealized Coupleable Atmospheric Model)
C icamsrc $Revision: 1.8 $ created $Date: 1996/05/02 21:31:03 $
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C aparams.F: Fortran source module
C Contains routines for initializing model parameters at the beginning
C of a run
C
 
#include "mmain.h"
 
C Routines: (all called by ATMINI)
C INICON: Initialize model constants, and unit numbers
C INIGRD: Initialize model grid
C INIMSK: Initialize "masks"
C ->INISEA: Initialize ocean basin details and sea-mask
C INIPR2: Initialize derived parameters
C INIPGQ: Default initialization of prognostic variables
C GETCOF: Read harmonic coefficients for time-dependent forcing
C DISPAR: Display model parameters
C
C General purpose routines:
C ATMERR: Abort execution, printing a message
C LJUST:  Left justify a string, removing leading spaces
C RJUST:  Right justify a string, removing trailing spaces
C
C Internal routines: LMASK, FMASK
C
 
      SUBROUTINE INICON( IUNIT1, DIRSTR )
C Initialize model constants, and unit numbers
C
C Input:  INTEGER IUNIT1 -- starting Fortran unit number for model use
C           (units IUNIT1 .. IUNIT1+9 are reserved for model use)
C         CHARACTER*(*) DIRSTR -- name of directory containing
C            the model parameter file and other initialization files
C Output: (mparams.h, Part 1)
C
#include "mparams.h"
C
      INTEGER IUNIT1
      CHARACTER*(*) DIRSTR
C
      REAL OMEGA, R, CPRESS, GRAVIT
C
C     Initialize program name
#ifdef SLAB_OCEAN
C     Internal (mixed-layer) ocean version
      PRGNAM= 'ICAM_SLAB'
#else
C     External ocean version
      PRGNAM= 'ICAM'
#endif
C     Initialize program version number
      PRGVER= '$Revision: 1.8 $, $Date: 1996/05/02 21:31:03 $'
C
C     Initialize directory name for parameter/initialization files
      PARDIR= DIRSTR
      CALL RJUST( PARDIR )
C
C     Initialize planetary parameters (in S.I. units)
C
C     Length of day (sec)
      DDAYL= 86400.
      DAYL= DDAYL
C     Robert time-filter factor (usually 0.01 or 0.02)
      TIMFIL= 0.02
C
C     Angular velocity of rotation (1/sec)
      OMEGA= 2.0 * PI / DAYL
C     Dry air gas constants (J/(kg*K))
      R= 287.0
      CPRESS= 1004.0
C     Surface gravity (m/s**2)
      GRAVIT= 9.81
C
C     Initialize planetary parameters (in mgrid.h)
      CALL PLINI( OMEGA, R, CPRESS, GRAVIT )
C
C     Planetary radius (m)
#ifdef OLD_VERSION
      AEARTH= 6.4E6
#else
      AEARTH= 6.37E6
#endif
C     Surface pressure (Pa)
      PBOT= 1000.0E2
C     Solar constant (in W/m**2)
      SOLFLX= 1360.0
C     Surface exchange coefficient (non-dimensional)
      EXCH= 1.1E-3
C     Density of freshwater (kg/m**3)
      RHOWAT= 1000.0
C     Density of ocean (kg/m**3)
      OCNRHO= 1000.0
C     Specific heat of ocean water (J/(kg*K))
#ifdef OLD_VERSION
      OCNCP= 4200.0
#else
      OCNCP= 4000.0
#endif
C
C     Fortran unit numbers
      IUPAR= IUNIT1
      IUPAK= IUNIT1 + 1
      IUTEM= IUNIT1 + 2
      IUDAA= IUNIT1 + 3
      IUTSA= IUNIT1 + 4
      IUHSA= IUNIT1 + 5
      IUVSA= IUNIT1 + 6
      IUZSA= IUNIT1 + 7
C
C     Formats to be used for save file
      AFMT= '(A72)'
      CFMT= '(A1,1X,A)'
      EFMT= '(1X,1P,5E15.6)'
      IFMT= '(1X,5I15)'
      LFMT= '(1X,32L2)'
C
      RETURN
      END
 
      SUBROUTINE INIGRD( LTROP )
C Initialize model grid (horizontal/vertical), and Del**8 damping
C
C Input:  (mparams.h: Parts 1-2)
C Output: (mgrid.h)
C         INTEGER LTROP -- model tropopause level number
C
#include "mgrid.h"
#include "mparams.h"
C
      INTEGER LTROP
C
C     Vertical discretisation
      REAL PFULL1(L1MAX+1), ZFULL1(L1MAX+1), TZMEN1(L1MAX+1)
      REAL PTHICK(L1MAX), PHALF(L1MAX), PFULL(L1MAX), PTROP0
      REAL PKFULL(L1MAX), ZFULL(L1MAX), ZNOND(L1MAX)
      REAL TRADEQ(K2MAX,L1MAX), TZMEAN(L1MAX), TZREF(L1MAX), TSTEEP
      INTEGER LSPNGE
C     Vertical damping
      REAL TDAMP(L1MAX), UDAMP(L1MAX), VERVIS(L1MAX), VISCB
C
      REAL PFAC, DEL8DF, TEMP
      INTEGER I, K, L
C
C     Initialize horizontal discretisation
      CALL SPINI( MMAX, NMAX, AEARTH )
C
C     Compute pressure level thicknesses
      IF ((PBOT/PLID).GT.FLOAT(NLEV)) THEN
C       Level thickness to increase with increasing pressure
C       Compute pressure-level thickness amplification factor PFAC
        PFAC= (PBOT/PLID)**(1.0/(NLEV-1))
        DO 110 I=1,10
  110     PFAC= (1.0 + (PFAC - 1.0)*(PBOT/PLID))**(1.0/NLEV)
C       Determine level thicknesses
        PTHICK(NLEV)= PLID * PFAC**(NLEV-1)
        TEMP= PTHICK(NLEV)
        DO 120 L=NLEV-1,2,-1
          PTHICK(L)= PTHICK(L+1) / PFAC
          TEMP= TEMP + PTHICK(L)
  120     CONTINUE
C       Ensure that the thicknesses add up to the surface pressure
        PTHICK(1)= PBOT - TEMP
      ELSE IF ((PBOT/PLID).GT.0.99*FLOAT(NLEV)) THEN
C       Equally spaced pressure levels
        PFAC= 1.0
        TEMP= PBOT / NLEV
        DO 130 L=1,NLEV
  130     PTHICK(L)= TEMP
      ELSE
        CALL ATMERR( 'INIGRD', 'Uppermost pressure level too thick')
      ENDIF
C     Compute pressure-levels
      PHALF(NLEV)= PBOT
      LTROP= NLEV
      LSPNGE= NLEV
      DO 140 L=NLEV,2,-1
        PHALF(L-1)= PHALF(L) -     PTHICK(L)
        PFULL(L)=   PHALF(L) - 0.5*PTHICK(L)
        IF (PFULL(L).GT.PTROP) LTROP= L
        IF (PFULL(L).GT.DPTOP) LSPNGE= L
  140   CONTINUE
C     Ensure that the upper-boundary is at zero pressure
      PFULL(1)= PHALF(1) - 0.5*PTHICK(1)
      IF (PFULL(1).GT.PTROP) LTROP= 1
C Note: LTROP is the highest level within the troposphere
      IF (PFULL(1).GT.DPTOP) LSPNGE= 1
C Note: LSPNGE is the highest level below the sponge layer
C     Compute effective tropopause location (lying between model levels)
      IF (LTROP.GT.1) THEN
        PTROP0= SQRT(PFULL(LTROP)*PFULL(LTROP-1))
      ELSE
        PTROP0= PFULL(LTROP)
      ENDIF
C     Compute U.S. Reference Atmosphere temperatures at all levels
      DO 150 L=1,NLEV
  150   PFULL1(L)= PFULL(L)
      PFULL1(NLEV+1)= PBOT
      CALL TPPROF( TZMEN1, ZFULL1, PFULL1, NLEV+1, 6, RGAS, G0)
C     Initialize standard potential temperature profile
      DO 170 L=1,NLEV
C       Compute auxiliary vertical coordinate
        PKFULL(L)= (PFULL(L)/PBOT)**(RGAS/CP)
C       Compute "non-dimensional height"
        ZNOND(L)= AMIN1(1.0, ALOG(PBOT/PFULL(L)) / ALOG(PBOT/PTROP0) )
C       Copy standard height values
        ZFULL(L)= ZFULL1(L)
C       Standard THETA profile
        TZMEAN(L)=  TZMEN1(L) / PKFULL(L)
C       Radiative equilibrium THETA (zero => no forcing)
        DO 160 K=1,K2
  160     TRADEQ(K,L)= 0.0
  170   CONTINUE
C     Initialize reference temperature profile
C     Reference Theta coincides with standard THETA at surface
      TZREF(NLEV)= TZMEAN(NLEV)
      DO 180 L=NLEV-1,1,-1
C       Steepness factor for reference temperature profile
C       (As compared to standard temperature profile)
        IF (L+1.GE.LTROP) THEN
C         Previous level in the troposphere
          TSTEEP= REFFAC * ( 1.0 + 0.0*ZNOND(L+1) )
        ELSE
C         Previous level above the troposphere
          TSTEEP= REFFAC * ( 0.75 + 0.25*SQRT(PFULL(L+1)/PTROP0) )
        ENDIF
        TZREF(L)= TZREF(L+1) + TSTEEP * (TZMEAN(L)-TZMEAN(L+1))
  180   CONTINUE
C
C     Initialize vertical damping parameters
C
      DO 200 L=1,NLEV
C       No radiative relaxation in the interior
        TDAMP(L)= 0.0
C       No Rayleigh friction (as yet)
        UDAMP(L)= 0.0
C       Vertical viscosity in the interior
        VERVIS(L)= VISC0
  200   CONTINUE
C
C     Viscosity near the surface (effectively, the Ekman drag)
      VISCB= DMPBOT*(CP*(1.0-PKFULL(NLEV))*TZMEAN(NLEV)/G0)**2
     1             *(PTHICK(NLEV)/(PHALF(NLEV) - PFULL(NLEV)))
C
      IF (DZBOT.GT.0.) THEN
C       Viscous boundary layer near the bottom
        DO 210 L=1,NLEV
          VERVIS(L)= VERVIS(L) + (VISCB-VISC0)*
     1                AMAX1(0., 1.0-(ZFULL(L)-ZFULL(NLEV))/DZBOT)
  210     CONTINUE
      ELSE
C       Only surface friction
        VERVIS(NLEV)= VISCB
      ENDIF
C
      IF (DPTOP.GT.0.) THEN
C       Sponge layer near the top
        DO 220 L=1,NLEV-1
C         Increased Newtonian cooling
          TDAMP(L)= TDAMP(L) + DMPTOP*
     1               AMAX1(0., 1.0-(PFULL(L)-PFULL(1))/DPTOP)
C         Increased Rayleigh friction
          UDAMP(L)= UDAMP(L) + DMPTOP*
     1               AMAX1(0., 1.0-(PFULL(L)-PFULL(1))/DPTOP)
  220     CONTINUE
      ENDIF
C
C     Quad-harmonic horizontal diffusion
      DEL8DF= D8DAMP / (FLOAT(NMAX*(NMAX+1))**4)
C
C     Initialize vertical truncation
      CALL VERINI( NLEV, PFULL, PHALF, TZMEAN, TZREF,
     1             DEL8DF, TRADEQ, TDAMP, UDAMP, IPRNDL, VERVIS )
C     Display vertical truncation details
      WRITE(*,*)
      WRITE(*,*) 'Vertical truncation details'
      WRITE(*,*) 'PFAC= ', PFAC
      WRITE(*,*)
      WRITE(*,'(1X,A3,5A8,3A11)') 'L', 'DP mb', 'P mb', 'Z km',
     1                            'Th-st K', 'Th-rf K', 'TDAMP 1/d',
     2                            'UDAMP 1/d','Visc m^2/s'
      WRITE(*,*)
      DO 230 L=1,L1
        WRITE(*,'(1X,I3,5F8.2,1P,3E11.2)') L, DP(L)/100.,PLV(L)/100.,
     1                         ZSTDLV(L)/1000., TSTDLV(L), TREFLV(L),
     2                         TDAMP(L)*DAYL, UDAMP(L)*DAYL, VERVIS(L)
  230   CONTINUE
      WRITE(*,*)
      WRITE(*,*) 'LTROP= ', LTROP,
     1           ', PTROP0= ', NINT(PTROP0/100.),' mb'
      WRITE(*,*)
      RETURN
      END
 
      SUBROUTINE INISEA
C Initialize ocean basin details and sea-mask
C Input:  (mparams.h: Parts 1-2)
C Output: (mparams.h: Part 3)
C
#include "mgrid.h"
#include "mparams.h"
C
      REAL DELLON
      INTEGER JLO, JHI, KLO, KHI
      INTEGER IB, JB, KB, J, K
      CHARACTER FILNAM*160
C
C     Set default sea mask values (no sea points)
      DO 200 K=1,K2
        DO 200 J=1,K1
  200     SEAMSK(J,K)= .FALSE.
C
C     Locate longitude/latitude boundaries for each ocean basin
C     (Basin boundaries correspond to velocity grid-points)
      DO 290 IB=1,NBASIN
C
C       Locate latitudinal boundaries of ocean basin
        KLO= 1
        KHI= 0
        DO 210 K=1,K2
          IF (DEGLAT(K) .LT. LATBLO(IB)) KLO= K+1
          IF (DEGLAT(K) .LT. LATBHI(IB)) KHI= K
  210     CONTINUE
C
        IF ( (KLO .GT. KHI) .OR. (KLO .GT. K2) ) THEN
          CALL ATMERR( 'INISEA',
     1                'Error in latitudinal boundary of basin' )
        ENDIF
C
C       Longitude grid-length (degrees)
        DELLON= 360. / K1
C
C       Locate longitudinal boundaries of ocean basin
        JLO= 1 + NINT( 0.5 + LONBLO(IB)/DELLON)
        JHI= 1 + NINT(-0.5 + LONBHI(IB)/DELLON)
C
C       (Note: JLO should range from 2 to K1+1; JHI from JLO to 2*K1)
        IF ( (JLO .LT. 2)   .OR. (JLO .GT. K1+1) .OR.
     1       (JHI .LT. JLO) .OR. ((JHI-JLO+1) .GT. K1) ) THEN
          CALL ATMERR( 'INISEA',
     1                'Error in longitudinal boundary of basin' )
        ENDIF
C
#ifdef DEBUG
        WRITE(*,*)
        WRITE(*,*) 'Subroutine INISEA: debugging information'
        WRITE(*,'(1X,A,I2)') 'Basin IB=', IB
        WRITE(*,*) 'JLO, JHI, KLO, KHI= ', JLO, JHI, KLO, KHI
#endif
C
C       Check that basin fits within declared array dimensions
        IF ( ((KHI-KLO+1) .GT. KBMAX) .OR.
     1       ((JHI-JLO+1) .GT. JBMAX)      ) THEN
          CALL ATMERR( 'INISEA', 'Ocean basin too large; recompile' )
        ENDIF
C
C       Extent of ocean basin (in atmospheric grid)
        NATMX(IB)= JHI - JLO + 1
        NATMY(IB)= KHI - KLO + 1
C       Basin index offsets
        JB0(IB)= JLO - 1
        KB0(IB)= KLO - 1
C
C       Set sea mask over ocean basin, check for overlaps
        DO 240 KB=1,NATMY(IB)
          K= KB + KB0(IB)
          DO 240 JB=1,NATMX(IB)
            J= 1 + MOD(JB-1+JB0(IB), K1)
            IF (.NOT.SEAMSK(J,K)) THEN
              SEAMSK(J,K)= .TRUE.
            ELSE
              CALL ATMERR( 'INISEA',
     1                    'Error: overlapping ocean basins')
            ENDIF
  240     CONTINUE
  290   CONTINUE
C
C     Read and overwrite sea mask, if filename SEAFIL was specified
      IF (SEASPF) THEN
        FILNAM= PARDIR//SEAFIL
        CALL LJUST( FILNAM )
        OPEN( IUTEM, FILE= FILNAM, STATUS='OLD' )
        READ(IUTEM,*) ((SEAMSK(J,K),J=1,K1),K=1,K2)
        CLOSE( IUTEM)
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE INIMSK
C Initialize "masks"
C (sea mask, bucket heights, and orography)
C
C Input:  (mparams.h, Parts 1-2)
C Output: (mparams.h, Part 3)
C
#include "mgrid.h"
#include "mparams.h"
C
      INTEGER J, K
      CHARACTER FILNAM*160
C
C     Initialize ocean basin details, and sea-mask
      CALL INISEA
C
C     Initialize bucket heights (m)
      DO 10 K=1,K2
        DO 10 J=1,K1
   10     BUCKHT(J,K)= BUKDEP
C
      IF (OROSPF) THEN
C       Read orography height (in meters) from file
        FILNAM= PARDIR//OROFIL
        CALL LJUST( FILNAM )
        OPEN( IUTEM, FILE= FILNAM, STATUS='OLD' )
        READ(IUTEM,*) ((OROHT(J,K),J=1,K1),K=1,K2)
        CLOSE( IUTEM)
      ELSE
C       No orography (default)
        DO 20 K=1,K2
          DO 20 J=1,K1
   20       OROHT(J,K)= 0.0
      ENDIF
      RETURN
      END
 
      SUBROUTINE INIPR2
C Initialize derived parameters
C Input:  (mparams.h: Parts 1-3)
C Output: (mparams.h: Part 4)
C
#include "mgrid.h"
#include "mparams.h"
C
      INTEGER IB, J, K, JA, KA, JB, KB
C
C     Initialize atmospheric coordinates used in interpolation
C
C     Longitude values of edges of atmospheric grid-points (degrees)
C     (Range: 0 to 720)
      DO 10 J=1,K1
       DGATM(J)= DEGLON(J)
   10  DGATM(J+K1)= DEGLON(J) + 360.
      DGATM(2*K1+1)= DGATM(K1+1) + 360.
C     Longitude values of atmospheric grid box edges (degrees)
C     (Range: 0- to 720+)
      DO 20 J=1,2*K1
   20   DGAMID(J)= 0.5*( DGATM(J) + DGATM(J+1) )
      DGAMID(0)= DGATM(K1) - 360.
      DGAMID(2*K1+1)= DGAMID(K1+1) + 360.
C
C     Sine-latitude (mu) values of atmospheric grid box "edges"
      MUAMID(0)= -1.0
      MUAMID(K2)= 1.0
      DO 30 K=1,K2-1
   30   MUAMID(K)= MUAMID(K-1) + 2.0*G(K)
C
#ifdef SLAB_OCEAN
C     Internal (mixed-layer) ocean
C     Initialize mixed layer heat capacity (J/m2K)
      DO 40 K=1,K2
        DO 40 J=1,K1
   40     MIXHCP(J,K)= OCNRHO * OCNCP * MIXDEP
#endif
C
C     Compute no. of sea points for each latitude in each ocean basin
      DO 70 IB=1,NBASIN
        DO 60 KB=1,NATMY(IB)
          NOCNP(KB,IB)= 0
          KA= KB + KB0(IB)
          DO 50 JB=1,NATMX(IB)
            JA= 1 + MOD(JB-1+JB0(IB), K1)
            IF (SEAMSK(JA,KA)) THEN
              NOCNP(KB,IB)= NOCNP(KB,IB) + 1
            ENDIF
   50       CONTINUE
          IF (NOCNP(KB,IB) .EQ. 0) THEN
            CALL ATMERR( 'INIPR2',
     1                  'All land latitude in ocean basin')
          ENDIF
   60     CONTINUE
C
#ifdef DEBUG
        WRITE(*,*) 'INIPR2: NOCNP(*,IB), IB=', IB
        WRITE(*,'(1X,16I4)') (NOCNP(KB,IB),KB=1,NATMY(IB))
#endif
C
   70   CONTINUE
C
C     Initialize progostic SST mask
C    (i.e., sea mask for coupled basins only)
      DO 80 K=1,K2
        DO 80 J=1,K1
   80       PROSST(J,K)= .FALSE.
C
      DO 90 IB=1,NBCOUP
        DO 90 KB=1,NATMY(IB)
          KA= KB + KB0(IB)
          DO 90 JB=1,NATMX(IB)
            JA= 1 + MOD(JB-1+JB0(IB), K1)
   90       PROSST(JA,KA)= SEAMSK(JA,KA)
C
      RETURN
      END
 
 
      SUBROUTINE INIPGQ
C Default initialization of prognostic variables,
C at both time-level 0 and time-level 1.
C (SST, land/ice temperatures, land water level, ice mask, ice height,
C  and all atmospheric variables)
C
C Input:  (none)
C Output: (mstate.h: Part 2)
C
#include "mgrid.h"
#include "mparams.h"
#include "mstate.h"
C
      REAL FPH(K1MAX,K2MAX), TS(K2MAX)
      CHARACTER*160 FILNAM
      INTEGER J, K, L, IPGQ
C
C     Reference zonally-symmetric surface temperatures
      DO 110 K=1,K2
  110   TS(K)= 301.0 - 31.5*MU(K)**2
C
C     Initialize zonally-symmetric SST, land/ice temperatures,
C     and land water level
      DO 120 K=1,K2
        DO 120 J=1,K1
          IF (.NOT.SEAMSK(J,K)) THEN
C           Land surface
            LAND1(J,K,JLANT)= TS(K)
            SST1(J,K)= 0.
          ELSE IF (PROSST(J,K)) THEN
C           Prognostic sea surface
            SST1(J,K)= TS(K)
            LAND1(J,K,JLANT)= 0.
          ELSE
C           Non-prognostic sea surface
            SST1(J,K)= 0.
            LAND1(J,K,JLANT)= 0.
          ENDIF
C         Ice temperature
          ICE1(J,K,JICET)= 0.0
C         Zero land water level
  120     LAND1(J,K,JLANWL)= 0.0
C
      IF (ICESPF) THEN
C       Read specified ice mask (T/F), ice height (m) from file
        FILNAM= PARDIR//ICEFIL
        CALL LJUST( FILNAM )
        OPEN( IUTEM, FILE= FILNAM, STATUS='OLD' )
        READ(IUTEM,*) ((ICEMSK(J,K),J=1,K1),K=1,K2)
        READ(IUTEM,*) ((ICE1(J,K,JICEHT),J=1,K1),K=1,K2)
        CLOSE( IUTEM)
      ELSE
C       Set default ice mask, ice height (no ice)
        DO 130 K=1,K2
          DO 130 J=1,K1
            ICEMSK(J,K)= .FALSE.
  130       ICE1(J,K,JICEHT)= 0.0
      ENDIF
C
C     Copy "surface" quantities to time-level 0
      CALL PHCOPY( SST0(1,1), SST1(1,1) )
      DO 140 IPGQ=JLANT,JLANWL
  140   CALL PHCOPY( LAND0(1,1,IPGQ), LAND1(1,1,IPGQ) )
      DO 150 IPGQ=JICET,JICEHT
  150   CALL PHCOPY( ICE0(1,1,IPGQ), ICE1(1,1,IPGQ) )
C
C     Zero all atmospheric prognostic quantities
      DO 200 L=1,L1
        DO 200 IPGQ=1,NPGQ
  200     CALL ZEROSP( PGQSP1(0,0,IPGQ,L), N1 )
C
C     Initial potential temperature distribution (vertically isothermal)
      DO 220 L=1,L1
        DO 210 K=1,K2
          DO 210 J=1,K1
  210       FPH(J,K)= TS(K) / PKLV(L)
          CALL SPECTR( PGQSP1(0,0,JPOT,L), FPH )
  220     CONTINUE
C
      IF (M1.GT.0) THEN
C       Initial perturbation: a "drop" of vorticity at all levels
        DO 240 L=1,L1
          DO 230 K=1,K2
            DO 230 J=1,K1
              FPH(J,K)= (PLV(L)/PSURF)*(0.1/DAYL)*COS(PHI(K))*
     1                           EXP( -( ((DEGLAT(K)-15.0)/5.)**2 +
     2                                   ((DEGLON(J)-180.)/5.)**2   ) )
  230       CONTINUE
          CALL SPECTR( PGQSP1(0,0,JVOR,L), FPH )
C         Zero global-mean rel-vorticity (for kinematic consistency)
          PGQSP1(0,0,JVOR,L)= (0., 0.)
  240     CONTINUE
      ENDIF
C
C     Copy atmospheric quantities to time-level 0
      DO 250 L=1,L1
        DO 250 IPGQ=1,NPGQ
  250     CALL SPCOPY( PGQSP0(0,0,IPGQ,L), PGQSP1(0,0,IPGQ,L), N1)
      RETURN
      END
 
      SUBROUTINE GETCOF
C Read harmonic coefficients for time-dependent forcing
C (i.e., SST, heat/E-P flux adjustments).
C Initialize common blocks in mcoef.h
C
C Input:  (read from files)
C Output: (mcoef.h)
C
#include "mgrid.h"
#include "mparams.h"
#include "mcoef.h"
C
      CHARACTER*160 FILNAM
      REAL TEMZAV(K2MAX,5)
      INTEGER I, J, K, ICOUNT
C
      IF (SSTSPF) THEN
C       Read (zonally-symmetric) SST coefficients from file
        FILNAM= PARDIR//SSTFIL
        CALL LJUST( FILNAM )
        OPEN( IUTEM, FILE= FILNAM, STATUS='OLD' )
C
        READ(IUTEM,*)
     1   ((TSRF0(J,K),TSRF1A(J,K),TSRF2A(J,K),
     2                TSRF1P(J,K),TSRF2P(J,K),J=1,K1),K=1,K2)
        CLOSE( IUTEM )
C
C       Compute zonal average over ocean points
        DO 140 K=1,K2
          DO 110 I=1,5
  110       TEMZAV(K,I)= 0.0
          ICOUNT= 0
          DO 120 J=1,K1
            IF (SEAMSK(J,K) .AND. .NOT. PROSST(J,K)) THEN
               ICOUNT= ICOUNT + 1
               TEMZAV(K,1)= TEMZAV(K,1) + TSRF0(J,K)
               TEMZAV(K,2)= TEMZAV(K,2) + TSRF1A(J,K)
               TEMZAV(K,3)= TEMZAV(K,3) + TSRF2A(J,K)
               TEMZAV(K,4)= TEMZAV(K,4) + TSRF1P(J,K)
               TEMZAV(K,5)= TEMZAV(K,5) + TSRF2P(J,K)
            ENDIF
  120       CONTINUE
          IF (ICOUNT .GT. 0) THEN
            DO 130 I=1,5
  130         TEMZAV(K,I)= TEMZAV(K,I)/ICOUNT
          ENDIF
  140     CONTINUE
C
        WRITE(*,*)
        WRITE(*,'(A,A)') 'SSTFIL=', FILNAM
        WRITE(*,*) 'Zonally-averaged SST coefficients:'
        WRITE(*,'(1X,5A8)')
     1 '   TSRF0', '  TSRF1A', '  TSRF2A', '  TSRF1P', '  TSRF2P'
        WRITE(*,'(1X,5F8.3)') ((TEMZAV(K,I),I=1,5),K=1,K2)
        WRITE(*,*)
      ENDIF
C
      IF (FADSPF) THEN
C       Read (zonally-symmetric) flux adjustment coefficients from file
        FILNAM= PARDIR//FADFIL
        CALL LJUST( FILNAM )
        OPEN( IUTEM, FILE= FILNAM, STATUS='OLD' )
C
C       Read heat flux adjustment coefficients (in W/m**2)
        READ(IUTEM,*)
     1   ((OTFA0(J,K),OTFA1A(J,K),OTFA2A(J,K),
     2                OTFA1P(J,K),OTFA2P(J,K),J=1,K1),K=1,K2)
C
C       Read E-P flux adjustment coefficients (kg/(s*m**2))
        READ(IUTEM,*)
     1   ((OSFA0(J,K),OSFA1A(J,K),OSFA2A(J,K),
     2                OSFA1P(J,K),OSFA2P(J,K),J=1,K1),K=1,K2)
C
        CLOSE( IUTEM )
C
C       Compute zonal average of heat flux adjustment over ocean points
        DO 240 K=1,K2
          DO 210 I=1,5
  210       TEMZAV(K,I)= 0.0
          ICOUNT= 0
          DO 220 J=1,K1
            IF (SEAMSK(J,K) .AND. .NOT. PROSST(J,K)) THEN
               ICOUNT= ICOUNT + 1
               TEMZAV(K,1)= TEMZAV(K,1) + OTFA0(J,K)
               TEMZAV(K,2)= TEMZAV(K,2) + OTFA1A(J,K)
               TEMZAV(K,3)= TEMZAV(K,3) + OTFA2A(J,K)
               TEMZAV(K,4)= TEMZAV(K,4) + OTFA1P(J,K)
               TEMZAV(K,5)= TEMZAV(K,5) + OTFA2P(J,K)
            ENDIF
  220       CONTINUE
          IF (ICOUNT .GT. 0) THEN
            DO 230 I=1,5
  230         TEMZAV(K,I)= TEMZAV(K,I)/ICOUNT
          ENDIF
  240     CONTINUE
C
        WRITE(*,*)
        WRITE(*,'(A,A)') 'FADFIL=', FILNAM
        WRITE(*,*) 'Heat flux adjustment (W/m**2)'
        WRITE(*,'(1X,5A8)')
     1   '   OTFA0', '  OTFA1A', '  OTFA2A', '  OTFA1P', '  OTFA2P'
        WRITE(*,'(1X,5F8.3)') ((TEMZAV(K,I),I=1,5),K=1,K2)
C
C       Compute zonal average of heat flux adjustment over ocean points
        DO 340 K=1,K2
          DO 310 I=1,5
  310       TEMZAV(K,I)= 0.0
          ICOUNT= 0
          DO 320 J=1,K1
            IF (SEAMSK(J,K) .AND. .NOT. PROSST(J,K)) THEN
               ICOUNT= ICOUNT + 1
               TEMZAV(K,1)= TEMZAV(K,1) + OSFA0(J,K)
               TEMZAV(K,2)= TEMZAV(K,2) + OSFA1A(J,K)
               TEMZAV(K,3)= TEMZAV(K,3) + OSFA2A(J,K)
               TEMZAV(K,4)= TEMZAV(K,4) + OSFA1P(J,K)
               TEMZAV(K,5)= TEMZAV(K,5) + OSFA2P(J,K)
            ENDIF
  320       CONTINUE
          IF (ICOUNT .GT. 0) THEN
            DO 330 I=1,5
  330         TEMZAV(K,I)= TEMZAV(K,I)/ICOUNT
          ENDIF
  340     CONTINUE
C
        WRITE(*,*)
        WRITE(*,*) 'E-P flux adjustment (kg/(s*m**2))'
        WRITE(*,'(1X,5A8)')
     1   '   OSFA0', '  OSFA1A', '  OSFA2A', '  OSFA1P', '  OSFA2P'
        WRITE(*,'(1X,5F8.3)') ((TEMZAV(K,I),I=1,5),K=1,K2)
        WRITE(*,*)
C
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE DISPAR
C Display model parameters and timing information
C Input: (mstate.h: Part 1; mparams.h: Parts 1-3)
C Output: (standard output)
C
#include "mparams.h"
#include "mstate.h"
C
C
      INTEGER IB
C
C     Display information from "mstate.h", Part 1
      WRITE(*,'(A,A)') 'Program version: ',  INIPRG
      WRITE(*,'(A,A)') 'Initialized:     ',  INITIM
      WRITE(*,'(1X,A,F12.2,A)') 'Current model time = ',
     1                           TIM/DAYL,  ' days'
      WRITE(*,'(1X,A,I12)')     'Current time-step  = ', ISTEP
      WRITE(*,'(1X,A,I12)')     'Current run part   = ', IPART
C
C     Display information from "mparams.h", Part 2
      WRITE(*,*)
      WRITE(*,'(A,A)') 'Run title:       ',  RTITLE
      WRITE(*,'(A,A)') 'Sea mask file:   ',  SEAFIL
      WRITE(*,'(A,A)') 'Orography file:  ',  OROFIL
      WRITE(*,'(A,A)') 'Clim. SST file:  ',  SSTFIL
      WRITE(*,'(A,A)') 'Flux adj. file:  ',  FADFIL
      WRITE(*,'(A,A)') 'Ice  file:       ',  ICEFIL
C
      WRITE(*,*)
      WRITE(*,'(1X,3A12)') 'NSPDAY', 'NDPYR', 'NSYNC'
      WRITE(*,'(1X,3I12)')  NSPDAY,   NDPYR, NSYNC
      WRITE(*,'(1X,4A12)') 'MMAX', 'NMAX', 'NLEV', 'NTRC'
      WRITE(*,'(1X,4I12)')  MMAX,   NMAX,   NLEV,   NTRC
C
      WRITE(*,*)
      WRITE(*,'(1X,3A12)') 'COUFLG', 'SSNFLG', 'ICEFLG'
      WRITE(*,'(1X,3L12)')  COUFLG,   SSNFLG,   ICEFLG
C
      WRITE(*,*)
      WRITE(*,'(1X,2A14)')  'PLID(mb)', 'PTROP(mb)'
      WRITE(*,'(1X,2G14.4)') PLID/100.,  PTROP/100.
      WRITE(*,'(1X,3A14)')  'EQPFAC', 'PPFAC', 'YPHASE'
      WRITE(*,'(1X,3G14.4)') EQPFAC,   PPFAC,   YPHASE
      WRITE(*,'(1X,4A14)')
     1        'IMPLCT', 'REFFAC', 'D8DAMP(1/d)', 'IPRNDL'
      WRITE(*,'(1X,4G14.4)')
     1         IMPLCT, REFFAC, D8DAMP*DAYL, IPRNDL
      WRITE(*,'(1X,5A14)') 'VISC0(m**2/s)',
     1        'DMPBOT(1/d)', 'DZBOT(km)', 'DMPTOP(1/d)', 'DPTOP(mb)'
      WRITE(*,'(1X,5G14.4)') VISC0,
     1         DMPBOT*DAYL, DZBOT/1000., DMPTOP*DAYL, DPTOP/100.
      WRITE(*,'(1X,5A14)')
     1        'LWFRAC', 'DLWAIR', 'DLWH2O', 'DSWAIR', 'DSWO3'
      WRITE(*,'(1X,5G14.4)')
     1         LWFRAC,   DLWAIR,   DLWH2O,   DSWAIR,   DSWO3
      WRITE(*,'(1X,3A14)')  'DLWCO2', 'MULCO2', 'MULTIM(yrs)'
      WRITE(*,'(1X,3G14.4)') DLWCO2,   MULCO2,   MULTIM/(NDPYR*DAYL)
      WRITE(*,'(1X,3A14)')  'ALBSEA', 'ALBLAN', 'ALBICE'
      WRITE(*,'(1X,3G14.4)') ALBSEA,   ALBLAN,   ALBICE
      WRITE(*,'(1X,4A14)')
     1        'LANHCP(J/m2K)', 'SATURF', 'BUKDEP(m)', 'MIXDEP(m)'
      WRITE(*,'(1X,4G14.4)')
     1         LANHCP,          SATURF,   BUKDEP,      MIXDEP
C
      WRITE(*,*)
      WRITE(*,'(1X,A,I12)') 'Number of ocean basins:       ', NBASIN
      WRITE(*,'(1X,A,I12)') 'Number coupled to atmosphere: ', NBCOUP
      WRITE(*,'(1X,A,I12)') 'Run-off distribution option:  ',  IRUNOF
      DO 200 IB=1,NBASIN
        WRITE(*,'(1X,A,I2)') 'OCEAN BASIN ', IB
        WRITE(*,'(1X,A,F7.3)') 'Run off factor= ', ROFAC(IB)
        WRITE(*,'(1X,A,F7.3,A,F7.3)')
     1          'Ocean-lat range: ', LATBLO(IB),
     2                       ' to ', LATBHI(IB)
        WRITE(*,'(1X,A,F7.3,A,F7.3)')
     1          'Ocean-mu  range: ', SIN(LATBLO(IB)*PI/180.),
     2                       ' to ', SIN(LATBHI(IB)*PI/180.)
        WRITE(*,'(1X,A,F7.3,A,F7.3)')
     1          'Atmos-mu  range: ', MUAMID(KB0(IB)),
     2                       ' to ', MUAMID(KB0(IB)+NATMY(IB))
        WRITE(*,'(1X,A,F7.3,A,F7.3)')
     1          'Ocean-lon range: ', LONBLO(IB),
     2                       ' to ', LONBHI(IB)
        WRITE(*,'(1X,A,F7.3,A,F7.3)')
     1          'Atmos-lon range: ', DGAMID(JB0(IB)),
     2                       ' to ', DGAMID(JB0(IB)+NATMX(IB))
  200   CONTINUE
      WRITE(*,*)
C
C     WRITE(*,*) 'Derived flags'
      WRITE(*,'(1X,5A12)') 'SEASPF', 'SSTSPF', 'FADSPF', 'OROSPF',
     1                     'ICESPF'
      WRITE(*,'(1X,5L12)')  SEASPF,   SSTSPF,   FADSPF,   OROSPF,
     1                      ICESPF
C
C     Display masks
      WRITE(*,*) 'Sea mask'
      CALL LMASK( SEAMSK )
C
      IF (NBCOUP .GT. 0) THEN
        WRITE(*,*) 'Prognostic SST mask'
        CALL LMASK( PROSST )
      ENDIF
C
      IF (OROSPF) THEN
        WRITE(*,*) 'Orography height (km)'
        CALL FMASK( OROHT, 1000. )
      ENDIF
C
      RETURN
      END
 
      SUBROUTINE LMASK( MASK )
C Prints out logical mask MASK
C
#include "mgrid.h"
      LOGICAL MASK(K1MAX,K2MAX)
C
      INTEGER NPART, I, J, K, J0
      CHARACTER LINE*64
C
C     Number of parts to display
      NPART= 1 + INT((K1-1)/64)
C
      DO 30 I=1,NPART
        IF (NPART .GT. 0) WRITE(*,'(1X,A,I2)') 'Part ', I
        J0= (I-1)*64
        DO 20 K=K2,1,-1
          LINE= ' '
          DO 10 J=1,MIN0(64,K1-J0)
            IF (MASK(J+J0,K)) THEN
              LINE(J:J)= '+'
            ELSE
              LINE(J:J)= '.'
            ENDIF
   10       CONTINUE
          WRITE(*,'(1X,F6.2,A2,A64)') DEGLAT(K), ': ', LINE
   20     CONTINUE
   30   WRITE(*,*)
      RETURN
      END
 
      SUBROUTINE FMASK( MASK, SCALE )
C Prints out floating value mask MASK, after scaling by SCALE
C
#include "mgrid.h"
      REAL MASK(K1MAX,K2MAX), SCALE
C
      INTEGER NPART, I, J, K, J0
      CHARACTER LINE*64
C
C     Number of parts to display
      NPART= 1 + INT((K1-1)/64)
C
      DO 20 I=1,NPART
        IF (NPART .GT. 0) WRITE(*,'(1X,A,I2)') 'Part ', I
        J0= (I-1)*64
        DO 10 K=K2,1,-1
          LINE= ' '
          WRITE(LINE,'(64I1)') (NINT(MASK(J+J0,K)/SCALE),
     1                          J=1,MIN0(64,K1-J0) )
          WRITE(*,'(1X,F6.2,A2,A64)') DEGLAT(K), ': ', LINE
   10     CONTINUE
   20   WRITE(*,*)
      RETURN
      END
 
      SUBROUTINE ATMERR( SUBNAM, MSG )
C Abort execution, printing a message
C Input: CHARACTER*(*) SUBNAM -- name of calling routine
C        CHARACTER*(*) MSG    -- message to be printed
      CHARACTER*(*) SUBNAM, MSG
C
      WRITE(*,'(A,A)') 'ATMERR: ', MSG
      WRITE(*,'(A,A)') 'ATMERR called from routine ', SUBNAM
      STOP
      END
 
