C ICAM (Idealized Coupleable Atmospheric Model)
C icamsrc $Revision: 1.8 $ created $Date: 1996/05/02 21:31:03 $
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C atmos.F: Fortran source module
C Contains driver routines for initializing, running, and saving
C the end state of the atmospheric model
C
 
#include "mmain.h"
 
C Routines:
C ATMINI: Initialize parameters for model integration/diagnostics
C         (called by ATMBGN)
C ATMDIN: Initialize (or re-initialize) model diagnostics
C         (called by ATMINI; may also be called directly)
C ATMRUN: Time-march atmospheric model
C         (called by ATMINT)
C ATMFIN: Terminate atmospheric model integration, write out save file
C         (called by ATMEND)
 
      SUBROUTINE ATMINI( RESTRT, IUNIT1, IUSAV, PARFIL,
     1                   DIRSTR, NSKIP, NSCFRQ, DIASAV,
     2                   IASTEP, ERRFLG )
C Initialize parameters for model integration/diagnostics.
C 1. New run: read model parameters from PARFIL, and set prognostic
C    variables to default values.
C 2. Continuation run: read model parameters and prognostic
C    variables from the save file (PARFIL is not used).
C 3. Restart run: read new model parameters from PARFIL, and the
C    prognostic variables from the save file.
C
C Input:  LOGICAL RESTRT -- TRUE for restart run,
C                           FALSE for new/continuation run
C         INTEGER IUNIT1 -- starting Fortran unit number for model use
C           (units IUNIT1 .. IUNIT1+9 are reserved for model use)
C         INTEGER IUSAV -- unit no. from which to read model save file
C            The save file should be opened before calling this routine,
C            and positioned at the beginning of the atmospheric save
C            information. It remains open on return, but positioned
C            after the atmospheric save information.
C            (Set to zero for new run)
C         CHARACTER*(*) PARFIL -- name of model parameter file
C                              (set to blank for continuation run)
C         CHARACTER*(*) DIRSTR -- name of directory containing
C            the model parameter file and other initialization files
C         [The following two parameters, NSKIP and NSCFRQ,
C          are meaningful only if the C-preprocessor DIAGNOS option
C          is defined]
C         INTEGER NSKIP -- sampling interval, in time-steps (> 0)
C             Model data are sampled every NSKIP time-steps
C             for computing diagnostics/saving horizontal section data.
C         INTEGER NSCFRQ -- horizontal section data saving interval,
C             in units of NSKIP time-steps (>=0). Horizontal
C             section data are saved every NSKIP*NSCFRQ time-steps,
C             after averaging over the NSCFRQ samples
C            [set to zero to suppress horizontal section saves]
C         LOGICAL DIASAV -- set to TRUE to write out diagnostic
C                           information as files
C Output: INTEGER IASTEP -- current atmospheric model time-step count
C         LOGICAL ERRFLG -- error flag (TRUE on error)
C
#include "mgrid.h"
#include "mparams.h"
#include "mstate.h"
C
      LOGICAL RESTRT, DIASAV, ERRFLG
      INTEGER IUNIT1, IUSAV, NSKIP, NSCFRQ, IASTEP
      CHARACTER*(*) PARFIL, DIRSTR
C
C --- LOCAL VARIABLES ---
      CHARACTER FILNAM*160
C
#ifdef ON_CRAY
      CHARACTER*8 DATE0, CLOCK0
#endif
C
C     Clear error flag
      ERRFLG= .FALSE.
C
C     Initialize constants
      CALL INICON( IUNIT1, DIRSTR )
C
C     Display program name, version etc.
      WRITE(*,*)
      WRITE(*,'(A,A,A)') PRGNAM, '-', PRGVER
      WRITE(*,'(A,A)') 'Parameter directory: ', PARDIR
C
C     Initialize model parameters and time-step count
C
      IF (RESTRT .AND. (IUSAV .EQ. 0)) THEN
        CALL ATMERR( 'ATMINI',
     1  'Save file should be specified for restart run' )
      ENDIF
C
      IF (IUSAV .NE. 0) THEN
C       Continuation/restart run;
C       Read model timing information from save file
        CALL GETTIM( IUSAV )
C       Read model parameters from save file
        CALL GETPAR( IUSAV )
      ENDIF
C
      IF ( (IUSAV .EQ. 0) .OR. RESTRT ) THEN
C       New/restart run;
C       Open model parameter file
        FILNAM= PARDIR//PARFIL
        CALL LJUST( FILNAM )
        OPEN( IUPAR, FILE= FILNAM, STATUS='OLD' )
C
C       Read new model parameters
        CALL INPAR( IUPAR, RESTRT )
        CLOSE( IUPAR )
C
C       Reset time-step count and run-part count
        ISTEP= 0
        IPART= 0
C
C       Set initialization program version string
        INIPRG= PRGNAM//PRGVER
C
C       Set initialization time string
#ifdef ON_CRAY
        CALL DATE( DATE0 )
        CALL CLOCK( CLOCK0 )
        INITIM= DATE0//' '//CLOCK0
#else
        INITIM= 'sometime'
#endif
      ENDIF
C
C     Compute timing parameters
C     Time-step
      DT1= DAYL / NSPDAY
C     Re-compute current model time
      DTIM= DBLE(ISTEP)*DBLE(DT1)
      TIM= DTIM
C     Length of "year"
      DYEARL= NDPYR * DDAYL
      YEARL= DYEARL
C
C     Copy time-step count for output
      IASTEP= ISTEP
C
C     Initialize file flags
      SEASPF= SEAFIL(1:2) .NE. 'no'
      OROSPF= OROFIL(1:2) .NE. 'no'
      SSTSPF= SSTFIL(1:2) .NE. 'no'
      FADSPF= FADFIL(1:2) .NE. 'no'
      ICESPF= ICEFIL(1:2) .NE. 'no'
C     Check for unimplemented options
      IF (OROSPF) STOP 'OROFIL'
      IF (ICESPF) STOP 'ICESPF'
C
C     Initialize model grid (and Del**8 damping)
      CALL INIGRD( MTROPP )
C     Initialize physical parametrizations
      CALL PHYINI( 0.1E-3, SOLFLX, EXCH, SATURF, LWFRAC,
     1             DLWAIR, DLWH2O, DSWAIR, DSWO3, VISC0)
C
      IF (RESTRT) THEN
C       Restart run;
C       Read "masks" from save file, and discard
        CALL GETMSK( IUSAV )
C       Reset "masks"
        CALL INIMSK
C       Initialize derived parameters
        CALL INIPR2
C       Read current values of prognostic variables from save file
        CALL GETPGQ( IUSAV )
      ELSE IF (IUSAV .NE. 0) THEN
C       Continuation run;
C       Read "masks" from save file
        CALL GETMSK( IUSAV )
C       Initialize derived parameters
        CALL INIPR2
C       Read current values of prognostic variables from save file
        CALL GETPGQ( IUSAV )
      ELSE
C       New run;
C       Initialize "masks"
        CALL INIMSK
C       Initialize derived parameters
        CALL INIPR2
C       Initialize prognostic variables
        CALL INIPGQ
      ENDIF
C
C     Read harmonic coefficients for time-dependent forcing
C     (SST, heat/E-P flux adjustments)
      CALL GETCOF
C
C     Display model parameters
      CALL DISPAR
C
C     Initialize time-stepping (leap-frog)
      CALL DDTINI( DT1, IMPLCT )
C
#ifndef SLAB_OCEAN
C     Initialize flux/SST exchange module (for coupling to ocean)
      CALL INIFLX
#endif
C
#ifdef DIAGNOS
C     Initlalize model diagnostics
      CALL ATMDIN( NSKIP, NSCFRQ, DIASAV )
#endif
      RETURN
      END
 
      SUBROUTINE ATMDIN( NSKIP, NSCFRQ, DIASAV )
C Initialize atmospheric diagnostics
C (should be called after initializing the atmospheric model
C  parameters and prognostic variables)
C
C Input: INTEGER NSKIP -- sampling interval, in time-steps (> 0)
C             Model data are sampled every NSKIP time-steps
C             for computing diagnostics/saving horizontal section data.
C         INTEGER NSCFRQ -- horizontal section data saving interval,
C             in units of NSKIP time-steps (>=0). Horizontal
C             section data are saved every NSKIP*NSCFRQ time-steps,
C             after averaging over the NSCFRQ samples
C            [set to zero to suppress horizontal section saves]
C         LOGICAL DIASAV -- set to TRUE to write out diagnostic
C                           information as files
C
#include "mgrid.h"
#include "mparams.h"
#include "mstate.h"
C
      INTEGER NSKIP, NSCFRQ
      LOGICAL DIASAV
C
#ifdef DIAGNOS
      REAL LANWID
      INTEGER IDAFIL, IHSFIL, ITSFIL, ISET, IPGQ, K, L
      LOGICAL LEVSAV(L1MAX), LATSAV(K2MAX), SFSAV
      COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
#endif
C
#ifdef DIAGNOS
C
C     Diagnostic output unit numbers (initially zero)
      IDAFIL= 0
      ITSFIL= 0
      IHSFIL= 0
C
      IF (DIASAV) THEN
C       Create diagnostics file
        IDAFIL= IUDAA
        IF (NSCFRQ .GT. 0) THEN
C          Create time-series file
           ITSFIL= IUTSA
        ENDIF
      ENDIF
C
      IF (NSCFRQ .GT. 0) THEN
C       Create horizontal section save file and time-series file
        IHSFIL= IUHSA
      ENDIF
C
C     Select surface section save
      SFSAV= .TRUE.
C     Select all levels for section saves
      DO 210 L=1,L1
  210   LEVSAV(L)= .TRUE.
C     Select no latitudes for section saves
      DO 220 K=1,K2
  220   LATSAV(K)= .FALSE.
C
C     Compute width of land sector (for diagnostic purposes only)
      IF (NBASIN.EQ.1) THEN
#ifdef OLD_VERSION
        LANWID= 0.7
#else
        LANWID= 1.0 - (LONBHI(1)-LONBLO(1))/360.
#endif
      ELSE
        LANWID= 0.0
      ENDIF
C
C     Initialize diagnostics
      ISET= MOD(IPART,10000)
      CALL DIAINI( ISET, DT1, NSPDAY, NDPYR, ISTEP,
     1             NSKIP, NSCFRQ, NSCFRQ,
     2             IDAFIL, ITSFIL, IHSFIL, 0, 0,
     3             MTROPP, .FALSE., SFSAV, LEVSAV, LATSAV,
     4             YPHASE, LANWID, RTITLE, INITIM )
C
C     Zero tendencies
      DO 230 L=1,L1
        DO 230 IPGQ=1,NPGQ
  230     CALL ZEROSP( DPGQSP(0,0,IPGQ,L), N1 )
C     Sample current model state
      CALL DIASTP( ISTEP, PGQSP1, DPGQSP )
#endif
C
      RETURN
      END
 
      SUBROUTINE ATMRUN( SYNFLG, NSTEP, DISVAR, CINTVL, ERRFLG )
C Time-march atmospheric model until time-step NSTEP
C Input:  LOGICAL SYNFLG -- synchronous integration flag
C                           (set to TRUE for synchronous integrations)
C         INTEGER NSTEP  -- last time-step of the integration
C         (i.e. integrate the model from current step to NSTEP;
C          NSTEP must be a multiple of the synchronization
C          interval NSYNC; for asynchronous coupled integrations,
C          if NSTEP = ISTEP (the current time-step count), no
C          integrations are performed, but the initial SST values
C          are obtained from the ocean model.)
C         CHARACTER*(*) DISVAR -- display string, containing list
C           of variables to be displayed. Each character of the
C           string represents one of the following:
C             A=abs-vor, B=rel-vor, C=psi, D=div, E=EPFD,
C             L=Fevap, M=albedo, N=Tsurf, O=OLR, P=precip, Q=q-lev
C             U=u-lev, V=w-speed, W=T-lev, X=u-bar, Y=v-bar, Z=T-bar
C         Set to blank for no display.
C         [only works when the GRAPHICS and DIAGNOS options are defined]
C         CHARACTER*(*) CINTVL -- contour interval string, containing
C           list of contour intervals for variables to be displayed
C           (set to blank/zeros for default values)
C Output: LOGICAL ERRFLG -- error flag
C
#include "mgrid.h"
#include "mparams.h"
#include "mstate.h"
#ifdef DEBUG
#include "mphys.h"
#endif
C
      INTEGER NSTEP
      CHARACTER*(*) DISVAR, CINTVL
      LOGICAL SYNFLG, ERRFLG
C
C --- LOCAL VARIABLES ---
C     Surface temperature (K) (computed by TIMDEP)
      REAL TSURF(K1MAX,K2MAX)
C
C     CO2 multiplication factor, insolation (W/m**2)(computed by TIMDEP)
      REAL CO2FAC, SOLRAD(K1MAX,K2MAX)
C     Heat flux adjustment (W/m**2), E-P flux adjustment (kg/(s*m**2))
C     (computed by TIMDEP)
      REAL OTFADJ(K1MAX,K2MAX), OSFADJ(K1MAX,K2MAX)
C     Surface relative humidity, albedo (computed by SRFPAR)
      REAL RHSURF(K1MAX,K2MAX), ALBEDO(K1MAX,K2MAX)
C
C     Tendency of atmospheric variables (computed by ATMSTP)
      COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
C     Surface wind-stress (Pa), E-P (kg/(s*m**2)), computed by ATMSTP
      REAL TAUSRF(K1MAX,K2MAX,JX:JY), EMINP(K1MAX,K2MAX)
C
C     Net heat flux into the surface (W/m**2), returned by ATMSTP
      REAL FNET(K1MAX,K2MAX)
C     Other fields computed returned by ATMSTP
      REAL CDRAG(K1MAX,K2MAX),  QEVAP(K1MAX,K2MAX)
      REAL MSAPCP(K1MAX,K2MAX), MCAPCP(K1MAX,K2MAX)
      REAL FEVAP(K1MAX,K2MAX),  FSENS(K1MAX,K2MAX)
      REAL FRAD(K1MAX,K2MAX)
      REAL FPSSRF(K1MAX,K2MAX), FMSSRF(K1MAX,K2MAX)
      REAL FPLSRF(K1MAX,K2MAX), FMLSRF(K1MAX,K2MAX)
      REAL FPSTOP(K1MAX,K2MAX), FPLTOP(K1MAX,K2MAX)
C     Other heating profiles returned by ATMSTP
      REAL LWHET(K2MAX,L1MAX),  SWHET(K2MAX,L1MAX)
      REAL MSAHET(K2MAX,L1MAX), MCAHET(K2MAX,L1MAX)
      REAL DCAHET(K2MAX,L1MAX)
C     Runoff parameters (computed by LANSTP)
      REAL RUNOFF(K1MAX,K2MAX), EMPLAN(K2MAX), EMPLAV
C     Surface fluxes over each ocean basin (computed by FLXCAL)
      REAL FLUXB(JBMAX,KBMAX,JTAUX:NFOCN,MAXBAS)
C     Number of time-steps to integrate
      INTEGER NCOUNT
C     Temporary variables
      REAL TEMP
C
      INTEGER J, K, L, IB, IPGQ, IT
      LOGICAL STBFLG
C
#ifdef SLAB_OCEAN
      INTEGER JA, KA, JB, KB
#endif
C
#ifdef GRAPHICS
#ifdef DIAGNOS
C     Display variables
      INTEGER IDISP, NDISP
      REAL FDISP(6), PRECIP(K1MAX,K2MAX)
      INTEGER LENST
#endif
#endif
C
C     Clear error flag
      ERRFLG= .FALSE.
C
C     Check time-step specification
      IF (MOD(NSTEP,NSYNC) .NE. 0) THEN
          CALL ATMERR( 'ATMRUN',
     1                'Error: NSTEP should be a multiple of NSYNC' )
      ENDIF
C
#ifdef GRAPHICS
#ifdef DIAGNOS
C     Determine number of variables to be displayed
      CALL COMPRS( DISVAR, ' ' )
      NDISP= MIN0(6, LENST( DISVAR, ' '))
C     Initialize default display contour intervals
      DO 10 IDISP=1,NDISP
   10   FDISP(IDISP)= 0.0
C     Read display contour intervals
      READ(CINTVL,*,ERR=11,END=11) (FDISP(IDISP),IDISP=1,NDISP)
   11   CONTINUE
#endif
#endif
C
#ifdef DEBUG
C     Zero zonal-mean accumulators (for convection diagnostics)
      DO 20 K=1,K2
        NDRYAD(K)= 0
        NDRYLV(K)= 0
        NWETAD(K)= 0
        NWETLV(K)= 0
   20   CONTINUE
#endif
C     Compute number of time-steps to integrate
      NCOUNT= NSTEP - ISTEP
      IF (NCOUNT .LT. 0) THEN
          CALL ATMERR( 'ATMRUN', 'Negative integration interval' )
      ENDIF
C
      IF (NCOUNT .EQ. 0) THEN
C       Zero integration interval
        IF (COUFLG .AND. .NOT.SYNFLG) THEN
#ifndef SLAB_OCEAN
C         External ocean; asynchronous integration; SPECIAL CASE
C
C         Temporarily initialize time-dependent variables
          CALL TIMDEP( ISTEP, SST0(1,1), LAND0(1,1,JLANT),
     1                 CO2FAC, TSURF, SOLRAD, OTFADJ, OSFADJ )
C         Update TSURF with initial SST values received from ocean model
          CALL UPDSST( ISTEP, TSURF, ERRFLG)
          IF (ERRFLG) GOTO 990
C
C         Copy prognostic SST at time-level 1 from surface temperature
          DO 30 K=1,K2
            DO 30 J=1,K1
              IF (PROSST(J,K)) THEN
                SST1(J,K)= TSURF(J,K)
              ENDIF
   30         CONTINUE
C         Copy time-level 1 SSTs identically to time-level 0
          CALL PHCOPY( SST0(1,1), SST1(1,1) )
#endif
        ELSE
          WRITE(*,*) 'ATMRUN: Warning - zero integration interval'
        ENDIF
      ENDIF
C
C     Integrate model from current time-step upto time-step NSTEP
      DO 900 IT=1,NCOUNT
C
C       Initialize time-dependent variables;
C        CO2 factor, surface temperature, insolation,
C        and heat/E-P flux adjustments (if any)
C       [Since the surface temperature array TSURF is initialized
C        with the "old" (time-level 0) values of SST/LST to ensure
C        stability, the leap-frog time-stepping scheme actually
C        acts like a forward scheme!]
C
        CALL TIMDEP( ISTEP, SST0(1,1), LAND0(1,1,JLANT),
     1               CO2FAC, TSURF, SOLRAD, OTFADJ, OSFADJ )
C
#ifndef SLAB_OCEAN
C
        IF (COUFLG .AND. SYNFLG ) THEN
C         External ocean; synchronous integration
C         Update TSURF with current SST values received from ocean model
          CALL UPDSST( ISTEP, TSURF, ERRFLG)
          IF (ERRFLG) GOTO 990
C
C         Copy prognostic SST at time-level 1 from surface temperature
          DO 200 K=1,K2
            DO 200 J=1,K1
              IF (PROSST(J,K)) THEN
                SST1(J,K)= TSURF(J,K)
              ENDIF
  200         CONTINUE
C         Copy time-level 1 SSTs identically to time-level 0
          CALL PHCOPY( SST0(1,1), SST1(1,1) )
        ELSE IF (COUFLG .AND. (MOD(ISTEP, NSYNC) .EQ. 0)) THEN
C         External ocean; asynchronous integration;
C         possibly wait for next SST packet to arrive
C         (to simulate sequential integration when using PVM)
          CALL NXTSST( ISTEP/NSYNC )
        ENDIF
#endif
C
C       Compute surface parameters (rel. humidity, albedo)
C       (use time-level 1 for land water level, because of
C        forward time-differencing)
        CALL SRFPAR(RHSURF, ALBEDO, TSURF, LAND1(1,1,JLANWL), ICEMSK)
C
C       Integrate the atmospheric variables over one time-step,
C       and compute fluxes
        CALL ATMSTP( DT1, SEAMSK, ICEMSK, SOLRAD, ALBEDO,
     1               TSURF, RHSURF, CO2FAC, DMPBOT, MTROPP,
     2               PGQSP0, PGQSP1,
     3               PGQSP2, DPGQSP, TAUSRF, EMINP,
     4               CDRAG, QEVAP, MSAPCP, MCAPCP,
     5               FEVAP, FSENS, FRAD, FNET, FPSSRF, FMSSRF,
     6               FPLSRF, FMLSRF, FPSTOP, FPLTOP,
     7               SWHET, LWHET, MSAHET, MCAHET, DCAHET )
C
C       Integrate the aland surface variables over one time-step,
C       and compute runoff
        CALL LANSTP( DT1, FNET, EMINP, LAND0, LAND1, LAND2,
     1               RUNOFF, EMPLAN, EMPLAV,
     2               QEVAP, MSAPCP, MCAPCP )
 
        IF (COUFLG) THEN
C         Interactive ocean;
C
          DO 220 IB=1,NBCOUP
C
C           Compute surface fluxes for the ocean basin
            CALL FLXCAL( FLUXB(1,1,JTAUX,IB), IB, TAUSRF, FNET,
     1                   EMINP, EMPLAN, EMPLAV, OTFADJ, OSFADJ,
     2                   QEVAP, MSAPCP, MCAPCP )
C
#ifdef SLAB_OCEAN
C           Internal (mixed-layer) ocean;
C           time march prognostic mixed layer ocean temperature
            DO 210 KB=1,NATMY(IB)
              KA= KB + KB0(IB)
              DO 210 JB=1,NATMX(IB)
                JA= 1 + MOD(JB-1+JB0(IB), K1)
                IF (PROSST(JA,KA)) THEN
                  SST2(JA,KA)= SST0(JA,KA) + 2.0*DT1*FLUXB(JB,KB,JFS,IB)
     1                                      / MIXHCP(JA,KA)
                ENDIF
  210           CONTINUE
#endif
  220       CONTINUE
        ENDIF
C
C       Apply Robert filter on all atmospheric variables
        DO 310 L=1,L1
          DO 310 IPGQ=1,NPGQ
            CALL ROBFIL( PGQSP0(0,0,IPGQ,L), PGQSP1(0,0,IPGQ,L),
     1                   PGQSP2(0,0,IPGQ,L), TIMFIL)
  310       CONTINUE
C
C       Exchange storage for atmospheric prognostic quantities
        DO 320 L=1,L1
          DO 320 IPGQ=1,NPGQ
            CALL SPCOPY( PGQSP0(0,0,IPGQ,L), PGQSP1(0,0,IPGQ,L), N1)
            CALL SPCOPY( PGQSP1(0,0,IPGQ,L), PGQSP2(0,0,IPGQ,L), N1)
  320       CONTINUE
C
C       Apply Robert filter on land surface temperatures only (???)
        CALL ROBERT( LAND0(1,1,JLANT), LAND1(1,1,JLANT),
     1               LAND2(1,1,JLANT), K1MAX, K1, K2, TIMFIL)
C       (Robert filter on ice quantities???)
C
C       Exchange storage for land fields
        DO 330 IPGQ=JLANT,JLANWL
          CALL PHCOPY( LAND0(1,1,IPGQ), LAND1(1,1,IPGQ) )
 330      CALL PHCOPY( LAND1(1,1,IPGQ), LAND2(1,1,IPGQ) )
C
C       Exchange storage for ice fields
        DO 340 IPGQ=JICET,JICEHT
          CALL PHCOPY( ICE0(1,1,IPGQ), ICE1(1,1,IPGQ) )
  340     CALL PHCOPY( ICE1(1,1,IPGQ), ICE2(1,1,IPGQ) )
C
        IF (COUFLG) THEN
#ifdef SLAB_OCEAN
C         Internal (mixed-layer) ocean;
C         Apply Robert filter on mixed-layer SST (???)
          CALL ROBERT( SST0(1,1), SST1(1,1),
     1                 SST2(1,1), K1MAX, K1, K2, TIMFIL)
C
C         Exchange storage for mixed-layer SST
          CALL PHCOPY( SST0(1,1), SST1(1,1) )
          CALL PHCOPY( SST1(1,1), SST2(1,1) )
#endif
        ENDIF
C
C       Increment time-step counter
        ISTEP= ISTEP + 1
C       Re-compute current model time
        DTIM= DBLE(ISTEP)*DBLE(DT1)
        TIM= DTIM
C
#ifdef DIAGNOS
C       Compute diagnostics
        CALL DIAFLX( ISTEP, PGQSP1, DPGQSP, QEVAP, MSAPCP, MCAPCP,
     1               FEVAP, FSENS, FRAD, FNET, FPSSRF, FMSSRF,
     2               FPLSRF, FMLSRF, FPSTOP, FPLTOP,
     3               SWHET, LWHET, MSAHET, MCAHET, DCAHET,
     4               SEAMSK, ICEMSK, SOLRAD, ALBEDO,
     5               TSURF, TAUSRF(1,1,JX), TAUSRF(1,1,JY),
     6               ICE1(1,1,JICEHT) )
#endif
C
#ifndef SLAB_OCEAN
C
        IF (COUFLG .AND. .NOT.SYNFLG) THEN
C         External ocean; asynchronous integration, next time-step
C         Update TSURF with current SST values received from ocean model
          CALL UPDSST( ISTEP, TSURF, ERRFLG)
          IF (ERRFLG) GOTO 990
C
C         Copy prognostic SST at time-level 1 from surface temperature
          DO 380 K=1,K2
            DO 380 J=1,K1
              IF (PROSST(J,K)) THEN
                SST1(J,K)= TSURF(J,K)
              ENDIF
  380         CONTINUE
C         Copy time-level 1 SSTs identically to time-level 0
          CALL PHCOPY( SST0(1,1), SST1(1,1) )
        ENDIF
C
        IF (COUFLG) THEN
C         External ocean; next time-step
C         Accumulate/send surface fluxes to ocean model, for each basin
          DO 390 IB=1,NBCOUP
            CALL FLXOUT( ISTEP, IB, FLUXB(1,1,JTAUX,IB), ERRFLG )
            IF (ERRFLG) GOTO 990
  390       CONTINUE
        ENDIF
#endif
C
#ifdef DIAGNOS
#ifdef GRAPHICS
        IF (MOD(ISTEP,12).EQ.0) THEN
C         Display info
          IF (NDISP .GT. 0) THEN
C           Display diagnostics;
C           compute total precipitation (static+convective)
            DO 400 K=1,K2
              DO 400 J=1,K1
  400           PRECIP(J,K)= MSAPCP(J,K) + MCAPCP(J,K)
C           Create appropriate number of windows
            IF (NDISP .LE. 3) THEN
              CALL ZSPLIT(NDISP, 1, 1)
            ELSE IF (NDISP .EQ. 4) THEN
              CALL ZSPLIT(2, 2, 0)
            ELSE
              CALL ZSPLIT(3, 2, 0)
            ENDIF
            DO 410 IDISP=1,NDISP
              IF ( (DISVAR(IDISP:IDISP).GE.'A') .AND.
     1             (DISVAR(IDISP:IDISP).LE.'Z')       ) THEN
C               Display variable
                CALL SELWIN(IDISP)
                CALL DISRUN( DISVAR(IDISP:IDISP), FDISP(IDISP), PGQSP1,
     1                       FEVAP, ALBEDO, TSURF, FPLTOP, PRECIP )
              ENDIF
  410         CONTINUE
            CALL ZFRAME
          ENDIF
        ENDIF
#endif
#endif
C
C       Check for possible numerical instability
        STBFLG= .FALSE.
        DO 500 L=1,L1-1
C         Compute horizontally averaged static stability
          TEMP=  (REAL(PGQSP1(0,0,JPOT,L)) - REAL(PGQSP1(0,0,JPOT,L+1)))
          IF (TEMP.GT.3.0*(TREFLV(L)-TREFLV(L+1))) THEN
C           Global-mean static stability much larger than ref. value
            STBFLG= ISTEP .GT. 1
            WRITE(*,'(1X,A,I3,A,1P,E15.6,A)') 'Overstability at level ',
     1              L, '+1/2 at model time = ', DTIM/DDAYL, ' days'
          ENDIF
  500     CONTINUE
        IF (STBFLG) THEN
          WRITE(*,*) 'Error: static stability too large'
          GOTO 990
        ENDIF
  900   CONTINUE
C
#ifdef DEBUG
      IF (NCOUNT .GT. 0) THEN
        WRITE(*,*) 'Average no. of column dry adjustments per step'
        WRITE(*,'(10(F7.3,'',''))')(NDRYAD(K)/FLOAT(NCOUNT*K1),K=1,K2)
        WRITE(*,*) 'Average no. levels affected per dry adjustment'
        WRITE(*,'(10(F7.3,'',''))')
     1           (NDRYLV(K)/FLOAT(MAX0(1,NDRYAD(K))),K=1,K2)
        WRITE(*,*) 'Average no. of column wet adjustments per step'
        WRITE(*,'(10(F7.3,'',''))')(NWETAD(K)/FLOAT(NCOUNT*K1),K=1,K2)
        WRITE(*,*) 'Average no. levels affected per wet adjustment'
        WRITE(*,'(10(F7.3,'',''))')
     1         (NWETLV(K)/FLOAT(MAX0(1,NWETAD(K))),K=1,K2)
        WRITE(*,*)
      ENDIF
#endif
C
C     Normal exit
      RETURN
C
C     Error exit
C     (3 jumps to this statement)
  990 ERRFLG= .TRUE.
      RETURN
      END
 
      SUBROUTINE ATMFIN( IUSAV, DIAFIL)
C Terminates one part of atmospheric model integration, and increments
C run part counter.
C The model state is saved to unit IUSAV, if it is not set to zero.
C Model diagnostics are written out to a file, if diagnostics were
C initialized.
C
C Input:  INTEGER IUSAV -- Fortran unit number for saving model state.
C            The save file should be opened before calling this routine,
C            and positioned at the beginning of the atmospheric save
C            information. It remains open on return, but positioned
C            after the atmospheric save information.
C         (Set to zero to suppress output of model state)
C Output: CHARACTER*(*) DIAFIL -- Name of diagnostic file written
C                                 (Blank, if no file is written)
C
#include "mstate.h"
C
      INTEGER IUSAV
      CHARACTER*(*) DIAFIL
C
C     Write out diagnostics file (if diagnostics were initialized)
      DIAFIL= ' '
#ifdef DIAGNOS
      CALL DIAEND( DIAFIL )
#endif
C
C     Increment run part number
      IPART= IPART + 1
C
      IF (IUSAV .NE. 0) THEN
C       Save model state
        CALL ATMSAV( IUSAV )
      ENDIF
C
      RETURN
      END
 
