C ICAM (Idealized Coupleable Atmospheric Model)
C interpol.F $Revision: 2.4 $ created $Date: 1996/04/11 17:42:33 $
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C interpol.F: Fortran source module
C Quadratic (flux conserving) interpolation routines
C

C Top-level routines:
C STRETC: Stretch and translate coordinate systems
C FLXIN2: Initialize 2-D interpolation coefficient tables
C FLUX2D: Interpolate 2-D arrays using tables initialized by FLXIN2
C FLXIN1: Initialize 1-D interpolation coefficient tables
C FLUX1D: Interpolate 1-D arrays using tables initialized by FLXIN1
C
      SUBROUTINE STRETC( SFAC, Y, YEDGE, NY, YLONEW, YHINEW )
C Stretch and translate a coordinate system to fit within new boundaries
C The coordinate system is given by grid-point values Y(NY),
C and grid-box edge values YEDGE(NY+1). The original boundaries are
C YEDGE(1) and YEDGE(NY+1). Y and YEDGE values are stretched and
C translated so that the new boundaries are YLOWNEW and YHINEW.
C SFAC is the stretching factor that converts the old coordinate system
C to the new one.
C
C Input: INTEGER NY -- Number of grid-point values
C        REAL Y(NY) -- Grid-point coordinate values
C        REAL YEDGE(NY+1) -- Grid-box edge coordinate values
C        REAL YLONEW, YHINEW -- New coordinate range
C Output: INTEGER Y, YEDGE -- updated values, now YEDGE(1)= YLONEW,
C                                          and YEDGE(NY+1)= YHINEW
C         REAL SFAC -- coordinate stretch factor (> 0)
C
      INTEGER NY
      REAL SFAC, Y(NY), YEDGE(NY+1), YLONEW, YHINEW
C
      REAL YMID0, YMID1
      INTEGER I
C
      IF ((YEDGE(1) .EQ. YLONEW) .AND. (YEDGE(NY+1) .EQ. YHINEW)) THEN
C       No stretching or translation necessary
        SFAC= 1.0
      ELSE
C       Compute stratch factor
        SFAC= (YHINEW - YLONEW) / (YEDGE(NY+1) - YEDGE(1))
C       Compute mid-value of coordinate ranges
        YMID0= 0.5*( YEDGE(1) + YEDGE(NY+1) )
        YMID1= 0.5*( YLONEW   + YHINEW      )
C       Stretch and translate grid-boxes
        YEDGE(1)=    YLONEW
        YEDGE(NY+1)= YHINEW
        DO 10 I=2,NY
   10     YEDGE(I)= (YEDGE(I) - YMID0)*SFAC + YMID1
C
        DO 20 I=1,NY
   20     Y(I)= (Y(I) - YMID0)*SFAC + YMID1
      ENDIF
C
      RETURN
      END

      SUBROUTINE FLXIN2( MAPX, FRACX, CFRACX, DX,
     1                   MAPY, FRACY, CFRACY, DY, YWT,
     2                   XBNEW, NXNEW, XPT, NX,
     3                   YBNEW, NYNEW, YPT, NY, YB )
C Initializes arrays required for 2-D flux conserving interpolation.
C
C TERMINOLOGY
C
C Grid-point:    location where the flux is defined on the input grid
C Grid-box:      the region associated with each grid-point over which
C                the flux is distributed
C Grid-interval: the region bounded by two adjacent grid-points
C
C NOTES
C
C  1. A grid-interval is formed from the right part of the grid-box
C    associated with the left grid-point, and the left part of the
C    grid-box associated with the right grid-point.
C
C  2. In the X-direction, it is assumed that the interior grid-box
C    edges lie exactly half way between the grid-points. There is
C    no such restriction in the Y-direction.
C
C  3. It is assumed there are grid-points at the edge of the input
C    grid. That is, if the input grid extends from XMIN to XMAX,
C    one needs to define two "fictitious" grid-points located at XMIN
C    and XMAX, if they are not already defined.
C
C  4. For output, the grid-point locations do not matter, and only the
C    grid-box coordinates need to be specified. The input flux is
C    integrated within the output grid-box, using a quadratic flux
C    conserving interpolation formula to interpolate within each
C    input grid-interval, based on the flux values at the bounding
C    grid-points.
C
C  5. All coordinate arrays should be in ascending order.
C
C Input:  INTEGER NXNEW, NYNEW-- no. of output X/Y grid-boxes
C         REAL XBNEW(0:NXNEW) -- X-coordinate values of output grid-box
C                                edges (i.e. grid-box JNEW extends from
C                                XBNEW(JNEW-1) to XBNEW(JNEW) )
C         REAL YBNEW(0:NYNEW) -- Y-coordinate values of output grid-box
C                                edges (i.e. grid-box KNEW extends from
C                                YBNEW(KNEW-1) to YBNEW(KNEW) )
C         INTEGER NX, NY      -- no. of input X/Y grid-points
C                                (excluding the two edge grid-points)
C         REAL XPT(0:NX+1) -- X-coordinate values of input grid-points
C                             (including the two edge grid-points)
C                       [XPT(0) <= XBNEW(0), XBNEW(NXNEW) <= XPT(NX+1)]
C         REAL YPT(0:NY+1) -- Y-coordinate values of input grid-points
C                             (including the two edge grid-points)
C                       [YPT(0) <= YBNEW(0), YBNEW(NYNEW) <= YPT(NY+1)]
C         REAL YB(0:NY) -- Y-coordinate values of grid-box edges
C                          associated with the NY interior grid-points,
C                          i.e., excluding the edge grid-points.
C                       [YPT(0) <= YB(0), YB(NY) <= YPT(NY+1)]
C Output: INTEGER MAPX(0:NXNEW) -- index map giving the nearest left
C           input grid-point corresponding to each output grid-box edge
C         REAL FRACX(2,0:NXNEW) -- fractional distance of the nearest
C                 left input grid-point (linear and squared values)
C         REAL CFRACX(2,0:NXNEW)-- complementary version of FRACX
C         REAL DX(0:NX)         -- X-width of input grid-interval
C         INTEGER MAPY(0:NYNEW) -- index map giving the nearest lower
C           input grid-point corresponding to each output grid-box edge
C         REAL FRACY(3,0:NYNEW) -- fractional distance of the nearest
C             lower input grid-point (linear, squared, and cubed values)
C         (FRACY=0.0 => at that point, FRACY=1.0 => at next point)
C         REAL CFRACY(3,0:NYNEW) -- complementary version of FRACY
C         REAL DY(0:NY)          -- Y-width of input grid-interval
C         REAL YWT(0:NY) -- weight associated with grid-point K on
C                           the Y grid-interval K, where K=0,...,NY.
C                           (A weight of 1-YWT is associated with
C                            grid-point K+1 on the same grid-interval)
C
      INTEGER NYNEW, NXNEW, NX, NY
      REAL XBNEW(0:NXNEW), XPT(0:NX+1)
      REAL YBNEW(0:NYNEW), YPT(0:NY+1), YB(0:NY)
      INTEGER MAPY(0:NYNEW), MAPX(0:NXNEW)
      REAL FRACY(3,0:NYNEW), CFRACY(3,0:NYNEW), DY(0:NY), YWT(0:NY)
      REAL FRACX(2,0:NXNEW), CFRACX(2,0:NXNEW), DX(0:NX)
C
      INTEGER J, K, JNEW, KNEW
C
C     Input Y-grid parameters
      DO 10 K=0,NY
C       Width of input Y grid-interval
        DY(K)= YPT(K+1) - YPT(K)
C       Weighting factor associated with the left grid-point
        YWT(K)= (YB(K) - YPT(K)) / DY(K)
C       Check for ascending order
        IF (DY(K) .LE. 0.) STOP 'FLXIN2/1'
        IF ((YB(K).LT.YPT(K)) .OR. (YB(K).GT.YPT(K+1))) STOP 'FLXIN2/2'
   10   CONTINUE
C
C     Width of input X grid-interval
C     ( The weighting factor is assumed to be 0.5 for
C       both the left and the right grid-points of the interval)
      DO 20 J=0,NX
        DX(J)= XPT(J+1) - XPT(J)
C       Check for ascending order
        IF (DX(J) .LE. 0.) STOP 'FLXIN2/3'
   20   CONTINUE
C
C     Check that output coordinates are in ascending order
      DO 30 KNEW=0,NYNEW-1
        IF (YBNEW(KNEW+1) .LE. YBNEW(KNEW)) STOP 'FLXIN2/4'
   30   CONTINUE
C
      DO 40 JNEW=0,NXNEW-1
        IF (XBNEW(JNEW+1) .LE. XBNEW(JNEW)) STOP 'FLXIN2/5'
   40   CONTINUE
C
C     Check that output coordinates are within input range
      IF ((YBNEW(0) .LT. YPT(0)) .OR. (YBNEW(NYNEW) .GT. YPT(NY+1)))
     1  STOP 'FLXIN2/6'
      IF ((XBNEW(0) .LT. XPT(0)) .OR. (XBNEW(NXNEW) .GT. XPT(NX+1)))
     1  STOP 'FLXIN2/7'
C
C     Compute fractional Y-distance from nearest left input grid-point
C     (MAPY= 0,...,NY, FRACY=0,...,1.0)
      DO 60 KNEW=0,NYNEW
        DO 50 K=NY,0,-1
          IF (YBNEW(KNEW) .LE. YPT(K+1)) THEN
            MAPY(KNEW)= K
            FRACY(1,KNEW)= (YBNEW(KNEW) - YPT(K)) / DY(K)
          ENDIF
   50     CONTINUE
C       Compute higher powers of fractional distance
        FRACY(2,KNEW)= FRACY(1,KNEW)**2
        FRACY(3,KNEW)= FRACY(1,KNEW)**3
C       Compute complementary fractional distance
        CFRACY(1,KNEW)= 1.0 - FRACY(1,KNEW)
        CFRACY(2,KNEW)= CFRACY(1,KNEW)**2
        CFRACY(3,KNEW)= CFRACY(1,KNEW)**3
   60   CONTINUE
C
C     Compute fractional X-distance from nearest left input grid-point
C     (MAPX= 0,...,NX, FRACX=0,...,1.0)
      DO 80 JNEW=0,NXNEW
        DO 70 J=NX,0,-1
          IF (XBNEW(JNEW) .LE. XPT(J+1)) THEN
            MAPX(JNEW)= J
            FRACX(1,JNEW)= (XBNEW(JNEW) - XPT(J)) / DX(J)
          ENDIF
   70     CONTINUE
C       Compute higher powers of fractional distance
        FRACX(2,JNEW)= FRACX(1,JNEW)**2
C       Compute complementary fractional distance
        CFRACX(1,JNEW)= 1.0 - FRACX(1,JNEW)
        CFRACX(2,JNEW)= CFRACX(1,JNEW)**2
   80   CONTINUE
C
#ifdef DEBUG
      WRITE(*,*)
      WRITE(*,*) 'FLXIN2: Debugging information'
      WRITE(*,*)
      WRITE(*,'(1X,5A14)')    'K', 'YPT', 'YB', 'DY', 'YWT'
      WRITE(*,'(1X,I14,4G14.4)') (K,YPT(K),YB(K),DY(K),YWT(K),K=0,NY)
      WRITE(*,'(1X,I14,G14.4)') NY+1, YPT(NY+1)
      WRITE(*,*)
      WRITE(*,'(1X,3A14)')    'J', 'XPT', 'DX'
      WRITE(*,'(1X,I14,2G14.4)') (J,XPT(J),DX(J),J=0,NX)
      WRITE(*,'(1X,I14,G14.4)') NX+1, XPT(NX+1)
      WRITE(*,*)
      WRITE(*,'(1X,4A14)')    'KNEW', 'YBNEW', 'MAPY', 'FRACY'
      WRITE(*,'(1X,I14,G14.4,I14,G14.4)')
     1    (KNEW, YBNEW(KNEW), MAPY(KNEW), FRACY(1,KNEW), KNEW=0,NYNEW)
      WRITE(*,*)
      WRITE(*,'(1X,4A14)')    'JNEW', 'XBNEW', 'MAPX', 'FRACX'
      WRITE(*,'(1X,I14,G14.4,I14,G14.4)')
     1    (JNEW, XBNEW(JNEW), MAPX(JNEW), FRACX(1,JNEW), JNEW=0,NXNEW)
#endif
C
      RETURN
      END

      SUBROUTINE FLUX2D( FNEW, NXNMAX, NXNEW, NYNEW, F, NXMAX, NX, NY,
     1                   MAPX, FRACX, CFRACX, DX,
     2                   MAPY, FRACY, CFRACY, DY, YWT )
C
C FLUX2D uses arrays MAPX, FRACX, CFRACX, DX, MAPY, CFRACY, FRACY,
C DY, YWT initialized by FLXIN2 to interpolate flux F in coordinate
C system (X,Y) to flux FNEW in coordinate system (XNEW,YNEW), conserving
C the area integrated flux values over the coarser grid resolution.
C
C Input:  INTEGER NXNEW,NYNEW   -- no. of output grid-boxes
C         INTEGER NXNMAX, NXMAX -- declared X-dimension parameters
C         REAL F(0:NXMAX+1,0:NY+1) -- flux values at input grid-points
C         (It is assumed there are extra grid-point values at the
C          edge of the grid, i.e, F(0,*), F(NX+1,*), F(*,0), F(*,NY+1).
C          If these "padding" grid-points don't actually exist, then
C          the corresponding nearest interior values should simply be
C          duplicated to pad the array, e.g, F(0,*)= F(1,*),
C          F(NX+1,*)= F(NX,*) etc. For periodic boundary conditions
C          in the X-direction (say), setting F(0,*)= F(NX,*),
C          and F(NX+1,*)= F(1,*) may be more approriate.)
C         INTEGER NX,NY         -- actual no. of input grid-points
C                       (excluding the "padding" points at the edges)
C         INTEGER MAPX(0:NXNEW) -- index map giving the nearest left
C           input grid-point corresponding to each output grid-box edge
C         REAL FRACX(2,0:NXNEW) -- fractional distance of the nearest
C                 left input grid-point (linear and squared values)
C         REAL CFRACX(2,0:NXNEW)-- complementary version of FRACX
C         REAL DX(0:NX)         -- X-width of input grid-interval
C         INTEGER MAPY(0:NYNEW) -- index map giving the nearest lower
C           input grid-point corresponding to each output grid-box edge
C         REAL FRACY(3,0:NYNEW) -- fractional distance of the nearest
C             lower input grid-point (linear, squared, and cubed values)
C         REAL CFRACY(3,0:NYNEW) -- complementary version of FRACY
C         REAL DY(0:NY)          -- Y-width of input grid-interval
C         REAL YWT(0:NY) -- weight associated with grid-point K on
C                           the Y grid-interval K, where K=0,...,NY.
C                           (A weight of 1-YWT is associated with
C                            grid-point K+1 on the same grid-interval)
C
C Output: REAL FNEW(NXNMAX,NYNEW) -- flux values at the output grid-boxes
C
      INTEGER NXNMAX, NXNEW, NYNEW, NXMAX, NX, NY
      REAL FNEW(NXNMAX,NYNEW), F(0:NXMAX+1,0:NY+1)
      INTEGER MAPY(0:NYNEW), MAPX(0:NXNEW)
      REAL FRACY(3,0:NYNEW), CFRACY(3,0:NYNEW), DY(0:NY), YWT(0:NY)
      REAL FRACX(2,0:NXNEW), CFRACX(2,0:NXNEW), DX(0:NX)
C
      REAL FVAL0, FVAL1, FVAL, FACC, DYACC, DXACC
      REAL AWT, DELX(2), DELY(3)
      INTEGER I, J, K, J0, J1, K0, K1, JA, JB, KA, KB, JNEW, KNEW
C
      REAL FINTP
C
#ifdef DEBUG
      WRITE(*,*) 'FLUX2D: Debugging information'
      WRITE(*,*) 'NXNMAX, NXNEW, NYNEW=', NXNMAX, NXNEW, NYNEW
      WRITE(*,*) 'NXMAX, NX, NY=', NXMAX, NX, NY
#endif
C     Compute integrated output grid-box values
      DO 500 KNEW=1,NYNEW
        K0= MAPY(KNEW-1)
        K1= MAPY(KNEW)
        DO 500 JNEW=1,NXNEW
          J0= MAPX(JNEW-1)
          J1= MAPX(JNEW)
C         Initialize Y-accumulators
          FACC= 0.0
          DYACC= 0.0
          DO 400 K=K0,K1
            KA= K
            KB= K+1
            AWT= YWT(K)
C
            IF ( (K .EQ. K0) .AND. (K .EQ. K1) ) THEN
C             Output grid box enclosed Y-direction
              DO 110 I=1,3
  110           DELY(I)= FRACY(I,KNEW) - FRACY(I,KNEW-1)
            ELSE IF (K .EQ. K0) THEN
C             Lower-end grid box; flip direction
              KA= K+1
              KB= K
              AWT= 1.0 - YWT(K)
              DO 120 I=1,3
  120           DELY(I)= CFRACY(I,KNEW-1)
            ELSE IF (K .EQ. K1) THEN
C             Upper-end grid box
              DO 130 I=1,3
  130           DELY(I)= FRACY(I,KNEW)
            ELSE
C             Interior grid box
              DO 140 I=1,3
  140           DELY(I)= 1.0
            ENDIF
C           Initialize X-accumulators
            DXACC= 0.0
            DO 300 J=J0,J1
C
              IF ( (K .EQ. K0) .AND. (K .EQ. K1) .AND.
     1             (J .EQ. J0) .AND. (J .EQ. J1)       ) THEN
C
C              SPECIAL CASE: Output grid box totally enclosed
C                            (in both X and Y directions)
C
                JA= J
                JB= J+1
C               First, integrate X=0..FRACX(1,JNEW-1)
                DO 210 I=1,2
  210             DELX(I)= FRACX(I,JNEW-1)
C
                FVAL0= FINTP( F, NXMAX, NX, NY, AWT, JA, JB, KA, KB,
     1                        DELX, DELY )
C
C               Next, integrate X=0..FRACX(1,JNEW),
                DO 220 I=1,2
  220             DELX(I)= FRACX(I,JNEW)
C
                FVAL1= FINTP( F, NXMAX, NX, NY, AWT, JA, JB, KA, KB,
     1                        DELX, DELY )
C
C               Take the difference
                FVAL= FVAL1 - FVAL0
C
C               Compute DELX(1) etc. properly (for use later)
                DO 230 I=1,2
  230             DELX(I)= FRACX(I,JNEW) - FRACX(I,JNEW-1)
              ELSE
C
                JA= J
                JB= J+1
C
                IF ( (J .EQ. J0) .AND. (J .EQ. J1) ) THEN
C                 Output grid box enclosed X-direction only
                  DO 240 I=1,2
  240               DELX(I)= FRACX(I,JNEW) - FRACX(I,JNEW-1)
                ELSE IF (J .EQ. J0) THEN
C                 Left-end grid box; flip direction
                  JA= J+1
                  JB= J
                  DO 250 I=1,2
  250               DELX(I)= CFRACX(I,JNEW-1)
                ELSE IF (J .EQ. J1) THEN
C                 Right-end grid box
                  DO 260 I=1,2
  260               DELX(I)= FRACX(I,JNEW)
                ELSE
C                 Interior grid box
                  DO 270 I=1,2
  270               DELX(I)= 1.0
                ENDIF
C
C               Compute interpolated contribution (from input grid-box)
                FVAL= FINTP( F, NXMAX, NX, NY, AWT, JA, JB, KA, KB,
     1                       DELX, DELY )
              ENDIF
C
C             Integrate over X-direction
              FACC= FACC + DX(J)*DY(K)*FVAL
  300         DXACC= DXACC + DX(J)*DELX(1)
C           Integrate over Y-direction
  400       DYACC= DYACC + DY(K)*DELY(1)
C         Unweight output box value
  500     FNEW(JNEW,KNEW)= FACC / (DXACC*DYACC)
C
      RETURN
      END

      REAL FUNCTION FINTP( F, NXMAX, NX, NY, AWT, JA, JB, KA, KB,
     1                     DELX, DELY )
C
C Do flux-conserving interpolation for the overlapping region
C between an output grid-box and a single input grid-box
C
      INTEGER NXMAX, NX, NY
      REAL F(0:NXMAX+1,0:NY+1), AWT
      INTEGER JA, JB, KA, KB
      REAL DELX(2), DELY(3)
C
      REAL FBAR, FMEAN
C
C     Compute equal-weighted average of corner values
      FBAR= 0.5*(  0.5*(F(JA,KA) + F(JB,KA)  )
     1           + 0.5*(F(JA,KB) + F(JB,KB)) )
C
C     Compute area-weighted average over grid-box
      FMEAN=         AWT*0.5*(F(JA,KA) + F(JB,KA)  )
     1       + (1.0-AWT)*0.5*(F(JA,KB) + F(JB,KB))
C
C     Flux-conserving quadratic interpolation
      FINTP=
     1   DELY(3)*DELX(1)*2.0*(FBAR - FMEAN)
     2
     3 + DELY(2)*DELX(2)*0.25*(  F(JA,KA) + F(JB,KB)
     4                         - F(JA,KB) - F(JB,KA)  )
     5 + DELY(2)*DELX(1)*(  0.5*(F(JA,KB) - F(JA,KA))
     6                    - 3.0*(FBAR - FMEAN)        )
     7 + DELY(1)*DELX(2)*0.5*(F(JB,KA) - F(JA,KA))
     8
     9 + DELY(1)*DELX(1)*F(JA,KA)
C
      RETURN
      END

      SUBROUTINE FLXIN1( MAP, FRAC, DYNEW, DY, LWT, RWT,
     1                   YBNEW, NYNEW, YB, YPT, NY )
C Initializes arrays required for 1-D flux conserving interpolation
C
C Input:  REAL YBNEW(0:NYNEW) -- coordinate values of output grid-box edges
C           (i.e. grid-box K extends from YBNEW(K-1) to YBNEW(K))
C         INTEGER NYNEW -- no. of output grid-boxes
C         REAL YB(0:NY) -- coordinate values of input grid-box edges
C         REAL YPT(0:NY+1) -- coordinate values of input grid-points
C (The first and last values should be identical to YB(0), YB(NY))
C         INTEGER NY -- no. of input grid-points
C (Note: YBNEW, YB, should be in ascending order)
C Output: INTEGER MAP(0:NYNEW) -- index map giving the nearest left input
C                  grid-point corresponding to each output grid-box edge
C         REAL FRAC(0:NYNEW) -- fractional distance of the nearest lower
C                               left input grid-point
C         (FRAC=0.0 => at that point, FRAC=1.0 => at next point)
C         REAL DYNEW(NYNEW) -- width of output grid-box
C         REAL DY(0:NY) -- width of modified input grid-box
C         REAL LWT(NY) -- left-side weight of grid-point
C         REAL RWT(NY) -- right-side weight of grid-point
C
      INTEGER NYNEW, NY
      INTEGER MAP(0:NYNEW)
      REAL FRAC(0:NYNEW), DYNEW(NYNEW), DY(0:NY), LWT(NY), RWT(NY)
      REAL YBNEW(0:NYNEW), YB(0:NY), YPT(0:NY+1)
C
      INTEGER J, K
C
C     Output grid-box widths
      DO 10 K=1,NYNEW
   10   DYNEW(K)= YBNEW(K) - YBNEW(K-1)
C
C     Input grid-parameters
      DY(0)= YPT(1) - YPT(0)
      DO 20 J=1,NY
C       Width
        DY(J)= YPT(J+1) - YPT(J)
C       Left weighting factor for flux
        LWT(J)= (YPT(J) - YB(J-1)) / DY(J-1)
C       Right weighting factor for flux
   20   RWT(J)= (YB(J) - YPT(J)) / DY(J)
C
C     Compute fractional distance from nearest left input grid-point
      MAP(0)= 0
      FRAC(0)= 0.0
      MAP(NYNEW)= NY
      FRAC(NYNEW)= 1.0
C
      K= 1
      DO 40 J=0,NY
   30   IF (YBNEW(K) .LE. YPT(J+1)) THEN
          MAP(K)= J
          FRAC(K)= (YBNEW(K) - YPT(J)) / DY(J)
          IF (K .EQ. NYNEW-1) GOTO 50
          K= K+1
          GOTO 30
        ENDIF
   40   CONTINUE
C
   50 RETURN
      END

      SUBROUTINE FLUX1D( FNEW, NYNEW, F, NY,
     1                   MAP, FRAC, DYNEW, DY, LWT, RWT )
C Uses the arrays MAP, FRAC, DYNEW, DY, LWT, RWT initialized by FLXIN1
C to interpolate flux F in coordinate system Y to flux FNEW in
C coordinate system YNEW, conserving the integrated flux values over
C the coarsest grid resolution.
C
C Input:  INTEGER NYNEW -- no. of output grid-boxes
C         REAL F(NY) -- flux values at the input grid-boxes
C         INTEGER NY -- no. of input grid-boxes
C         INTEGER MAP(0:NYNEW) -- index map giving the nearest left input
C                  grid-point corresponding to each output grid-box edge
C         REAL FRAC(0:NYNEW) -- fractional distance of the nearest lower
C                               left input grid-point
C         REAL DYNEW(NYNEW) -- width of output grid-box
C         REAL DY(0:NY) -- width of modified input grid-box
C         REAL LWT(NY) -- left-side weight of grid-point
C         REAL RWT(NY) -- right-side weight of grid-point
C Output: REAL FNEW(NYNEW) -- flux values at the output grid-boxes
C
      INTEGER NYNEW, NY
      REAL FNEW(NYNEW), F(NY)
      REAL FRAC(0:NYNEW), DYNEW(NYNEW), DY(0:NY), LWT(NY), RWT(NY)
      INTEGER MAP(0:NYNEW)
C
      INTEGER J, K, J0, J1
      REAL FBAR0, FMEAN0, FVAL0, FBAR1, FMEAN1, FVAL1
C
C     Compute integrated output box values
      DO 20 K=1,NYNEW
C       Left-end grid-box
        J0= MAP(K-1)
        IF (J0 .EQ. 0) THEN
          FBAR0= F(1)
          FMEAN0= F(1)
          FVAL0= F(1)*FRAC(K-1)
        ELSE IF (J0 .EQ. NY) THEN
          FBAR0= F(NY)
          FMEAN0= F(NY)
          FVAL0= F(NY)*FRAC(K-1)
        ELSE
          FBAR0= 0.5*(F(J0+1) + F(J0))
          FMEAN0= RWT(J0)*F(J0) + LWT(J0+1)*F(J0+1)
          FVAL0= 2.0*(FBAR0 - FMEAN0)*FRAC(K-1)**3
     1          + (  0.5*(F(J0+1) - F(J0))
     2             - 3.0*(FBAR0 - FMEAN0)        )*FRAC(K-1)**2
     3          + F(J0)*FRAC(K-1)
        ENDIF
C       Right-end grid-box
        J1= MAP(K)
        IF (J1 .EQ. 0) THEN
          FBAR1= F(1)
          FMEAN1= F(1)
          FVAL1= F(1)*FRAC(K)
        ELSE IF (J1 .EQ. NY) THEN
          FBAR1= F(NY)
          FMEAN1= F(NY)
          FVAL1= F(NY)*FRAC(K)
        ELSE
          FBAR1= 0.5*(F(J1+1) + F(J1))
          FMEAN1= RWT(J1)*F(J1) + LWT(J1+1)*F(J1+1)
          FVAL1= 2.0*(FBAR1 - FMEAN1)*FRAC(K)**3
     1          + (  0.5*(F(J1+1) - F(J1))
     2             - 3.0*(FBAR1 - FMEAN1)        )*FRAC(K)**2
     3          + F(J1)*FRAC(K)
        ENDIF
C       Contribution from the extreme ovelapping input grid-boxes
        FNEW(K)= -DY(J0)*FVAL0 + DY(J1)*FVAL1
        IF (J1 .GT. J0) THEN
C         Integrate over overlapping whole input grid-boxes
          IF (J0 .EQ. 0) THEN
C           Left-extreme grid-box
            FNEW(K)= FNEW(K) + DY(J0)*F(J0+1)
          ELSE
            FNEW(K)= FNEW(K) + DY(J0)*(RWT(J0)*F(J0)+LWT(J0+1)*F(J0+1))
          ENDIF
          DO 10 J=J0+1,J1-1
   10       FNEW(K)= FNEW(K) + DY(J)*(RWT(J)*F(J)+LWT(J+1)*F(J+1))
          ENDIF
   20     CONTINUE
C     Unweight output box values
      DO 30 K=1,NYNEW
   30   FNEW(K)= FNEW(K) / DYNEW(K)
      RETURN
      END

C --------------------------------------------------------------------
C For GNU Emacs:
C $Msg-digest-checksum: 4d91f53c66363e12972afc5bdbe58ef0 $
C --------------------------------------------------------------------
C Local Variables:
C msg-digest-active: t
C End:
C --------------------------------------------------------------------
