#ifndef JDEBUG
#   define JDEBUG 1
#endif
C Simphys version 3.8 created 18 Jan 1995
C ------------- Description of major subroutines -----------------------
C SURFAC -- Computes surface parametrizations for a set of columns
C RADIAT -- Computes radiative heating rates for a set of columns
C DCADJ  -- performs dry adiabatic adjustment in a single column
C MSADJ  -- Performs moist static adjustment for a set of columns
C MCADJ  -- Performs moist convective adjustment in a single column
C DIFFUS -- Computes vertical diffusion tendencies for a set of columns
C
C -------------- Description of major functions ------------------------
C AVGSOL -- Returns zonally-averaged incoming solar radiation
C AVGALB -- Returns zonally-averaged climatological albedo
C AVGSST -- Returns zonally-averaged climatological surface temperature
C
      SUBROUTINE SURFAC( CDRAG, QEVAP, FEVAP, FSENS, FRAD,
     1                   TLOW, TLOWM1, QLOW, VLOW, PLV, NCOLMX, NCOL,
     2                   L1, TSURF, RHSURF, PSURF, GRAVIT, CSURF )
C
C SURFAC computes computes the surface parametrizations.
C
C The sensible heat flux FSENS and the evaporation flux QEVAP are
C computed using a simple bulk aerodynamic formula, with the exchange
C coefficient proportional to the density and the wind speed. Linear
C extrapolation (in log-pressure) is used to estimate the surface air
C temperature, given the air temperature at the two lowest model
C levels. The surface air relative humidity is assumed to be the average
C of the surface relative humidity and lowest model-level relative
C humidity. The sensible heat flux is proportional the difference
C between the surface temperature and the surface air temperature. The
C evaporation is proportional to difference between the surface
C specific humidity and the surface air specific humidity. A minimum
C speed of 5m/s is assumed while computing the exchange coefficient.
C The surface drag coefficient CDRAG is simply the exchange coefficient
C divided by the mass of the lowest layer. The upward LW radiation at
C the surface FRAD is determined by the simple black-body radiation
C formula.
C (Held & Suarez, JAS 1978)
C
C Input:  REAL TLOW(NCOL) -- temperature at lowest model level (in K)
C                            (TLOW(J) corresponds to value at column J)
C         REAL TLOWM1(NCOL) -- temperature at level above lowest level
C         REAL QLOW(NCOL) -- mass mixing ratio of water vapour
C                            at lowest level
C         REAL VLOW(NCOLMX,2) -- lowest level wind components
C         REAL PLV(NCOLMX,L1) -- full level pressure
C         INTEGER NCOLMX -- inner dimension for column arrays
C         INTEGER NCOL -- no. of columns to be handled
C         INTEGER L1 -- no. of vertical levels
C         REAL TSURF(NCOL) -- Surface temperature for each column
C         REAL RHSURF(NCOL) -- Surface rel. humidity for each column
C         REAL PSURF(NCOL) -- Surface pressure
C         REAL GRAVIT -- Gravitational acceleration at surface (m/s**2)
C         REAL CSURF -- Non-dimensional surface exchange coefficient
C                       (typically 1.1E-3)
C Output: REAL CDRAG(NCOL) -- Surface drag coefficient (1/s)
C         REAL QEVAP(NCOL) -- Evaporative flux of water vapour
C                             (kg/(s*m**2))
C         REAL FEVAP(NCOL) -- Heat flux associated with the
C                             evaporative flux of water vapour
C                             (W/m**2)
C         REAL FSENS(NCOL) -- Upward sensible heat flux from surface
C                             (W/m**2)
C         REAL FRAD(NCOL) -- Upward radiative heat flux from surface
C                            (W/m**2)
C
      INTEGER NCOLMX, NCOL, L1
      REAL CDRAG(NCOL), QEVAP(NCOL), FEVAP(NCOL), FSENS(NCOL)
      REAL FRAD(NCOL), TLOW(NCOL), TLOWM1(NCOL), QLOW(NCOL)
      REAL VLOW(NCOLMX,2), PLV(NCOLMX,L1), TSURF(NCOL), RHSURF(NCOL)
      REAL PSURF(NCOL), GRAVIT, CSURF
C
      REAL ES, QSSURF, QSLOW, RHLOW, TAIR, QSAIR, RHAIR, CTEMP
      INTEGER J
C
C     Stefan's constant (in W/(m**2*K**4))
      REAL STEFAN
      PARAMETER (STEFAN= 5.669E-8)
C     Molecular weights ofr air, water, ozone (kg/kgmole)
      REAL AIRWT, H2OWT, O3WT
      PARAMETER (AIRWT=28.97, H2OWT=18.016, O3WT=48.)
C     Coefficients for water-air systems
C     (R, Cp for dry air (in J/(kg*K)),
C      latent heat of condendation (in J/kg) )
      REAL RDRY, CPDRY, LHEAT
      PARAMETER (RDRY= 287.0, CPDRY=1004.0, LHEAT=2.5E6)
C     Coefficients for Clausius-Clapeyron relation
      REAL CCFAC0, CCFAC1
      PARAMETER (CCFAC0=26.2612, CCFAC1=5420.3)
C     Derived coefficients
      REAL LBYCP, RBYCP
      PARAMETER (LBYCP=LHEAT/CPDRY, RBYCP=RDRY/CPDRY)
      REAL H2OAIR, O3AIR
      PARAMETER (H2OAIR=H2OWT/AIRWT, O3AIR=O3WT/AIRWT)
C     Magnus formula for Clausius-Clapeyron realtion
      REAL CCMF0, CCMF1, CCMF2
      PARAMETER (CCMF0=58.8352, CCMF1=6763.6, CCMF2=4.9283)
C
      DO 100 J=1,NCOL
C       Compute surface saturation mixing ratio
        ES= EXP(CCMF0-CCMF1/TSURF(J)-CCMF2*ALOG(TSURF(J)))
        QSSURF= H2OAIR*ES/(PSURF(J) - ES)
C
C       Compute low-level saturation mixing ratio
        ES= EXP(CCMF0-CCMF1/TLOW(J)-CCMF2*ALOG(TLOW(J)))
        QSLOW= H2OAIR*ES/(PLV(J,L1) - ES)
C       Compute low-level relative humidity (>= 0)
        RHLOW= AMAX1(0.0, QLOW(J)) / QSLOW
C
C       Compute surface air temperature (extrapolate in log-p)
        TAIR= TLOW(J) +  (TLOW(J) - TLOWM1(J))
     1                  *ALOG( PSURF(J)   / PLV(J,L1)  )
     2                  /ALOG( PLV(J,L1) / PLV(J,L1-1))
C       Compute surface air relative humidity (>= 0)
        RHAIR= AMAX1( 0.0, 0.5*(RHSURF(J) + RHLOW) )
C       Compute surface air saturation mixing ratio
        ES= EXP(CCMF0-CCMF1/TAIR-CCMF2*ALOG(TAIR))
        QSAIR= H2OAIR*ES/(PSURF(J) - ES)
#ifdef DEBUG
        IF (J. EQ. JDEBUG) THEN
          WRITE(*,*) 'TSURF, TAIR, TLOW, TLOWM1: ', TSURF(JDEBUG), TAIR,
     1                                      TLOW(JDEBUG), TLOWM1(JDEBUG)
          WRITE(*,*) 'RHSURF, RHAIR, RHLOW: ', RHSURF(JDEBUG),
     1                                         RHAIR, RHLOW
        ENDIF
#endif
C
C       Compute dimensional bulk exchange coefficient
C       (Assume 5m/s minimum wind speed)
        CTEMP= CSURF * (PSURF(J)/(RDRY*TSURF(J)))
     1               * AMAX1(5.0, SQRT(VLOW(J,1)**2 + VLOW(J,2)**2))
C       Compute evaporative Q flux, and associated heat flux
C       (Negative evaporative flux is not allowed)
        QEVAP(J)= AMAX1( 0.0, CTEMP*(RHSURF(J)*QSSURF - RHAIR*QSAIR) )
        FEVAP(J)= LHEAT*QEVAP(J)
C       Compute sensible heat flux (could be negative)
        FSENS(J)= CTEMP*CPDRY*(TSURF(J) - TAIR)
C       Compute upward radiative flux
        FRAD(J)= STEFAN*TSURF(J)**4
C       Compute surface drag coefficient
        CDRAG(J)= CTEMP*GRAVIT/(2.0*(PSURF(J) - PLV(J,L1)))
  100   CONTINUE
#ifdef DEBUG
      WRITE(*,*) 'CDRAG*86400., CTEMP*86400., QEVAP*1000.:',
     1    CDRAG(JDEBUG)*86400., CTEMP*86400., QEVAP(JDEBUG)*1000.
      WRITE(*,*) 'FEVAP, FSENS, FRAD:',
     1            FEVAP(JDEBUG), FSENS(JDEBUG), FRAD(JDEBUG)
#endif
      RETURN
      END

      SUBROUTINE RADIAT( FPSW, FMSW, FPLW, FMLW,
     1                   T, Q, O3, DP, NCOLMX, NCOL, L1,
     2                   FIXST, FBOT, FTOP, ALBEDO, WINLW,
     3                   KLWAIR, KLWH2O, KSWAIR, KSWO3,
     4                   KAPA, FAC )
C
C RADIAT computes SW and LW radiative fluxes, given the temperature
C and the mass mixing ratio of water vapour and ozone.
C
C For the LW calculation, the atmosphere is assumed to be grey, with a
C "picket fence" structure, i.e., with a window which allows a fraction
C WINLW of the upward LW radiation at the surface to escape directly
C to space. There are two radiatively active constituents in the LW:
C one well-mixed (AIR) and the other with spatially varying mixing
C ratio (H2O). KLWAIR denotes the LW absorptivity of the well-mixed
C constituents, and KLWH2O denotes the LW absorptivity of water vapour.
C If FIXST is true, FBOT contains the upward LW flux at the surface.
C If FIXST is false, FBOT contains the net (upward) LW flux at the
C surface.
C
C For the SW calculation, the atmosphere is assumed to be grey, but with
C no windows. There are two radiatively active constituents in the SW:
C one well-mixed (AIR) and the other with spatially varying mixing
C ratio (O3). KSWAIR denotes the SW absorptivity of the well-mixed
C constituents, and KSWO3 denotes the SW absorptivity of ozone. FTOP
C denotes the incident solar radiation at the top of the atmosphere.
C At the surface, a fraction ALBEDO of the downward SW flux is assumed
C to be reflected upwards.
C
C NOTE: Mass mixing ratios Q/O3 may occasionally be negative,
C       leading to reduced LW/SW absorptivity. In that case,
C       a constraint is imposed so this reduction does not exceed
C       50% of the base air absorptivity values KLWAIR/KSWAIR.
C
C Input:  REAL T(NCOLMX,L1) -- Temperature field (in K)
C              (T(J,L) corresponds to level L of column J)
C         REAL Q(NCOLMX,L1) -- mass mixing ratio of water vapour
C         REAL O3(NCOLMX,L1) -- mass mixing ratio of ozone
C         REAL DP(NCOLMX,L1) -- pressure thicknesses of levels (in Pa)
C         INTEGER NCOLMX -- inner dimension for column arrays
C         INTEGER NCOL -- no. of columns to be handled
C         INTEGER L1 -- no. of vertical levels
C         LOGICAL FIXST  -- IF .TRUE., FBOT equals upward longwave
C                           radiative flux at surface ("fixed surface
C                           temperature"). If .FALSE., FBOT equals the
C                           net (upward - downward) longwave radiative
C                           flux at surface.
C         REAL FBOT(NCOL) -- upward longwave radiative flux at bottom
C         REAL FTOP(NCOL) -- downward shortwave radiative flux at top
C         REAL ALBEDO(NCOL) -- surface albedo
C         REAL WINLW -- longwave window fraction (0 <= WINLW < 1)
C         REAL KLWAIR -- longwave absorptivity of air (in 1/Pa)
C         REAL KLWH2O -- longwave absorptivity of H2O (in 1/Pa)
C         REAL KSWAIR -- shortwave absorptivity of air (in 1/Pa)
C         REAL KSWO3 -- shortwave absorptivity of ozone (in 1/Pa)
C         REAL KAPA(NCOLMX,L1) -- work array
C         REAL FAC(NCOLMX,L1,2) -- work array
C Output: REAL FPSW(NCOLMX,0:L1)--upward shortwave flux at half-levels
C         REAL FMSW(NCOLMX,0:L1)--downward shortwave flux at half-levels
C         REAL FPLW(NCOLMX,0:L1)--upward longwave flux at half-levels
C         REAL FMLW(NCOLMX,0:L1)--downward longwave flux at half-levels
C         (All in W/m**2)
C
      INTEGER NCOLMX, NCOL, L1
      REAL FPSW(NCOLMX,0:L1), FMSW(NCOLMX,0:L1) 
      REAL FPLW(NCOLMX,0:L1), FMLW(NCOLMX,0:L1)
      REAL T(NCOLMX,L1), Q(NCOLMX,L1), O3(NCOLMX,L1), DP(NCOLMX,L1)
      REAL FBOT(NCOL), FTOP(NCOL), ALBEDO(NCOL)
      REAL WINLW, KLWAIR, KLWH2O, KSWAIR, KSWO3
      REAL KAPA(NCOLMX,L1), FAC(NCOLMX,L1,2)
      LOGICAL FIXST
C
      INTEGER J, L
C
C     Stefan's constant (in W/(m**2*K**4))
      REAL STEFAN
      PARAMETER (STEFAN= 5.669E-8)
C     Molecular weights ofr air, water, ozone (kg/kgmole)
      REAL AIRWT, H2OWT, O3WT
      PARAMETER (AIRWT=28.97, H2OWT=18.016, O3WT=48.)
C     Coefficients for water-air systems
C     (R, Cp for dry air (in J/(kg*K)),
C      latent heat of condendation (in J/kg) )
      REAL RDRY, CPDRY, LHEAT
      PARAMETER (RDRY= 287.0, CPDRY=1004.0, LHEAT=2.5E6)
C     Coefficients for Clausius-Clapeyron relation
      REAL CCFAC0, CCFAC1
      PARAMETER (CCFAC0=26.2612, CCFAC1=5420.3)
C     Derived coefficients
      REAL LBYCP, RBYCP
      PARAMETER (LBYCP=LHEAT/CPDRY, RBYCP=RDRY/CPDRY)
      REAL H2OAIR, O3AIR
      PARAMETER (H2OAIR=H2OWT/AIRWT, O3AIR=O3WT/AIRWT)
C
C     Compute height/column dependent longwave absorptivity
      DO 110 L=1,L1
        DO 110 J=1,NCOL
          KAPA(J,L)= KLWAIR + AMAX1(-0.5*KLWAIR, KLWH2O*Q(J,L)/H2OAIR)
  110     CONTINUE
C     Compute longwave fluxes
      CALL GREYLW( FPLW, FMLW, T, KAPA, DP,
     1             NCOLMX, NCOL, L1, FIXST, FBOT, WINLW, FAC )
C
C     Compute height/column dependent shortwave absorptivity
      DO 120 L=1,L1
        DO 120 J=1,NCOL
          KAPA(J,L)= KSWAIR + AMAX1(-0.5*KSWAIR, KSWO3*O3(J,L)/O3AIR)
  120     CONTINUE
C     Compute shortwave fluxes
      CALL GREYSW( FPSW, FMSW, T, KAPA, DP,
     1             NCOLMX, NCOL, L1, FTOP, ALBEDO, FAC )
#ifdef DEBUG
      WRITE(*,*) 'FPLW, FMLW, FPSW, FMSW:',
     1            FPLW(JDEBUG,L1), FMLW(JDEBUG,L1),
     2            FPSW(JDEBUG,L1), FMSW(JDEBUG,L1)
#endif
      RETURN
      END

      SUBROUTINE DCADJ( PTCOL, IADJ, IADLEV, WT, L1 )
C DCADJ performs dry adiabatic adjustment in a single column.
C Input:  REAL PTCOL(L1) -- column Theta values (from top to bottom)
C         REAL WT(L1) -- Weight factor (for heat conservation)
C         INTEGER L1 -- No. of model levels
C Output: REAL PTCOL(L1) -- column Theta values, adjusted to be
C          statically stable from level L1 upward. The adjustment
C          is carried out so as to conserve sum of PTCOL(L)*WT(L).
C         INTEGER IADJ -- no. of adjustment iterations carried out
C                         (>= 0, useful for diagnostic purposes)
C         INTEGER IADLEV -- total no. of levels adjusted in all
C                            iterations (>=0)
C
      INTEGER IADJ, IADLEV, L1
      REAL PTCOL(L1), WT(L1)
C
      REAL HEAT, CUWT, PTBAR
      INTEGER L, LLOW, LHIGH
      LOGICAL EXTEND
C
C ----------------------------------------------------------------------
C (NOTE: It is important for the dry adjustment not to re-adjust
C        previously adjusted regions, unless they have been extended
C        vertically. Otherwise, infinite-looping might occur. The
C        current dry adjustment scheme results in the exact equality
C        PTCOL(L).EQ.PTCOL(L-1) in the adjusted regions. If the
C        adjustment results in adjoining regions becoming unstable,
C        it should simply lead to adjustment over an extended
C        region in the next iteration. So the above equality may
C        be interpreted either as denoting stability, or instability,
C        depending on the context.)
C ----------------------------------------------------------------------
C
C     Beginning of iteration
      IADJ= 0
      IADLEV= 0
      LHIGH= 1
C     Scan downward for statically unstable region
C     (1 jump to this statement)
  200 DO 210 L=LHIGH,L1-1
        IF (PTCOL(L+1).GT.PTCOL(L)) GOTO 300
  210   CONTINUE
C
C     Whole column statically stable
      RETURN
C
C     Unstable region encountered
C     (1 jump to this statement)
  300 LHIGH= L
C     Determine vertical extent of unstable region,
      HEAT= PTCOL(LHIGH)*WT(LHIGH)
      CUWT= WT(LHIGH)
      DO 310 L=LHIGH+1,L1
        IF (PTCOL(L).GE.PTCOL(L-1)) THEN
C         Accumulate heat content, and cumulative weight factor
          LLOW= L
          HEAT= HEAT + PTCOL(L)*WT(L)
          CUWT= CUWT + WT(L)
        ELSE
C         Reached stable region
          GOTO 400
        ENDIF
  310   CONTINUE
C
C     Compute mean Theta
C     (2 jumps to this statement)
  400 PTBAR= HEAT/CUWT
      EXTEND= .FALSE.
      IF (LLOW.LT.L1) THEN
C       Unstable region above bottom of model
        IF (PTBAR.LT.PTCOL(LLOW+1)) THEN
C         Adjustment will destabilize bottom of unstable region
C         Extend region downward
          LLOW= LLOW+1
          HEAT= HEAT + PTCOL(LLOW)*WT(LLOW)
          CUWT= CUWT + WT(LLOW)
          EXTEND= .TRUE.
        ENDIF
      ENDIF
      IF (LHIGH.GT.1) THEN
C       Unstable region below top of model
        IF (PTCOL(LHIGH-1).LT.PTBAR) THEN
C         Adjustment will destabilize top of unstable region
C         Extend region upward
          LHIGH= LHIGH-1
          HEAT= HEAT + PTCOL(LHIGH)*WT(LHIGH)
          CUWT= CUWT + WT(LHIGH)
          EXTEND= .TRUE.
        ENDIF
      ENDIF
C     Check if unstable region has been extended
      IF (EXTEND) GOTO 400
C     Adjust temperature profile to be (exactly) isentropic
      DO 410 L=LHIGH,LLOW
  410   PTCOL(L)= PTBAR
C     Adjustment diagnostics
      IADJ= IADJ+1
      IADLEV= IADLEV + (LLOW - LHIGH + 1)
      LHIGH= MIN0(L1, LLOW+1)
C     End of iteration
      GOTO 200
      END

      SUBROUTINE DCADJ2(PT, IADJ, IADLEV, WT, NCOLMX, NCOL, SGNDPT)
C DCADJ2 performs dry adiabatic adjustment for a set of columns
C (Fast version of DCADJ for two-level models)
C Input:  REAL PT(NCOLMX,2) -- Theta values
C         REAL WT(2) -- Weight factor (for heat conservation)
C         INTEGER NCOLMX -- inner dimension for column arrays
C         INTEGER NCOL -- no. of columns to be handled
C         REAL SGNDPT(NCOL) -- work array
C Output: REAL PT(NCOLMX,2) -- Theta values, adjusted to be dry
C          convectively stable from level L1 upward. The adjustment
C          is carried out so as to conserve sum of PT(J,L)*WT(L).
C         INTEGER IADJ -- no. of adjustment iterations carried out
C                         (>= 0, useful for diagnostic purposes)
C         INTEGER IADLEV -- total no. of levels adjusted in all
C                            iterations (>=0)
C
      INTEGER IADJ, IADLEV, NCOLMX, NCOL
      REAL PT(NCOLMX,2), WT(2), SGNDPT(NCOL)
C
      REAL COEF1, COEF2, PTBAR, FADJ
      INTEGER J
C
      COEF1= WT(1) / (WT(1) + WT(2))
      COEF2= WT(2) / (WT(1) + WT(2))
C     Compute mean Theta, sign of dTheta/dz, adjusted Theta
      DO 110 J=1,NCOL
        PTBAR= COEF1*PT(J,1) + COEF2*PT(J,2)
        SGNDPT(J)= 0.5+SIGN(0.5, PT(J,2) - PT(J,1))
        PT(J,1)=   (1.0-SGNDPT(J))*PT(J,1)
     1              + SGNDPT(J)*PTBAR
        PT(J,2)=   (1.0-SGNDPT(J))*PT(J,2)
     1              + SGNDPT(J)*PTBAR 
  110   CONTINUE
C     Adjustment diagnostics
      FADJ= 0.0
      DO 120 J=1,NCOL
  120   FADJ= FADJ + SGNDPT(J)
      IADJ= NINT(FADJ)
      IADLEV= 2*IADJ
      RETURN
      END

      SUBROUTINE MSADJ( T, Q, QS, DQSDT, SATFLG,
     1                  COLPLV, NCOLMX, NCOL, L1, SATFAC )
C MSADJ performs moist static adjustment for a set of columns.
C Input:  REAL T(NCOLMX,L1) -- Temperature field (in K)
C              (T(J,L) corresponds to level L of column J)
C         REAL Q(NCOLMX,L1) -- mass mixing ratio of water vapor
C         REAL QS(NCOLMX,L1) -- saturation mass mixing ratio
C         REAL DQSDT(NCOLMX,L1) -- T-derivative of QS
C         LOGICAL SATFLG(NCOLMX,L1) -- saturation flag
C                                     (Q(J,L) >= QS(J,L))
C         REAL COLPLV(NCOLMX,L1) -- full level pressure
C         INTEGER NCOLMX -- inner dimension for column arrays
C         INTEGER NCOL -- no. of columns to be handled
C         INTEGER L1 -- No. of vertical levels
C         REAL SATFAC -- attenuation factor for computation of eS
C                      (usually 0.8; determines saturation threshold)
C Output: REAL T(NCOLMX,L1) -- Temperature values, adjusted to be
C          moist statically stable. The adjustment is carried out so
C          as to conserve total heat content in each column.
C         REAL Q(NCOLMX,L1) -- water vapor mass mixing ratio,
C                              after moist static adjustment
C         REAL QS(NCOLMX,L1) -- new saturation mass mixing ratio
C         REAL DQSDT(NCOLMX,L1) -- new T-derivative of QS
C         LOGICAL SATFLG(NCOLMX,L1) -- new saturation flag
C
      INTEGER NCOLMX, NCOL, L1
      REAL T(NCOLMX,L1), Q(NCOLMX,L1), QS(NCOLMX,L1), DQSDT(NCOLMX,L1)
      REAL COLPLV(NCOLMX,L1), SATFAC
      LOGICAL SATFLG(NCOLMX,L1)
C
      REAL ES, DESDT, Q0, T0
      INTEGER I, J, L
C
C     Stefan's constant (in W/(m**2*K**4))
      REAL STEFAN
      PARAMETER (STEFAN= 5.669E-8)
C     Molecular weights ofr air, water, ozone (kg/kgmole)
      REAL AIRWT, H2OWT, O3WT
      PARAMETER (AIRWT=28.97, H2OWT=18.016, O3WT=48.)
C     Coefficients for water-air systems
C     (R, Cp for dry air (in J/(kg*K)),
C      latent heat of condendation (in J/kg) )
      REAL RDRY, CPDRY, LHEAT
      PARAMETER (RDRY= 287.0, CPDRY=1004.0, LHEAT=2.5E6)
C     Coefficients for Clausius-Clapeyron relation
      REAL CCFAC0, CCFAC1
      PARAMETER (CCFAC0=26.2612, CCFAC1=5420.3)
C     Derived coefficients
      REAL LBYCP, RBYCP
      PARAMETER (LBYCP=LHEAT/CPDRY, RBYCP=RDRY/CPDRY)
      REAL H2OAIR, O3AIR
      PARAMETER (H2OAIR=H2OWT/AIRWT, O3AIR=O3WT/AIRWT)
C
C
#ifdef DEBUG
      WRITE(*,'(A8,(10F7.2))')'MSA-T0: ', (T(JDEBUG,L),L=1,L1)
      WRITE(*,'(A8,(10F7.3))')'MSA-Q0: ', (Q(JDEBUG,L)*1000.0,L=1,L1)
      WRITE(*,'(A8,(10F7.3))')'MSA-QS: ', (QS(JDEBUG,L)*1000.0,L=1,L1)
#endif
C     Perform moist static adjustment (2 iterations)
      DO 110 L=1,L1
        DO 110 I=1,2
          DO 110 J=1,NCOL
            IF (SATFLG(J,L)) THEN
C             Level is supersaturated
C             Copy current Q, T values
              Q0= Q(J,L)
              T0= T(J,L)
C             Adjust water vapour mixing ratio
              Q(J,L)=  (QS(J,L) + Q0*LBYCP*DQSDT(J,L))
     1               / (1.0 + LBYCP*DQSDT(J,L))
C             Adjust temperature
              T(J,L)= T0 + LBYCP * (Q0 - Q(J,L))
C             Recompute saturation mixing ratio
C             Saturation vapor pressure
C             (An attenuation factor of SATFAC is used)
              ES=SATFAC*EXP(CCFAC0-CCFAC1/T(J,L))
C             dES/dT
              DESDT= ES*(CCFAC1/T(J,L)**2)
C             Saturation mixing ratio QS
              QS(J,L)= H2OAIR*ES
     1                /AMAX1(COLPLV(J,L), COLPLV(J,L) - ES)
C             dQS/dT
              DQSDT(J,L)= H2OAIR*COLPLV(J,L)*DESDT
     1                   /AMAX1(COLPLV(J,L), COLPLV(J,L) - ES)**2
            ENDIF
  110       CONTINUE
C     End of adjustment
#ifdef DEBUG
      WRITE(*,'(A8,(10F7.2))') 'MSA-T1: ', (T(JDEBUG,L),L=1,L1)
      WRITE(*,'(A8,(10F7.3))') 'MSA-Q1: ', (Q(JDEBUG,L)*1000.0,L=1,L1)
#endif
      RETURN
      END

      SUBROUTINE MCADJ( TCOL, QCOL, QS, DQSDT, SATFLG, GAMMAS, 
     1                  IADJ, IADLEV, CIFLAG, PLV, PHLV, DP,
     2                  L1, SATFAC, DELTAT )
C MCADJ performs moist convective adjustment in a single column.
C Input:  REAL TCOL(L1) -- column temperature values
C         REAL QCOL(L1) -- mass mixing ratio of water vapor
C         REAL QS(L1) -- saturation mass mixing ratio
C         REAL DQSDT(L1) -- T-derivative of QS
C         LOGICAL SATFLG(L1) -- saturation flag (QCOL(L) >= QS(L))
C         REAL GAMMAS(L1-1) -- moist adiabat at half-levels
C                              i.e. GAMMAS(L)= dT/dp at L+1/2
C         LOGICAL CIFLAG(L1-1) -- moist convective instability flag
C           (if CIFLAG(L) is .TRUE., then TCOL(L+1)-TCOL(L) exceeds
C            0.01+GAMMAS(L)*(PLV(L+1)-PLV(L)), and level L+1 is
C            saturated)
C         REAL PLV(L1) -- full level pressure
C         REAL PHLV(0:L1) -- half level pressure values
C         REAL DP(L1) -- pressure thickness at model levels
C         INTEGER L1 -- No. of model levels
C         REAL SATFAC -- attenuation factor for computation of eS
C                      (usually 0.8; determines saturation threshold)
C         REAL DELTAT(L1) -- work array
C Output: REAL TCOL(L1) -- column temperature values, adjusted to be
C          moist convectively stable. The adjustment is carried out so
C          as to conserve total column heat content
C         REAL QCOL(L1) -- column water vapor mass mixing ratio values,
C                          after moist convective adjustment
C         REAL QS(L1) -- new saturation mass mixing ratio
C         REAL DQSDT(L1) -- new T-derivative of QS
C         LOGICAL SATFLG(L1) -- new saturation flag (QCOL(L) >= QS(J,L))
C         REAL GAMMAS(L1-1) -- new moist adiabat at half-levels
C                              i.e. GAMMAS(L)= dT/dp at L+1/2
C         INTEGER IADJ -- no. of adjustment iterations carried out
C                         (>= 0)
C         INTEGER IADLEV -- total no. of levels adjusted in all
C                           iterations (>=0)
C
      INTEGER IADJ, IADLEV, L1
      REAL TCOL(L1), QCOL(L1), QS(L1), DQSDT(L1), GAMMAS(L1-1)
      REAL PLV(L1), PHLV(0:L1), DP(L1), SATFAC
      REAL DELTAT(L1)
      LOGICAL SATFLG(L1), CIFLAG(L1-1)
C
      REAL ES, DESDT, THLV, HEAT, CUWT, WT
      REAL TLOW, THIGH, GAMMAH
      INTEGER I, L, LLOW, LHIGH
      LOGICAL EXTEND
C
C     Stefan's constant (in W/(m**2*K**4))
      REAL STEFAN
      PARAMETER (STEFAN= 5.669E-8)
C     Molecular weights ofr air, water, ozone (kg/kgmole)
      REAL AIRWT, H2OWT, O3WT
      PARAMETER (AIRWT=28.97, H2OWT=18.016, O3WT=48.)
C     Coefficients for water-air systems
C     (R, Cp for dry air (in J/(kg*K)),
C      latent heat of condendation (in J/kg) )
      REAL RDRY, CPDRY, LHEAT
      PARAMETER (RDRY= 287.0, CPDRY=1004.0, LHEAT=2.5E6)
C     Coefficients for Clausius-Clapeyron relation
      REAL CCFAC0, CCFAC1
      PARAMETER (CCFAC0=26.2612, CCFAC1=5420.3)
C     Derived coefficients
      REAL LBYCP, RBYCP
      PARAMETER (LBYCP=LHEAT/CPDRY, RBYCP=RDRY/CPDRY)
      REAL H2OAIR, O3AIR
      PARAMETER (H2OAIR=H2OWT/AIRWT, O3AIR=O3WT/AIRWT)
C
C
#ifdef DEBUG
      WRITE(*,'(A8,(10F7.2))') 'MCA-T0: ', (TCOL(L),L=1,L1)
      WRITE(*,'(A8,(10F7.3))') 'MCA-Q0: ', (QCOL(L)*1000.0,L=1,L1)
      WRITE(*,'(A8,(10F7.3))') 'MCA-QS: ', (QS(L)*1000.0,L=1,L1)
      WRITE(*,'(A8,(10F7.2))')
     1        'MCA-GS: ', (GAMMAS(L)*(PLV(L+1) - PLV(L)),L=1,L1-1)
#endif
C
C     Beginning of iteration
      IADJ= 0
      IADLEV= 0
      LHIGH= 1
C     Scan downward for moist convectively unstable region
C     (1 jump to this statement)
  200 DO 210 L=LHIGH,L1-1
        IF (CIFLAG(L)) GOTO 300
  210   CONTINUE
C     Whole column is moist convectively stable; return
      RETURN
C
C     Unstable region encountered
C     (1 jump to this statement)
  300 LHIGH= L
C     Determine vertical extent of unstable region,
      DO 310 L=LHIGH+1,L1-1
        IF (.NOT. CIFLAG(L)) GOTO 390
  310   CONTINUE
      L= L1
C     (1 jump to this statement)
  390 LLOW= L
#ifdef DEBUG
      WRITE(*,*) 'LHIGH, LLOW:', LHIGH, LLOW
#endif
C     Iterate moist convective adjustment over unstable region
      DO 600 I=1,3
C       Integrate the moist adiabat vertically upward
        DELTAT(LLOW)= 0.0
        DO 410 L=LLOW-1,LHIGH+1,-1
  410     DELTAT(L)= DELTAT(L+1) + GAMMAS(L)*(PLV(L+1) - PLV(L))
C       Compute total heat content in region (except top of region)
        HEAT= 0.0
        CUWT= 0.0
        DO 420 L=LHIGH+1,LLOW
          WT= 1.0 + LBYCP*DQSDT(L)
          HEAT= HEAT + DP(L)*(  WT*(TCOL(L) + DELTAT(L))
     1                        + LBYCP*(QCOL(L) - QS(L)) )
          CUWT= CUWT + DP(L)*WT
  420     CONTINUE
C
C       (1 jump to this statement)
  500   EXTEND= .FALSE.
        DELTAT(LHIGH)=  DELTAT(LHIGH+1)
     1                + GAMMAS(LHIGH)*(PLV(LHIGH+1) - PLV(LHIGH))
#ifdef DEBUG
        WRITE(*,*) 'LHIGH=', LHIGH
#endif
        IF (SATFLG(LHIGH) .AND. (L1 .NE. 2)) THEN
C         Top of region is saturated
          WT= 1.0 + LBYCP*DQSDT(LHIGH)
          HEAT= HEAT + DP(LHIGH)*(  WT*(TCOL(LHIGH) + DELTAT(LHIGH))
     1                            + LBYCP*(QCOL(LHIGH) - QS(LHIGH)) )
          CUWT= CUWT + DP(LHIGH)*WT
C         Compute new temperature at bottom of region
          TLOW= HEAT/CUWT
          IF (LHIGH.GT.1) THEN
C           Compute new temperature at top of region
            THIGH= TLOW - DELTAT(LHIGH)
C           Half-level temperature at LHIGH-1/2
            THLV= 0.5 * (THIGH + TCOL(LHIGH-1))
C           Saturation vapor pressure for evaporation
            ES= SATFAC*EXP(CCFAC0-CCFAC1/THLV)
C           dES/dT
            DESDT= ES*(CCFAC1/THLV**2)
C           Moist adiabatic lapse rate (at LHIGH-1/2)
            GAMMAH=  RBYCP*(THLV/PHLV(LHIGH-1))
     1             * (PHLV(LHIGH-1) + H2OAIR*LBYCP*ES/(RBYCP*THLV) )
     2             / (PHLV(LHIGH-1) + H2OAIR*LBYCP*DESDT )
            IF ( (THIGH-TCOL(LHIGH-1)) .GT.
     1           (0.01 + GAMMAH*(PLV(LHIGH) - PLV(LHIGH-1))) ) THEN
C             Adjustment will destabilize top of unstable region
C             Extend region upward
              LHIGH= LHIGH-1
              EXTEND= .TRUE.
            ENDIF
          ENDIF
        ELSE
C         Top of region is not saturated, keep Q fixed
C         (if L1=2, this is always assumed to be true)
          WT= 1.0
          HEAT= HEAT + DP(LHIGH)*WT*(TCOL(LHIGH) + DELTAT(LHIGH))
          CUWT= CUWT + DP(LHIGH)*WT
C         Compute new temperature at bottom of region
          TLOW= HEAT/CUWT
        ENDIF
C
C       Check if unstable region has been extended
        IF (EXTEND) GOTO 500
C       Adjust Q, T profiles
        IF (SATFLG(LHIGH) .AND. (L1 .NE. 2)) THEN
C         Top level initially saturated
          QCOL(LHIGH)=  QS(LHIGH)
     1                + DQSDT(LHIGH)*(TLOW-DELTAT(LHIGH)-TCOL(LHIGH))
        ENDIF
        TCOL(LHIGH)= TLOW - DELTAT(LHIGH)
        DO 510 L=LHIGH+1,LLOW
          QCOL(L)= QS(L) + DQSDT(L)*(TLOW - DELTAT(L) - TCOL(L))
  510     TCOL(L)= TLOW - DELTAT(L)
C       Re-compute saturation mixing ratios (at L)
        DO 520 L=LHIGH,LLOW
C         Saturation vapor pressure
C         (An attenuation factor of SATFAC is used)
          ES=SATFAC*EXP(CCFAC0-CCFAC1/TCOL(L))
C         dES/dT
          DESDT= ES*(CCFAC1/TCOL(L)**2)
C         Saturation mixing ratio QS
          QS(L)= H2OAIR*ES
     1          /AMAX1(PLV(L), PLV(L) - ES)
C         dQS/dT
          DQSDT(L)= H2OAIR*PLV(L)*DESDT
     1             /AMAX1(PLV(L), PLV(L) - ES)**2
  520     CONTINUE
C       Re-compute moist adiabatic lapse rate (at L+1/2)
        DO 530 L=MAX0(1,LHIGH-1),MIN0(L1-1,LLOW)
C         Half-level temperature
          THLV= 0.5 * (TCOL(L) + TCOL(L+1))
C         Saturation vapor pressure for evaporation
          ES= SATFAC*EXP(CCFAC0-CCFAC1/THLV)
C         dES/dT
          DESDT= ES*(CCFAC1/THLV**2)
C         Moist adiabatic lapse rate (at half-levels)
          GAMMAS(L)=  RBYCP*(THLV/PHLV(L))
     1              * (PHLV(L) + H2OAIR*LBYCP*ES/(RBYCP*THLV) )
     2              / (PHLV(L) + H2OAIR*LBYCP*DESDT )
  530     CONTINUE
  600   CONTINUE
C     Set saturation flag (except at top of region)
      DO 610 L=LHIGH+1,LLOW
  610   SATFLG(L)= .TRUE.
C     Two-level case is handled differently
      IF (L1 .EQ. 2) SATFLG(1)= QCOL(1) .GT. QS(1)
      IF (LLOW .LT. L1) THEN
C       Recompute convective instability flag at LLOW+1/2
        CIFLAG(LLOW)= ( (TCOL(LLOW+1)-TCOL(LLOW)) .GT.
     1                  (0.01 + GAMMAS(LLOW)*(PLV(LLOW+1)-PLV(LLOW))) )
     2          .AND. ( SATFLG(LLOW+1) .OR.
     3                  (QCOL(LLOW+1) .GE. QS(LLOW+1)) )
      ENDIF
C     Adjustment diagnostics
      IADJ= IADJ+1
      IADLEV= IADLEV + (LLOW - LHIGH + 1)
      LHIGH= LLOW
C     End of adjustment
#ifdef DEBUG
      WRITE(*,'(A8,(10F7.2))') 'MCA-T1: ', (TCOL(L),L=1,L1)
      WRITE(*,'(A8,(10F7.3))') 'MCA-Q1: ', (QCOL(L)*1000.0,L=1,L1)
      WRITE(*,'(A8,(10F7.3))') 'MCA-QS: ', (QS(L)*1000.0,L=1,L1)
      WRITE(*,'(A8,(10F7.2))')
     1        'MCA-GS: ', (GAMMAS(L)*(PLV(L+1) - PLV(L)),L=1,L1-1)
      WRITE(*,*)
#endif
C     Scan further down
      GOTO 200
      END

      SUBROUTINE MCADJ2( T, Q, QS, DQSDT, SATFLG, GAMMA1,
     1                   IADJ, IADLEV, NCOLMX, NCOL,
     2                   CIFLG1, PLV, PHLV, DP, SATFAC )
C MCADJ2 performs moist convective adjustment for a set of columns.
C (Fast version of MCADJ for 2-level model)
C Input:  REAL T(NCOLMX,2) --  Temperature field (in K)
C         REAL Q(NCOLMX,2) -- mass mixing ratio of water vapour
C         REAL QS(NCOLMX,2) -- saturation mass mixing ratio
C         REAL DQSDT(NCOLMX,2) -- T-derivative of QS
C         LOGICAL SATFLG(NCOLMX,2) -- saturation flag
C              (Q(J,L) >= QS(J,L))
C         REAL GAMMA1(NCOL) -- moist adiabat at level 1+1/2
C                              i.e. GAMMA1= dT/dp at 1+1/2
C         INTEGER NCOLMX -- inner dimension for column arrays
C         INTEGER NCOL -- no. of columns to be handled
C         LOGICAL CIFLG1(NCOL) -- moist convective instability flag
C           (if CIFLG1(J) is .TRUE., then TCOL(J,2)-TCOL(J,1) exceeds
C            0.01+GAMMA1(J)*(PLV(2)-PLV(1)), and level 2 is
C            saturated)
C         REAL PLV(2) -- full level pressure
C         REAL PHLV(0:2) -- half level pressure values
C         REAL DP(2) -- pressure thickness at model levels
C         REAL SATFAC -- attenuation factor for computation of eS
C                      (usually 0.8; determines saturation threshold)
C Output: REAL T(NCOLMX,2) -- Temperature values, adjusted to be
C          moist convectively stable. The adjustment is carried out so
C          as to conserve total heat content in each column.
C         REAL Q(NCOLMX,2) -- water vapor mass mixing ratio,
C                             after moist convective adjustment
C         REAL QS(NCOLMX,2) -- new saturation mass mixing ratio
C         REAL DQSDT(NCOLMX,2) -- new T-derivative of QS
C         LOGICAL SATFLG(NCOLMX,2) -- new saturation flag
C         REAL GAMMA1(NCOL) -- new moist adiabat at level 1+1/2
C         INTEGER IADJ -- no. of adjustment iterations carried out
C                         (>= 0)
C         INTEGER IADLEV -- total no. of levels adjusted in all
C                            iterations (>=0)
C
      INTEGER IADJ, IADLEV, NCOLMX, NCOL
      REAL T(NCOLMX,2), Q(NCOLMX,2), QS(NCOLMX,2), DQSDT(NCOLMX,2)
      REAL GAMMA1(NCOL)
      REAL PLV(2), PHLV(0:2), DP(2), SATFAC
      LOGICAL SATFLG(NCOLMX,2), CIFLG1(NCOL)
C
      REAL ES, DESDT, THLV, HEAT, CUWT
      REAL TLOW, DELTA1
      INTEGER I, J
#ifdef DEBUG
      INTEGER L
#endif
C
C     Stefan's constant (in W/(m**2*K**4))
      REAL STEFAN
      PARAMETER (STEFAN= 5.669E-8)
C     Molecular weights ofr air, water, ozone (kg/kgmole)
      REAL AIRWT, H2OWT, O3WT
      PARAMETER (AIRWT=28.97, H2OWT=18.016, O3WT=48.)
C     Coefficients for water-air systems
C     (R, Cp for dry air (in J/(kg*K)),
C      latent heat of condendation (in J/kg) )
      REAL RDRY, CPDRY, LHEAT
      PARAMETER (RDRY= 287.0, CPDRY=1004.0, LHEAT=2.5E6)
C     Coefficients for Clausius-Clapeyron relation
      REAL CCFAC0, CCFAC1
      PARAMETER (CCFAC0=26.2612, CCFAC1=5420.3)
C     Derived coefficients
      REAL LBYCP, RBYCP
      PARAMETER (LBYCP=LHEAT/CPDRY, RBYCP=RDRY/CPDRY)
      REAL H2OAIR, O3AIR
      PARAMETER (H2OAIR=H2OWT/AIRWT, O3AIR=O3WT/AIRWT)
C
#ifdef DEBUG
      WRITE(*,'(A8,(10F7.2))') 'MC2-T0: ', (T(JDEBUG,L),L=1,2)
      WRITE(*,'(A8,(10F7.3))') 'MC2-Q0: ', (Q(JDEBUG,L)*1000.0,L=1,2)
#endif
C
C     Beginning of iteration
      IADJ= 0
      IADLEV= 0
C     Iterate moist convective adjustment over unstable regions
      DO 600 I=1,3
        DO 600 J=1,NCOL
          IF (CIFLG1(J)) THEN
C           Integrate the moist adiabat vertically upward
            DELTA1=  GAMMA1(J)*(PLV(2) - PLV(1))
C           Compute total heat content in region (except top of region)
            HEAT=  DP(2)*(  (1.0 + LBYCP*DQSDT(J,2))*T(J,2)
     1                           + LBYCP*(Q(J,2) - QS(J,2)) )
     2           + DP(1)*(T(J,1) + DELTA1)
            CUWT=  DP(2)*(1.0 + LBYCP*DQSDT(J,2)) + DP(1)
C
C           Keep Q fixed in the top level
C           Compute new temperature at bottom of region
            TLOW= HEAT/CUWT
C           Adjust Q, T profiles
            T(J,1)= TLOW - DELTA1
            Q(J,2)= QS(J,2) + DQSDT(J,2)*(TLOW - T(J,2))
            T(J,2)= TLOW
C           Re-compute saturation mixing ratios at level 2
C           Saturation vapor pressure
C           (An attenuation factor of SATFAC is used)
            ES=SATFAC*EXP(CCFAC0-CCFAC1/T(J,2))
C           dES/dT
            DESDT= ES*(CCFAC1/T(J,2)**2)
C           Saturation mixing ratio QS
            QS(J,2)= H2OAIR*ES
     1              /AMAX1(PLV(2), PLV(2) - ES)
C           dQS/dT
            DQSDT(J,2)= H2OAIR*PLV(2)*DESDT
     1                 /AMAX1(PLV(2), PLV(2) - ES)**2
C           Re-compute moist adiabatic lapse rate (at 1+1/2)
C           Half-level temperature
            THLV= 0.5 * (T(J,1) + T(J,2))
C           Saturation vapor pressure for evaporation
            ES= SATFAC*EXP(CCFAC0-CCFAC1/THLV)
C           dES/dT
            DESDT= ES*(CCFAC1/THLV**2)
C           Moist adiabatic lapse rate (at half-levels)
            GAMMA1(J)=  RBYCP*(THLV/PHLV(1))
     1                * (PHLV(1) + H2OAIR*LBYCP*ES/(RBYCP*THLV) )
     2                / (PHLV(1) + H2OAIR*LBYCP*DESDT )
          ENDIF
  600     CONTINUE
C     Set saturation flag in level 2
      DO 610 J=1,NCOL
  610   SATFLG(J,2)= .TRUE.
      DO 620 J=1,NCOL
        IF (CIFLG1(J)) THEN
C         Re-compute saturation mixing ratios at level 1
C         Saturation vapor pressure
C         (An attenuation factor of SATFAC is used)
          ES=SATFAC*EXP(CCFAC0-CCFAC1/T(J,1))
C         dES/dT
          DESDT= ES*(CCFAC1/T(J,1)**2)
C         Saturation mixing ratio QS
          QS(J,1)= H2OAIR*ES
     1            /AMAX1(PLV(1), PLV(1) - ES)
C         dQS/dT
          DQSDT(J,1)= H2OAIR*PLV(1)*DESDT
     1               /AMAX1(PLV(1), PLV(1) - ES)**2
C         Set saturation flag at level 1
          SATFLG(J,1)= Q(J,1) .GT. QS(J,1)
        ENDIF
  620   CONTINUE
C     Adjustment diagnostics
      DO 630 J=1,NCOL
        IF (CIFLG1(J)) THEN
          IADJ= IADJ+1
          IADLEV= IADLEV + 2
        ENDIF
  630   CONTINUE
C     End of adjustment
#ifdef DEBUG
      WRITE(*,'(A8,(10F7.2))') 'MC2-T1: ', (T(JDEBUG,L),L=1,2)
      WRITE(*,'(A8,(10F7.3))') 'MC2-Q1: ', (Q(JDEBUG,L)*1000.0,L=1,2)
      WRITE(*,'(A8,(10F7.3))') 'MC2-QS: ', (QS(JDEBUG,L)*1000.0,L=1,2)
      WRITE(*,'(A8,(10F7.2))')
     1        'MC2-GS: ', GAMMA1(JDEBUG)*(PLV(L+1) - PLV(L))
      WRITE(*,*)
#endif
      RETURN
      END

      SUBROUTINE GREYLW( FPLUS, FMINUS, T, KAPALW, DP,
     1                   NCOLMX, NCOL, L1, FIXST, FBOT, WINLW, FAC )
C GREYLW computes net longwave radiative flux deposited in each level,
C in the "grey" atmosphere approximation.
C
C Input:  REAL T(NCOLMX,L1) -- Temperature field (in K)
C              (T(J,L) corresponds to level L of column J)
C         REAL KAPALW(NCOLMX,L1) -- longwave absorptivity (in 1/Pa)
C         REAL DP(NCOLMX,L1) -- pressure thicknesses of levels (in Pa)
C         INTEGER NCOLMX -- inner dimension for column arrays
C         INTEGER NCOL -- no. of columns to be handled
C         INTEGER L1 -- no. of vertical levels
C         LOGICAL FIXST  -- IF .TRUE., FBOT equals upward longwave
C                           radiative flux at surface ("fixed surface
C                           temperature"). If .FALSE., FBOT equals the
C                           net (upward - downward) longwave radiative
C                           flux at surface.
C         REAL FBOT(NCOL) -- Upward/net surface longwave radiative flux,
C                            for each column
C         REAL WINLW -- longwave window fraction (0 <= WINLW < 1)
C         REAL FAC(NCOLMX,L1,2) -- work array
C Output: REAL FPLUS(NCOLMX,0:L1) --upward radiative flux at half-levels
C         REAL FMINUS(NCOLMX,0:L1) --downward radiative flux
C
      INTEGER NCOLMX, NCOL, L1
      REAL FPLUS(NCOLMX,0:L1), FMINUS(NCOLMX,0:L1)
      REAL T(NCOLMX,L1), KAPALW(NCOLMX,L1), DP(NCOLMX,L1)
      REAL FBOT(NCOL), WINLW, FAC(NCOLMX,L1,2)
      LOGICAL FIXST
C
      INTEGER J, L
C     Stefan's constant (in W/(m**2*K**4))
      REAL STEFAN
      PARAMETER (STEFAN= 5.669E-8)
C
C     Compute LW level attenuation and emission factors
      DO 110 L=1,L1
        DO 110 J=1,NCOL
          FAC(J,L,1)= EXP(-KAPALW(J,L)*DP(J,L))
  110     FAC(J,L,2)= STEFAN*T(J,L)**4
C
C     No downward flux at upper boundary
      DO 210 J=1,NCOL
  210   FMINUS(J,0)= 0.0
C     Compute downward flux in the interior
      DO 220 L=1,L1
        DO 220 J=1,NCOL
          FMINUS(J,L)=  FAC(J,L,1)*(FMINUS(J,L-1) - FAC(J,L,2))
     1                + FAC(J,L,2)
  220     CONTINUE
C
C     Determine upward flux at lower boundary
      IF (FIXST) THEN
C       Fixed surface temperature b.c.
        DO 310 J=1,NCOL
  310     FPLUS(J,L1)= (1.0 - WINLW) * FBOT(J)
      ELSE
C       Fixed surface flux b.c.
        DO 320 J=1,NCOL
  320     FPLUS(J,L1)= (1.0 - WINLW) * (FBOT(J) + FMINUS(J,L1))
      ENDIF
C     Compute upward flux in the interior
      DO 330 L=L1,1,-1
        DO 330 J=1,NCOL
          FPLUS(J,L-1)=  FAC(J,L,1)*(FPLUS(J,L) - FAC(J,L,2))
     1                 + FAC(J,L,2)
  330     CONTINUE
C     Add constant component escaping through the "longwave window"
      IF (FIXST) THEN
C       Fixed surface temperature b.c.
        DO 340 L=0,L1
          DO 340 J=1,NCOL
  340       FPLUS(J,L)= FPLUS(J,L) + WINLW * FBOT(J)
      ELSE
C       Fixed surface flux b.c.
        DO 350 L=0,L1
          DO 350 J=1,NCOL
  350       FPLUS(J,L)= FPLUS(J,L) + WINLW * (FBOT(J) + FMINUS(J,L1))
      ENDIF
#ifdef DEBUG
      WRITE(*,'(A9,(10F7.2))') 'LW-T:',(T(JDEBUG,L),L=1,L1)
      WRITE(*,'(A9,(10F7.3))') 'LW-KAPA:',(KAPALW(JDEBUG,L)*1.E5,L=1,L1)
      WRITE(*,'(A9,(10F7.2))') 'LW-PLUS:',(FPLUS(JDEBUG,L),L=0,L1)
      WRITE(*,'(A9,(10F7.2))') 'LW-MINUS:',(FMINUS(JDEBUG,L),L=0,L1)
#endif
      RETURN
      END

      SUBROUTINE GREYSW( FPLUS, FMINUS, T, KAPASW, DP,
     1                   NCOLMX, NCOL, L1, FTOP, ALBEDO, FAC )
C GREYSW computes net shortwave radiative flux deposited in each level,
C in the cold uniform absorber approximation, with no scattering or
C emission within the atmosphere
C
C Input:  REAL T(NCOLMX,L1) -- Temperature field (in K)
C              (T(J,L) corresponds to level L of column J)
C         REAL KAPASW(NCOLMX,L1) -- shortwave absorptivity (in 1/Pa)
C         REAL DP(NCOLMX,L1) -- pressure thicknesses of levels (in Pa)
C         INTEGER NCOLMX -- inner dimension for column arrays
C         INTEGER NCOL -- no. of columns to be handled
C         INTEGER L1 -- no. of vertical levels
C         REAL FTOP(NCOL) -- downward shortwave flux at the top
C                            of each column
C         REAL ALBEDO(NCOL) -- surface albedo for each column
C         REAL FAC(NCOLMX,L1) -- work array
C Output: REAL FPLUS(NCOLMX,0:L1) --upward radiative flux at half-levels
C         REAL FMINUS(NCOLMX,0:L1) --downward radiative flux
C
      INTEGER NCOLMX, NCOL, L1
      REAL FPLUS(NCOLMX,0:L1), FMINUS(NCOLMX,0:L1)
      REAL T(NCOLMX,L1), KAPASW(NCOLMX,L1), DP(NCOLMX,L1)
      REAL FTOP(NCOL), ALBEDO(NCOL), FAC(NCOLMX,L1)
C
      INTEGER J, L
C
C     Compute SW level attenuation factor
      DO 110 L=1,L1
        DO 110 J=1,NCOL
  110     FAC(J,L)= EXP(-KAPASW(J,L)*DP(J,L))
C
C     Initialize downward flux at upper boundary
      DO 210 J=1,NCOL
  210   FMINUS(J,0)= FTOP(J)
C     Compute downward flux in the interior
      DO 220 L=1,L1
        DO 220 J=1,NCOL
          FMINUS(J,L)= FAC(J,L) * FMINUS(J,L-1)
  220     CONTINUE
C
C     Determine upward flux at lower boundary (partial reflection)
      DO 310 J=1,NCOL
  310   FPLUS(J,L1)= ALBEDO(J) * FMINUS(J,L1)
C     Compute upward flux in the interior
      DO 320 L=L1,1,-1
        DO 320 J=1,NCOL
          FPLUS(J,L-1)= FAC(J,L) * FPLUS(J,L)
  320     CONTINUE
#ifdef DEBUG
      WRITE(*,'(A9,(10F7.3))') 'SW-KAPA:',(KAPASW(JDEBUG,L)*1.E5,L=1,L1)
      WRITE(*,'(A9,(10F7.2))') 'SW-PLUS:',(FPLUS(JDEBUG,L),L=0,L1)
      WRITE(*,'(A9,(10F7.2))') 'SW-MINUS:',(FMINUS(JDEBUG,L),L=0,L1)
#endif
      RETURN
      END

      SUBROUTINE DIFFUS( DIFTEN, Q, PLV, DP, NCOLMX, NCOL, L1,
     1                   DIFFC, FBOT, FTOP, GRAVIT, F )
C DIFFUS computes diffusive tendencies in the vertical.
C
C Input:  REAL Q(NCOLMX,L1) -- Quantity to be diffused
C              (Q(J,L) corresponds to level L of column J)
C         REAL PLV(NCOLMX,L1) -- full level pressure (in Pa)
C         REAL DP(NCOLMX,L1) -- pressure thicknesses of levels (in Pa)
C         INTEGER NCOLMX -- inner dimension for column arrays
C         INTEGER NCOL -- no. of columns to be handled
C         INTEGER L1 -- no. of vertical levels
C         REAL DIFFC(L1) -- pressure-dependent diffusivity
C                           at level L+1/2 (in Pa**2/s)
C         REAL FBOT(NCOL) -- inward flux of Q at the bottom
C                            (in kg/(s*m**2))
C         REAL FTOP(NCOL) -- inward flux of Q at the top
C                            (in kg/(s*m**2))
C         REAL GRAVIT -- Gravitational acceleration at surface (m/s**2)
C         REAL F(NCOLMX,L1-1) -- work array
C Output: REAL DIFTEN(NCOLMX,L1) -- diffusive tendency of Q
C
      INTEGER NCOLMX, NCOL, L1
      REAL DIFTEN(NCOLMX,L1)
      REAL Q(NCOLMX,L1), PLV(NCOLMX,L1), DP(NCOLMX,L1), DIFFC(L1)
      REAL FBOT(NCOL), FTOP(NCOL), GRAVIT, F(NCOLMX,L1-1)
C
      INTEGER J, L
C
C     Compute diffusive flux of Q at level L+1/2
      DO 110 L=1,L1-1
        DO 110 J=1,NCOL
  110     F(J,L)= -DIFFC(L)*(Q(J,L+1) - Q(J,L))/(PLV(J,L+1) - PLV(J,L))
C     Compute diffusive tendency at level L
      DO 120 L=2,L1-1
        DO 120 J=1,NCOL
  120     DIFTEN(J,L)= -(F(J,L) - F(J,L-1)) / DP(J,L)
C     Handle boundaries separately
      DO 130 J=1,NCOL
        DIFTEN(J,1)=  -(F(J,1) - FTOP(J)*GRAVIT) / DP(J,1)
        DIFTEN(J,L1)= -(-FBOT(J)*GRAVIT - F(J,L1-1)) / DP(J,L1)
  130   CONTINUE
#ifdef DEBUG
      WRITE(*,'(A9,(10F7.2))') 'DIFFC:',(DIFFC(L),L=1,L1)
      WRITE(*,'(A9,1P,(5E14.5))')
     1 'DIFTEN:',(DIFTEN(JDEBUG,L)*8.64E7,L=1,L1)
      WRITE(*,'(A9,1P,(5E14.5))') 'F:',(F(JDEBUG,L)*1000.,L=1,L1-1)
      WRITE(*,*) 'FBOT, FTOP:',FBOT(JDEBUG)*GRAVIT,FTOP(JDEBUG)*GRAVIT
#endif
      RETURN
      END
 
      REAL FUNCTION AVGSOL( YRDAY, SINLAT, SOLCON )
C Returns zonally-averaged incoming solar radiation
C (North & Coakley, JAS 1979)
C Input:  REAL YRDAY -- fractional time of the year
C         (0.0 => 22 December, ranges from 0.0 to 1.0)
C         REAL SINLAT -- sine of the latitude (-1 .. +1)
C         REAL SOLCON -- solar constant (W/m**2)
C Output: Zonally-averaged incoming solar radiation
C         for that day and latitude (in W/m**2)
C
      REAL YRDAY, SINLAT, SOLCON
C
      REAL PI
      PARAMETER (PI=3.141592653589793)
C
      AVGSOL= 0.25*SOLCON*AMAX1(0.0,
     1         1.0 - 0.796*COS(2.0*PI*YRDAY)*SINLAT
     2             + (0.147*COS(4.0*PI*YRDAY) - 0.477)
     3               *0.5*(3.0*SINLAT**2 - 1.0) )
      RETURN
      END

      REAL FUNCTION AVGALB( YRDAY, SINLAT )
C Returns zonally and diurnally averaged symmetrized
C Northern Hemisphere climatological albedo values
C (North & Coakley, JAS 1979)
C Input:  REAL YRDAY -- fractional time of the year
C         (0.0 => 22 December, ranges from 0.0 to 1.0)
C         REAL SINLAT -- sine of the latitude (-1 .. +1)
C Output: Zonally-averaged albedo for that day and latitude
C         (as a fraction of unity)
C
      REAL YRDAY, SINLAT
C
      REAL PI
      PARAMETER (PI=3.141592653589793)
C
      AVGALB=   0.32
     1        + 0.09*COS(2.0*PI*(YRDAY-0.104))*SINLAT
     2        + 0.20*0.5*(3.0*SINLAT**2 - 1.0)
      RETURN
      END

      REAL FUNCTION AVGSST( YRDAY, SINLAT )
C Returns zonally and diurnally averaged symmetrized
C Northern Hemisphere climatological surface temperature values
C (North & Coakley, JAS 1979)
C Input:  REAL YRDAY -- fractional time of the year
C         (0.0 => 22 December, ranges from 0.0 to 1.0)
C         REAL SINLAT -- sine of the latitude (-1 .. +1)
C Output: Zonally-averaged surface temperature
C         for that day and latitude (in K)
C
      REAL YRDAY, SINLAT
C
      REAL PI
      PARAMETER (PI=3.141592653589793)
C
      AVGSST=  288.0
     1       - 15.5*COS(2.0*PI*(YRDAY-0.088))*SINLAT
     2       - 28.0*0.5*(3.0*SINLAT**2 - 1.0)
      RETURN
      END

C --------------------------------------------------------------------
C For GNU Emacs:
C $Msg-digest-checksum: 119c2274dba9a163465f92d33cd9765c $
C --------------------------------------------------------------------
C Local Variables:
C msg-digest-active: t
C End:
C --------------------------------------------------------------------
