## Makefile for ICAM (Idealized Coupleable Atmospheric Model)
## Created 11 Sep 1996
##
## Author: R. Saravanan, Climate and Global Dynamics Division
##                       National Center for Atmospheric Research
##                       Boulder, CO 80307-3000, U.S.A.
##                       E-mail: svn@ncar.ucar.edu
##                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
##
#
#  NOTES
#
# 1. The environment variable ARCH, denoting machine/operating system
#   type, has to be defined before this makefile can be used.
#   Recognized values for ARCH are SUN4, SUN4SOL2, CRAY, SGI, RS6K
#   denoting SunOS, Sun/Solaris, Cray/UNICOS, Silicon Graphics/IRIX,
#   and IBM/RS6000 respectively.
#
#   (**NOTE** For IBM/RS6000, see the make include file RS6K.mk for special
#             instructions)
#
# 2. Main object modules:
#
#   a: acouple_atm_A.o - "atmospheric" version of coupling module acouple.F
#   b: acouple_ocn_A.o - "oceanic" version of coupling module acouple.F
#   c: packet_A.o      - packet communications module packet.F
#   d: libatop_A.a     - top-level ICAM library
#   e: libamid_A.a     - mid-level ICAM library
#   f: libalow.a     - low-level ICAM library
#   g: liblinpk.a  - LINPACK linear algebra package (OPTIONAL)
#
#   (The suffix "_A" denotes the model configuration;
#    it would be different for different configurations)
#
# 3. Creating object modules:
#
#      To create the ".o" files, type
#
#             make acouple_atm_A.o acouple_ocn_A.o packet_A.o
#
#      To create the top/mid libraries, type
#
#             make libatop_A.a libamid_A.a
#
#      To create the low-level library, and the optional LINPACK library,
#      cd to the "low" subdirectory and type
#
#             make libalow.a [liblinpk.a]
#
#      (The LINPACK library would not be needed the CRAY)
#
# 4. Coupling options: There are two coupling options for using ICAM:
#
#   A: The "trivial coupling" option
#      The atmospheric model and ocean model are part of the same UNIX process,
#      i.e., the atmospheric model is simply a set of subroutines
#      called by the ocean model. These subroutines can be found in the
#      coupling module acouple.F. There are only three of them:
#       ATMBGN--to begin the integration;
#       ATMINT--to integrate the atmosphere through a synchronization interval;
#       ATMEND--to terminate the integration.
#
#      The ocean model, which would communicate with the atmosphere through
#      these subroutines, needs to be linked to the following files:
#
#             acouple_atm_A.o packet_A.o -latop_A -lamid_A -lalow ${LINPACK}
#
#   B: The "packet exchange" option
#      In this case, the atmospheric model and the ocean model are two
#      separate UNIX processes--possibly even on different machines--
#      that communicate with each other by exchanging packets of data
#      containing the SST and the surface flux.
#
#      The ocean model would call the subroutines ATMBGN/ATMINT/ATMEND
#      just as in option A, but it needs to be be linked only to the following
#      two files:
#
#             acouple_ocn.o packet.o
#
#      The driver routine for the atmospheric model would call only
#      the subroutines ATMBGN/ATMEND from acouple.F, just like the ocean
#      model (with the same arguments), but instead of calling ATMINT, it
#      should call subroutine ATMRUN of module atmos.F. The driver
#      routine for the atmospheric model should be linked to the following
#      files, just as in option A:
#
#             acouple_atm.o packet.o -latop_A -lamid_A -lalow ${LINPACK}
#
#     (atmsep.F contains an example of such an atmospheric driver routine)
#
# 5. Test programs
#
#  a: In subdirectory "ocn", "make coam" creates a test coupled
#     ocean-atmosphere model
#
#  b: In subdirectory "atm", "make atmsep" creates a test atmospheric driver
#     program
#
#==============================================================================
## CONTENTS of makefile
#
# Part 1: Read machine dependent parameters from include file
#      2: Set switches that determine the model configuration
#      3: Define model parameters
#       a. Atmospheric vertical resolution (Pressure levels)
#       b. Atmospheric horizontal resolution (Triangular truncation)
#       c. Oceanic resolution parameters
#       d. Set diagnostic switches for saving model data
#       e. Other parameters
#      4: Define compilation flags
#      5: Compilation of coupling interface routines
#      6: Compilation of libraries
#       a: Top-level library
#       b: Mid-level library
#      7: Implicit rules for compiling object files and creating libraries
#
#==============================================================================
## Part 1: Read machine dependent parameters from include file

include ${ARCH}.mk

## The following parameters should be defined in the include file:
## FC     = Fortran compile command
## CPPOPT = default C preprocessor options
## FCOPT  = default Fortran compiler options
## FCHEK  = default Fortran checking options
## AR     = library archive command
## ARFLAGS= library archive flags
## LINPACK= LINPACK linear algebra package option
## RM     = "rm" command

#==============================================================================
## Part 2: Set switches that determine the model configuration

## Atmospheric model configuration name
##

MC= B

# B:  T31/2-level/2-basin "Atlantic/Pacific" configuration
#     (with double precision communication with the ocean)
#
# => MSWITCH= -DDIAGNOS -DDOUBLEPREC
# => L1MAX=2
# => N1MAX=31, M1MAX=31, K1MAX=96, K2MAX=48, JBMAX=32, KBMAX=K2MAX
# => MAXBAS=2, JOMAX=16, KOMAX=48, NFOCN=4
# => NTRACE=1, MXSAMP=7201

##
## Set make parameter MSWITCH to reflect compile command-line model switches
## e.g. MSWITCH= -DDEBUG
##

MSWITCH= -DDIAGNOS -DDOUBLEPREC

# List of allowed model switches (i.e. C preprocessor options):

# ZONAVG         If defined, the ocean model is assumed to be
#                zonally-averaged, i.e., a meridional plane model

# DIAGNOS        If defined, the diagnostics module is linked,
#                allowing horizontal section saves etc.

# DOUBLEPREC     If defined, the ocean model is assumed to be running with
#                twice the floating-point precision of the atmospheric model

# PVM_LINK       If defined, PVM routines are linked
#                (for packet communications)

# SLAB_OCEAN     If defined, couple atmosphere to an internal slab ocean
#                model, rather than to an external ocean model.

# DEBUG          If defined, a *lot* of debugging information is displayed

# OLD_VERSION    For compatibility with old version (COAM, version 2.0.*.*)


#==============================================================================
## Part 3: Define model parameters

#------------------------------------------------------------------------------
#  Part 3a. Atmospheric vertical resolution (Pressure levels)

L1MAX=2
                                             # Max no. of model levels

AVRES= -DL1MAX=${L1MAX}

#------------------------------------------------------------------------------
#  Part 3b. Atmospheric horizontal resolution (Triangular truncation)

N1MAX=31
                                             # Max meridional "wavenumber"
M1MAX=${N1MAX}
                                             # Max zonal wavenumber
K1MAX=96
                                             # Max no. of longitudes
K2MAX=48
                                             # Max no. of gaussian latitudes
KL0MAX=${K2MAX}
                                             # Max no. of latitudes/levels
JBMAX=32
                                             # Max. no of ocean points on
                                             # atmospheric X-grid (per basin)
KBMAX=${K2MAX}
                                             # Max. no of ocean points on
                                             # atmospheric Y-grid (per basin)

AHRES= -DN1MAX=${N1MAX} -DM1MAX=${M1MAX} -DK1MAX=${K1MAX} -DK2MAX=${K2MAX} \
       -DKL0MAX=${KL0MAX} -DJBMAX=${JBMAX} -DKBMAX=${KBMAX}

#------------------------------------------------------------------------------
#  Part 3c. Oceanic resolution parameters

MAXBAS=2
                                             # Max no. of ocean basins
JOMAX=16
                                             # Max. no temperature points on
                                             # oceanic X-grid
KOMAX=48
                                             # Max. no of temperature points on
                                             # oceanic Y-grid
NFOCN=4
                                             # No. of flux fields passed
                                             # from atmosphere to ocean

ORES= -DMAXBAS=${MAXBAS} -DJOMAX=${JOMAX} -DKOMAX=${KOMAX} -DNFOCN=${NFOCN}

#------------------------------------------------------------------------------
#  Part 3d. Set diagnostic switches to determine which fields are saved
#           as horizontal sections to a direct access file

DSWITCH= -DSAVE_FLX_BOT -DSAVE_FLX_TOP -DSAVE_TUV -DSAVE_Q

# List of allowed save switches (i.e. C preprocessor options):
#
# SAVE_FLX_BOT   If defined, save surface fluxes (TSURF, PRECIP, EVAP, FSURF)
#
# SAVE_FLX_TOP   If defined, save top fluxes (OLR, ASR)
#
# SAVE_TUV       If defined, save T, U, V at each level
#
# SAVE_Q         If defined, save Q at each level
#
# SAVE_EDDY_HI   If defined, save upper level eddy statistics
#                            (TEKE, PSIFR1)
#
# SAVE_EDDY_LO   If defined, save lower level eddy statistics
#                            (VT1, DIVUT1, DIVUQ1. PSIFR1)

#------------------------------------------------------------------------------
#  Part 3e. Other parameters

NTRACE=1
                                             # No. of tracers (incl. moisture)

MXSAMP=7201
                                             # Max no. of diagnostic samples

OTHER= -DNEVMAX=${MXSAMP} -DNTRACE=${NTRACE}


#==============================================================================
## Part 4: Define compilation flags

# CPP flags (default options, configuration switches, and all parameters)
CPPFLAGS= ${CPPOPT} ${MSWITCH} ${AHRES} ${AVRES} ${ORES} ${DSWITCH} ${OTHER}
          
# Fortran compiler flags (as defined in the include file)
FFLAGS= ${FCOPT}
          
#==============================================================================
## Part 5: Compilation of coupling interface routines

# Coupling interface for the atmospheric model

acouple_atm_${MC}.o: acouple.F mcouple.h
	${FC} $(FFLAGS) $(FCHEK) $(CPPFLAGS) -c acouple.F
	/bin/mv acouple.o acouple_atm_${MC}.o


# Coupling interface for use with a separate ocean model process

acouple_ocn_${MC}.o: acouple.F mcouple.h
	${FC} $(FFLAGS) $(FCHEK) $(CPPFLAGS) -DOCEAN_ONLY -c acouple.F
	/bin/mv acouple.o acouple_ocn_${MC}.o


# Packet communication routines

packet_${MC}.o: packet.F mpacket.h
	${FC} $(FFLAGS) $(FCHEK) $(CPPFLAGS) -c packet.F
	/bin/mv packet.o packet_${MC}.o

#==============================================================================
## Part 6: Compilation of libraries

#------------------------------------------------------------------------------
#  Part 6a: Top-level library "libatop_A.a"
# (Routines that are modified often, or call lots of other routines)

LTOP=libatop_${MC}.a

${LTOP}: ${LTOP}(atmos.o)   ${LTOP}(aparams.o)  \
         ${LTOP}(astate.o)  ${LTOP}(amarch.o)   \
         ${LTOP}(aflux.o)
.PRECIOUS: ${LTOP}

atmos.F: mmain.h mgrid.h mparams.h mstate.h mphys.h
                                         # Driver for initializing/running
                                         # idealized atmospheric model

aparams.F: mmain.h mgrid.h mparams.h mstate.h mcoef.h
                                         # Parameter initialization routines

astate.F: mmain.h mgrid.h mparams.h mstate.h
                                         # Model save file routines

amarch.F: mmain.h mgrid.h mparams.h mstate.h mcoef.h
                                         # Time-marching routines

aflux.F: mmain.h mgrid.h mparams.h mflux.h minterp.h
                                         # Flux/SST exchange routines

#------------------------------------------------------------------------------
#  Part 6b: Mid-level library "libamid_A.a"
# (Routines that are not likely to be modified often;
#  may be resolution dependent)

LMID=libamid_${MC}.a

${LMID}: ${LMID}(physic.o)   ${LMID}(prognos.o) \
         ${LMID}(splib.o)    ${LMID}(diagnos.o)
.PRECIOUS: ${LMID}
            

# (Module diagnos.o is optional, and may be left out if option DIAGNOS is
#  not defined)

physic.F: mcons.h mgrid.h mphys.h
                                         # Physics interface with SIMPHYS

prognos.F: mcons.h mgrid.h tmarch.h spgrid.h
                                         # Prognostic module

splib.F: spcons.h spgrid.h sppoly.h spfftb.h
                                         # Spectral transforms module

diagnos.F: mcons.h mgrid.h tmarch.h diacom.h spgrid.h
                                         # Diagnostic module (OPTIONAL)

#==============================================================================
## Part 7: Implicit rules for compiling object files and creating libraries

# Recognize only suffixes ".F", ".f" and ".o"

.SUFFIXES:
.SUFFIXES: .F .f .o

# Use default Fortran flags (FFLAGS) + checking + CPP options for ".F" files

.F.o:
	$(FC) $(FFLAGS) $(FCHEK) $(CPPFLAGS) -c $<

.F.a:
	$(FC) $(FFLAGS) $(FCHEK) $(CPPFLAGS) -c $<
	$(AR) $(ARFLAGS) $@ $*.o
	$(RM) $*.o

# Use only default Fortran flags (FFLAGS) options for ".f" files

.f.o:
	$(FC) $(FFLAGS) -c $<

.f.a:
	$(FC) $(FFLAGS) -c $<
	$(AR) $(ARFLAGS) $@ $*.o
	$(RM) $*.o

# --------------------------------------------------------------------
# For GNU Emacs:
# $Msg-digest-checksum: 31a20df57de1df7d2fc342f715585721 $
# --------------------------------------------------------------------
# Local Variables:
# msg-digest-active: t
# End:
# --------------------------------------------------------------------
