#ifndef OCONS
#   include "ocons.h"
#endif
C
C Routines for two-dimensional (meridional plane) ocean model
C [The formulation is described in Thual and McWilliams, 1992
C  and Saravanan and McWilliams, 1995]
C
C Version 7.3 created 19 Apr 96
C
      SUBROUTINE ARAKWA(JACOB, Q, PSI, NX0, NY0, NX, NY, NQ, DX, DY)
C Compute the Arakawa Jacobian for advection by 2-D nondivergent flow.
C The grid is assumed to be rectangular, with uniform grid spacing.
C It is also assumed that there is no normal flow across boundaries,
C i.e., the PSI field should have all the boundary values set
C identically equal to zero.
C Reference:
C   Arakawa, A., 1966: Computational design for long-term numerical
C     integration of the equations of fluid motion: two-dimensional
C     incompressible flow. Part I, J. Computational Phys., v.1,
C     pp 119-143.
C
C Input:  REAL Q(0:NX0, 0:NY0, NQ) -- scalar quantities to be advected
C         REAL PSI(0:NX0, 0:NY) -- streamfunction (no normal flow)
C              (PSI(0,*)=PSI(NX,*)=PSI(*,0)=PSI(*,NY)=0)
C         INTEGER NX0, NY0 -- declared upper bounds of arrays
C         INTEGER NX, NY -- operative upper bounds of arrays
C         INTEGER NQ -- no. of scalars to be advected
C         REAL DX, DY -- grid spacing in X, Y directions
C Output: REAL JACOB(0:NX0, 0:NY0, NQ) -- advective time-tendencies
C                                         for the scalars (J(Q,PSI))
C
      INTEGER NX0, NY0, NX, NY, NQ
      REAL JACOB(0:NX0, 0:NY0, NQ), Q(0:NX0, 0:NY0, NQ)
      REAL PSI(0:NX0, 0:NY), DX, DY
C
      INTEGER I, J, K
      REAL U(0:NXMAX, 0:NYMAX), V(0:NXMAX, 0:NYMAX), TEMP
      REAL EFLUX(-1:NXMAX, 0:NYMAX), NFLUX(0:NXMAX, -1:NYMAX)
      REAL NEFLUX(-1:NXMAX, -1:NYMAX), NWFLUX(-1:NXMAX, -1:NYMAX)
C
      IF ((NX .GT. NXMAX) .OR. (NY .GT. NYMAX)) STOP 'ARAKWA'
C
C     Velocities in the interior (centred difference)
      DO 110 J=1,NY-1
        DO 110 I=1,NX-1
          U(I,J)= PSI(I,   J-1) - PSI(I,   J+1)
          V(I,J)= PSI(I+1, J  ) - PSI(I-1, J  )
  110     CONTINUE
C     Velocities at Y boundaries (no normal flow)
      DO 120 I=0,NX
        U(I,0 )= PSI(I, 0   ) - PSI(I, 1 )
        U(I,NY)= PSI(I, NY-1) - PSI(I, NY)
        V(I,0 )= 0.0
        V(I,NY)= 0.0
  120   CONTINUE
C     Velocities at X boundaries (no normal flow)
      DO 130 J=1,NY-1
        U(0, J)= 0.0
        U(NX,J)= 0.0
        V(0, J)= PSI(1,  J) - PSI(0,    J)
        V(NX,J)= PSI(NX, J) - PSI(NX-1, J)
  130   CONTINUE
C
C     No northward flux at Y boundaries (J=-1/2,NY+1/2)
      DO 140 I=0,NX
        NFLUX(I,-1)= 0.0
        NFLUX(I,NY)= 0.0
  140   CONTINUE
C     No eastward flux at X boundaries (I=-1/2,NX+1/2)
      DO 150 J=0,NY
         EFLUX(-1,J)= 0.0
         EFLUX(NX,J)= 0.0
  150   CONTINUE
C     No diagonal fluxes at X/Y boundaries
C     (I=-1/2,NX+1/2, J=-1/2,NY+1/2)
      DO 160 I=-1,NX
        NEFLUX(I,-1)= 0.0
        NEFLUX(I,NY)= 0.0
        NWFLUX(I,-1)= 0.0
        NWFLUX(I,NY)= 0.0
  160   CONTINUE
      DO 170 J=0,NY-1
        NEFLUX(-1,J)= 0.0
        NEFLUX(NX,J)= 0.0
        NWFLUX(-1,J)= 0.0
        NWFLUX(NX,J)= 0.0
  170   CONTINUE
C     For each advected scalar K
      DO 300 K=1,NQ
C       Compute interior fluxes
C       Eastward flux (at I+1/2,J)
        DO 210 J=0,NY
          DO 210 I=0,NX-1
            EFLUX(I,J)=  ( U(I,J)   + U(I+1,J)   )
     1                 * ( Q(I,J,K) + Q(I+1,J,K) )
  210       CONTINUE
C       Northward flux (at I,J+1/2)
        DO 220 J=0,NY-1
          DO 220 I=0,NX
            NFLUX(I,J)=  ( V(I,J)   + V(I,J+1)   )
     1                 * ( Q(I,J,K) + Q(I,J+1,K) )
  220       CONTINUE
C       Diagonal fluxes (at I+1/2, J+1/2)
        DO 230 J=0,NY-1
          DO 230 I=0,NX-1
C           Northeastward flux
            NEFLUX(I,J)=  ( PSI(I+1,J)   - PSI(I,J+1)     )
     1                  * (   Q(I,J,K)   +   Q(I+1,J+1,K) )
C           Northwestward flux
            NWFLUX(I,J)=  ( PSI(I+1,J+1) - PSI(I,J)     )
     1                  * (   Q(I+1,J,K) +   Q(I,J+1,K) )
  230       CONTINUE
C       Compute the Arakawa Jacobian (flux form)
        TEMP= -1.0/(12.0*DX*DY)
        DO 240 J=0,NY
          DO 240 I=0,NX
            JACOB(I,J,K)= TEMP*(   EFLUX(I,J)   -  EFLUX(I-1,J)
     1                          +  NFLUX(I,J)   -  NFLUX(I,J-1)
     2                          + NEFLUX(I,J)   - NEFLUX(I-1,J-1)
     3                          + NWFLUX(I-1,J) - NWFLUX(I,J-1)   )
  240       CONTINUE
  300   CONTINUE
      RETURN
      END
      SUBROUTINE BUOYAN( ZTEND, T, S, NX0, NX, NY, DX, TCOEF, SCOEF )
C Given the temperature and salinity fields, compute the vorticity
C tendency due to buoyancy, and add it to the other tendencies.
C The grid is assumed to be rectangular, with uniform grid spacing.
C
C Input:  REAL ZTEND(0:NX0, 0:NY) -- other tendency of vorticity
C         REAL T(0:NX0, 0:NY) -- Temperature field
C         REAL S(0:NX0, 0:NY) -- Salinity field
C         INTEGER NX0 -- declared upper bound
C         INTEGER NX, NY -- operative upper bounds
C         REAL DX -- grid spacing in the X direction
C         REAL TCOEF -- buoyancy gradient coefficient for T
C         REAL SCOEF -- buoyancy gradient coefficient for S
C Output: REAL ZTEND(0:NX0, 0:NY) --buoyancy+other tendency of vorticity
C
      INTEGER NX0, NX, NY
      REAL ZTEND(0:NX0, 0:NY), T(0:NX0, 0:NY), S(0:NX0, 0:NY)
      REAL DX, TCOEF, SCOEF
C
      INTEGER I, J
      REAL TCOEF0, SCOEF0
C
C     Scaled coefficients
      TCOEF0= TCOEF/(2.0*DX)
      SCOEF0= SCOEF/(2.0*DX)
C
C     Accumulate interior buoyancy tendency in X direction
      DO 110 J=0,NY
        DO 110 I=1,NX-1
          ZTEND(I,J)=  ZTEND(I,J) + TCOEF0*(T(I+1,J) - T(I-1,J))
     1                            + SCOEF0*(S(I+1,J) - S(I-1,J))
  110     CONTINUE
C     Accumulate boundary buoyancy tendency in the X direction
      DO 120 J=0,NY
        ZTEND(0, J)=  ZTEND(0, J) + 2.0*TCOEF0*(T(1, J) - T(0,   J))
     1                            + 2.0*SCOEF0*(S(1, J) - S(0,   J))
        ZTEND(NX,J)=  ZTEND(NX,J) + 2.0*TCOEF0*(T(NX,J) - T(NX-1,J))
     1                            + 2.0*SCOEF0*(S(NX,J) - S(NX-1,J))
  120   CONTINUE
C
      RETURN
      END
      SUBROUTINE DIFUSE( QTEND, Q, NX0, NX, NY, DX, DY,
     1                   DIFFX, DIFFY, TOPFLX)
C Given a field Q, and diffusive flux of Q at the "top",
C compute the diffusive tendency and add it to the adiabatic tendency.
C The grid is assumed to be rectangular, with uniform grid spacing.
C
C Input:  REAL QTEND(0:NX0, 0:NY) -- adiabatic tendency of Q
C         REAL Q(0:NX0, 0:NY) -- scalar field
C         INTEGER NX0 -- declared upper bound
C         INTEGER NX, NY -- operative upper bounds
C         REAL DIFFX, DIFFY -- diffusion coefficient in X/Y directions
C         REAL TOPFLX(0:NX) -- diffusive flux at the "top" (y=ymax)
C         REAL DX, DY -- grid spacing in X, Y directions
C Output: REAL QTEND(0:NX0, 0:NY) -- adiabatic+diffusive tendency of Q
C
      INTEGER NX0, NX, NY
      REAL QTEND(0:NX0, 0:NY), Q(0:NX0, 0:NY), DX, DY
      REAL DIFFX, DIFFY, TOPFLX(0:NX)
C
      INTEGER I, J
      REAL DIFFX0, DIFFY0
C
C     Scaled diffusion coefficients
      DIFFX0= DIFFX/DX**2
      DIFFY0= DIFFY/DY**2
C     Accumulate interior diffusion in X direction
      DO 110 J=0,NY
        DO 110 I=1,NX-1
          QTEND(I,J)=  QTEND(I,J)
     1               + DIFFX0*(Q(I+1,J) - 2.0*Q(I,J) + Q(I-1,J))
  110     CONTINUE
C     Accumulate boundary diffusion in the X direction (free slip)
      DO 120 J=0,NY
        QTEND(0, J)= QTEND(0, J) + DIFFX0*(Q(1, J) - Q(0,   J))
        QTEND(NX,J)= QTEND(NX,J) - DIFFX0*(Q(NX,J) - Q(NX-1,J))
  120   CONTINUE
C
C     Accumulate interior diffusion in the Y direction
      DO 210 J=1,NY-1
        DO 210 I=0,NX
          QTEND(I,J)=  QTEND(I,J)
     1               + DIFFY0*(Q(I,J+1) - 2.0*Q(I,J) + Q(I,J-1))
  210     CONTINUE
C     Accumulate boundary diffusion in the Y direction
      DO 220 I=0,NX
C       Free slip at bottom
        QTEND(I, 0)= QTEND(I, 0) + DIFFY0*(Q(I, 1) - Q(I,   0))
C       Specified flux at top
        QTEND(I,NY)= QTEND(I,NY) - DIFFY0*(Q(I,NY) - Q(I,NY-1))
     1                           + TOPFLX(I)/DY
  220   CONTINUE
C
      RETURN
      END

#ifdef DPRECISION
#   define REAL8 REAL
#else
#   define REAL8 REAL*8
#endif

      SUBROUTINE INVLP0(WCOEF, WFST, NX, NY, DX, DY)
C Compute the coefficient matrices used by subroutine INVLAP.
C Input:  INTEGER NX, NY -- X/Y dimension of grid
C         REAL DX, DY -- grid spacing in X, Y directions
C         (DX=0 => No variation of the function in the X-direction)
C Output: REAL8 WCOEF(NX-1, NY-1) -- coefficient array for solving
C                                    tridiagonal system
C         REAL WFST(5*(NX/2)+15) -- coefficient array for
C                                   fast sine transform
C
      INTEGER NX, NY
      REAL8 WCOEF(NX-1, NY-1)
      REAL WFST(5*(NX/2)+15), DX, DY
C
      INTEGER J, K
      REAL8 LAMBDA(NXMAX-1)
      REAL PI
      PARAMETER (PI=3.141592653589793D0)
C
      IF ((NX .GT. NXMAX) .OR. (NY .GT. NYMAX)) STOP 'INVLP0'
C     Compute coefficients for vectorized sine-transform
      CALL VSINTI(NX-1, WFST)
      IF (DX .GT. 0.) THEN
C       Compute diagonal coefficients
        DO 10 K=1,NX-1
          LAMBDA(K)=  DBLE((COS(K*PI/FLOAT(NX)) - 1.0)*2.0*(DY/DX)**2)
     1              - 2.0D0
   10     WCOEF(K,1)= 1.0D0 / LAMBDA(K)
      ELSE
C       No horizontal variation
        DO 20 K=1,NX-1
          LAMBDA(K)= - 2.0D0
   20     WCOEF(K,1)= 1.0D0 / LAMBDA(K)
      ENDIF
C     Compute coefficients for tri-diagonal matrix solution
      DO 30 J=2,NY-1
        DO 30 K=1,NX-1
   30     WCOEF(K,J)= 1.0D0/(LAMBDA(K) - WCOEF(K,J-1))
      RETURN
      END
C
      SUBROUTINE INVLAP(PSI, ZETA, NX0, NX, NY, DX, DY, WCOEF, WFST)
C Given the vorticity, compute streamfunction for 2-D nondivergent flow.
C The grid is assumed to be rectangular, with uniform grid spacing.
C It is also assumed that there is no normal flow across boundaries,
C i.e., the PSI field has all the boundary values set identically equal
C to zero.
C Reference:
C   Temperton, C., 1979: Direct methods for the solution of the
C     discrete Poisson equation: some comparisons, J. Computational
C     Phys., v.31, pp 1-20.
C
C Note: Before calling this routine, the coefficient matrices WCOEF and
C       WFST should be precomputed by calling routine INVLP0.
C Input:  REAL ZETA(0:NX0, 0:NY) -- vorticity field
C         INTEGER NX0 -- declared upper bound
C         INTEGER NX, NY -- operative upper bounds
C         REAL DX, DY -- grid spacing in X, Y directions
C         (DX=0 => No variation of the function in the X-direction)
C         REAL8 WCOEF(NX-1, NY-1) -- solution coefficient matrix
C         REAL WFST(5*(NX/2)+15)-- fast sine transform coeff. matrix
C Output: REAL PSI(0:NX0, 0:NY) -- streamfunction (no normal flow)
C              (PSI(0,*)=PSI(NX,*)=PSI(*,0)=PSI(*,NY)=0)
C
      INTEGER NX0, NX, NY
      REAL PSI(0:NX0, 0:NY), ZETA(0:NX0, 0:NY), DX, DY
      REAL8 WCOEF(NX-1, NY-1)
      REAL WFST(5*(NX/2)+15)
C
      INTEGER I, J, K
      REAL BTRAN(NYMAX-1,NXMAX)
      REAL8 PSICAP(NXMAX-1,NYMAX-1)
C     Work arrays
      REAL WORK1(NYMAX-1,NXMAX)
C
      IF ((NX .GT. NXMAX) .OR. (NY .GT. NYMAX)) STOP 'INVLAP'
C
C     Set up transposed array for vectorizable sine-transform
      DO 110 J=1,NY-1
        DO 110 I=1,NX-1
  110     BTRAN(J,I)= ZETA(I,J)*DY**2
C     Compute sine-transform
      CALL VSINT(NY-1, NX-1, BTRAN, WORK1, NYMAX-1, WFST)
C     Untranspose transformed array
      DO 120 J=1,NY-1
        DO 120 K=1,NX-1
  120     PSICAP(K,J)= BTRAN(J,K)
C
C     Use up-down sweep to solve the tri-diagonal system
      DO 210 K=1,NX-1
  210   PSICAP(K,1)= WCOEF(K,1)*PSICAP(K,1)
C
      DO 220 J=2,NY-1
        DO 220 K=1,NX-1
  220     PSICAP(K,J)= WCOEF(K,J)*(PSICAP(K,J) - PSICAP(K,J-1))
C
      DO 230 J=NY-2,1,-1
        DO 230 K=1,NX-1
  230     PSICAP(K,J)= PSICAP(K,J) - WCOEF(K,J)*PSICAP(K,J+1)
C
C     Set up transposed array for inverse sine-transform
      DO 310 J=1,NY-1
        DO 310 K=1,NX-1
  310     BTRAN(J,K)= PSICAP(K,J)
C     Compute inverse sine-transform
      CALL VSINT(NY-1, NX-1, BTRAN, WORK1, NYMAX-1, WFST)
C     Untranspose streamfunction
      DO 320 J=1,NY-1
        DO 320 I=1,NX-1
  320     PSI(I,J)= BTRAN(J,I)
C     Zero boundary values
      DO 330 I=0,NX
        PSI(I, 0)= 0.0
  330   PSI(I,NY)= 0.0
      DO 340 J=1,NY-1
        PSI(0, J)= 0.0
  340   PSI(NX,J)= 0.0
C
      RETURN
      END

C --------------------------------------------------------------------
C For GNU Emacs:
C $Msg-digest-checksum: 48ccded1725830ac01c9ac9a7ff2a9c3 $
C --------------------------------------------------------------------
C Local Variables:
C msg-digest-active: t
C End:
C --------------------------------------------------------------------
