C ICAM (Idealized Coupleable Atmospheric Model)
C packetsrc $Revision: 1.6 $ created $Date: 1996/05/03 22:50:18 $
C
C Author: R. Saravanan, Climate and Global Dynamics Division
C                       National Center for Atmospheric Research
C                       Boulder, CO 80307-3000, U.S.A.
C                       E-mail: svn@ncar.ucar.edu
C                       URL: http://www.cgd.ucar.edu:/gds/svn/svn.html
C
C packet.F: Fortran source module
C Contains data packet send/receive routines for communication
C between the atmosphere and the ocean models
C
 
C Top-level routines:
C INIPAK: Initialize packet communications (called by ATMBGN)
C ENDPAK: Terminate packet communications (called by ATMEND)
C
C RCVSST: Receive SST and coordinate data from ocean model
C          (called by UPDSST)
C SNDFLX: Send surface flux data to ocean model
C          (called by FLXOUT)
C RCVFLX: Receive surface flux data from atmospheric model
C          (called by ATMINT)
C SNDSST: Send SST and coordinate data to atmospheric model
C          (called by ATMINT)
C
 
C C preprocessor options
C  MAXBAS: Maximum number of (rectangular) ocean basins
C
C  JOMAX:  Maximum number of temperature grid-points in X-direction
C           in ocean basin (>= 1)
C  KOMAX:  Maximum number of temperature grid-points in Y-direction
C           in ocean basin (>= 2)
C  NFOCN:  Maximum number of fluxes passed from atmosphere to ocean
C           (Tau-x, Tau-y, heat flux, E-P, tracer fluxes)
C
C  PVM_LINK: Define this option to link the Parallel Virtual Machine
C            (PVM) library for packet communications
C 
C  MPI_LINK: Define this option to link the Message Passing Interface
C            (MPI) library for packet communications
 
#ifndef MAXBAS
#   define MAXBAS 1
#endif
#ifndef JOMAX
#   define JOMAX 1
#endif
#ifndef KOMAX
#   define KOMAX 65
#endif
#ifndef NFOCN
#   define NFOCN 4
#endif
 
      SUBROUTINE INIPAK( COMOPT, RUNNAM )
C Initialize packet communications
C
C Input: CHARACTER*(*) COMOPT -- communication option string
C  [Options 1: <blank>  => use COMMON blocks for communication;
C           2: 'PVM'    => use PVM (Parallel Virtual Machine);
C           3: 'MPI'    => use MPI (Message Passing Interface)
C           4: <other>  => use disk files for communication
C                        (COMOPT is assumed to contain the name of
C                         the directory where these files reside)]
C        CHARACTER*5 RUNNAM -- run name (used to generate packet names)
C Output: (mpacket.h: Part 1)
C
#if defined(MPI_LINK)
#include "mpif.h"
      INTEGER IERR
#endif
#include "mpacket.h"
C
      CHARACTER*(*) COMOPT
      CHARACTER RUNNAM*5
C
      INTEGER IB
C
C     Save option string, right justified
      COMDIR= COMOPT
      CALL RJUST( COMDIR )
C
C     Save run name
      RUNPAK= RUNNAM
C
C     Copy communication options
      IF (COMOPT .EQ. ' ') THEN
C       Use COMMON blocks for communication
        IPKOPT= 1
      ELSE IF (COMOPT .EQ. 'PVM') THEN
C       Use PVM for communications
        IPKOPT= 2
#ifdef PVM_LINK
C
C       Enroll into PVM 3
        CALL PVMFMYTID( MYTID )
C
C       Join a process group called <run-name>
        CALL PVMFJOINGROUP( RUNPAK, MYINST )
C
C       Instance number of other task
        OINST= 1-MYINST
C       Clear other task ID (to be determined after synchronization)
        OTHTID= -1
C
C       Define data type codes
C       Always pass integers as 8-byte reals
        RIOPT= 6
C       Pass 4-byte or 8-byte reals, depending on word length
        IF (LENWRD .EQ. 4) THEN
          RROPT= 4
        ELSE IF (LENWRD .EQ. 8) THEN
          RROPT= 6
        ENDIF
C       Clear SST synchronization flag
        SSTSYN= .FALSE.
C       Set up parallel integration
        SEQFLG= .FALSE.
C       Display PVM info
        WRITE(*,'(A,A)') 'INIPAK: PVM group ', RUNPAK
        WRITE(*,*) 'INIPAK: MYTID  =', MYTID
        WRITE(*,*) 'INIPAK: MYINST =', MYINST
        WRITE(*,*) 'INIPAK: RIOPT  =', RIOPT
        WRITE(*,*) 'INIPAK: RROPT  =', RROPT
#else
        WRITE(*,*) 'INIPAK: PVM routines not linked'
        STOP
#endif
      ELSE IF (COMOPT .EQ. 'MPI') THEN
C       Use MPI for communications
        IPKOPT= 3
#ifdef MPI_LINK
C
C       Establish MPI environment
        CALL MPI_INIT(IERR)
C
        CALL MPI_COMM_RANK( MPI_COMM_WORLD, MYID, IERR )
C
        CALL MPI_COMM_SIZE( MPI_COMM_WORLD, NUMPROCS, IERR )
C
C       Rank of other process
        OTHERID= 1-MYID
C       Clear SST synchronization flag
        SSTSYN= .FALSE.
C       Set up parallel integration
        SEQFLG= .FALSE.
C
C       Display MPI info
        WRITE(*,'(A,A)') 'INIPAK: MPI world communicator ', RUNPAK
        WRITE(*,*) 'INIPAK: MYID  =', MYID
        WRITE(*,*) 'INIPAK: NUMPROCS =', NUMPROCS
#else
        WRITE(*,*) 'INIPAK: MPI routines not linked'
        STOP
#endif
      ELSE
C       Use disk files for communication (in directory COMOPT)
        IPKOPT= 4
      ENDIF
C
C     Clear receive/send packet common blocks
      DO 10 IB=1,MAXBAS
        IPACK0(IB)= -1
        IPACK1(IB)= -1
   10   CONTINUE
C
      RETURN
      END
 
      SUBROUTINE ENDPAK
C Terminate packet communications
C
#include "mpacket.h"
C
#ifdef PVM_LINK
      INTEGER INFO
#elif defined(MPI_LINK)
      INTEGER IERR
#endif
C
      IF (IPKOPT .EQ. 2) THEN
C       End PVM communication
#ifdef PVM_LINK
C       Leave process group
        CALL PVMFLVGROUP( RUNPAK, INFO )
C       Exit PVM
        CALL PVMFEXIT( INFO )
#endif
      ELSE IF (IPKOPT .EQ. 3) THEN
C       End MPI communication
#ifdef MPI_LINK
C
        CALL MPI_FINALIZE(IERR)
#endif
      ENDIF
      RETURN
      END
 
      SUBROUTINE OSYNC( ERRFLG )
C Synchronize with other task, and update other task's ID;
C return error flag
C
#include "mpacket.h"
C
      LOGICAL ERRFLG
C
#ifdef PVM_LINK
      INTEGER INFO, NEWTID
#elif defined(MPI_LINK)
#include "mpif.h"
      INTEGER IERR
#endif
C
      ERRFLG= .FALSE.
C
#ifdef PVM_LINK
#ifdef DEBUG
      WRITE(*,*) 'OSYNC: waiting ...'
#endif
C     Synchronize; i.e., wait for other task
      CALL PVMFBARRIER( RUNPAK, 2, INFO )
      IF (INFO .LT. 0) THEN
        ERRFLG= .TRUE.
        RETURN
      ENDIF
C
C     Get task ID of other task
      CALL PVMFGETTID( RUNPAK, OINST, NEWTID )
      IF (NEWTID .LT. 0) THEN
        WRITE(*,*) 'OSYNC: error code ', NEWTID
        ERRFLG= .TRUE.
        RETURN
      ENDIF
C     Update other task ID
      OTHTID= NEWTID
#ifdef DEBUG
      WRITE(*,*) 'OSYNC: OTHTID= ', OTHTID
#endif

#elif defined(MPI_LINK)
#ifdef DEBUG
      WRITE(*,*) 'OSYNC: waiting ...'
#endif
C     Synchronize; i.e., wait for other task
      CALL MPI_BARRIER( MPI_COMM_WORLD, IERR )
      IF (IERR .NE. MPI_SUCCESS) THEN
        ERRFLG= .TRUE.
        RETURN
      ENDIF
#endif
      RETURN
      END
 
      SUBROUTINE NXTSST( IPACKT )
C Receive SST packet in advance, if a file named "SEQUENTIAL"
C exists in the local directory. This forces synchronous
C integration. (Works only with PVM?MPI; does nothing otherwise)
 
C Input:  INTEGER IPACKT -- packet number (>= 0)
C
#include "mpacket.h"
C
      INTEGER IPACKT
C
#if defined(PVM_LINK) || defined(MPI_LINK)
      LOGICAL ERRFLG
#endif
C
#if defined(PVM_LINK) || defined(MPI_LINK)
      IF (MOD(IPACKT,10) .EQ. 0) THEN
C       Update sequential integration flag every 10 synch. intervals
        INQUIRE(FILE='SEQUENTIAL', EXIST=SEQFLG)
      ENDIF
C
      IF (SEQFLG) THEN
C       Sequential integration; synchronize
        CALL OSYNC( ERRFLG )
C       Set SST synchronization flag
        SSTSYN= .TRUE.
      ENDIF
#endif
      RETURN
      END
 
      SUBROUTINE NAMGEN( IB, IPACKT, CODE,
     1                   SEMAPH, PAKNAM, SEMFIL, PAKFIL )
C Generate semaphore name SEMAPH, packet name PAKNAM
C and the corresponding filenames SEMFIL, PAKFIL,
C using basin number IB, packet number IPACKT (0,1,...),
C data code (3 characters), and the run prefix/communication
C directory info stored in mpacket.h.
C
#include "mpacket.h"
      INTEGER IB, IPACKT
      CHARACTER*3 CODE
      CHARACTER*20 SEMAPH, PAKNAM
      CHARACTER*100 SEMFIL, PAKFIL
C
      CHARACTER HEADER*10
C
C     Create packet name header
      WRITE(HEADER,'(A5,A1,A3,I1)') RUNPAK, '-', CODE, IB
C
C     Create semaphore name
      SEMAPH= HEADER//'-semaphore'
      SEMFIL= COMDIR//SEMAPH
      CALL LJUST( SEMFIL )
C
C     Create data packet name
      WRITE(PAKNAM,'(A10,1X,I9)') HEADER, 100000000+IPACKT
      PAKNAM(11:12)= '-p'
      PAKFIL= COMDIR//PAKNAM
      CALL LJUST( PAKFIL )
C
      RETURN
      END
 
      SUBROUTINE RCVSST( IUNIT, IPACKT, IB, NXMAX, NYMAX,
     1                   NBASX, NBASY, XBAS, XBMID, YBAS, YBMID,
     2                   SSTBAS, ERRFLG )
C Receive SST and coordinate data from ocean model
C
C Input:  INTEGER IUNIT  -- Fortran unit number to be used, if need be
C         INTEGER IPACKT -- packet number (>= 0)
C         INTEGER IB     -- ocean basin number
C         INTEGER NXMAX, NYMAX -- declared X/Y dimensions
C Output: INTEGER NBASX, NBASY -- no. of X/Y oceanic grid-points
C         REAL XBAS(NXMAX)   -- Longitudes of grid-points (degrees)
C         REAL XBMID(0:NXMAX)-- Longitude of grid-box edges (degrees)
C         REAL YBAS(NYMAX)   -- Sine-latitude (mu) of grid-points
C         REAL YBMID(0:NYMAX)-- Sine-latitude (mu) grid-box edges
C         REAL SSTBAS(NXMAX,NYMAX) -- oceanic SST values (K)
C         LOGICAL ERRFLG -- set to TRUE, if any error occurs
C [NOTE: 1. Ocean grid-points correspond to the temperature grid;
C           ocean grid box edges correspond to the velocity grid.
C        2. SST values should be set to zero over land points     ]
C
#include "mpacket.h"
C
      INTEGER IUNIT, IPACKT, IB, NXMAX, NYMAX
      INTEGER NBASX, NBASY
      REAL XBAS(NXMAX), XBMID(0:NXMAX)
      REAL YBAS(NYMAX), YBMID(0:NYMAX)
      REAL SSTBAS(NXMAX,NYMAX)
      LOGICAL ERRFLG
C
      INTEGER JO, KO
      LOGICAL EXFLAG
      CHARACTER*20 SEMAPH, PAKNAM
      CHARACTER*100 SEMFIL, PAKFIL
C
#ifdef PVM_LINK
      INTEGER BUFID, INFO
      REAL*8 RITEM(5)
#elif defined(MPI_LINK)
#include "mpif.h"
      INTEGER OFFSET_XG, OFFSET_XM, OFFSET_YG, OFFSET_YM, OFFSET_SST
      INTEGER STAT(MPI_STATUS_SIZE), RSST_SIZE, IERR
#endif
C
C     Clear error flag
      ERRFLG= .FALSE.
C     Check for valid ocean basin number
      IF (IB .GT. MAXBAS) STOP 'MAXBAS'
C
C     Generate semaphore/packet names for SST data
      CALL NAMGEN( IB, IPACKT, 'SST',
     1             SEMAPH, PAKNAM, SEMFIL, PAKFIL )
C
      IF (IPKOPT .EQ. 2) THEN
C       PVM communication
#ifdef PVM_LINK
        IF (.NOT. SSTSYN) THEN
C         Synchronize; i.e., wait for other task
          CALL OSYNC( ERRFLG )
          IF (ERRFLG) RETURN
        ENDIF
C       Clear SST synchronization flag
        SSTSYN= .FALSE.
C       Receive SST packet from other task
        CALL PVMFRECV( OTHTID, -1, BUFID )
        IF (BUFID .LT. 0) THEN
          CALL PVMFPERROR( 'pvmfrecv', BUFID )
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Unpack integer data (from reals)
        CALL PVMFUNPACK( RIOPT, RITEM, 5, 1, INFO )
        IPACK0(IB)= NINT(RITEM(1))
        ISTP0(IB)=  NINT(RITEM(2))
        NSYNC0(IB)= NINT(RITEM(3))
        NOCNX0(IB)= NINT(RITEM(4))
        NOCNY0(IB)= NINT(RITEM(5))
C       Check packet size before unpacking
        IF ( (NOCNX0(IB) .GT. JOMAX) .OR.
     1       (NOCNY0(IB) .GT. KOMAX)      ) THEN
          WRITE(*,*) 'RCVSST: Packet too large for COMMON block'
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Unpack floating point data
        CALL PVMFUNPACK( RROPT,  XOCN(1,IB), NOCNX0(IB),   1, INFO )
        CALL PVMFUNPACK( RROPT, XOMID(0,IB), NOCNX0(IB)+1, 1, INFO )
        CALL PVMFUNPACK( RROPT,  YOCN(1,IB), NOCNY0(IB),   1, INFO )
        CALL PVMFUNPACK( RROPT, YOMID(0,IB), NOCNY0(IB)+1, 1, INFO )
        DO 210 KO=1,NOCNY0(IB)
  210     CALL PVMFUNPACK(RROPT, SSTOCN(1,KO,IB), NOCNX0(IB), 1, INFO)

#endif
      ELSE IF (IPKOPT .EQ. 3) THEN
C       MPI communication
#ifdef MPI_LINK
        IF (.NOT. SSTSYN) THEN
C         Synchronize; i.e., wait for other task
          CALL OSYNC( ERRFLG )
          IF (ERRFLG) RETURN
        ENDIF
C       Clear SST synchronization flag
        SSTSYN= .FALSE.
C
C       Receive integer data
        CALL MPI_RECV( ISST_BUF, ISST_SIZE, MPI_INTEGER,
     $                 OTHERID, ISST_TAG,
     $                 MPI_COMM_WORLD, STAT, IERR )
        IF (IERR .NE. MPI_SUCCESS) THEN
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Unpack integer data
        IPACK0(IB)= ISST_BUF(1)
        ISTP0(IB)=  ISST_BUF(2)
        NSYNC0(IB)= ISST_BUF(3)
        NOCNX0(IB)= ISST_BUF(4)
        NOCNY0(IB)= ISST_BUF(5)
C
C       Check anticipated size of real data
        IF ( (NOCNX0(IB) .GT. JOMAX) .OR.
     1       (NOCNY0(IB) .GT. KOMAX)      ) THEN
          WRITE(*,*) 'RCVSST: Packet too large for COMMON block'
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Compute offsets for real data buffer RSST_BUF
C       NOTE: The first element of the buffer is RSST_BUF(1)
        OFFSET_XG= 0
        OFFSET_XM= OFFSET_XG + NOCNX0(IB)
        OFFSET_YG= OFFSET_XM + NOCNX0(IB) + 1
        OFFSET_YM= OFFSET_YG + NOCNY0(IB)
        OFFSET_SST= OFFSET_YM + NOCNY0(IB) + 1
C
C       Receive real data
        RSST_SIZE=  2*NOCNX0(IB)+1 + 2*NOCNY0(IB)+1 +
     $                NOCNX0(IB)*NOCNY0(IB)
C
        CALL MPI_RECV( RSST_BUF, RSST_SIZE, MPI_REAL,
     $                 OTHERID, RSST_TAG,
     $                 MPI_COMM_WORLD, STAT, IERR )
C
C       Unpack real data
        DO 210 JO=1,NOCNX0(IB)
          XOCN(JO,IB)=  RSST_BUF(JO+OFFSET_XG)
  210     XOMID(JO,IB)= RSST_BUF(JO+1+OFFSET_XM)
        XOMID(0,IB)=    RSST_BUF(1+OFFSET_XM)
C
        DO 220 KO=1,NOCNY0(IB)
          YOCN(KO,IB)=  RSST_BUF(KO+OFFSET_YG)
  220     YOMID(KO,IB)= RSST_BUF(KO+1+OFFSET_YM)
        YOMID(0,IB)=    RSST_BUF(1+OFFSET_YM)
C
        DO 230 KO=1,NOCNY0(IB)
          DO 230 JO=1,NOCNX0(IB)
            SSTOCN(JO,KO,IB)= RSST_BUF(JO +
     $                        (KO-1)*NOCNX0(IB) + OFFSET_SST)
  230       CONTINUE
#endif
      ELSE IF (IPKOPT .EQ. 4) THEN
C         Disk file communication
C
#ifdef DEBUG
          WRITE(*,'(A,A)') 'RCVSST: waiting for semaphore ', SEMFIL
#endif
C         Wait for ocean model to create SST semaphore
C         (one jump to this statement)
  301     INQUIRE(FILE=SEMFIL, EXIST=EXFLAG)
          IF (.NOT. EXFLAG) THEN
#ifdef ON_SUN4
            CALL SLEEP( 1 )
#endif
            GOTO 301
          ENDIF
C
C       Open SST data packet file
        OPEN(IUNIT, FILE=PAKFIL, FORM='UNFORMATTED', STATUS='OLD')
C       Read header info
        READ(IUNIT) IPACK0(IB), ISTP0(IB), NSYNC0(IB)
        READ(IUNIT) NOCNX0(IB), NOCNY0(IB)
C       Check packet size before reading further
        IF ( (NOCNX0(IB) .GT. JOMAX) .OR.
     1       (NOCNY0(IB) .GT. KOMAX)      ) THEN
          WRITE(*,*) 'RCVSST: Packet too large for COMMON block'
          ERRFLG= .TRUE.
          CLOSE( IUNIT )
          RETURN
        ENDIF
C       Read coordinate values
        READ(IUNIT) (XOCN(JO,IB),JO=1,NOCNX0(IB)),
     1             (XOMID(JO,IB),JO=0,NOCNX0(IB)),
     2              (YOCN(KO,IB),KO=1,NOCNY0(IB)),
     3             (YOMID(KO,IB),KO=0,NOCNY0(IB))
C       Read SST
        READ(IUNIT)
     1       ((SSTOCN(JO,KO,IB),JO=1,NOCNX0(IB)),KO=1,NOCNY0(IB))
C       Close data packet file and delete it
        CLOSE(IUNIT, STATUS='DELETE' )
C
C       Open semaphore file
        OPEN(IUNIT, FILE=SEMFIL, FORM='UNFORMATTED', STATUS='OLD' )
C       Close semaphore file and delete it
        CLOSE(IUNIT, STATUS='DELETE')
      ENDIF
C
C     Check if the right packet has been received
      IF (IPACK0(IB) .NE. IPACKT) THEN
        WRITE(*,*) 'RCVSST: Wrong packet received for ', PAKNAM
        ERRFLG= .TRUE.
        RETURN
      ENDIF
C     Check packet size before copying data
      IF ( (NOCNX0(IB) .GT. NXMAX) .OR.
     1     (NOCNY0(IB) .GT. NYMAX)      ) THEN
        WRITE(*,*) 'RCVSST: Packet too large for supplied arrays'
        ERRFLG= .TRUE.
        CLOSE( IUNIT )
        RETURN
      ENDIF
C     Extract data from packet common block
C     Copy X/Y coordinates
      NBASX= NOCNX0(IB)
      NBASY= NOCNY0(IB)
C
      DO 10 JO=1,NBASX
   10   XBAS(JO)= XOCN(JO,IB)
      DO 20 JO=0,NBASX
   20   XBMID(JO)= XOMID(JO,IB)
C
      DO 30 KO=1,NBASY
   30   YBAS(KO)= YOCN(KO,IB)
      DO 40 KO=0,NBASY
   40   YBMID(KO)= YOMID(KO,IB)
C     Copy SST
      DO 50 KO=1,NBASY
        DO 50 JO=1,NBASX
   50     SSTBAS(JO,KO)= SSTOCN(JO,KO,IB)
      IF (IPKOPT .EQ. 1) THEN
C       COMMON block communication; clear packet common block
        IPACK0(IB)= -1
      ENDIF
C
#ifdef DEBUG
      WRITE(*,*)
      WRITE(*,*) 'Subroutine RCVSST: Debugging information'
      WRITE(*,'(1X,A,A)') 'Packet name: ', PAKNAM
      WRITE(*,'(1X,A,I2)') 'BASIN', IB
      WRITE(*,'(1X,A,3I10)') 'IPACK0,ISTP0,ISYNC0: ',
     1                        IPACK0(IB), ISTP0(IB), NSYNC0(IB)
      WRITE(*,'(1X,A,2I10)') 'NOCNX0,NOCNY0:       ',
     1                        NOCNX0(IB), NOCNY0(IB)
#endif
C
      RETURN
      END
 
      SUBROUTINE SNDFLX( IUNIT, IPACKT, IB, NXMAX, NYMAX,
     1                   FLXTOT, FLXBAS, NBASX, NBASY, NFLX,
     2                   IASTP, NASYNC, ERRFLG)
C Send surface flux data to ocean model
C
C Input:  INTEGER IUNIT  -- Fortran unit number to be used, if need be
C         INTEGER IPACKT       -- packet number (>= 0)
C         INTEGER IB           -- ocean basin number
C         INTEGER NXMAX, NYMAX -- declared X/Y dimensions
C         REAL FLXTOT(NFLX)    -- integrated fluxes over ocean basin
C         REAL FLXBAS(NXMAX,NYMAX,NFLX) -- fluxes over ocean basin
C         INTEGER NBASX, NBASY -- no. of X/Y oceanic grid-points
C         INTEGER NFLX         -- no. of different ocean fluxes
C         INTEGER IASTP        -- atmospheric model time-step count
C         INTEGER NASYNC -- no. of atmospheric time-steps between
C                           synchronizations (>= 1)
C Output: (packet containing flux data is sent to ocean model)
C         LOGICAL ERRFLG -- set to TRUE, if any error occurs
C
#include "mpacket.h"
C
      INTEGER IUNIT, IPACKT, IB, NXMAX, NYMAX
      INTEGER NBASX, NBASY, NFLX, IASTP, NASYNC
      REAL FLXTOT(NFLX), FLXBAS(NXMAX,NYMAX,NFLX)
      LOGICAL ERRFLG
C
      INTEGER IFX, JO, KO
      LOGICAL EXFLAG
      CHARACTER*20 SEMAPH, PAKNAM
      CHARACTER*100 SEMFIL, PAKFIL
C
#ifdef PVM_LINK
      INTEGER BUFID, INFO
      REAL*8 RITEM(6)
#elif defined(MPI_LINK)
#include "mpif.h"
      INTEGER RFLX_SIZE, IERR
#endif
C
C     Clear error flag
      ERRFLG= .FALSE.
C     Check for valid ocean basin number
      IF (IB .GT. MAXBAS) STOP 'MAXBAS'
C
C     Generate semaphore/packet names for flux data
      CALL NAMGEN( IB, IPACKT, 'FLX',
     1             SEMAPH, PAKNAM, SEMFIL, PAKFIL )
C
C     Check packet size
      IF ( (NBASX .GT. JOMAX) .OR.
     1     (NBASY .GT. KOMAX) .OR.
     2     (NFLX  .GT. NFOCN)      ) THEN
        WRITE(*,*) 'SNDFLX: Packet too large for COMMON block'
        ERRFLG= .TRUE.
        RETURN
      ENDIF
C
C     Insert data into packet common block
C
      IF (IPKOPT .EQ. 1) THEN
C       COMMON block communication
        IF (IPACK1(IB) .NE. -1) THEN
C         Packet common block not cleared
          WRITE(*,*) 'SNDFLX: Common not cleared for ', PAKNAM
          ERRFLG= .TRUE.
          RETURN
        ENDIF
      ENDIF
C     Copy header info
      NOCNX1(IB)= NBASX
      NOCNY1(IB)= NBASY
      NFLX1(IB)=  NFLX
C     Copy fluxes
      DO 10 IFX=1,NFLX
        DO 10 KO=1,NBASY
          DO 10 JO=1,NBASX
   10       FLXOCN(JO,KO,IFX,IB)= FLXBAS(JO,KO,IFX)
C     Copy flux integral
      DO 20 IFX=1,NFLX
   20   FXTOCN(IFX,IB)= FLXTOT(IFX)
C     Copy packet number etc.
      IPACK1(IB)= IPACKT
      ISTP1(IB)=  IASTP
      NSYNC1(IB)= NASYNC
C
      IF (IPKOPT .EQ. 2) THEN
C       PVM communication; clear send buffer
#ifdef PVM_LINK
        CALL PVMFINITSEND( 0, BUFID )
        IF (BUFID .LT. 0) THEN
          CALL PVMFPERROR( 'pvmfinitsend', BUFID )
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Pack integer data (to reals)
        RITEM(1)= IPACK1(IB)
        RITEM(2)= ISTP1(IB)
        RITEM(3)= NSYNC1(IB)
        RITEM(4)= NOCNX1(IB)
        RITEM(5)= NOCNY1(IB)
        RITEM(6)= NFLX1(IB)
        CALL PVMFPACK( RIOPT, RITEM, 6, 1, INFO )
C       Pack floating point data
        DO 210 IFX=1,NFLX1(IB)
          DO 210 KO=1,NOCNY1(IB)
  210       CALL PVMFPACK(RROPT,FLXOCN(1,KO,IFX,IB),NOCNX1(IB),1,INFO)
        CALL PVMFPACK( RROPT, FXTOCN(1,IB), NFLX1(IB), 1, INFO )
C       Synchronize; i.e., wait for other task
        CALL OSYNC( ERRFLG )
        IF (ERRFLG) RETURN
C       Send data to other task
        CALL PVMFSEND( OTHTID, 0, INFO )
        IF (INFO .LT. 0) THEN
          CALL PVMFPERROR( 'pvmfsend', INFO )
          ERRFLG= .TRUE.
          RETURN
        ENDIF
#endif
      ELSE IF (IPKOPT .EQ. 3) THEN
C       MPI communication
#ifdef MPI_LINK
C       Synchronize; i.e., wait for other task
        CALL OSYNC( ERRFLG )
        IF (ERRFLG) RETURN
C
C       Pack integer data
        IFLX_BUF(1) = IPACK1(IB)
        IFLX_BUF(2) = ISTP1(IB)
        IFLX_BUF(3) = NSYNC1(IB)
        IFLX_BUF(4) = NOCNX1(IB)
        IFLX_BUF(5) = NOCNY1(IB)
        IFLX_BUF(6) = NFLX1(IB)
C
C       Send integer data
        CALL MPI_SEND( IFLX_BUF, IFLX_SIZE, MPI_INTEGER,
     $                 OTHERID, IFLX_TAG,
     $                 MPI_COMM_WORLD, IERR )
        IF (IERR .NE. MPI_SUCCESS) THEN
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C
C       Check anticipated size of real data
        IF ( (NOCNX1(IB) .GT. JOMAX) .OR.
     1       (NOCNY1(IB) .GT. KOMAX) .OR.
     2       (NFLX1(IB)  .GT. NFOCN)     ) THEN
          WRITE(*,*) 'SNDFLX: Packet too large for COMMON block'
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C
C       Pack real data
        DO 210 IFX=1,NFLX1(IB)
          DO 210 KO=1,NOCNY1(IB)
            DO 210 JO=1,NOCNX1(IB)
              RFLX_BUF(JO + NOCNX1(IB)*((KO-1)+NOCNY1(IB)*(IFX-1)) ) =
     $                FLXOCN(JO,KO,IFX,IB)
  210       CONTINUE
C
C       Send real data
        RFLX_SIZE= NFLX1(IB) * NOCNX1(IB) * NOCNY1(IB)
C
        CALL MPI_SEND( RFLX_BUF, RFLX_SIZE, MPI_REAL,
     $                 OTHERID, RFLX_TAG,
     $                 MPI_COMM_WORLD, IERR )
        IF (IERR .NE. MPI_SUCCESS) THEN
          ERRFLG= .TRUE.
          RETURN
        ENDIF
#endif
      ELSE IF (IPKOPT .EQ. 4) THEN
C         Disk file communication;
C         Write data from packet common block into file
C
C         Wait until ocean model has disposed of previous packet, if any
C         (one jump to this statement)
  301     INQUIRE(FILE=SEMFIL, EXIST=EXFLAG)
          IF (EXFLAG) THEN
#ifdef ON_SUN4
            CALL SLEEP( 1 )
#endif
            GOTO 301
          ENDIF
C
C       Open flux data packet file
        OPEN(IUNIT, FILE=PAKFIL, FORM='UNFORMATTED', STATUS='NEW')
C       Write header
        WRITE(IUNIT) IPACK1(IB), ISTP1(IB), NSYNC1(IB)
        WRITE(IUNIT) NOCNX1(IB), NOCNY1(IB), NFLX1(IB)
C       Write data
        WRITE(IUNIT)
     1   (((FLXOCN(JO,KO,IFX,IB),JO=1,NOCNX1(IB)),KO=1,NOCNY1(IB)),
     2                           IFX=1,NFLX1(IB))
        WRITE(IUNIT) (FXTOCN(IFX,IB),IFX=1,NFLX1(IB))
C       Close data packet file
        CLOSE(IUNIT)
C
C       Create new semaphore file
        OPEN(IUNIT, FILE=SEMFIL, FORM='UNFORMATTED', STATUS='NEW' )
        CLOSE(IUNIT)
      ENDIF
C
#ifdef DEBUG
      WRITE(*,*)
      WRITE(*,*) 'Subroutine SNDFLX: Debugging information'
      WRITE(*,'(1X,A,A)') 'Packet name: ', PAKNAM
      WRITE(*,'(1X,A,I2)') 'BASIN', IB
      WRITE(*,'(1X,A,3I10)') 'IPACK1,ISTP1,ISYNC1: ',
     1                        IPACK1(IB), ISTP1(IB), NSYNC1(IB)
      WRITE(*,'(1X,A,3I10)') 'NOCNX1,NOCNY1,NFLX1: ',
     1                        NOCNX1(IB), NOCNY1(IB), NFLX1(IB)
#endif
C
      RETURN
      END
 
      SUBROUTINE SNDSST( IUNIT, IPACKT, IB, NXMAX, NBASX, NBASY,
     1                   XBAS, XBMID, YBAS, YBMID, SSTBAS,
     2                   IOSTP, NOSYNC, ERRFLG)
C Send SST and coordinate data to atmospheric model
C
C Input:  INTEGER IUNIT  -- Fortran unit number to be used, if need be
C         INTEGER IPACKT -- packet number (>= 0)
C         INTEGER IB     -- ocean basin number
C         INTEGER NXMAX  -- declared X dimension
C         INTEGER NBASX, NBASY -- actual no. of X/Y oceanic grid-points
C         REAL X(NBASX)        -- Longitudes of grid-points (degrees)
C         REAL XMID(0:NBASX)   -- Longitude of grid-box edges (degrees)
C         REAL Y(NBASY)        -- Sine-latitude of grid-points
C         REAL YMID(0:NBASY)   -- Sine-latitude of grid-box edges
C [NOTE:  Ocean grid-points correspond to the temperature grid;
C         ocean grid box edges correspond to the velocity grid ]
C         REAL SSTBAS(NXMAX,NBASY) -- oceanic SST values (K)
C         INTEGER IOSTP  -- ocean model time-step count
C         INTEGER NOSYNC -- no. of oceanic time-steps between
C                           synchronizations (>= 1)
C Output: LOGICAL ERRFLG -- set to TRUE, if any error occurs
C         (SST data, along with the coordinate information initialized
C          by INIPK2, are sent to the atmospheric model)
C
#include "mpacket.h"
C
      INTEGER IUNIT, IPACKT, IB
      INTEGER NXMAX, NBASX, NBASY, IOSTP, NOSYNC
      REAL XBAS(NBASX), XBMID(0:NBASX)
      REAL YBAS(NBASY), YBMID(0:NBASY), SSTBAS(NXMAX,NBASY)
      LOGICAL ERRFLG
C
      INTEGER JO, KO
      LOGICAL EXFLAG
      CHARACTER*20 SEMAPH, PAKNAM
      CHARACTER*100 SEMFIL, PAKFIL
C
#ifdef PVM_LINK
      INTEGER BUFID, INFO
      REAL*8 RITEM(5)
#elif defined(MPI_LINK)
#include "mpif.h"
      INTEGER OFFSET_XG, OFFSET_XM, OFFSET_YG, OFFSET_YM, OFFSET_SST
      INTEGER RSST_SIZE, IERR
#endif
C
C     Clear error flag
      ERRFLG= .FALSE.
C     Check for valid ocean basin number
      IF (IB .GT. MAXBAS) STOP 'MAXBAS'
C
C     Generate semaphore/packet names for SST data
      CALL NAMGEN( IB, IPACKT, 'SST',
     1             SEMAPH, PAKNAM, SEMFIL, PAKFIL )
C
C     Check packet size
      IF ( (NBASX .GT. JOMAX) .OR.
     1     (NBASY .GT. KOMAX)     ) THEN
        WRITE(*,*) 'SNDSST: Packet too large for COMMON block'
        ERRFLG= .TRUE.
        RETURN
      ENDIF
C
C     Insert data into packet common block
C
      IF (IPKOPT .EQ. 1) THEN
C       COMMON block communication
        IF (IPACK0(IB) .NE. -1) THEN
C         Packet common block not cleared
          WRITE(*,*) 'SNDSST: Common not cleared for ', PAKNAM
          ERRFLG= .TRUE.
          RETURN
        ENDIF
      ENDIF
C     Copy header info
      NOCNX0(IB)= NBASX
      NOCNY0(IB)= NBASY
C     Copy X/Y coordinates
      DO 10 JO=1,NBASX
   10   XOCN(JO,IB) = XBAS(JO)
      DO 20 JO=0,NBASX
   20   XOMID(JO,IB)= XBMID(JO)
      DO 30 KO=1,NBASY
   30   YOCN(KO,IB) = YBAS(KO)
      DO 40 KO=0,NBASY
   40   YOMID(KO,IB)= YBMID(KO)
C     Copy SST
      DO 50 KO=1,NBASY
        DO 50 JO=1,NBASX
   50     SSTOCN(JO,KO,IB)= SSTBAS(JO,KO)
C     Copy packet number etc.
      IPACK0(IB)= IPACKT
      ISTP0(IB)=  IOSTP
      NSYNC0(IB)= NOSYNC
C
      IF (IPKOPT .EQ. 2) THEN
C       PVM communication; clear send buffer
#ifdef PVM_LINK
        CALL PVMFINITSEND( 0, BUFID )
        IF (BUFID .LT. 0) THEN
          CALL PVMFPERROR( 'pvmfinitsend', BUFID )
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Pack integer data (to reals)
        RITEM(1)= IPACK0(IB)
        RITEM(2)= ISTP0(IB)
        RITEM(3)= NSYNC0(IB)
        RITEM(4)= NOCNX0(IB)
        RITEM(5)= NOCNY0(IB)
        CALL PVMFPACK( RIOPT, RITEM, 5, 1, INFO )
C       Pack floating point data
        CALL PVMFPACK( RROPT,  XOCN(1,IB), NOCNX0(IB),   1, INFO )
        CALL PVMFPACK( RROPT, XOMID(0,IB), NOCNX0(IB)+1, 1, INFO )
        CALL PVMFPACK( RROPT,  YOCN(1,IB), NOCNY0(IB),   1, INFO )
        CALL PVMFPACK( RROPT, YOMID(0,IB), NOCNY0(IB)+1, 1, INFO )
        DO 210 KO=1,NOCNY0(IB)
  210     CALL PVMFPACK( RROPT, SSTOCN(1,KO,IB), NOCNX0(IB), 1, INFO )
C
C       Synchronize; i.e., wait for other task
        CALL OSYNC( ERRFLG )
        IF (ERRFLG) RETURN
C       Send data to other task
        CALL PVMFSEND( OTHTID, 0, INFO )
        IF (INFO .LT. 0) THEN
          CALL PVMFPERROR( 'pvmfsend', INFO )
          ERRFLG= .TRUE.
          RETURN
        ENDIF
#endif
      ELSE IF (IPKOPT .EQ. 3) THEN
C       MPI communication
#ifdef MPI_LINK
C       Synchronize; i.e., wait for other task
        CALL OSYNC( ERRFLG )
        IF (ERRFLG) RETURN
C       Pack integer data
        ISST_BUF(1) = IPACK0(IB)
        ISST_BUF(2) = ISTP0(IB)
        ISST_BUF(3) = NSYNC0(IB)
        ISST_BUF(4) = NOCNX0(IB)
        ISST_BUF(5) = NOCNY0(IB)
C       Send integer data
        CALL MPI_SEND( ISST_BUF, ISST_SIZE, MPI_INTEGER,
     $                 OTHERID, ISST_TAG,
     $                 MPI_COMM_WORLD, IERR )
        IF (IERR .NE. MPI_SUCCESS) THEN
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Check anticipated size of real data
        IF ( (NOCNX0(IB) .GT. JOMAX) .OR.
     1       (NOCNY0(IB) .GT. KOMAX)      ) THEN
          WRITE(*,*) 'SNDSST: Packet too large for COMMON block'
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Compute offsets for real data buffer RSST_BUF
C       NOTE: The first element of the buffer is RSST_BUF(1)
        OFFSET_XG= 0
        OFFSET_XM= OFFSET_XG + NOCNX0(IB)
        OFFSET_YG= OFFSET_XM + NOCNX0(IB) + 1
        OFFSET_YM= OFFSET_YG + NOCNY0(IB)
        OFFSET_SST= OFFSET_YM + NOCNY0(IB) + 1
C
C       Pack real data
        DO 210 JO=1,NOCNX0(IB)
          RSST_BUF(JO+OFFSET_XG) = XOCN(JO,IB)
  210     RSST_BUF(JO+1+OFFSET_XM) = XOMID(JO,IB)
        RSST_BUF(1+OFFSET_XM) = XOMID(0,IB)
C
        DO 220 KO=1,NOCNY0(IB)
          RSST_BUF(KO+OFFSET_YG) = YOCN(KO,IB)
  220     RSST_BUF(KO+1+OFFSET_YM) = YOMID(KO,IB)
        RSST_BUF(1+OFFSET_YM) = YOMID(0,IB)
C
        DO 230 KO=1,NOCNY0(IB)
          DO 230 JO=1,NOCNX0(IB)
            RSST_BUF(JO + (KO-1)*NOCNX0(IB) + OFFSET_SST) =
     $      SSTOCN(JO,KO,IB)
  230       CONTINUE
C
C       Send real data
        RSST_SIZE=  2*NOCNX0(IB)+1 + 2*NOCNY0(IB)+1 +
     $                NOCNX0(IB)*NOCNY0(IB)
C
        CALL MPI_SEND( RSST_BUF, RSST_SIZE, MPI_REAL,
     $                 OTHERID, RSST_TAG,
     $                 MPI_COMM_WORLD, IERR )
#endif
      ELSE IF (IPKOPT .EQ. 4) THEN
C         Disk file communication
C         Write data from packet common block into file
C
C         Wait until atmospheric model has disposed of previous packet
C         (one jump to this statement)
  301     INQUIRE(FILE=SEMFIL, EXIST=EXFLAG)
          IF (EXFLAG) THEN
#ifdef ON_SUN4
            CALL SLEEP( 1 )
#endif
            GOTO 301
          ENDIF
C
C       Open SST data packet file
        OPEN(IUNIT, FILE=PAKFIL, FORM='UNFORMATTED', STATUS='NEW')
C       Write header info
        WRITE(IUNIT) IPACK0(IB), ISTP0(IB), NSYNC0(IB)
        WRITE(IUNIT) NOCNX0(IB), NOCNY0(IB)
C       Write coordinate values
        WRITE(IUNIT) (XOCN(JO,IB),JO=1,NOCNX0(IB)),
     1              (XOMID(JO,IB),JO=0,NOCNX0(IB)),
     2               (YOCN(KO,IB),KO=1,NOCNY0(IB)),
     3              (YOMID(KO,IB),KO=0,NOCNY0(IB))
C       Write SST
        WRITE(IUNIT)
     1       ((SSTOCN(JO,KO,IB),JO=1,NOCNX0(IB)),KO=1,NOCNY0(IB))
C       Close data packet file
        CLOSE(IUNIT)
C
C       Create new semaphore file
        OPEN(IUNIT, FILE=SEMFIL, FORM='UNFORMATTED', STATUS='NEW' )
        CLOSE(IUNIT)
      ENDIF
C
#ifdef DEBUG
      WRITE(*,*)
      WRITE(*,*) 'Subroutine SNDSST: Debugging information'
      WRITE(*,'(1X,A,A)') 'Packet name: ', PAKNAM
      WRITE(*,'(1X,A,I2)') 'BASIN', IB
      WRITE(*,'(1X,A,3I10)') 'IPACK0,ISTP0,ISYNC0: ',
     1                        IPACK0(IB), ISTP0(IB), NSYNC0(IB)
      WRITE(*,'(1X,A,2I10)') 'NOCNX0,NOCNY0:       ',
     1                        NOCNX0(IB), NOCNY0(IB)
#endif
C
      RETURN
      END
 
      SUBROUTINE RCVFLX( IUNIT, IPACKT, IB, NXMAX, NYMAX, NFMAX,
     1                   FLXTOT, FLXBAS, NBASX, NBASY, NFLX,
     2                   ERRFLG)
C Receive surface flux data from atmospheric model
C
C Input:  INTEGER IUNIT  -- Fortran unit number to be used, if need be
C         INTEGER IPACKT       -- packet number (>= 0)
C         INTEGER IB           -- ocean basin number
C         INTEGER NXMAX,NYMAX,NFMAX -- declared X/Y/flux dimensions
C Output: REAL FLXTOT(NFMAX)    -- integrated fluxes over ocean basin
C         REAL FLXBAS(NXMAX,NYMAX,NFMAX) -- fluxes over ocean basin
C         INTEGER NBASX, NBASY -- no. of X/Y oceanic grid-points
C         INTEGER NFLX         -- no. of different ocean fluxes
C         LOGICAL ERRFLG -- set to TRUE, if any error occurs
C
#include "mpacket.h"
C
      INTEGER IUNIT, IPACKT, IB, NXMAX, NYMAX, NFMAX
      REAL FLXTOT(NFMAX), FLXBAS(NXMAX,NYMAX,NFMAX)
      INTEGER NBASX, NBASY, NFLX
      LOGICAL ERRFLG
C
      INTEGER IFX, JO, KO
      LOGICAL EXFLAG
      CHARACTER*20 SEMAPH, PAKNAM
      CHARACTER*100 SEMFIL, PAKFIL
C
#ifdef PVM_LINK
      INTEGER BUFID, INFO
      REAL*8 RITEM(6)
#elif defined(MPI_LINK)
#include "mpif.h"
      INTEGER STAT(MPI_STATUS_SIZE), RFLX_SIZE, IERR
#endif
C
C     Clear error flag
      ERRFLG= .FALSE.
C     Check for valid ocean basin number
      IF (IB .GT. MAXBAS) STOP 'MAXBAS'
C
C     Generate semaphore/packet names for flux data
      CALL NAMGEN( IB, IPACKT, 'FLX',
     1             SEMAPH, PAKNAM, SEMFIL, PAKFIL )
C
      IF (IPKOPT .EQ. 2) THEN
C       PVM communication
#ifdef PVM_LINK
C       Synchronize; i.e., wait for other task
        CALL OSYNC( ERRFLG )
        IF (ERRFLG) RETURN
C       Receive packet from other task
        CALL PVMFRECV( OTHTID, -1, BUFID )
        IF (BUFID .LT. 0) THEN
          CALL PVMFPERROR( 'pvmfrecv', BUFID )
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Check packet size before unpacking
        IF ( (NOCNX1(IB) .GT. JOMAX) .OR.
     1       (NOCNY1(IB) .GT. KOMAX) .OR.
     2       (NFLX1(IB)  .GT. NFOCN)      ) THEN
          WRITE(*,*) 'RCVFLX: Packet too large for COMMON block'
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Unpack integer data (from reals)
        CALL PVMFUNPACK( RIOPT, RITEM, 6, 1, INFO )
        IPACK1(IB)= NINT(RITEM(1))
        ISTP1(IB)=  NINT(RITEM(2))
        NSYNC1(IB)= NINT(RITEM(3))
        NOCNX1(IB)= NINT(RITEM(4))
        NOCNY1(IB)= NINT(RITEM(5))
        NFLX1(IB)=  NINT(RITEM(6))
C       Unpack floating point data
        DO 210 IFX=1,NFLX1(IB)
          DO 210 KO=1,NOCNY1(IB)
  210       CALL PVMFUNPACK(RROPT,FLXOCN(1,KO,IFX,IB),NOCNX1(IB),1,INFO)
        CALL PVMFUNPACK( RROPT, FXTOCN(1,IB), NFLX1(IB), 1, INFO )
#endif
      ELSE IF (IPKOPT .EQ. 3) THEN
C       MPI communication
#ifdef MPI_LINK
C       Synchronize; i.e., wait for other task
        CALL OSYNC( ERRFLG )
        IF (ERRFLG) RETURN
C
C       Receive integer data
        CALL MPI_RECV( IFLX_BUF, IFLX_SIZE, MPI_INTEGER,
     $                 OTHERID, IFLX_TAG,
     $                 MPI_COMM_WORLD, STAT, IERR )
        IF (IERR .NE. MPI_SUCCESS) THEN
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C       Unpack integer data
        IPACK1(IB) = IFLX_BUF(1)
        ISTP1(IB)  = IFLX_BUF(2)
        NSYNC1(IB) = IFLX_BUF(3)
        NOCNX1(IB) = IFLX_BUF(4)
        NOCNY1(IB) = IFLX_BUF(5)
        NFLX1(IB)  = IFLX_BUF(6)
C
C       Check anticipated size of real data
        IF ( (NOCNX1(IB) .GT. JOMAX) .OR.
     1       (NOCNY1(IB) .GT. KOMAX) .OR.
     2       (NFLX1(IB)  .GT. NFOCN)     ) THEN
          WRITE(*,*) 'SNDFLX: Packet too large for COMMON block'
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C
C       Receive real data
        RFLX_SIZE= NFLX1(IB) * NOCNX1(IB) * NOCNY1(IB)
C
        CALL MPI_RECV( RFLX_BUF, RFLX_SIZE, MPI_REAL,
     $                 OTHERID, RFLX_TAG,
     $                 MPI_COMM_WORLD, STAT, IERR )
        IF (IERR .NE. MPI_SUCCESS) THEN
          ERRFLG= .TRUE.
          RETURN
        ENDIF
C
C       Unpack real data
        DO 210 IFX=1,NFLX1(IB)
          DO 210 KO=1,NOCNY1(IB)
            DO 210 JO=1,NOCNX1(IB)
              FLXOCN(JO,KO,IFX,IB) =
     $          RFLX_BUF(JO + NOCNX1(IB)*((KO-1)+NOCNY1(IB)*(IFX-1)) )
  210       CONTINUE
#endif
      ELSE IF (IPKOPT .EQ. 4) THEN
C         Disk file communication
C
#ifdef DEBUG
          WRITE(*,'(A,A)') 'RCVFLX: waiting for semaphore ', SEMFIL
#endif
C         Wait for atmospheric model to create FLUX semaphore
C         (one jump to this statement)
  301     INQUIRE(FILE=SEMFIL, EXIST=EXFLAG)
          IF (.NOT. EXFLAG) THEN
#ifdef ON_SUN4
            CALL SLEEP( 1 )
#endif
            GOTO 301
          ENDIF
C
C       Open flux data packet file
        OPEN(IUNIT, FILE=PAKFIL, FORM='UNFORMATTED', STATUS='OLD')
C       Read header
        READ(IUNIT) IPACK1(IB), ISTP1(IB), NSYNC1(IB)
        READ(IUNIT) NOCNX1(IB), NOCNY1(IB), NFLX1(IB)
C       Check packet size before reading further
        IF ( (NOCNX1(IB) .GT. JOMAX) .OR.
     1       (NOCNY1(IB) .GT. KOMAX) .OR.
     2       (NFLX1(IB)  .GT. NFOCN)      ) THEN
          WRITE(*,*) 'RCVFLX: Packet too large for COMMON block'
          ERRFLG= .TRUE.
          CLOSE( IUNIT )
          RETURN
        ENDIF
C       Read data
        READ(IUNIT)
     1   (((FLXOCN(JO,KO,IFX,IB),JO=1,NOCNX1(IB)),KO=1,NOCNY1(IB)),
     2                           IFX=1,NFLX1(IB))
        READ(IUNIT) (FXTOCN(IFX,IB),IFX=1,NFLX1(IB))
C       Close data packet file and delete it
        CLOSE(IUNIT, STATUS='DELETE' )
C
C       Open semaphore file
        OPEN(IUNIT, FILE=SEMFIL, FORM='UNFORMATTED', STATUS='OLD' )
C       Close semaphore file and delete it
        CLOSE(IUNIT, STATUS='DELETE')
      ENDIF
C
C     Check if the right packet has been received
      IF (IPACK1(IB) .NE. IPACKT) THEN
        WRITE(*,*) 'RCVFLX: Wrong packet received for ', PAKNAM
        ERRFLG= .TRUE.
        RETURN
      ENDIF
C     Check packet size before copying data
      IF ( (NOCNX1(IB) .GT. NXMAX) .OR.
     1     (NOCNY1(IB) .GT. NYMAX) .OR.
     2     (NFLX1(IB)  .GT. NFMAX)     ) THEN
        WRITE(*,*) 'RCVFLX: Packet too large for supplied arrays'
        ERRFLG= .TRUE.
        RETURN
      ENDIF
C     Extract data from packet common block
C     Copy header info
      NBASX= NOCNX1(IB)
      NBASY= NOCNY1(IB)
      NFLX= NFLX1(IB)
C     Copy fluxes
      DO 10 IFX=1,NFLX
        DO 10 KO=1,NBASY
          DO 10 JO=1,NBASX
   10       FLXBAS(JO,KO,IFX)= FLXOCN(JO,KO,IFX,IB)
C     Copy flux integral
      DO 20 IFX=1,NFLX
   20   FLXTOT(IFX)= FXTOCN(IFX,IB)
C
      IF (IPKOPT .EQ. 1) THEN
C       COMMON block communication; clear packet common block
        IPACK1(IB)= -1
      ENDIF
C
#ifdef DEBUG
      WRITE(*,*)
      WRITE(*,*) 'Subroutine RCVFLX: Debugging information'
      WRITE(*,'(1X,A,A)') 'Packet name: ', PAKNAM
      WRITE(*,'(1X,A,I2)') 'BASIN', IB
      WRITE(*,'(1X,A,3I10)') 'IPACK1,ISTP1,ISYNC1: ',
     1                        IPACK1(IB), ISTP1(IB), NSYNC1(IB)
      WRITE(*,'(1X,A,3I10)') 'NOCNX1,NOCNY1,NFLX1: ',
     1                        NOCNX1(IB), NOCNY1(IB), NFLX1(IB)
#endif
C
      RETURN
      END
