#ifndef MCONS
#   include "mcons.h"
#endif
C Physic $Revision: 3.5 $ created $Date: 1994/03/17 22:04:20 $
C ------------- Description of major subroutines -----------------------
C PHYINI -- Initializes physical parametrizations
C PHYCAL -- Computes physical parametrizations
C ------------- Description of C preprocessor parameters ---------------
C ------------- Description of common module mphys.h -------------------
C
C Physical parametrization details
C     REAL DPTADJ, SOLCON, CSURF, SATFAC, WINLW
C     REAL KLWAIR, KLWH2O, KSWAIR, KSWO3
C     COMMON/PHYPA1/ DPTADJ, SOLCON, CSURF, SATFAC, WINLW,
C    1               KLWAIR, KLWH2O, KSWAIR, KSWO3
C     REAL PTADJ, DPT2F, ZVISC
C     COMMON/PHYPA2/ PTADJ(L1MAX), DPT2F(L1MAX), ZVISC(L1MAX)
C     INTEGER NDRYAD, NDRYLV, NWETAD, NWETLV
C     COMMON/PHYPA3/ NDRYAD(K2MAX), NDRYLV(K2MAX),
C    1               NWETAD(K2MAX), NWETLV(K2MAX)
C     REAL COLPLV, COLDP, COLPS
C     COMMON/PHYPA4/ COLPLV(K1MAX,L1MAX), COLDP(K1MAX,L1MAX),
C    1               COLPS(K1MAX)
C
C DPTADJ -- Reference dTheta/dz for dry adiabatic adjustment.
C           i.e. dTheta/dz is adjusted to equal, or exceed, this value.
C           DPTADJ = 0 => constant Theta adjustment.
C SOLCON -- Solar constant (in W/m**2)
C CSURF -- Bulk aerodynamic surface exchange coefficient (non-dim)
C SATFAC -- Water vapour saturation threshold (non-dim)
C WINLW  -- Longwave window fraction (0 <= WINLW < 1)
C KLWAIR -- Longwave absorptivity of air (in 1/Pa)
C KLWH2O -- Longwave absorptivity of H2O (in 1/Pa)
C KSWAIR -- Shortwave absorptivity of air (in 1/Pa)
C KSWO3  -- Shortwave absorptivity of ozone (in 1/Pa)
C PTADJ(L)-- Dry adjustment reference Theta profile
C DPT2F(L)-- Theta tendency -> energy flux conversion factor
C ZVISC(L)-- Diffusion coefficient for vertical mixing (in Pa**2/s)
C NDRYAD(K)-- Count of dry adjustment iterations (cumulative)
C NDRYLV(K)-- Count of levels affected by dry adjustment (cumulative)
C NWETAD(K)-- Count of wet adjustment iterations (cumulative)
C NWETLV(K)-- Count of levels affected by wet adjustment (cumulative)
C COLPLV, COLDP, COLPS -- Copies of PLV(L), DP(L), PSURF for each column
C
C ------------------- End of comments ---------------------------------
C
      SUBROUTINE PHYINI( TSLOPE, SOLFLX, EXCH, SATURF, LWFRAC,
     1                   DLWAIR, DLWH2O, DSWAIR, DSWO3, VISC0 )
C PHYINI initializes the common blocks containing details of
C physical parametrizations.
C (PHYINI should be called after the vertical truncation details
C  have been initialized by calling VERINI)
C Input:  REAL TSLOPE -- Reference dTheta/dz for dry adiabatic
C                        adjustment. i.e. dTheta/dz is adjusted
C                        to equal, or exceed, this value.
C                        TSLOPE = 0 => constant Theta adjustment.
C           (TSLOPE is usually set to a small number like 1E-4)
C         REAL SOLFLX -- Solar constant (in W/m**2, usually around 1360)
C         REAL EXCH -- Bulk aerodynamic surface exchange coefficient
C                      (Non-dimensional, usually 1.1E-3)
C         REAL SATURF -- Water vapour saturation threshold
C           (True value is 1.0, but a value of 0.8 is often used
C            for the purpose of moist static/convective adjustment)
C         REAL LWFRAC -- Longwave window (LW) fraction (0 <= LWFRAC < 1)
C         REAL DLWAIR -- LW optical depth of well-mixed atmosphere
C         REAL DLWH2O -- LW optical depth of pure H2O atmosphere
C         REAL DSWAIR -- SW optical depth of well-mixed atmosphere
C         REAL DSWO3  -- SW optical depth of pure O3 atmosphere
C         REAL VISC0  -- Diffusion coefficient for vert. mixing (m**2/s)
C
C Output: Physical parametrization common blocks are initialized
C
#include "mgrid.h"
#include "mphys.h"
C
      REAL TSLOPE, SOLFLX, EXCH, SATURF, LWFRAC
      REAL DLWAIR, DLWH2O, DSWAIR, DSWO3, VISC0
C
      INTEGER J, K, L
C
C     Initialize slope of dry adjustment Theta profile
      DPTADJ= TSLOPE
C     Solar constant
      SOLCON= SOLFLX
C     Surface exchange coefficient
      CSURF= EXCH
C     Water vapour saturation threshold
      SATFAC= SATURF
C     Initialize Longwave window fraction
      WINLW= LWFRAC
C     Absorptivities
      KLWAIR= DLWAIR / PSURF
      KLWH2O= DLWH2O / PSURF
      KSWAIR= DSWAIR / PSURF
      KSWO3=  DSWO3  / PSURF
C
      DO 110 L=1,L1
C       Dry adjustment reference Theta profile
        PTADJ(L)= DPTADJ * (ZSTDLV(L) - ZSTDLV(L1))
C       Compute Theta tendency -> energy flux conversion factor
        DPT2F(L)= (CP/G0) * DP(L) * PKLV(L)
  110   CONTINUE
C     Diffusion coefficient for vertical mixing
      ZVISC(L1)= VISC0 * ((PHLV(L1)-PLV(L1))/ZSTDLV(L1))**2
      DO 120 L=1,L1-1
        ZVISC(L)= VISC0 * ( (PLV(L+1)    - PLV(L)   ) /
     1                    (ZSTDLV(L+1) - ZSTDLV(L))   )**2
  120   CONTINUE
C     Initialize dry/wet adjustment diagnostics
      DO 130 K=1,K2
        NDRYAD(K)= 0
        NDRYLV(K)= 0
        NWETAD(K)= 0
  130   NWETLV(K)= 0
C     Duplicate pressure, thickness values for each column
      DO 140 L=1,L1
        DO 140 J=1,K1
          COLPLV(J,L)= PLV(L)
  140     COLDP(J,L)= DP(L)
C     Duplicate surface pressure values for each column
      DO 150 J=1,K1
  150   COLPS(J)= PSURF
      RETURN
      END
 
      SUBROUTINE PHYCAL( DPGQSP, CDRAG, QEVAP, MSAPCP, MCAPCP,
     1                   FEVAP, FSENS, FRAD, FNET, FPSSRF, FMSSRF,
     2                   FPLSRF, FMLSRF, FPSTOP, FPLTOP,
     3                   SWHET, LWHET, MSAHET, MCAHET, DCAHET,
     4                   PGQSP, SEAMSK, ICEMSK, SOLRAD, ALBEDO,
     5                   TSURF, RHSURF, ULOW, CO2FAC, DTRAD,
     6                   LWET, LDRY)
C PHYCAL computes physical parametrization tendencies for all
C prognostic quantities PGQSP.
C
C Input:  COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C          original spectral tendency of prognostic quantities
C         COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C          spectral representation of prognostic quantities at all
C          levels at the appropriate model time
C         LOGICAL SEAMSK(K1MAX,K2MAX) -- sea mask (.TRUE. over sea)
C         LOGICAL ICEMSK(K1MAX,K2MAX) -- ice mask (.TRUE. over ice)
C         REAL SOLRAD(K1MAX,K2MAX) -- incident solar radiation (W/m**2)
C         REAL ALBEDO(K1MAX,K2MAX) -- surface albedo (0 .. 1)
C         REAL TSURF(K1MAX,K2MAX) -- surface temperature (in K)
C         REAL RHSURF(K1MAX,K2MAX) -- surface relative humidity (0 .. 1)
C         REAL ULOW(K1MAX,K2MAX,JX:JY) -- lowest level wind components
C         REAL CO2FAC -- increased "CO2" factor (> 0, usually = 1)
C         REAL DTRAD -- effective radiative-convective time-step
C         INTEGER LWET -- highest level for moist convective adjustment
C         INTEGER LDRY -- lowest level for dry convective adjustment
C Output: COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX) --
C          spectral tendency of prognostic quantities at all
C          levels, with the effects of radiative heating and
C          static/convective adjustment added to the original values
C         REAL CDRAG(K1MAX,K2MAX) -- surface drag coefficient (in 1/s)
C         REAL QEVAP(K1MAX,K2MAX) -- evaporative flux of water vapour
C                                    (in kg/(s*m**2))
C         REAL MSAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                static adjustment (in kg/(s*m**2))
C         REAL MCAPCP(K1MAX,K2MAX) -- precipitation due to moist
C                                convective adjustment (in kg/(s*m**2))
C         REAL FEVAP(K1MAX,K2MAX) -- heat flux associated with the
C                                    evaporative flux of water vapour
C                                    (in W/m**2)
C         REAL FSENS(K1MAX,K2MAX) -- upward surface sensible heat flux
C                                    (in W/m**2)
C         REAL FRAD(K1MAX,K2MAX) -- upward surface radiative heat flux
C                                   (in W/m**2)
C         REAL FNET(K1MAX,K2MAX) -- Net heat flux into land/sea
C                                   (in W/m**2)
C         REAL FPSSRF(K1MAX,K2MAX) -- upward SW radiation at surface
C         REAL FMSSRF(K1MAX,K2MAX) -- downward SW radiation at surface
C         REAL FPLSRF(K1MAX,K2MAX) -- upward LW radiation at surface
C         REAL FMLSRF(K1MAX,K2MAX) -- downward LW radiation at surface
C         REAL FPSTOP(K1MAX,K2MAX) -- upward SW radiation at model top
C         REAL FPLTOP(K1MAX,K2MAX) -- upward LW radiation at model top
C         REAL SWHET(K2MAX,L1MAX)  -- zonally-averaged vertical profile
C              of heating due to SW radiation (in K/s)
C         REAL LWHET(K2MAX,L1MAX)  -- zonally-averaged vertical profile
C              of heating due to LW radiation (in K/s)
C         REAL MSAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to moist static adjustment (in K/s)
C         REAL MCAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to moist convective adjustment (in K/s)
C         REAL DCAHET(K2MAX,L1MAX) -- zonally-averaged vertical profile
C              of heating due to dry convective adjustment (in K/s)
C
#include "mgrid.h"
#include "mphys.h"
      COMPLEX DPGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      COMPLEX PGQSP(0:M1MAX,0:N1MAX,NPGQ,L1MAX)
      REAL CDRAG(K1MAX,K2MAX), QEVAP(K1MAX,K2MAX)
      REAL MSAPCP(K1MAX,K2MAX), MCAPCP(K1MAX,K2MAX)
      REAL FEVAP(K1MAX,K2MAX), FSENS(K1MAX,K2MAX)
      REAL FRAD(K1MAX,K2MAX), FNET(K1MAX,K2MAX)
      REAL FPSSRF(K1MAX,K2MAX), FMSSRF(K1MAX,K2MAX)
      REAL FPLSRF(K1MAX,K2MAX), FMLSRF(K1MAX,K2MAX)
      REAL FPSTOP(K1MAX,K2MAX), FPLTOP(K1MAX,K2MAX)
      REAL SWHET(K2MAX,L1MAX), LWHET(K2MAX,L1MAX)
      REAL MSAHET(K2MAX,L1MAX), MCAHET(K2MAX,L1MAX)
      REAL DCAHET(K2MAX,L1MAX)
      REAL SOLRAD(K1MAX,K2MAX), ALBEDO(K1MAX,K2MAX)
      REAL TSURF(K1MAX,K2MAX), RHSURF(K1MAX,K2MAX)
      REAL ULOW(K1MAX,K2MAX,JX:JY), CO2FAC, DTRAD
      INTEGER LWET, LDRY
      LOGICAL SEAMSK(K1MAX,K2MAX), ICEMSK(K1MAX,K2MAX)
 
C
      REAL TCOL(L1MAX), QCOL(L1MAX)
      REAL QSCOL(L1MAX), DQSCOL(L1MAX), GAMCOL(L1MAX-1)
      REAL PTOLD(K1MAX,L1MAX), T(K1MAX,L1MAX), T0(K1MAX,L1MAX)
      REAL QOLD(K1MAX,L1MAX), Q(K1MAX,L1MAX), Q0(K1MAX,L1MAX)
      REAL QS(K1MAX,L1MAX), DQSDT(K1MAX,L1MAX), GAMMAS(K1MAX,L1MAX-1)
      REAL VLOW(K1MAX,2), O3(K1MAX,L1MAX), FBOT(K1MAX), QTOP(K1MAX)
      REAL HEAT(K1MAX,L1MAX), DIFTEN(K1MAX,L1MAX), LWHET0, SWHET0
      REAL FPSW(K1MAX,0:L1MAX), FMSW(K1MAX,0:L1MAX)
      REAL FPLW(K1MAX,0:L1MAX), FMLW(K1MAX,0:L1MAX)
      REAL KAPA(K1MAX,L1MAX), FAC(K1MAX,L1MAX,4), DELTAT(L1MAX)
      REAL DELPGQ(K1MAX,L1MAX)
      REAL ES, DESDT, THLV
      INTEGER IADJ, IADLEV, J, K, L
      LOGICAL SATFLG(K1MAX,L1MAX), CIFLAG(K1MAX,L1MAX-1)
      LOGICAL SATCOL(L1MAX), CIFCOL(L1MAX-1)
      LOGICAL MCAFLG(K1MAX), DCAFLG(K1MAX)
C     Stefan's constant (in W/(m**2*K**4))
      REAL STEFAN
      PARAMETER (STEFAN= 5.669E-8)
C     Molecular weights ofr air, water, ozone (kg/kgmole)
      REAL AIRWT, H2OWT, O3WT
      PARAMETER (AIRWT=28.97, H2OWT=18.016, O3WT=48.)
C     Coefficients for water-air systems
C     (R, Cp for dry air (in J/(kg*K)),
C      latent heat of condendation (in J/kg) )
      REAL RDRY, CPDRY, LHEAT
      PARAMETER (RDRY= 287.0, CPDRY=1004.0, LHEAT=2.5E6)
C     Coefficients for Clausius-Clapeyron relation
      REAL CCFAC0, CCFAC1
      PARAMETER (CCFAC0=26.2612, CCFAC1=5420.3)
C     Derived coefficients
      REAL LBYCP, RBYCP
      PARAMETER (LBYCP=LHEAT/CPDRY, RBYCP=RDRY/CPDRY)
      REAL H2OAIR, O3AIR
      PARAMETER (H2OAIR=H2OWT/AIRWT, O3AIR=O3WT/AIRWT)
C
      REAL PI
      PARAMETER (PI=3.141592653589793)
      REAL GBYCP
C
      GBYCP= G0/CP
C
C     No moisture flux at top
      DO 120 J=1,K1
  120   QTOP(J)= 0.0
C     No ozone
      DO 130 L=1,L1
        DO 130 J=1,K1
  130     O3(J,L)= 0.0
C
C     Initialize precipitation diagnostics
      DO 140 K=1,K2
        DO 140 J=1,K1
          MSAPCP(J,K)=0.0
  140     MCAPCP(J,K)=0.0
C     Initialize zonal-mean heating rate diagnostics
      DO 150 L=1,L1
        DO 150 K=1,K2
          LWHET(K,L)= 0.0
          SWHET(K,L)= 0.0
          MSAHET(K,L)= 0.0
          MCAHET(K,L)= 0.0
          DCAHET(K,L)= 0.0
  150     CONTINUE
C     Loop over latitudes
      DO 700 K=1,K2
C       Convert specific humidity to physical space
        CALL VPHYS(QOLD, PGQSP(0,0,JTR1,1), K, NPGQ, L1)
C       Convert Theta to physical space
        CALL VPHYS(PTOLD, PGQSP(0,0,JPOT,1), K, NPGQ, L1)
C       Convert Theta to Temperature
        DO 210 L=1,L1
          DO 210 J=1,K1
  210       T(J,L)= PTOLD(J,L) * PKLV(L)
        DO 220 J=1,K1
C         Initialize surface winds
          VLOW(J,1)= ULOW(J,K,JX)
          VLOW(J,2)= ULOW(J,K,JY)
  220     CONTINUE
C       Compute surface fluxes
        CALL SURFAC( CDRAG(1,K), QEVAP(1,K), FEVAP(1,K), FSENS(1,K),
     1               FRAD(1,K), T(1,L1), T(1,L1-1), QOLD(1,L1), VLOW,
     2               COLPLV, K1MAX, K1, L1, TSURF(1,K), RHSURF(1,K),
     3               COLPS, G0, CSURF )
C       Upward LW flux at bottom boundary
        DO 230 J=1,K1
  230     FBOT(J)= FRAD(J,K)
C       Compute radiative heating rates
        CALL RADIAT( FPSW, FMSW, FPLW, FMLW,
     1               T, QOLD, O3, COLDP, K1MAX, K1, L1,
     2               .TRUE., FBOT, SOLRAD(1,K), ALBEDO(1,K), WINLW,
     3               CO2FAC*KLWAIR, KLWH2O, KSWAIR, KSWO3,
     4               KAPA, FAC )
        DO 240 J=1,K1
C         Copy flux values at bottom/top boundaries
          FPSSRF(J,K)= FPSW(J,L1)
          FMSSRF(J,K)= FMSW(J,L1)
          FPLSRF(J,K)= FPLW(J,L1)
          FMLSRF(J,K)= FMLW(J,L1)
          FPSTOP(J,K)= FPSW(J,0)
          FPLTOP(J,K)= FPLW(J,0)
C         Compute net heat flux into land/sea
          FNET(J,K)= (  FMSW(J,L1) - FPSW(J,L1)
     1                + FMLW(J,L1) - FPLW(J,L1)
     2                - FSENS(J,K) - FEVAP(J,K) )
  240     CONTINUE
C       Compute water vapour diffusion tendencies
        CALL DIFFUS( DIFTEN, QOLD, COLPLV, COLDP, K1MAX, K1, L1,
     1               ZVISC, QEVAP(1,K), QTOP, G0, FAC )
C
C       Compute heating rates in the lowest level
        DO 300 J=1,K1
C         Compute LW heating rate
          LWHET0=  (   FPLW(J,L1) - FPLW(J,L1-1)
     1               -(FMLW(J,L1) - FMLW(J,L1-1)) )
     2           *GBYCP/COLDP(J,L1)
C         Compute SW heating rate
          SWHET0=  (   FPSW(J,L1) - FPSW(J,L1-1)
     1               -(FMSW(J,L1) - FMSW(J,L1-1)) )
     2           *GBYCP/COLDP(J,L1)
C         Compute net radiative + sensible heating rate
          HEAT(J,L1)= LWHET0 + SWHET0 + GBYCP*FSENS(J,K)/COLDP(J,L1)
C         Radiation, diffusion time-step (forward)
          T(J,L1)= T(J,L1) + DTRAD * HEAT(J,L1)
          Q(J,L1)= QOLD(J,L1) + DTRAD * DIFTEN(J,L1)
C         Accumulate heating rate diagnostics
          LWHET(K,L1)= LWHET(K,L1) + LWHET0/K1
          SWHET(K,L1)= SWHET(K,L1) + SWHET0/K1
  300     CONTINUE
C       Compute heating rates in all other levels
        DO 310 L=1,L1-1
          DO 310 J=1,K1
C           Compute LW heating rate
            LWHET0=  (   FPLW(J,L) - FPLW(J,L-1)
     1                 -(FMLW(J,L) - FMLW(J,L-1)) )
     2             *GBYCP/COLDP(J,L)
C           Compute SW heating rate
            SWHET0=  (   FPSW(J,L) - FPSW(J,L-1)
     1                 -(FMSW(J,L) - FMSW(J,L-1)) )
     2             *GBYCP/COLDP(J,L)
C           Compute net radiative heating rate
            HEAT(J,L)= LWHET0 + SWHET0
C           Radiation, diffusion time-step (forward)
            T(J,L)= T(J,L) + DTRAD * HEAT(J,L)
            Q(J,L)= QOLD(J,L) + DTRAD * DIFTEN(J,L)
C           Accumulate heating rate diagnostics
            LWHET(K,L)= LWHET(K,L) + LWHET0/K1
            SWHET(K,L)= SWHET(K,L) + SWHET0/K1
  310       CONTINUE
C       Copy temperature and moisture values
        DO 320 L=1,L1
          DO 320 J=1,K1
            T0(J,L)= T(J,L)
  320       Q0(J,L)= Q(J,L)
C       Compute initial saturation mixing ratios, and set flag
        DO 330 L=LWET,L1
          DO 330 J=1,K1
C           Saturation vapor pressure
C           (An attenuation factor of SATFAC is used)
            ES= SATFAC*EXP(CCFAC0-CCFAC1/T(J,L))
C           dES/dT
            DESDT= ES*(CCFAC1/T(J,L)**2)
C           Saturation mixing ratio QS
            QS(J,L)= H2OAIR*ES
     1              /AMAX1(PLV(L), PLV(L) - ES)
C           dQS/dT
            DQSDT(J,L)= H2OAIR*PLV(L)*DESDT
     1                 /AMAX1(PLV(L), PLV(L) - ES)**2
C           Set saturation flag
            SATFLG(J,L)= Q(J,L) .GT. QS(J,L)
  330       CONTINUE
        DO 340 L=1,LWET-1
          DO 340 J=1,K1
C           Assume levels above LWET are unsaturated
            QS(J,L)= 0.0
            DQSDT(J,L)= 0.0
            SATFLG(J,L)= .FALSE.
  340       CONTINUE
C       Perform moist static adjustment
        CALL MSADJ( T, Q, QS, DQSDT, SATFLG,
     1              COLPLV, K1MAX, K1, L1, SATFAC )
C       Compute effect of MSA, and copy temperature and moisture values
        DO 350 L=1,L1
          DO 350 J=1,K1
            MSAHET(K,L)= MSAHET(K,L) + (  (Q0(J,L) - Q(J,L))*LBYCP
     1                                  + (T(J,L) - T0(J,L)) )
     2                                /(K1*DTRAD)
            MSAPCP(J,K)= MSAPCP(J,K) + DP(L)*(Q0(J,L) - Q(J,L))
     1                                /(G0*DTRAD)
            T0(J,L)= T(J,L)
  350       Q0(J,L)= Q(J,L)
C       Perform moist convective adjustment from level LWET downward
        IF (LWET .LT. L1) THEN
C         Compute initial moist adiabatic lapse rate (at L+1/2)
          DO 360 L=LWET,L1-1
            DO 360 J=1,K1
C             Half-level temperature
              THLV= 0.5 * (T(J,L) + T(J,L+1))
C             Saturation vapor pressure for evaporation
              ES= SATFAC*EXP(CCFAC0-CCFAC1/THLV)
C             dES/dT
              DESDT= ES*(CCFAC1/THLV**2)
C             Moist adiabatic lapse rate (at half-levels)
              GAMMAS(J,L)=  RBYCP*(THLV/PHLV(L))
     1                  * (PHLV(L) + H2OAIR*LBYCP*ES/(RBYCP*THLV) )
     2                  / (PHLV(L) + H2OAIR*LBYCP*DESDT )
C             Compute convective instability flag
C             Convergence criterion is 0.01
              CIFLAG(J,L)= ( (T(J,L+1)-T(J,L)) .GT.
     1                       (0.01 + GAMMAS(J,L)*(PLV(L+1) - PLV(L))) )
     2                 .AND. ( SATFLG(J,L+1) .OR.
     3                       (Q(J,L+1) .GE. QS(J,L+1)) )
  360       CONTINUE
C         Determine whether there is any moist convective instability at
C         each latitude
          DO 410 J=1,K1
  410       MCAFLG(J)= .FALSE.
          DO 420 L=LWET,L1-1
            DO 420 J=1,K1
  420         MCAFLG(J)= MCAFLG(J) .OR. CIFLAG(J,L)
          DO 460 J=1,K1
            IF (MCAFLG(J)) THEN
C             Copy column values
              DO 430 L=LWET,L1
                TCOL(L)= T(J,L)
                QCOL(L)= Q(J,L)
                QSCOL(L)= QS(J,L)
                DQSCOL(L)= DQSDT(J,L)
  430           SATCOL(L)= SATFLG(J,L)
              DO 440 L=LWET,L1-1
                GAMCOL(L)= GAMMAS(J,L)
  440           CIFCOL(L)= CIFLAG(J,L)
C             Perform moist convective adjustment
              CALL MCADJ( TCOL(LWET), QCOL(LWET), QSCOL(LWET),
     1                    DQSCOL(LWET), SATCOL(LWET), GAMCOL(LWET),
     2                    IADJ, IADLEV, CIFCOL(LWET), PLV(LWET),
     3                    PHLV(LWET-1), DP(LWET), L1-LWET+1, SATFAC,
     4                    DELTAT(LWET) )
              NWETAD(K)= NWETAD(K) + IADJ
              NWETLV(K)= NWETLV(K) + IADLEV
C             Copy column values back
              DO 450 L=LWET,L1
                T(J,L)= TCOL(L)
                Q(J,L)= QCOL(L)
                QS(J,L)= QSCOL(L)
                DQSDT(J,L)= DQSCOL(L)
  450           SATFLG(J,L)= SATCOL(L)
            ENDIF
  460       CONTINUE
        ENDIF
 
C       Compute effect of MCA, and copy temperature and moisture values
        DO 470 L=1,L1
          DO 470 J=1,K1
            MCAHET(K,L)= MCAHET(K,L) + (  (Q0(J,L) - Q(J,L))*LBYCP
     1                                  + (T(J,L) - T0(J,L)) )
     2                                /(K1*DTRAD)
            MCAPCP(J,K)= MCAPCP(J,K) + DP(L)*(Q0(J,L) - Q(J,L))
     1                                /(G0*DTRAD)
            T0(J,L)= T(J,L)
  470       Q0(J,L)= Q(J,L)
C       Perform dry convective adjustment from level LDRY upward
        IF (LDRY .GT. 1) THEN
C         Convert Temperature to Theta, and subtract ref. Theta
          DO 510 L=1,LDRY
            DO 510 J=1,K1
  510         T(J,L)= T(J,L)/PKLV(L) - PTADJ(L)
C         Determine whether there is any dry convective instability at
C         each latitude
          DO 520 J=1,K1
  520       DCAFLG(J)= .FALSE.
          DO 530 L=1,LDRY-1
            DO 530 J=1,K1
  530         DCAFLG(J)= DCAFLG(J) .OR. (T(J,L+1) .GT. T(J,L))
          DO 560 J=1,K1
            IF (DCAFLG(J)) THEN
C             Copy column values
              DO 540 L=1,LDRY
  540           TCOL(L)= T(J,L)
C             Perform dry adiabatic adjustment
              CALL DCADJ( TCOL, IADJ, IADLEV, DPT2F, LDRY )
              NDRYAD(K)= NDRYAD(K) + IADJ
              NDRYLV(K)= NDRYLV(K) + IADLEV
C             Copy column values back
              DO 550 L=1,LDRY
  550           T(J,L)= TCOL(L)
            ENDIF
  560       CONTINUE
C         Convert Theta back to temperature
          DO 570 L=1,LDRY
            DO 570 J=1,K1
  570         T(J,L)= (T(J,L) + PTADJ(L)) * PKLV(L)
        ENDIF
C       Compute effect of DCA
        DO 580 L=1,L1
          DO 580 J=1,K1
            DCAHET(K,L)= DCAHET(K,L) + (T(J,L) - T0(J,L))
     1                                /(K1*DTRAD)
  580       CONTINUE
C
C       Compute Theta tendency due to radiative-convective adjustment
        DO 610 L=1,L1
          DO 610 J=1,K1
  610       DELPGQ(J,L)= (T(J,L)/PKLV(L) - PTOLD(J,L)) / DTRAD
C       Convert Theta tendency to spectral space
        CALL VSPEC(DPGQSP(0,0,JPOT,1), DELPGQ, K, NPGQ, L1)
C       Compute specific humidity tendency due to rad-con adjustment
        DO 620 L=1,L1
          DO 620 J=1,K1
  620       DELPGQ(J,L)= (Q(J,L) - QOLD(J,L)) / DTRAD
C       Convert specific humidity tendency to spectral space
        CALL VSPEC(DPGQSP(0,0,JTR1,1), DELPGQ, K, NPGQ, L1)
  700   CONTINUE
C ----------------------------------------------------------------------
      RETURN
      END
