#ifndef SPCONS
#   include "spcons.h"
#endif
C Splib -- A Spherical-harmonic transforms package
C          version 6.2 created 23 April 1996
C
C Author: R. Saravanan, D.A.M.T.P., University of Cambridge,
C         Silver Street, Cambridge CB3 9EW, U.K.
C         E-mail: svn@atm.amtp.cam.ac.uk
C (Please send bug-reports/comments to above address)
C
C --------------------- Beginning of Preface ---------------------------
C
C The Splib package consists of the following modules-
C   spcons.h -- include module defining truncation parameters
C   spgrid.h -- common module containing details of the transform-grid
C   sppoly.h -- common module containing Legendre polynomial tables
C   spfftb.h -- common module for table used by the FFT routines
C   splib.F  -- Fortran77 module containing transform routines
C
C DEPENDENCY NOTE: splib.F uses three routines from VFFTPK v2.1,
C                  the vectorized version of the NCAR FFT package.
C                  VFFTPK can be obtained by sending a mail message to
C                  netlib@research.att.com containing the single message
C                  line "send all from vfftpk". The routines used are
C                  as follows: VRFFTI, VRFFTF, and VRFFTB.
C
C The modules contain references to the following C-preprocessor
C parameters:
C
C DPRECISION: This parameter, if defined, indicates that the default
C        floating point precision on the machine for REAL variables
C        is equivalent to REAL*8. This parameter is needed because some
C        of the initializations need to be carried out at double
C        precision on 32-bit machines. This could be avoided on 64-bit
C        machines by defining this parameter.
C
C UDEF:  The whole package is supposed to be compilable with the
C        IMPLICIT NONE option (f77 -u). A few dummy declarations were
C        needed to achieve this on UNIX compilers. These dummy
C        declarations may potentially generate errors on other
C        compilers. Each of these declarations is preceded by a comment
C        line of the form:
C  C Dummy declaration (sometimes needed with the IMPLICIT NONE option)
C       Defining preprocessor option UDEF enables these declarations.
C
C COMPLEXOP: This parameter, if defined, indicates that complex
C        arithmetic operations may be used during the Legendre
C        transform operations. The default is to use real arithmetic
C        operations only.
C
C COMPACTPOLY: This parameter, if defined, indicates that the Legendre
C        polynomial tables should be stored in a compact form, to
C        minimize memory requirements. By default, the polynomial
C        tables are stored in a "twinned" form, for efficient real
C        arithmetic operations.
C        Notes: 1. Defining COMPLEXOP automatically causes COMPACTPOLY
C                  to be defined.
C               2. Defining COMPACTPOLY without defining COMPLEXOP
C                  causes polynomial tables to be stored with the
C                  n-index, rather than the m-index, varying most
C                  rapidly. The order of the two innermost loops in
C                  the Legendre transform is also reversed. This may be
C                  more efficient for non-triangular truncations
C                  with very few zonal wavenumbers.
C
C FIXTRUNC: This parameter, if defined, fixes the truncation order
C        to be (N1MAX,M1MAX) at compile time. This may lead to slightly
C        more efficient execution of code using only fixed truncation.
C
C SINGLELAT: This parameter, if defined, causes the Legendre polynomial
C        values to be computed at each latitude circle as and when
C        required. If SINGLELAT is not defined, then Legendre
C        polynomials for all latitudes are calculated initially
C        and stored in a table, allowing all-latitudes-at-a-time
C        transforms. But for triangular truncation orders exceeding T42,
C        the memory required for such a table may be prohibitive.
C        Defining SINGLELAT would considerably reduce storage
C        requirements in this case. But several routines that perform
C        transforms at all latitudes at a time would become
C        unavailable. Also, the user should take care to bunch
C        together all transforms at the same latitude, to avoid
C        redundant Legendre polynomial computations.
C
C (The following integer parameters have default values, which can be
C  overridden by explicit definitions)
C
C N1MAX: This is the maximum order of triangular truncation (i.e.
C        order of Legendre polynomials). The default value is 21.
C
C M1MAX: This is the maximum value of zonal wavenumber resolved by the
C        truncation. The default value is N1MAX.
C
C K1MAX: Number of equally spaced longitudes in the transform grid
C        [should be the lowest integral power of 2 >= (3*N1MAX+1) ].
C        The default value is 64.
C
C K2MAX: Number of gaussian latitudes in the transform grid [should
C        be the lowest integer >= (3*N1MAX+1)/2. The default value
C        is 32.
C (NOTE: If K2MAX is an odd number, then K2 is always chosen to be
C  an odd number, so that the equator is one of the gaussian latitudes.
C  Similarly, if K2MAX is an even number, then K2 is always chosen to be
C  an even number.)
C
C [Other typical choices of (N1MAX, M1MAX, K1MAX, K2MAX) would be
C     (10, 10,  32, 16) or (21, 21,  64,  32)
C  or (42, 42, 128, 64) or (85, 85, 256, 128) ]
C
C KL0MAX: Maximum of either K2MAX or the number of model levels.
C         The default value is K2MAX.
C (NOTE: For a multi-level model, this allows transforms to be
C   optionally vectorized over levels, rather than latitudes)
C
C (The above options can be defined, e.g., by the command
C   "f77 -DDPRECISION -DM1MAX=0 -DK1MAX=1 ...")
C
C Bugs:
C 1. Some subroutines accept subroutine-names as parameters. This seems
C    to generate spurious warning messages on some compilers.
C 2. As described before, passing subroutine-names as parameters
C    produces error messages when compiled with the f77 "-u" option.
C    As a fix, dummy declarations are introduced declaring these
C    subroutine-names as INTEGERs.
C 3. The C preprocessor sometimes fails to substitute for parameters.
C    This probably has to do with differing syntactic conventions for
C    for Fortran and C. When that happens, enclosing the relevant
C    parameter in a redundant pair of parentheses seems to help.
C
C ----------- The Spherical Harmonic Spectral Transforms --------------
C
C The notation and the method are based on "The Spectral Method" by
C Bennert Machenhauer (Chapter 3 of "Numerical Methods used in
C Atmospheric Models", vol.2, G.A.R.P. publications series no.17,
C W.M.O.(1979).
C
C The routines in this package use triangular truncation of order N1
C i.e. N = 0,...,N1  and M = -N,...,+N.
C the range of index M can be further restricted by specifying
C another parameter M1 that may be less than N1. Then we have
C                        M = -MIN(N,M1),...,+MIN(N,M1)
C e.g. The extreme case of M1=0 would correspond to axially
C      symmetric truncation.
C
C LAMBDA denotes longitude and PHI denotes latitude. Instead of PHI,
C we prefer to use MU = SIN(PHI). In the discussion that follows,
C we will use the term "latitude" to denote MU, rather than PHI.
C LAMBDA = 0 ... 2*PI, PHI = -PI/2 ... PI/2, MU = -1 ... +1
C
C We can expand a general function F(LAMBDA,MU) as
C   F(LAMBDA,MU) = Sigma(N=0 to N1) Sigma(M=-N to N) F[M,N] * Y[M,N]
C where F[M,N] are complex coefficients, Y[M,N] are the spherical
C harmonics,
C       Y[M,N](LAMBDA,MU) = P[M,N](MU) * EXP(I*M*LAMBDA),
C and P[M,N] are the associated legendre polynomials.
C The spherical harmonics satisfy the normalization condition
C (1/(4*PI)) * <<CONJG(Y[M,N])*Y[M2,N2]>> = Kronecker-delta[M,M2][N,N2]
C where << ... >> denotes double integration over MU, LAMBDA.
C
C Note: For components of vectors, The truncated series will be one
C       order higher in N as compared to scalars. i.e. Order N1+1
C
C For a real function F(LAMBDA,MU), we have the relation
C       F[-M,N] = (-1)**M * CONJG(F[M,N])
C So we need to store the coefficients F[M,N] only for M >= 0
C
C                    **** Transform grid ****
C
C To use these routines, we must choose the truncation order N1
C and the transform grid of K1 longitudes and K2 latitudes.
C i.e. We have the following ranges
C N = 0,...,N1        (upto N1+1 for vector components)
C M = 0,...,MIN(N,M1) (because we are interested in real functions)
C M2= 0,...,2*N+1     (This strange index is used only for storing the
C Legendre polynomials and their derivatives. Since they always occur
C in the form complex-number * real-polynomial, one can treat the
C complex multiplication as two real multiplications [for efficiency
C and vectorization]. So the polynomial values are stored in duplicate)
C LAMBDA[J] = 2*PI*(J-1)/K1, J = 1,...,K1  (longitudes)
C MU[K],                     K = 1,...,K2  (gaussian latitudes)
 
C Any real function F(LAMBDA,MU) defined on a sphere can therefore
C be represented in three different 2-D spaces:
C 1. Spectral space: Here the function is represented by its spectral
C    coefficients F[M,N]
C    (here F[-M,N] = (-1)**M * CONJG( F[M,N] ) )
C 2. Latitude space: Here the function is represented by its
C    latitude-dependent fourier coefficient F[M](MU[K])
C    (here F[-M](MU[K]) = CONJG( F[M](MU[K]) ) )
C 3. Physical space: Here the function is represented by its values
C    at the grid points F(LAMBDA[J],MU[K])
C
C The routines in this package basically enable you to move from
C one representation to another
C
C --------------- representation of a real function F ----------------
C
C For the transform operations, it would be convenient to declare the
C various representations of function F in the following manner:
C 1. Spectral space-
C       COMPLEX FSP(0:M1MAX,0:N1MAX)
C       (declare as FSP(0:M1MAX,0:N2MAX) for vector components)
C 2. Latitude space-
C       COMPLEX FLA(0:M1MAX,K2MAX)
C    For a single latitude, i.e., single value of K, declare as
C       COMPLEX FLAK(0:M1MAX)
C 3. Physical space-
C       REAL FPH(K1MAX,K2MAX)
C    For a single latitude, i.e., single value of K, declare as
C       REAL FPHK(K1MAX)
C
C ---------------- Description of various modules -------------------
C
C ---------------------- module spgrid.h ----------------------------
C All information about the spectral truncation is contained in
C common module spgrid.h in the following form
C
C Basic common module for sptransforms
#ifndef FIXTRUNC
C     INTEGER M1, N1, K1, K2, N2
C     COMMON/SPGRD1/ M1, N1, K1, K2, N2
#endif
C
C     REAL DEGLON, DEGLAT, LAMBDA, PHI, MU, COSPHI, COSINV, G, D
C     COMPLEX CIM
C     COMMON/SPGRD2/ DEGLON(K1MAX), DEGLAT(K2MAX),
C    1 LAMBDA(K1MAX), PHI(K2MAX), MU(K2MAX), COSPHI(K2MAX),
C    2 COSINV(K2MAX), G(K2MAX), D(0:M1MAX, 0:N2MAX), CIM(0:M1MAX)
C
C     REAL A0, A0Q, A0INV, A0QINV
C     COMMON/SPGRD3/ A0, A0Q, A0INV, A0QINV
C
C Polynomial table size parameters
C     INTEGER MN1MAX, MN2MAX, NP2MAX, NH2MAX
C     PARAMETER (MN1MAX= ((N1MAX+1)*(N1MAX+2)
C    1                  -(N1MAX-M1MAX)*(N1MAX-M1MAX+1))/2 - 1)
C     PARAMETER (MN2MAX=MN1MAX+M1MAX+1)
C
#ifdef COMPACTPOLY
C     PARAMETER (NP2MAX=MN2MAX, NH2MAX=MN1MAX)
#else
C     PARAMETER (NP2MAX=2*MN2MAX+1, NH2MAX=2*MN1MAX+1)
#endif
C
C (M1MAX, N1MAX, K1MAX, K2MAX) are parameters defining maximum storage.
C  They correspond to maximum values of M1, N1, K1, K2.
C
#ifndef FIXTRUNC
C M1        = maximum value of zonal wavenumber M
C N1        = order of triangular truncation  (for scalars)
C K1        = no. of equally spaced longitudes   ( K1 >= 3*M1 + 1 )
C (K1 is usually chosen to contain prime factors 2 or 3 only)
C K2        = no. of gaussian latitudes          ( K2 >= (3*N1 + 1)/2 )
C (If K2MAX is an odd number, K2 is always chosen to be an odd number)
C (If K2MAX is an even number, K2 is always chosen to be an even number)
C N2        = N1 + 1  (order of truncation for vector components)
#endif
C
C DEGLON(J) = longitude values (in degrees, 0 ... 360)
C DEGLAT(K) = gaussian latitude values (in degrees, -90 ... +90)
C LAMBDA(J) = longitude values (in radians, 0 ... 2*Pi)
C PHI(K)    = gaussian latitude values (in radians, -Pi/2 ... +Pi/2)
C MU(K)     = gaussian "latitude" values ( SIN(PHI(K)), -1 ... +1 )
C COSPHI(K) = cosine of latitude ( COS(PHI(K)), 0 ... 1 )
C COSINV(K) = reciprocal of cosine of latitude ( 1 / COS(PHI(K)) )
C G(K)      = gaussian quadrature weights (with factor of 1/2 included)
C D(M,N)    = coefficients of recurrence relation for H[M,N]
C CIM(M) = CMPLX(0.0, M)  (coefficient for evaluating LAMBDA-derivative)
C
C A0        = planetary radius (in metres)
C A0Q       = A0**2
C A0INV     = 1 / A0
C A0QINV    = 1 / A0Q
C
C (MN1MAX, MN2MAX are derived parameters defining the maximum number of
C  polynomial values that can be stored. NP2MAX, NH2MAX are similar
C  parameters, except that they take into account any duplication
C  of values.)
C
C ---------------------- module sppoly.h ----------------------------
C Common module for Legendre polynomial tables
C     REAL8 PNORM, PRECI, PRECC1, PRECC2, HRECC1, HRECC2, PK, HK
C     DOUBLE PRECISION FAC
C     COMMON/SPPOL1/ PNORM(0:M1MAX, 0:N2MAX), PRECI(0:M1MAX, K2MAX),
C    1       PRECC1(0:M1MAX, 0:N2MAX), PRECC2(0:M1MAX, 0:N2MAX),
C    2       HRECC1(0:M1MAX, 0:N1MAX), HRECC2(0:M1MAX, 0:N1MAX),
C    3           PK(0:M1MAX, 0:N2MAX),     HK(0:M1MAX, 0:N1MAX),
C    4       FAC(0:2*N2MAX+2)
C
C     REAL PK2, HK2
C     INTEGER KCUR, MN1, MN2
C     COMMON/SPPOL2/ PK2(0:NP2MAX),
C    1               HK2(0:NH2MAX), KCUR, MN1, MN2
C
#ifndef SINGLELAT
C     REAL P2, H2
C     COMMON/SPPOL3/ P2(0:NP2MAX, K2MAX),
C    1               H2(0:NH2MAX, K2MAX)
#endif
C
C PNORM(M,N)= normalization factor for Legendre polynomial
C PRECI(M,K)= "value" of Legendre polynomial P[M,M] at latitude MU(K)
C             (used as the starting value for the recurrence relation)
C PRECC1(M,N)= recurrence coefficient [( 2*N-1)/(N-M)]  for P[M,N]
C PRECC2(M,N)= recurrence coefficient [-(N+M-1)/(N-M)]  for P[M,N]
C HRECC1(M,N)= recurrence coefficient [     N*D(M,N+1)] for H[M,N]
C HRECC2(M,N)= recurrence coefficient [-(N+1)*D(M,  N)] for H[M,N]
C PK(M,N)   = value of associated legendre polynomial at MU(KCUR)
C HK(M,N)   = value of -(1-MU**2) times the MU-derivative of the above
C FAC(I)    = factorial I [I <= 2*N2MAX+2]
C (Note: PK is defined for N=0,..,N2, but HK is defined only upto N=N1)
C
C PK2(MN)   = value of associated legendre polynomial at MU(KCUR)
C HK2(MN)   = value of -(1-MU**2) times the MU-derivative of the above
C (Note: PK2 and HK2 contain the same values as PK and HK, but without
C        any gaps and in single precision. Usually the values are stored
C        with the m-index varying most rapidly. If COMPACTPOLY is not
C        defined, then each value is duplicated in consecutive locations
C        for efficient real arithmetic. If COMPACTPOLY is defined, but
C        COMPLEXOP is not defined, then the values are stored with the
C        n-index varying most rapidly.)
C KCUR      = latitude-index (K) for which PK and HK are stored
C             (KCUR = 0 if PK and HK are undefined)
C MN1       = number of (non-duplicated) polynomial values in HK2 - 1
C MN2       = number of (non-duplicated) polynomial values in PK2 - 1
C
#ifndef SINGLELAT
C P2(MN,K)  = same as PK2, but at all latitudes
C H2(MN,K)  = same as HK2, but at all latitudes
#endif
C
C ---------------------- module spfftb.h ----------------------------
C Common module for f.f.t. table used by sptransforms
C      REAL FFTAB1, FFTABV, FFWORK
C      COMMON/SPFFTB/ FFTAB1(2*K1MAX+15),  FFTABV(K1MAX+15),
C     1               FFWORK(KL0MAX,K1MAX)
C
C FFTAB1  -- table of coefficients used for the FFTs involved in
C            single latitude-circle spectral transforms
C FFTABV  -- table of coefficients used for the FFTs involved in
C            multiple latitude-circle spectral transforms
C FFWORK  -- work space for multiple latitude-circle spectral transforms
C
C ---------------------- module splib.F ----------------------------
C
C Initialization routines
C
C     SPINI  -- Initializes the many tables required for the transforms.
C
C The following routines convert a function from spectral to physical
C space, and back, through latitude space. To save storage, they often
C operate on function values for one latitude at a time. Given the
C full spectral representation, one can concentrate on one latitude
C at a time for all operations except one. i.e. The conversion from
C latitude space to spectral space. This obviously requires function
C values at all latitudes. Subroutine LA2SP, which is responsible
C for this conversion, does not actually perform the full conversion,
C but accumulates contributions to the spectral coefficients from each
C latitude. To use LA2SP, one should first set the matrix of spectral
C coefficients to zero, using subroutine ZEROSP, and then successively
C call LA2SP for each latitude.
C
C The following routines operate on one latitude circle at a time:
C
C     SP2LA  -- converts from spectral space to latitude space (S -> L)
C     XSP2LA -- SP2LA of the LAMBDA-derivative                 (S -> L)
C     YSP2LA -- SP2LA of the MU-derivative                     (S -> L)
C     LA2SP  -- converts from latitude space to spectral space (L -> S)
C               (Note: This routine actually accumulates)
C     XLA2SP -- LA2SP of the LAMBDA-derivative                 (L -> S)
C     YLA2SP -- LA2SP of thE MU-derivative (by parts!)         (L -> S)
C
C The following routines are operators in spectral space. i.e. They do
C not transform to any other kind of space.
C
C     ZEROSP -- zeroes spectral space representation           (S -> S)
C     SPCOPY -- copies spectral space representations          (S -> S)
C     DELSQ  -- del-squared (Laplacian) operator               (S -> S)
C     IDELSQ -- inverse of DELSQ                               (S -> S)
C
C The following set of routines are adapted from the book
C Numerical Recipes, W.H.Press et al, Cambridge(1986). They are used
C by the routines described above, but the user may never need to
C bother about them.
C
C    GAULEG -- This subroutine finds the abcissas and weights for the
C              n-point Gauss-Legendre quadrature.
C
C The following routines consist of operators and mappings useful in
C solving the primitive equations for a "thin" planetary atmosphere.
C These are likely to be the most useful routines of all. These
C routines can be used only if the order of the spectral truncation is
C "low", because they involve all-latitudes-at-a-time transforms. These
C routines are based on the more primitive routines described above.
C Before using these routines, the spectral transform grid must be
C initialized by calling SPINI.
C
#ifndef SINGLELAT
C     PHCOPY -- copies physical representations                (P -> P)
C     MULCOS -- multiplies by a power of COSPHI                (P -> P)
C
C     PHYSIC -- convert from spectral to physical space        (S -> P)
C     HVELOC -- calculate horizontal velocity from PSI and CHI (S -> P)
C
C The following routines all accumulate in spectral space. So take care
C to initialize the spectral coefficients using ZEROSP, if necessary.
C     SPECTR -- convert from physical to spectral space        (P -> S)
C     DIVERG -- computes the divergence of a vector field      (P -> S)
C     CURLZ  -- computes the curl of a vector field            (P -> S)
C
C The following routines offer more flexible transforms, but they may
C not need to be used very often.
C     S2P -- converts from spectral space to physical space
C     P2S -- converts from physical space to spectral space
C            (Note: This routine actually accumulates)
#endif
C     L2P -- converts from latitude space to physical space
C     P2L -- converts from physical space to latitude space
C
C The following routines may be used to perform multiple transforms
C at the same latitude circle (for different model levels etc.)
C     VPHYS  -- converts from spectral space to physical space (S -> P)
C     VSPEC  -- converts from physical space to spectral space (P -> S)
C               (Note: This routine actually accumulates)
C
C The following are some functions/subroutines useful in diagnostic
C calculations. Before using these routines, the spectral transform
C grid must be initialized by calling SPINI.
C
C     ZAVGE  -- calculate zonal average of a function in physical space
C     EDDY   -- determine eddy part of a function in physical space
C     GAVGE  -- calculate global average of a function in physical space
C     TOTAL  -- totals up an array of numbers
C     WTOTAL -- computes the weighted total of an array of numbers
C     WAVGE  -- computes the weighted average of an array of numbers
C
C ----------------------- End of Preface ----------------------------
C
      SUBROUTINE SPINI(MMAX, NMAX, RADIUS)
C SPINI initializes the transform grid for triangular truncation of
C order NMAX. It chooses the minimum possible K1, K2 to avoid aliasing
C errors. For simplicity, K1 is chosen to be a power of 2. SPINI also
C initializes the tables required for efficient fast fourier
C transforms.
C Input:  INTEGER MMAX, NMAX -- maximum value of spherical harmonic
C                               indices [M,N]
C                 (MMAX should always be <= NMAX.
C                  MMAX=NMAX for triangular truncation.
C                  MMAX=0 for axially symmetric truncation.)
C         REAL RADIUS -- planetary radius (in metres)
C
#include "spgrid.h"
#include "sppoly.h"
#include "spfftb.h"
      INTEGER MMAX, NMAX
      REAL RADIUS
      INTEGER I,J,K,M,N,K1TEM,K2TEM
      REAL DLAM, PI
#ifndef SINGLELAT
#ifdef COMPACTPOLY
      INTEGER MN
#else
      INTEGER M2N
#endif
#endif
      PARAMETER (PI=3.141592653589793)
C
C No. of longitudinal intervals
      IF (MMAX.GT.NMAX) CALL SPERR('SPINI', 'MMAX must be <= NMAX' )
      IF (MMAX.EQ.0) THEN
        K1TEM= 1
      ELSE IF (MMAX.EQ.1) THEN
        K1TEM= 4
      ELSE IF (MMAX.EQ.2) THEN
        K1TEM= 8
      ELSE IF (MMAX.LE.5) THEN
        K1TEM= 16
      ELSE IF (MMAX.LE.10) THEN
        K1TEM= 32
      ELSE IF (MMAX.LE.13) THEN
        K1TEM= 40
      ELSE IF (MMAX.LE.15) THEN
        K1TEM= 48
      ELSE IF (MMAX.LE.21) THEN
        K1TEM= 64
      ELSE IF (MMAX.LE.31) THEN
        K1TEM= 96
      ELSE IF (MMAX.LE.42) THEN
        K1TEM= 128
      ELSE IF (MMAX.LE.63) THEN
        K1TEM= 192
      ELSE IF (MMAX.LE.85) THEN
        K1TEM= 256
      ELSE IF (MMAX.LE.106) THEN
        K1TEM= 320
      ELSE
        CALL SPERR('SPINI', 'Recode for resolution higher than T106' )
      ENDIF
C No. of gaussian latitudes
      K2TEM= INT( (3*NMAX+1.)/2. + 0.9999 )
C If K2MAX is an odd number, force K2 to also be an odd number
C (So that the equator is always included in the gaussian latitudes)
      IF ((MOD(K2MAX,2).EQ.1) .AND. (MOD(K2TEM,2).EQ.0)) K2TEM=K2TEM+1
C If K2MAX is an even number, force K2 to also be an even number
      IF ((MOD(K2MAX,2).EQ.0) .AND. (MOD(K2TEM,2).EQ.1)) K2TEM=K2TEM+1
C
#ifdef FIXTRUNC
C
C *** Fixed Truncation ***
C Check if truncation is compatible with cpp parameters
      IF ( (MMAX.NE.M1) .OR. (NMAX.NE.N1) )
     1 CALL SPERR('SPINI', 'Truncation incompatible with parameters' )
C Check if derived parameters are specified correctly
      IF ( (K1.NE.K1TEM) .OR. (K2.NE.K2TEM) )
     1 CALL SPERR('SPINI', 'Incompatible parameter specification' )
C
#else
C
C No. of longitudinal intervals
      K1= K1TEM
C No. of gaussian latitudes
      K2= K2TEM
C Maximum zonal wavenumber
      M1= MMAX
C Maximum Legendre polynomial order (for scalars)
      N1= NMAX
C Maximum truncation order (for components of vectors)
      N2= N1+1
C Check if truncation fits into available storage
      IF (     (M1.GT.M1MAX) .OR. (N1.GT.N1MAX)
     1    .OR. (K1.GT.K1MAX) .OR. (K2.GT.K2MAX) )
     2 CALL SPERR('SPINI', 'Truncation too large for available storage')
C
#endif
C
C Initialize table of factorials
      FAC(0)= 1.D0
      DO 10 I=1,2*N2MAX+2
   10   FAC(I)= I * FAC(I-1)
C
      IF (K1.GT.1) THEN
C Set-up tables for F.F.T's (latitude space <-> physical space)
        CALL VRFFTI( K1, FFTABV)
      ENDIF
C
C Initialize longitude values
      DLAM= 2.*PI / K1
      DO 20 J=1,K1
        LAMBDA(J)= (J-1)*DLAM
        DEGLON(J)= LAMBDA(J) * 180./PI
   20   CONTINUE
C Initialize gaussian "latitudes" and gaussian quadrature weights
      CALL GAULEG( -1.0, 1.0, MU, G, K2)
C Initialize latitude angles etc.
      DO 30 K=1,K2
        PHI(K)= SIGN(ASIN( ABS(MU(K)) ), MU(K))
        DEGLAT(K)= PHI(K) * 180./PI
        COSPHI(K)= SQRT(1.0-MU(K)**2)
        COSINV(K)= 1.0/COSPHI(K)
C (Include factor of 1/2 in weight G(K) )
        G(K)= 0.5*G(K)
   30   CONTINUE
C Initialize coefficient for evaluating LAMBDA-derivative
      DO 40 M=0,M1
   40   CIM(M)= CMPLX( 0.0, FLOAT(M) )
C Initialize recurrence coeff. for MU-derivative of Legdr. polynomial
      D(0,0)= 0.0
      DO 50 N=1,N2
        DO 50 M=0,MIN0(N,M1)
          D(M,N)= SQRT( FLOAT(N**2 - M**2) / (4 * N**2 - 1) )
   50     CONTINUE
C Initialize recurrence relations for computing Legendre polynomials
C (see Numerical Recipes, p.182)
      DO 60 K=1,K2
        PRECI(0,K)= 1.0
        DO 60 M=1,M1
            PRECI(M,K)= -(2*M-1) * COSPHI(K) * PRECI(M-1,K)
   60     CONTINUE
C
      DO 70 N=0,N2
        DO 70 M=0,MIN0(N-1,M1)
          PRECC1(M,N)=  FLOAT(2*N-1)/(N-M)
          PRECC2(M,N)= -FLOAT(N+M-1)/(N-M)
   70     CONTINUE
C Compute normalization factor for Legendre polynomial
C (as in Machenhauer 1979)
      DO 80 N=0,N2
        DO 80 M=0,MIN0(N,M1)
          PNORM(M,N)= ((-1)**M) * DSQRT(DBLE(2*N+1) * FAC(N-M)/FAC(N+M))
   80     CONTINUE
C Compute coefficients for finding MU-derivative of Legendre polynomial
      DO 90 N=0,N1
        DO 90 M=0,MIN0(N,M1)
            HRECC1(M,N)=      N * D(M,N+1)
            HRECC2(M,N)= -(N+1) * D(M,N)
   90       CONTINUE
C Initialize planetary radius and related constants
      A0= RADIUS
      A0Q= A0**2
      A0INV= 1.0 / A0
      A0QINV= 1.0 / A0Q
C Number of Associated Legendre polynomial and derivative values
      MN1= ((N1+1)*(N1+2) - (N1-M1)*(N1-M1+1))/2 - 1
      MN2= MN1 + M1 + 1
#ifndef SINGLELAT
C Initialize table of Associated Legendre polynomials and derivatives
      DO 120 K=1,K2
         CALL POLINI( K )
#ifdef COMPACTPOLY
         DO 100 MN=0,MN2
  100      P2(MN,K)= PK2(MN)
         DO 110 MN=0,MN1
  110      H2(MN,K)= HK2(MN)
#else
         DO 100 M2N=0,2*MN2+1
  100      P2(M2N,K)= PK2(M2N)
         DO 110 M2N=0,2*MN1+1
  110      H2(M2N,K)= HK2(M2N)
#endif
  120  CONTINUE
#endif
C Clear polynomial buffer
      KCUR= 0
      RETURN
      END
 
      SUBROUTINE POLINI(K)
C POLINI calculates the Legendre polynomials, and their MU-derivatives,
C at latitude MU(K).
C (Note: This is an auxiliary routine used by SPINI)
C Input:  INTEGER K -- Latitude number at which polynomials and their
C                      derivatives are to be evaluated (MU = MU(K))
C Output: (The Associated Legendre polynomials, with normalization as
C          in Machenhauer(1979), are stored in array PK of common blocks
C          /SPGRD2/, /SPGRD3/. The corresponding MU-derivatives are
C          stored in array HK. PK2 and HK2 are also initialized.
C          KCUR is set to K.)
C
#include "spgrid.h"
#include "sppoly.h"
      INTEGER K
      INTEGER M, N, MN
      REAL8 MUK
C Computing polynomials for latitude K
      KCUR= K
      MUK= MU(K)
C Initialize starting values for recurrence
      DO 10 M=0,M1
   10   PK(M,M)= PRECI(M,K)
C (N can vary between 0 and N2, but M can only vary between 0 and N1)
      DO 20 M=0,M1
   20   PK(M,M+1)= MUK*PRECC1(M,M+1)*PRECI(M,K)
C Use recurrence relation to compute other values
      DO 30 N=2,N2
        DO 30 M=0,MIN0(N-2,M1)
          PK(M,N)= MUK*PRECC1(M,N)*PK(M,N-1)
     1               + PRECC2(M,N)*PK(M,N-2)
   30     CONTINUE
C Normalize Associated Legendre polynomial (Machenhauer's convention)
      DO 40 N=0,N2
        DO 40 M=0,MIN0(N,M1)
   40     PK(M,N)= PNORM(M,N) * PK(M,N)
C Initialize table of MU-derivative of Associated Legendre polynomials
      HK(0,0)= 0.0
      DO 50 N=1,N1
        DO 50 M=0,MIN0(N,M1)
          HK(M,N)=  HRECC1(M,N) * PK(M,N+1)
     1            + HRECC2(M,N) * PK(M,N-1)
   50     CONTINUE
C Copy table to alternate  storage
#ifdef COMPLEXOP
C Compact form (no duplication, M-index varying most rapidly)
      MN= 0
      DO 70 N=0,N2
        DO 60 M=0,MIN0(N,M1)
   60     PK2(M+MN)= PK(M,N)
   70   MN= MN + MIN0(N,M1) + 1
C
      MN= 0
      DO 90 N=0,N1
        DO 80 M=0,MIN0(N,M1)
   80     HK2(M+MN)= HK(M,N)
   90   MN= MN + MIN0(N,M1) + 1
#else
#ifdef COMPACTPOLY
C Compact form (no duplication, N-index varying most rapidly)
      MN= 0
      DO 70 M=0,M1
        DO 60 N=M,N2
   60     PK2(N-M+MN)= PK(M,N)
   70   MN= MN - M + N2 + 1
C
      MN= 0
      DO 90 M=0,M1
        DO 80 N=M,N1
   80     HK2(N-M+MN)= HK(M,N)
   90   MN= MN - M + N1 + 1
#else
C Non-compact form (M-index varying most rapidly)
C Duplicate consecutive values
      MN= 0
      DO 70 N=0,N2
        DO 60 M=0,MIN0(N,M1)
          PK2(2*M+2*MN)= PK(M,N)
   60     PK2(2*M+2*MN+1)= PK(M,N)
   70   MN= MN + MIN0(N,M1) + 1
C
      MN= 0
      DO 90 N=0,N1
        DO 80 M=0,MIN0(N,M1)
          HK2(2*M+2*MN)= HK(M,N)
   80     HK2(2*M+2*MN+1)= HK(M,N)
   90   MN= MN + MIN0(N,M1) + 1
#endif
#endif
      RETURN
      END
 
      SUBROUTINE UVLAK(ULAK,VLAK,PSISP,CHISP,K)
C UVLAK calculates the zonal and meridional components in latitude
C space (ULAK, VLAK) of the horizontal velocity for a single latitude,
C given the spectral space representation of streamfunction PSISP and
C velocity-potential CHISP.
C Note: This is an auxiliary routine used by routine HVEL)
C Input:  INTEGER K -- latitude number ( MU = MU(K) )
C         COMPLEX PSISP(0:M1MAX,0:N1) -- spectral streamfunction
C         COMPLEX CHISP(0:M1MAX,0:N1) -- spectral velocity-potential
C Output: COMPLEX ULAK(0:M1MAX), VLAK(0:M1MAX) -- zonal and meridional
C         components of velocity in latitude space at latitude MU(K)
C
#include "spgrid.h"
#include "sppoly.h"
      INTEGER K
      COMPLEX ULAK(0:M1MAX), VLAK(0:M1MAX)
      COMPLEX PSISP(0:M1MAX,0:N1), CHISP(0:M1MAX,0:N1)
      COMPLEX FTMK1(0:M1MAX), FTMK2(0:M1MAX)
      REAL RTEMP1, RTEMP2
      INTEGER M
      RTEMP1= A0INV * COSINV(K)
      RTEMP2= A0INV * COSPHI(K)
C Calculate scalar "velocity" U*COS(PHI)
#ifndef SINGLELAT
      CALL XSP2LA(FTMK1,CHISP,P2(0,K),H2(0,K),N1,K)
      CALL YSP2LA(FTMK2,PSISP,P2(0,K),H2(0,K),N1,K)
#else
      IF(K.NE.KCUR)
     1  CALL SPERR('UVLAK', 'Polynomials not initialized for latitude')
      CALL XSP2LA(FTMK1,CHISP,PK2,HK2,N1)
      CALL YSP2LA(FTMK2,PSISP,PK2,HK2,N1)
#endif
      DO 10 M=0,M1
        ULAK(M)= RTEMP1*FTMK1(M) - RTEMP2*FTMK2(M)
   10   CONTINUE
C Calculate scalar "velocity" V*COS(PHI)
#ifndef SINGLELAT
      CALL XSP2LA(FTMK1,PSISP,P2(0,K),H2(0,K),N1,K)
      CALL YSP2LA(FTMK2,CHISP,P2(0,K),H2(0,K),N1,K)
#else
      IF(K.NE.KCUR)
     1  CALL SPERR('UVLAK', 'Polynomials not initialized for latitude')
      CALL XSP2LA(FTMK1,PSISP,PK2,HK2,N1,K)
      CALL YSP2LA(FTMK2,CHISP,PK2,HK2,N1,K)
#endif
      DO 20 M=0,M1
        VLAK(M)= RTEMP1*FTMK1(M) + RTEMP2*FTMK2(M)
   20   CONTINUE
      RETURN
      END
 
      SUBROUTINE SP2LA(FLAK,FSP,P0,H0,N0,K)
C SP2LA converts the spectral space representation FSP to latitude
C space representation FLAK, for a single latitude MU.
C Input:  COMPLEX FSP(0:M1MAX,0:N0) -- Spectral representation of F.
C         REAL P0(0:NP2MAX), H0(0:NH2MAX) --
C         (Legendre polynomials and their MU-derivatives at some
C          latitude MU. Usually {PK2, HK2} or {P2(0,K), H2(0,K)} )
C         INTEGER N0 -- order of truncation (N1 or N2).
C         INTEGER K -- latitude number ( MU = MU(K) )
C Output: COMPLEX FLAK(0:M1MAX) -- latitude representation of F,
C                                  at latitude MU.
C
#include "spgrid.h"
#ifdef COMPLEXOP
C
C Compact form (M-index varying most rapidly)
C
      INTEGER N0,K
      COMPLEX FLAK(0:M1MAX), FSP(0:M1MAX,0:N0)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M,N,MN
C
C Zero latitude space coefficients
      DO 10 M=0,M1
   10   FLAK(M)= (0., 0.)
C Multiply by associated legendre polynomial, and accumulate
      MN= 0
      DO 30 N=0,N0
        DO 20 M=0,MIN0(N,M1)
   20     FLAK(M)= FLAK(M) + FSP(M,N) * P0(M+MN)
   30   MN= MN + MIN0(N,M1) + 1
C
      RETURN
#else
#ifdef COMPACTPOLY
C
C Compact form (N-index varying most rapidly)
C
      INTEGER N0,K
C Pretend that the complex arrays are actually real arrays
      REAL FLAK(0:2*M1MAX+1), FSP(0:2*M1MAX+1,0:N0)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M,M2,N,MN
C
C Zero latitude space coefficients
      DO 10 M2=0,2*M1+1
   10   FLAK(M2)= 0.0
C Multiply by associated legendre polynomial, and accumulate
      DO 20 N=0,N0
   20   FLAK(0)= FLAK(0) + FSP(0,N) * P0(N)
C
      MN= N2 + 1
      DO 40 M=1,M1
        DO 30 M2=2*M,2*M+1
          DO 30 N=M,N0
   30       FLAK(M2)= FLAK(M2) + FSP(M2,N) * P0(N-M+MN)
   40   MN= MN - M + N2 + 1
C
      RETURN
#else
C
C Non-compact form (M-index varying most rapidly)
C
      INTEGER N0,K
C Pretend that the complex arrays are actually real arrays
      REAL FLAK(0:2*M1MAX+1), FSP(0:2*M1MAX+1,0:N0)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M2,N,M2N
C
C Zero latitude space coefficients
      DO 10 M2=0,2*M1+1
   10   FLAK(M2)= 0.0
C Multiply by associated legendre polynomial, and accumulate
      M2N= 0
      DO 30 N=0,N0
        DO 20 M2=0,2*MIN0(N,M1)+1
   20     FLAK(M2)= FLAK(M2) + FSP(M2,N) * P0(M2+M2N)
   30   M2N= M2N + 2*MIN0(N,M1) + 2
C
      RETURN
#endif
#endif
      END
 
      SUBROUTINE XSP2LA(FLAK,FSP,P0,H0,N0,K)
C XSP2LA returns FLAK, the latitude space representation of the
C LAMBDA-Derivative of a function F, given FSP, the spectral space
C represention of F. All this for a single latitude MU.
C i.e. XSP2LA is SP2LA combined with d/dLAMBDA.
C Input:  COMPLEX FSP(0:M1MAX,0:N0) -- Spectral representation of F.
C         REAL P0(0:NP2MAX), H0(0:NH2MAX) --
C         (Legendre polynomials and their MU-derivatives at some
C          latitude MU. Usually {PK2, HK2} or {P2(0,K), H2(0,K)} )
C         INTEGER N0 -- order of truncation (N1 or N2).
C         INTEGER K -- latitude number ( MU = MU(K) )
C Output: COMPLEX FLAK(0:M1MAX) -- latitude representation of
C                         LAMBDA-derivative of F, at latitude MU.
C
#include "spgrid.h"
      INTEGER N0,K
      COMPLEX FLAK(0:M1MAX), FSP(0:M1MAX,0:N0)
      REAL P0(0:2*M1MAX+1,0:N0), H0(0:2*M1MAX+1,0:N0)
      INTEGER M
      IF (M1.GT.0) THEN
C Do a normal transform from spectral space to latitude space
        CALL SP2LA(FLAK,FSP,P0,H0,N0,K)
C Evaluate lambda-derivative
        DO 10 M=0,M1
          FLAK(M)= CIM(M) * FLAK(M)
   10     CONTINUE
      ELSE
        FLAK(0)= (0., 0.)
      ENDIF
      RETURN
      END
 
      SUBROUTINE YSP2LA(FLAK,FSP,P0,H0,N0,K)
C YSP2LA returns FLAK, the latitude space representation of the
C MU-derivative of a function F, given FSP, the spectral space
C representation of F. All this for a single latitude MU.
C i.e. YSP2LA is SP2LA combined with d/dMU.
C Input:  COMPLEX FSP(0:M1MAX,0:N0) -- Spectral representation of F.
C         REAL P0(0:NP2MAX), H0(0:NH2MAX) --
C         (Legendre polynomials and their MU-derivatives at some
C          latitude MU. Usually {PK2, HK2} or {P2(0,K), H2(0,K)} )
C         INTEGER N0 -- order of truncation (N1 or N2).
C         INTEGER K -- latitude number ( MU = MU(K) )
C Output: COMPLEX FLAK(0:M1MAX) -- latitude representation of
C                             MU-derivative of F, at some latitude MU.
C
#include "spgrid.h"
#ifdef COMPLEXOP
C
C Compact form (M-index varying most rapidly)
C
      INTEGER N0,K
      COMPLEX FLAK(0:M1MAX), FSP(0:M1MAX,0:N0)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M,N,MN
      REAL RTEMP
C
C Zero latitude space coefficients
      DO 10 M=0,M1
   10   FLAK(M)= (0., 0.)
C Multiply by -(1-MU**2) times the MU-derivative of associated
C legendre polynomial, and accumulate.
      MN= 0
      DO 30 N=0,N0
        DO 20 M=0,MIN0(N,M1)
   20     FLAK(M)= FLAK(M) + FSP(M,N) * H0(M+MN)
   30   MN= MN + MIN0(N,M1) + 1
C Divide by -(1-MU**2) to obtain MU-derivative
      RTEMP= - (COSINV(K)**2)
      DO 40 M=0,M1
   40   FLAK(M)= RTEMP * FLAK(M)
C
      RETURN
#else
#ifdef COMPACTPOLY
C
C Compact form (N-index varying most rapidly)
C
      INTEGER N0,K
C Pretend that the complex arrays are actually real arrays
      REAL FLAK(0:2*M1MAX+1), FSP(0:2*M1MAX+1,0:N0)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M,M2,N,MN
      REAL RTEMP
C
C Zero latitude space coefficients
      DO 10 M2=0,2*M1+1
   10   FLAK(M2)= 0.0
C Multiply by -(1-MU**2) times the MU-derivative of associated
C legendre polynomial, and accumulate.
      DO 20 N=0,N0
   20   FLAK(0)= FLAK(0) + FSP(0,N) * H0(N)
C
      MN= N1 + 1
      DO 40 M=1,M1
        DO 30 M2=2*M,2*M+1
          DO 30 N=M,N0
   30       FLAK(M2)= FLAK(M2) + FSP(M2,N) * H0(N-M+MN)
   40   MN= MN - M + N1 + 1
C Divide by -(1-MU**2) to obtain MU-derivative
      RTEMP= - (COSINV(K)**2)
      DO 50 M2=0,2*M1+1
   50   FLAK(M2)= RTEMP * FLAK(M2)
C
      RETURN
#else
C
C Non-compact form (M-index varying most rapidly)
C
      INTEGER N0,K
C Pretend that the complex arrays are actually real arrays
      REAL FLAK(0:2*M1MAX+1), FSP(0:2*M1MAX+1,0:N0)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M2,N,M2N
      REAL RTEMP
C
C Zero latitude space coefficients
      DO 10 M2=0,2*M1+1
   10   FLAK(M2)= 0.0
C Multiply by -(1-MU**2) times the MU-derivative of associated
C legendre polynomial, and accumulate.
      M2N= 0
      DO 30 N=0,N0
        DO 20 M2=0,2*MIN0(N,M1)+1
   20     FLAK(M2)= FLAK(M2) + FSP(M2,N) * H0(M2+M2N)
   30   M2N= M2N + 2*MIN0(N,M1) + 2
C Divide by -(1-MU**2) to obtain MU-derivative
      RTEMP= - (COSINV(K)**2)
      DO 40 M2=0,2*M1+1
   40   FLAK(M2)= RTEMP * FLAK(M2)
C
      RETURN
#endif
#endif
      END
 
      SUBROUTINE LA2SP(FSP,FLAK,P0,H0,N0,K)
C LA2SP adds to spectral space representation FSP the contribution
C from latitude space representation FLAK, for a single latitude
C MU. (When contributions from all the latitudes have been added
C up, the result will be the complete spectral space representation.)
C Input:  COMPLEX FSP(0:M1MAX,0:N0) -- Spectral representation of F.
C         COMPLEX FLAK(0:M1MAX) -- Latitude representation of F,
C                                  at some latitude MU.
C         REAL P0(0:NP2MAX), H0(0:NH2MAX) --
C         (Legendre polynomials and their MU-derivatives at some
C          latitude MU. Usually {PK2, HK2} or {P2(0,K), H2(0,K)} )
C         INTEGER N0 -- order of truncation (N1 or N2).
C         INTEGER K -- latitude number ( MU = MU(K) )
C Output: COMPLEX FSP(0:M1MAX,0:N0) -- Spectral representation of F,
C                           with contribution from latitude MU added.
C
#include "spgrid.h"
#ifdef COMPLEXOP
C
C Compact form (M-index varying most rapidly)
C
      INTEGER N0,K
      COMPLEX FSP(0:M1MAX,0:N0), FLAK(0:M1MAX)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M,N,MN
      COMPLEX GFLAKC(0:M1MAX)
C
C Evaluate temporary array
      DO 10 M=0,M1
   10   GFLAKC(M)= G(K) * FLAK(M)
C Evaluate gaussian quadrature contribution from latitude K.
C Remember, the factor of 1/2 is included in G(K)
      MN= 0
      DO 30 N=0,N0
        DO 20 M=0,MIN0(N,M1)
   20     FSP(M,N)= FSP(M,N) + GFLAKC(M) * P0(M+MN)
   30   MN= MN + MIN0(N,M1) + 1
C
      RETURN
#else
#ifdef COMPACTPOLY
C
C Compact form (N-index varying most rapidly)
C
      INTEGER N0,K
C Pretend that the complex arrays are actually real arrays
      REAL FSP(0:2*M1MAX+1,0:N0), FLAK(0:2*M1MAX+1)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M,M2,N,MN
      REAL GFLAK(0:2*M1MAX+1)
C
C Evaluate temporary array
      DO 10 M2=0,2*M1+1
   10   GFLAK(M2)= G(K) * FLAK(M2)
C Evaluate gaussian quadrature contribution from latitude K.
C Remember, the factor of 1/2 is included in G(K)
      DO 20 N=0,N0
   20   FSP(0,N)= FSP(0,N) + GFLAK(0) * P0(N)
C
      MN= N2 + 1
      DO 40 M=1,M1
        DO 30 M2=2*M,2*M+1
          DO 30 N=M,N0
   30       FSP(M2,N)= FSP(M2,N) + GFLAK(M2) * P0(N-M+MN)
   40   MN= MN - M + N2 + 1
C
      RETURN
#else
C
C Non-compact form (M-index varying most rapidly)
C
      INTEGER N0,K
C Pretend that the complex arrays are actually real arrays
      REAL FSP(0:2*M1MAX+1,0:N0), FLAK(0:2*M1MAX+1)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M2,N,M2N
      REAL GFLAK(0:2*M1MAX+1)
C
C Evaluate temporary array
      DO 10 M2=0,2*M1+1
   10   GFLAK(M2)= G(K) * FLAK(M2)
C Evaluate gaussian quadrature contribution from latitude K.
C Remember, the factor of 1/2 is included in G(K)
      M2N= 0
      DO 30 N=0,N0
        DO 20 M2=0,2*MIN0(N,M1)+1
   20     FSP(M2,N)= FSP(M2,N) + GFLAK(M2) * P0(M2+M2N)
   30   M2N= M2N + 2*MIN0(N,M1) + 2
C
      RETURN
#endif
#endif
      END
 
      SUBROUTINE XLA2SP(FSP,FLAK,P0,H0,N0,K)
C XLA2SP does almost the same thing as LA2SP. The only difference
C is that instead of transforming the function F (given by FLAK), it
C transforms the LAMBDA-derivative of F to spectral space. i.e. It
C returns (accumulates) (0.,1.)*M times the result given by LA2SP.
C
#include "spgrid.h"
      INTEGER N0,K
      COMPLEX FSP(0:M1MAX,0:N0), FLAK(0:M1MAX)
      REAL P0(0:2*M1MAX+1,0:N0), H0(0:2*M1MAX+1,0:N0)
      COMPLEX FTMKC(0:M1MAX)
      INTEGER M
      IF (M1.GT.0) THEN
C Evaluate temporary array (taking LAMBDA-derivative)
        DO 10 M=0,M1
   10     FTMKC(M)= CIM(M) * FLAK(M)
C Call LA2SP to do the rest of the job
        CALL LA2SP(FSP,FTMKC,P0,H0,N0,K)
      ENDIF
      RETURN
      END
 
      SUBROUTINE YLA2SP(FSP,FLAK,P0,H0,N0,K)
C YLA2SP does almost the same thing as LA2SP. the only difference
C is that instead of transforming the function F (given by FLAK), it
C transforms the MU-derivative of F to spectral space. It does this
C by integrating by parts, assuming the boundary terms vanish, and
C using H[M,N] instead of P[M,N] in the transform.
C i.e. YLA2SP returns (accumulates) dF/dMU for a single latitude.
C **CAUTION** YLA2SP will work only for functions which vanish at the
C             poles. e.g. COS(PHI) * some-function
C
#include "spgrid.h"
#ifdef COMPLEXOP
C
C Compact form (M-index varying most rapidly)
C
      INTEGER N0,K
      COMPLEX FSP(0:M1MAX,0:N0), FLAK(0:M1MAX)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M,N,MN
      COMPLEX GFLAKC(0:M1MAX)
      REAL RTEMP
C
C Evaluate temporary array (incorporates division by (1-MU**2) )
      RTEMP= G(K) * (COSINV(K)**2)
      DO 10 M=0,M1
   10   GFLAKC(M)= RTEMP * FLAK(M)
C Evaluate gaussian quadrature contribution from latitude K.
C Remember, the factor of 1/2 is included in G(K)
      MN= 0
      DO 30 N=0,N0
        DO 20 M=0,MIN0(N,M1)
   20     FSP(M,N)= FSP(M,N) + GFLAKC(M) * H0(M+MN)
   30   MN= MN + MIN0(N,M1) + 1
C
      RETURN
#else
#ifdef COMPACTPOLY
C
C Compact form (N-index varying most rapidly)
C
      INTEGER N0,K
C Pretend that the complex arrays are actually real arrays
      REAL FSP(0:2*M1MAX+1,0:N0), FLAK(0:2*M1MAX+1)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M,M2,N,MN
      REAL GFLAK(0:2*M1MAX+1), RTEMP
C
C Evaluate temporary array (incorporates division by (1-MU**2) )
      RTEMP= G(K) * (COSINV(K)**2)
      DO 10 M2=0,2*M1+1
   10   GFLAK(M2)= RTEMP * FLAK(M2)
C Evaluate gaussian quadrature contribution from latitude K.
C Remember, the factor of 1/2 is included in G(K)
      DO 20 N=0,N0
   20   FSP(0,N)= FSP(0,N) + GFLAK(0) * H0(N)
C
      MN= N1 + 1
      DO 40 M=1,M1
        DO 30 M2=2*M,2*M+1
          DO 30 N=M,N0
   30       FSP(M2,N)= FSP(M2,N) + GFLAK(M2) * H0(N-M+MN)
   40   MN= MN - M + N1 + 1
C
      RETURN
#else
C
C Non-compact form (M-index varying most rapidly)
C
      INTEGER N0,K
C Pretend that the complex arrays are actually real arrays
      REAL FSP(0:2*M1MAX+1,0:N0), FLAK(0:2*M1MAX+1)
      REAL P0(0:NP2MAX), H0(0:NH2MAX)
      INTEGER M2,N,M2N
      REAL GFLAK(0:2*M1MAX+1), RTEMP
C
C Evaluate temporary array (incorporates division by (1-MU**2) )
      RTEMP= G(K) * (COSINV(K)**2)
      DO 10 M2=0,2*M1+1
   10   GFLAK(M2)= RTEMP * FLAK(M2)
C Evaluate gaussian quadrature contribution from latitude K.
C Remember, the factor of 1/2 is included in G(K)
      M2N= 0
      DO 30 N=0,N0
        DO 20 M2=0,2*MIN0(N,M1)+1
   20     FSP(M2,N)= FSP(M2,N) + GFLAK(M2) * H0(M2+M2N)
   30   M2N= M2N + 2*MIN0(N,M1) + 2
C
      RETURN
#endif
#endif
      END
 
      SUBROUTINE ZEROSP(FSP,N0)
C ZEROSP zeroes the spectral coefficient matrix FSP, for truncation
C order N0.
C Input:  INTEGER N0 -- order of truncation (N1 or N2).
C Output: COMPLEX FSP(0:M1MAX,0:N0) -- spectral representation of F.
C
#include "spgrid.h"
      INTEGER N0
      COMPLEX FSP(0:M1MAX,0:N0)
      INTEGER M,N
      DO 10 N=0,N0
        DO 10 M=0,MIN0(N,M1)
          FSP(M,N)= (0.,0.)
   10     CONTINUE
      RETURN
      END
 
      SUBROUTINE SPCOPY(FSP2,FSP1,N0)
C SPCOPY copies spectral coefficient matrix FSP1 to FSP2
C Input:  INTEGER N0 -- order of truncation (N1 or N2).
C         COMPLEX FSP1(0:M1MAX,0:N0) -- spectral representation of F.
C Output: COMPLEX FSP2(0:M1MAX,0:N0) -- copy of FSP1.
C
#include "spgrid.h"
      INTEGER N0
      COMPLEX FSP2(0:M1MAX,0:N0),FSP1(0:M1MAX,0:N0)
      INTEGER M,N
      DO 10 N=0,N0
        DO 10 M=0,MIN0(N,M1)
          FSP2(M,N)= FSP1(M,N)
   10     CONTINUE
      RETURN
      END
 
      SUBROUTINE DELSQ(DSQFSP,FSP,N0)
C DELSQ calculates DSQFSP, del-squared (Laplacian) of a function F,
C given its spectral representation FSP.
C Input:  INTEGER N0 -- order of truncation (N1 or N2).
C         COMPLEX FSP(0:M1MAX,0:N0) -- spectral representation of F.
C Output: COMPLEX DSQFSP(0:M1MAX,0:N0) -- spectral representation
C                                         of del-squared F.
C
#include "spgrid.h"
      INTEGER N0
      COMPLEX DSQFSP(0:M1MAX,0:N0),FSP(0:M1MAX,0:N0)
      INTEGER M,N
      REAL TEMP
      DO 10 N=0,N0
        TEMP= -N * (N+1) * A0QINV
        DO 10 M=0,MIN0(N,M1)
          DSQFSP(M,N)= FSP(M,N) * TEMP
   10     CONTINUE
      RETURN
      END
 
      SUBROUTINE IDELSQ(FSP,DSQFSP,N0)
C IDELSQ calculates FSP, the spectral representation of a function F,
C given the spectral representation of del-squared (Laplacian) of F.
C (Note: Since the inverse laplacian is not unique, FSP(0,0) is always
C        set to zero)
C Input:  INTEGER N0 -- order of truncation (N1 or N2).
C         COMPLEX DSQFSP(0:M1MAX,0:N0) -- spectral representation OF
C                                         del-squared F.
C Output: COMPLEX FSP(0:M1MAX,0:N0) -- spectral representation of F.
C
#include "spgrid.h"
      INTEGER N0
      COMPLEX FSP(0:M1MAX,0:N0),DSQFSP(0:M1MAX,0:N0)
      INTEGER M,N
      REAL TEMP
C Inverse laplacian is non-unique to upto a constant
      FSP(0,0)= (0.,0.)
      DO 10 N=1,N0
        TEMP= -A0Q / (N * (N+1) )
        DO 10 M=0,MIN0(N,M1)
          FSP(M,N)= DSQFSP(M,N) * TEMP
   10     CONTINUE
      RETURN
      END
 
      SUBROUTINE GAULEG(X1,X2,X,W,N)
C
C Given the lower and upper limits of integration X1 and X2, and
C given N, this routine returns arrays X and W of length N, containing
C the abcissas and weights of the Gauss-Legendre n-point quadrature
C formula.
C (Numerical Recipes - W.H.Press et al, Cambridge(1986), p.125-126)
C
      INTEGER N
      REAL X1,X2,X(N),W(N)
C
C High precision is a good idea for this routine.
      DOUBLE PRECISION EPS
      PARAMETER (EPS=3.D-14)
C
      DOUBLE PRECISION P1, P2, P3, PP, XL, XM, Z, Z1
      INTEGER I, J, M
C Increase if you don't have this precision.
C
      M=(N+1)/2
C The roots are symmetric in the interval, so we only find half of them.
C
      XM=0.5D0*(X2+X1)
      XL=0.5D0*(X2-X1)
C
C Loop over the desired roots
      DO 12 I=1,M
        Z=COS(3.141592654D0*(I-.25D0)/(N+.5D0))
C Starting with the above approximation to the I-th root, we enter the
C main loop of refinement by Newton's method.
C
    1   CONTINUE
        P1=1.D0
        P2=0.D0
C
C Loop up the recurrence relation to get the Legendre polynomial
C evaluated at Z.
        DO 11 J=1,N
          P3=P2
          P2=P1
          P1=((2.D0*J-1.D0)*Z*P2-(J-1.D0)*P3)/J
   11     CONTINUE
C P1 is the now desired Legendre polynomial. We next compute PP, its
C derivative, by a standard relation involving also P2, the polynomial
C of one lower order.
C
        PP=N*(Z*P1-P2)/(Z*Z-1.D0)
        Z1=Z
        Z=Z1-P1/PP
C Newton's method.
C
        IF(ABS(Z-Z1).GT.EPS)GO TO 1
        X(I)=XM-XL*Z
        X(N+1-I)=XM+XL*Z
C Scale the root to the desired interval
C and put in its symmetric counterpart.
C
        W(I)=2.D0*XL/((1.D0-Z*Z)*PP*PP)
        W(N+1-I)=W(I)
C Compute the weight and its symmetric counterpart.
C
   12   CONTINUE
      RETURN
      END
 
      SUBROUTINE SPERR(PROC, MESG)
C SPERR aborts the program with a diagnostic error message.
C Input: CHARACTER*(*) PROC -- name of calling function/subroutine
C        CHARACTER*(*) MESG -- diagnostic message
C
      CHARACTER*(*) PROC, MESG
      WRITE(*,'(A,A,A,A)') '**Error in ', PROC, ': ', MESG
      STOP 'SPERR'
      END
 
C All-latitudes-at-a-time spectral transforms begin here
      SUBROUTINE PHCOPY(FPH2,FPH1)
C PHCOPY copies physical space representation FPH1 to FPH2
C Input:  REAL FPH1(K1MAX,K2MAX) -- physical representation oF F.
C Output: REAL FPH2(K1MAX,K2MAX) -- copy of FPH1.
C
#include "spgrid.h"
      REAL FPH2(K1MAX,K2MAX), FPH1(K1MAX,K2MAX)
      INTEGER J,K
      DO 10 K=1,K2
        DO 10 J=1,K1
          FPH2(J,K)= FPH1(J,K)
   10     CONTINUE
      RETURN
      END
 
      SUBROUTINE MULCOS(FPH2,FPH1,C0,I0)
C MULCOS multiplies physical space representations by a coefficient
C C0 and by COS(PHI)**I0.
C Input:  REAL FPH1(K1MAX,K2MAX) -- physical representation of F
C         REAL C0 -- coefficient to multiply F by
C         INTEGER I0 -- exponent of COS(PHI)
C Output: REAL FPH2(K1MAX,K2MAX) -- C0*COS(PHI)**I0 times FPH1
C (Note: FPH2 may be the same array as FPH1)
C
#include "spgrid.h"
      REAL FPH2(K1MAX,K2MAX), FPH1(K1MAX,K2MAX), C0
      INTEGER I0
      REAL RTEMP(K2MAX)
      INTEGER J,K
      IF (I0.EQ.0) THEN
        DO 10 K=1,K2
   10     RTEMP(K)= C0
      ELSE IF (I0.EQ.1) THEN
        DO 20 K=1,K2
   20     RTEMP(K)= C0 * COSPHI(K)
      ELSE IF (I0.EQ.-1) THEN
        DO 30 K=1,K2
   30     RTEMP(K)= C0 * COSINV(K)
      ELSE
        DO 40 K=1,K2
   40     RTEMP(K)= C0 * (COSPHI(K)**I0)
      ENDIF
      DO 50 K=1,K2
        DO 50 J=1,K1
          FPH2(J,K)= RTEMP(K) * FPH1(J,K)
   50     CONTINUE
      RETURN
      END
 
#ifndef SINGLELAT
      SUBROUTINE PHYSIC(FPH,FSP)
C PHYSIC converts the spectral space representation FSP to physical
C space representation FPH.
C Note: Truncation order is assumed to be N1 (i.e. as for a scalar)
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX) -- spectral representation of F
C Output: REAL FPH(K1MAX,K2MAX) -- physical space representation of F
C
#include "spgrid.h"
      REAL FPH(K1MAX,K2MAX)
      COMPLEX FSP(0:M1MAX,0:N1MAX)
      EXTERNAL SP2LA
      CALL S2P( FPH, FSP, SP2LA, N1)
      RETURN
      END
 
      SUBROUTINE HVELOC(UPH,VPH,PSISP,CHISP)
C HVELOC calculates the horizontal velocity components U, V (UPH, VPH)
C in physical space, given the spectral space representation of
C streamfunction PSISP and velocity-potential CHISP.
C Input:  COMPLEX PSISP(0:M1MAX,0:N1MAX) -- spectral streamfunction
C         COMPLEX CHISP(0:M1MAX,0:N1MAX) -- spectral velocity-potential
C Output: REAL UPH(K1MAX,K2MAX) -- zonal component of velocity
C         REAL VPH(K1MAX,K2MAX) -- meridional component of velocity
C
#include "spgrid.h"
      REAL UPH(K1MAX,K2MAX), VPH(K1MAX,K2MAX)
      COMPLEX PSISP(0:M1MAX,0:N1MAX), CHISP(0:M1MAX,0:N1MAX)
      COMPLEX ULA(0:M1MAX,K2MAX), VLA(0:M1MAX,K2MAX)
      INTEGER K
C Calculate velocity components (ULA,VLA) in latitude space
      DO 10 K=1,K2
   10   CALL UVLAK(ULA(0,K),VLA(0,K),PSISP,CHISP,K)
C Convert to physical space
      CALL L2P(UPH,ULA,K2)
      CALL L2P(VPH,VLA,K2)
      RETURN
      END
 
      SUBROUTINE SPECTR(FSP,FPH)
C SPECTR converts the physical space representation FPH to spectral
C space, and accumulates in FSP.
C Note: Truncation order is assumed to be N1 (i.e. as for a scalar)
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX) -- old spectral coefficients
C         REAL FPH(K1MAX,K2MAX) -- physical space representation of F
C Output: COMPLEX FSP(0:M1MAX,0:N1MAX) -- spectral representation of F
C                                         plus old coefficients
C
#include "spgrid.h"
      COMPLEX FSP(0:M1MAX,0:N1MAX)
      REAL FPH(K1MAX,K2MAX)
      EXTERNAL LA2SP
      CALL P2S( FSP, FPH, LA2SP, N1)
      RETURN
      END
 
      SUBROUTINE DIVERG(FSP,FPH1,FPH2)
C Given the component physical space representations FPH1 and FPH2
C of a vector field (say U,V), DIVERG computes the divergence in
C spectral space, and accumulates in FSP.
C Note: Truncation order is assumed to be N1 (i.e. as for a scalar)
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX) -- old spectral coefficients
C         REAL FPH1(K1MAX,K2MAX), FPH2(K1MAX,K2MAX) -- physical
C         representation of vector components (zonal, meridional)
C Output: COMPLEX FSP(0:M1MAX,0:N1MAX) -- spectral divergence of F
C                                         plus old coefficients
C
#include "spgrid.h"
      COMPLEX FSP(0:M1MAX,0:N1MAX)
      REAL FPH1(K1MAX,K2MAX), FPH2(K1MAX,K2MAX)
      REAL TEMPH(K1MAX,K2MAX)
      EXTERNAL XLA2SP, YLA2SP
C Accumulate zonal-derivative of FPH1 / COSPHI
      CALL MULCOS( TEMPH, FPH1, A0INV, -1)
      CALL P2S( FSP, TEMPH, XLA2SP, N1 )
C Accumulate (1/A0) * MU-derivative of COSPHI * FPH2
      CALL MULCOS( TEMPH, FPH2, A0INV, +1)
      CALL P2S( FSP, TEMPH, YLA2SP, N1 )
      RETURN
      END
 
      SUBROUTINE CURLZ(FSP,FPH1,FPH2)
C Given the component physical space representations FPH1 and FPH2
C of a vector field (say U,V), CURLZ computes the curl (z-component) in
C spectral space, and accumulates in FSP.
C Note: Truncation order is assumed to be N1 (i.e. as for a scalar)
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX) -- old spectral coefficients
C         REAL FPH1(K1MAX,K2MAX), FPH2(K1MAX,K2MAX) -- physical
C         representation of vector components (zonal, meridional)
C Output: COMPLEX FSP(0:M1MAX,0:N1MAX) -- spectral curl of F
C                                         plus old coefficients
C
#include "spgrid.h"
      COMPLEX FSP(0:M1MAX,0:N1MAX)
      REAL FPH1(K1MAX,K2MAX), FPH2(K1MAX,K2MAX)
      REAL TEMPH(K1MAX,K2MAX)
      EXTERNAL XLA2SP, YLA2SP
C Accumulate zonal-derivative of FPH2 / COSPHI
      CALL MULCOS( TEMPH, FPH2, A0INV, -1)
      CALL P2S( FSP, TEMPH, XLA2SP, N1 )
C Accumulate (1/A0) * MU-derivative of -COSPHI * FPH1
      CALL MULCOS( TEMPH, FPH1, -A0INV, +1)
      CALL P2S( FSP, TEMPH, YLA2SP, N1 )
      RETURN
      END
 
      SUBROUTINE S2P(FPH,FSP,FSP2LA,N0)
C S2P converts the spectral space representation FSP to physical
C space representation FPH, for all latitudes. FSP2LA is the
C intermediate subroutine to be used to convert from spectral to
C latitude space. i.e. FSP2LA = SP2LA or XSP2LA or YSP2LA
C **NOTE** Remember to declare FSP2LA as external to the calling routine
C
C Input:  COMPLEX FSP(0:M1MAX,0:N0) -- spectral representation of F.
C         SUBROUTINE FSP2LA(FLAK,FSP,P0,H0,N0,K) -- spectral to latitude
C                                            space conversion subroutine
C         INTEGER N0 -- order of truncation (N1 or N2).
C Output: REAL FPH(K1MAX,K2MAX) -- physical space representation of F
C
#include "spgrid.h"
#include "sppoly.h"
      INTEGER N0
      REAL FPH(K1MAX,K2MAX)
      COMPLEX FSP(0:M1MAX,0:N0)
      INTEGER K
      COMPLEX FLA(0:M1MAX,K2MAX)
#ifdef UDEF
C Dummy declaration (sometimes needed with the IMPLICIT NONE option)
      INTEGER FSP2LA
#endif
C Convert to latitude space in temporary storage, using FSP2LA
      DO 10 K=1,K2
   10   CALL FSP2LA(FLA(0,K),FSP,P2(0,K),H2(0,K),N0,K)
C Convert to physical space
      CALL L2P(FPH,FLA,K2)
      RETURN
      END
 
      SUBROUTINE P2S(FSP,FPH,FLA2SP,N0)
C P2S adds to spectral space representation FSP the contribution
C from physical space representation FPH, for all latitudes. FLA2SP is
C the intermediate subroutine to be used to convert from latitude to
C spectral space. i.e. FLA2SP = LA2SP or XLA2SP or YLA2SP
C **NOTE** Remember to declare FLA2SP as external to the calling routine
C
C Input:  COMPLEX FSP(0:M1MAX,0:N0) -- old spectral coefficients
C         REAL FPH(K1MAX,K2MAX) -- physical space representation of F
C         SUBROUTINE FLA2SP(FSP,FLAK,P0,H0,N0,K) -- latitude to spectral
C                                            space conversion subroutine
C         INTEGER N0 -- order of truncation (N1 or N2)
C Output: COMPLEX FSP(0:M1MAX,0:N0) -- spectral representation of F,
C                                      plus the old coefficients
C
#include "spgrid.h"
#include "sppoly.h"
      INTEGER N0
      COMPLEX FSP(0:M1MAX,0:N0)
      REAL FPH(K1MAX,K2MAX)
      INTEGER K
      COMPLEX FLA(0:M1MAX,K2MAX)
#ifdef UDEF
C Dummy declaration (sometimes needed with the IMPLICIT NONE option)
      INTEGER FLA2SP
#endif
C Convert to latitude space in temporary storage
      CALL P2L(FLA,FPH,K2)
C Convert to spectral space using FLA2SP (accumulate)
      DO 10 K=1,K2
   10   CALL FLA2SP(FSP,FLA(0,K),P2(0,K),H2(0,K),N0,K)
      RETURN
      END
#endif
 
      SUBROUTINE L2P(FPH,FLA,KL0)
C L2P converts a function F from latitude space representation FLA
C to physical space representation FPH, for all latitudes/levels.
C Input:  COMPLEX FLA(0:M1MAX,KL0) -- latitude representation of F
C         INTEGER KL0 -- No. of latitudes/levels
C Output: REAL FPH(K1MAX,KL0) -- physical representation of F
C
#include "spgrid.h"
#include "spfftb.h"
      INTEGER KL0
      REAL FPH(K1MAX,KL0), FLA(0:2*M1MAX+1,KL0)
      REAL TEMPH(KL0MAX, K1MAX), RTEM
C Note that TEMPH is in transpose form
      INTEGER J,KL
C
      IF (KL0.GT.KL0MAX)
     1 CALL SPERR('L2P', 'Recompile with increased KL0MAX' )
      IF (K1.EQ.1) THEN
C Axially symmetric case; handle separately
        DO 10 KL=1,KL0
  10      FPH(1,KL)= FLA(0,KL)
      ELSE
C Copy the first M1+1 fourier coefficients to transposed array
        DO 20 KL=1,KL0
   20     TEMPH(KL,1)= FLA(0,KL)
        DO 30 J=2,2*M1+1
          DO 30 KL=1,KL0
   30       TEMPH(KL,J)= FLA(J,KL)
C Zero all extra coefficients
        DO 40 J=2*M1+2,K1
          DO 40 KL=1,KL0
   40       TEMPH(KL,J)= 0.0
C Since FLA(*,KL) is the fourier transform of real function
C TEMPH(KL,*), invert the transform.
        CALL VRFFTB(KL0, K1, TEMPH, FFWORK, KL0MAX, FFTABV)
C Transpose temporary matrix and normalize to obtain proper result
        RTEM= SQRT(FLOAT(K1))
        DO 50 KL=1,KL0
          DO 50 J=1,K1
   50       FPH(J,KL)= TEMPH(KL,J) * RTEM
      ENDIF
      RETURN
      END
 
      SUBROUTINE P2L(FLA,FPH,KL0)
C P2L converts a function F from physical space representation FPH
C to latitude space representation FLA, for all latitudes/levels.
C Input:  REAL FPH(K1MAX,KL0) -- physical representation of F
C         INTEGER KL0 -- No. of latitudes/levels
C Output: COMPLEX FLA(0:M1MAX,KL0) -- latitude representation of F
C
#include "spgrid.h"
#include "spfftb.h"
      INTEGER KL0
      REAL FPH(K1MAX,KL0), FLA(0:2*M1MAX+1,KL0)
      REAL TEMPH(KL0MAX, K1MAX), RTEM
C Note that TEMPH is in transpose form
      INTEGER J,KL
C
      IF (KL0.GT.KL0MAX)
     1 CALL SPERR('P2L', 'Recompile with increased KL0MAX' )
      IF (K1.EQ.1) THEN
C Axially symmetric case; handle separately
        DO 10 KL=1,KL0
          FLA(0,KL)= FPH(1,KL)
  10      FLA(1,KL)= 0.0
      ELSE
C Copy physical values into temporary storage
        DO 20 KL=1,KL0
          DO 20 J=1,K1
   20       TEMPH(KL,J)= FPH(J,KL)
C Fourier transforms of real functions TEMPH(KL,*)
        CALL VRFFTF(KL0, K1, TEMPH, FFWORK, KL0MAX, FFTABV)
C Iterate for each latitude
C Retrieve the real first fourier coefficient
C (Divide by sqrt(K1) for proper normalization)
        RTEM= 1.0 / SQRT(FLOAT(K1))
        DO 30 KL=1,KL0
          FLA(0,KL)= TEMPH(KL,1) * RTEM
   30     FLA(1,KL)= 0.0
C
C Retrieve the next M1 fourier coefficients
        DO 40 KL=1,KL0
          DO 40 J=2,2*M1+1
   40       FLA(J,KL)= TEMPH(KL,J) * RTEM
      ENDIF
      RETURN
      END
 
      SUBROUTINE VPHYS(VFPHK,FSP,K,NQ,L1)
C VPHYS converts the spectral space representation FSP(*,*,1,*) to
C physical space representation VFPHK at all model levels, but for a
C single latitude MU=MU(K).
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX,NQ,L1) -- spectral representation
C                                        of all quantities at all levels
C         INTEGER K -- latitude number (MU = MU(K))
C         INTEGER NQ -- inner dimension of FSP
C                       (usually no. of prognostic quantities)
C         INTEGER L1 -- No. of model levels
C Output: REAL VFPHK(K1MAX,L1) -- physical space representation of
C                                 quantity FSP(*,*,1,*), for MU=MU(K)
C
#include "spgrid.h"
#include "sppoly.h"
      INTEGER K,NQ,L1
      REAL VFPHK(K1MAX,L1)
      COMPLEX FSP(0:M1MAX,0:N1MAX,NQ,L1)
      INTEGER L
      COMPLEX FLA(0:M1MAX,KL0MAX)
C
      IF (L1.GT.KL0MAX)
     1 CALL SPERR('VPHYS', 'Recompile with increased KL0MAX' )
C Convert to latitude space in temporary storage, using SP2LA
#ifndef SINGLELAT
      DO 10 L=1,L1
   10   CALL SP2LA(FLA(0,L),FSP(0,0,1,L),P2(0,K),H2(0,K),N1,K)
#else
      IF(K.NE.KCUR)
     1  CALL SPERR( 'VPHYS', 'Polynomials not initialized for latitude')
      DO 10 L=1,L1
   10   CALL SP2LA(FLA(0,L),FSP(0,0,1,L),PK2,HK2,N1,K)
#endif
C Convert to physical space
      CALL L2P(VFPHK,FLA,L1)
      RETURN
      END
 
      SUBROUTINE VSPEC(FSP,VFPHK,K,NQ,L1)
C VSPEC adds to spectral space representation FSP(*,*,1,*) the
C contribution from physical space representation VFPHK at all model
C levels, but for a single latitude MU=MU(K).
C Input:  COMPLEX FSP(0:M1MAX,0:N1MAX,NQ,L1) -- old spectral
C                         representation of all quantities at all levels
C         REAL VFPHK(K1MAX,L1) -- physical space representation of
C                                 quantity FSP(*,*,1,*), for MU=MU(K)
C         INTEGER K -- latitude number (MU = MU(K))
C         INTEGER NQ -- inner dimension of FSP
C                       (usually no. of prognostic quantities)
C         INTEGER L1 -- No. of model levels
C Output: COMPLEX FSP(0:M1MAX,0:N1MAX,NQ,L1) -- new spectral
C                 representation of all quantities at all levels,
C                 with contribution from VPHK added to FSP(*,*,1,*)
C
#include "spgrid.h"
#include "sppoly.h"
      INTEGER K,NQ,L1
      REAL VFPHK(K1MAX,L1)
      COMPLEX FSP(0:M1MAX,0:N1MAX,NQ,L1)
      INTEGER L
      COMPLEX FLA(0:M1MAX,KL0MAX)
C
      IF (L1.GT.KL0MAX)
     1 CALL SPERR('VSPEC', 'Recompile with increased KL0MAX' )
C Convert to latitude space in temporary storage
      CALL P2L(FLA,VFPHK,L1)
C Convert to spectral space using LA2SP (accumulate)
#ifndef SINGLELAT
      DO 10 L=1,L1
   10   CALL LA2SP(FSP(0,0,1,L),FLA(0,L),P2(0,K),H2(0,K),N1,K)
#else
      IF(K.NE.KCUR)
     1  CALL SPERR( 'VSPEC', 'Polynomials not initialized for latitude')
      DO 10 L=1,L1
   10   CALL LA2SP(FSP(0,0,1,L),FLA(0,L),PK2,HK2,N1,K)
#endif
      RETURN
      END
 
C Diagnostic routines begin here
      SUBROUTINE ZAVGE(FPROF,FPH)
C ZAVGE computes the zonal average FPROF of a global field FPH.
C Input:  REAL FPH(K1MAX,K2MAX) -- values of a function on the globe
C Output: REAL FPROF(K2MAX) -- zonally averaged profile of the function
C                           (FPROF(K) is the average at latitude MU(K) )
C
#include "spgrid.h"
      REAL FPROF(K2MAX), FPH(K1MAX,K2MAX)
      INTEGER K
      REAL TOTAL
      DO 10 K=1,K2
        FPROF(K)= TOTAL(FPH(1,K),K1) / K1
   10   CONTINUE
      RETURN
      END
 
      SUBROUTINE EDDY(EPH,FPH)
C EDDY returns the eddy component EPH of a global field FPH.
C i.e. It subtracts out the zonal mean part.
C Input:  REAL FPH(K1MAX,K2MAX) -- values of a function on the globe
C Output: REAL EPH(K1MAX,K2MAX) -- eddy part of the function FPH
C
#include "spgrid.h"
      REAL EPH(K1MAX,K2MAX), FPH(K1MAX,K2MAX)
      INTEGER J,K
      REAL ZMEAN
      REAL TOTAL
      DO 10 K=1,K2
        ZMEAN= TOTAL(FPH(1,K),K1) / K1
        DO 10 J=1,K1
          EPH(J,K)= FPH(J,K) - ZMEAN
   10     CONTINUE
      RETURN
      END
 
      REAL FUNCTION GAVGE(FPH)
C GAVGE returns the global average of a global field FPH.
C Input:  REAL FPH(K1MAX,K2MAX) -- values of a function on the globe
C Output: Global average of the function
C
#include "spgrid.h"
      REAL FPH(K1MAX,K2MAX)
      REAL FPROF(K2MAX)
      REAL WTOTAL
      CALL ZAVGE(FPROF,FPH)
      GAVGE= WTOTAL(G,FPROF,K2)
      RETURN
      END
 
      REAL FUNCTION TOTAL(F,N)
C TOTAL computes the sum of elements of real array F, of length N
      INTEGER N
      REAL F(N)
      INTEGER J
      REAL SUM
      SUM= 0.0
      DO 10 J=1,N
   10   SUM= SUM + F(J)
      TOTAL=SUM
      RETURN
      END
 
      REAL FUNCTION WTOTAL(W,F,N)
C WTOTAL computes the weighted sum of elements of real array F, using
C the real array of weights W, both of length N.
      INTEGER N
      REAL W(N),F(N)
      INTEGER J
      REAL SUM
      SUM= 0.0
      DO 10 J=1,N
   10   SUM= SUM + W(J) * F(J)
      WTOTAL= SUM
      RETURN
      END
 
      REAL FUNCTION WAVGE(W,F,N)
C WAVGE computes the weighted average of elements of real array F,
C using the real array of weights W, both of length N.
      INTEGER N
      REAL W(N),F(N)
      REAL WTOTAL, TOTAL
      WAVGE= WTOTAL(W,F,N) / TOTAL(W,N)
      RETURN
      END
 
C --------------------------------------------------------------------
C For GNU Emacs:
C $Msg-digest-checksum: a83c5ec6ba89a262a4444bd14db4b3cc $
C --------------------------------------------------------------------
C Local Variables:
C msg-digest-active: t
C End:
C --------------------------------------------------------------------
